package com.example.juanalonsoastruga.cargascontrol.CapaTransversal;


import java.util.HashMap;
import java.util.Map;

/**
 * Clase CodigoCombustibleVehiculo, establece el codigo de combustible de un vehiculo a representar en la aplicacion.
 *
 */
public enum CodigoCombustibleVehiculo {
    CPRIM(0,"CPRIM"),
    CSEC(1,"CSEC");

    private final String descripcion;
    private final int id;

    private static final Map<String,CodigoCombustibleVehiculo> valuesByCode;

    static {
        valuesByCode = new HashMap<>(values().length);
        for(CodigoCombustibleVehiculo value : values()) {
            valuesByCode.put(value.id+"", value);
        }
    }

    /**
     * Constructor de clase.
     *
     * @param descripcion
     * @param descripcion
     */
    CodigoCombustibleVehiculo(int id,String descripcion){
        this.id=id;
        this.descripcion=descripcion;
    }

    /**
     * Metodo getDescripcion, devuelve un String con la descripcion del combustible.
     *
     * @param id Un int con el id del elemento por el que se pregunta
     * @return  Un String con la descripcion
     */
    public  static String getDescripcion(int id){
        return valuesByCode.get(id+"").getDescripcion();
    }

    /**
     * Metodo getDescripcion, devuelve un String con la descripcion del combustible.
     *
     * @return  Un String con la descripcion
     */
    public String getDescripcion(){
        return descripcion;
    }

    /**
     * Metodo getId, devuelve el id asignado a la grafica.
     *
     * @return  Un int con el id
     */
    public int getId(){
        return id;
    }
}