package com.example.juanalonsoastruga.cargascontrol.CapaTransversal;

import java.util.HashMap;
import java.util.Map;

/**
 * Clase CodigoGrafica, establece el codigo de grafica a representar en la aplicacion.
 *
 */
public enum CodigoGrafica {
    GRAFICA1(0,"CONSUMO_COMBUSTIBLE"),
    GRAFICA2(1,"CONSUMO_COMBUSTIBLE_KM"),
    GRAFICA3(2,"COSTE_COMBUSTIBLE_RECARGA"),
    GRAFICA4(3,"KM"),
    GRAFICA5(4,"COSTE_RECARGA");

    private final String descripcion;
    private final int id;

    private static final Map<String,CodigoGrafica> valuesByCode;

    static {
        valuesByCode = new HashMap<>(values().length);
        for(CodigoGrafica value : values()) {
            valuesByCode.put(value.id+"", value);
        }
    }

    /**
     * Constructor de clase.
     *
     * @param descripcion
     * @param descripcion
     */
    CodigoGrafica(int id,String descripcion){
       this.id=id;
       this.descripcion=descripcion;
    }


    /**
     * Metodo getDescripcion, devuelve un String con la descripcion de la grafica.
     *
     * @param id Un int con el id del elemento por el que se pregunta
     * @return  Un String con la descripcion
     */
    public  static String getDescripcion(int id){
        return valuesByCode.get(id+"").getDescripcion();
    }

    /**
     * Metodo getDescripcion, devuelve un String con la descripcion de la grafica.
     *
     * @return  Un String con la descripcion
     */
    public String getDescripcion(){
        return descripcion;
    }

    /**
     * Metodo getId, devuelve el id asignado a la grafica.
     *
     * @return  Un int con el id
     */
    public int getId(){
        return id;
    }
}
