package com.example.juanalonsoastruga.cargascontrol.CapaTransversal;

import java.util.HashMap;
import java.util.Map;

/**
 * Clase CodigoPeriocidadGrafica, establece el codigo de periocidad de la grafica
 *
 */
public enum CodigoPeriocidadGrafica {

    PERIOCIDAD_GRAFICA1(0,"MM"),
    PERIOCIDAD_GRAFICA2(1,"YYYY"),;

    private final String descripcion;
    private final int id;

    private static final Map<String,CodigoPeriocidadGrafica> valuesByCode;

    static {
        valuesByCode = new HashMap<>(values().length);
        for(CodigoPeriocidadGrafica value : values()) {
            valuesByCode.put(value.id+"", value);
        }
    }

    /**
     * Constructor de clase.
     *
     * @param id Un int con el id del elemento por el que se pregunta
     * @param descripcion Un String con la descripcion
     */
    CodigoPeriocidadGrafica (int id,String descripcion){
        this.id=id;
        this.descripcion=descripcion;
    }


    /**
     * Metodo getDescripcion, devuelve un String con la descripcion de la periocidad de la grafica a representar.
     *
     * @param id Un int con el id del elemento por el que se pregunta
     * @return  Un String con la descripcion
     */
    public  static String getDescripcion(int id){
        return valuesByCode.get(id+"").getDescripcion();
    }

    /**
     * Metodo getDescripcion, devuelve un String con la descripcion de la periocidad de la grafica a representar.
     *
     * @return  Un String con la descripcion
     */
    public String getDescripcion(){
        return descripcion;
    }

    /**
     * Metodo getId, devuelve el id asignado a la periocidad de la grafica.
     *
     * @return  Un int con el id
     */
    public int getId(){
        return id;
    }
}
