package com.example.juanalonsoastruga.cargascontrol.Controlador.CUBorrarVehiculo;


import com.example.juanalonsoastruga.cargascontrol.Formateador.CUBorrarVehiculo.InterfaceFBorradoVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Modelo.CUBorrarVehiculo.InterfaceMBorradoVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Modelo.CUBorrarVehiculo.ModeloBorrarVehiculo;

/**
 * Clase ControladorBorradoVehiculo, representa al controlador en el CU Borrar Vehiculo
 *
 */
public class ControladorBorradoVehiculo implements InterfaceCBorradoVehiculo {
    private InterfaceFBorradoVehiculo formateador;
    private InterfaceMBorradoVehiculo modelo;

    /**
     * Contructor de clase
     *
     * @param formateador Objeto InterfaceFBorradoVehiculo
     */
    public ControladorBorradoVehiculo(InterfaceFBorradoVehiculo formateador){
        setFormateador(formateador);
        setModel(new ModeloBorrarVehiculo() );
    }

    /**
     * Establece la formateador asociada a este controlador.
     *
     * @param formateador Objeto que implementa InterfaceFBorradoVehiculo
     */
    @Override
    public void setFormateador(InterfaceFBorradoVehiculo formateador){
        if(formateador==null){
            throw new IllegalArgumentException();
        }else {
            this.formateador = formateador;
        }
    }
    /**
     * Establece el modelo asociada a este controlador.
     *
     * @param modelo Objeto que implementa InterfaceMBorradoVehiculo
     */
    @Override
    public void setModel(InterfaceMBorradoVehiculo modelo){
        if(modelo==null){
            throw new IllegalArgumentException();
        }else {
            this.modelo = modelo;
        }
    }

    /**
     * Metodo getControlador, devuelve la formateador asociada a este controlador.
     *
     * @return  Un objeto InterfaceFBorradoVehiculo
     */
    @Override
    public InterfaceFBorradoVehiculo getFormateador(){
        return formateador;
    }
    /**
     * Metodo getModel, devuelve la formateador asociada a este controlador.
     *
     * @return  Un objeto InterfaceMBorradoVehiculo
     */
    @Override
    public InterfaceMBorradoVehiculo getModel(){
        return modelo;
    }

    /**
     * Metodo botonBorrarSeleccionado, informa al controlador que el boton borrado se ha seleccionado.
     *
     */
    @Override
    public boolean botonBorrarSeleccionado() {
        formateador.muestraDialogoConfirmacionBorrado();
        return true;
    }

    /**
     * Metodo borradoConfirmado, informa al controlador que el borrado se ha confirmado.
     *
     * @param matricula String con la matricula del vehiculo
     */
    @Override
    public boolean borradoConfirmado(String matricula) {
        modelo.borrarVehiculo(matricula);
        return true;
    }
}
