package com.example.juanalonsoastruga.cargascontrol.Controlador.CUCalcularCosteTrayecto;


import com.example.juanalonsoastruga.cargascontrol.Formateador.CUCalcularCosteTrayecto.InterfaceFCalcularCosteTrayecto;
import com.example.juanalonsoastruga.cargascontrol.Modelo.CUCalcularCosteTrayecto.InterfaceMCalcularCosteTrayecto;
import com.example.juanalonsoastruga.cargascontrol.Modelo.CUCalcularCosteTrayecto.ModeloCalcularCosteTrayecto;

/**
 * Clase ControladorCalcularCosteTrayecto, representa al controlador del caso de uso ControladorCalcularCosteTrayecto
 *
 */
public class ControladorCalcularCosteTrayecto implements InterfaceCCalcularCosteTrayecto {

    private InterfaceFCalcularCosteTrayecto formateador;
    private InterfaceMCalcularCosteTrayecto modelo;

    /**
     * Constructor de clase.
     *
     * @param formateador   Objeto InterfaceFCalcularCosteTrayecto
     */
    public ControladorCalcularCosteTrayecto(InterfaceFCalcularCosteTrayecto formateador){
       setFormateador(formateador);
       setModelo(new ModeloCalcularCosteTrayecto());
    }

    /**
     * Metodo setFormateador asocia el formateador especificado a este controlador.
     *
     * @param formateador Objeto InterfaceFCalcularCosteTrayecto
     * @throws IllegalArgumentException si el argumento es nulo
     */
    @Override
    public  void setFormateador(InterfaceFCalcularCosteTrayecto formateador) {
        if(formateador==null){
            throw  new IllegalArgumentException();
        }else{
            this.formateador=formateador;
        }
    }

    /**
     * Metodo getFormateador, devuelve la instancia del formateador asociado a este controlador.
     *
     * @return Objeto InterfaceFCalcularCosteTrayecto
     */
    @Override
    public InterfaceFCalcularCosteTrayecto getFormateador() {
        return formateador;
    }
    /**
     * Metodo getModelo, devuelve la instancia del modelo asociado a este controlador.
     *
     * @return Objeto InterfaceMCalcularCosteTrayecto
     */
    @Override
    public InterfaceMCalcularCosteTrayecto getModelo() {
        return modelo;
    }

    /**
     * Metodo setModelo, establece el modelo asociado a este controlador.
     *
     * @param modelo Objeto InterfaceMCalcularCosteTrayecto
     * @throws IllegalArgumentException si el argumento es nulo
     */
    @Override
    public  void setModelo(InterfaceMCalcularCosteTrayecto modelo) {
        if(modelo==null){
            throw  new IllegalArgumentException();
        }else{
            this.modelo=modelo;
        }
    }

    /**
     * Metodo solicitarGuardado, avisa al controlador del suceso de una solicitud de guardado.
     *
     * @param distancia String con la distancia del trayecto
     * @param precio String con el precio por unidad de recarga
     * @param consumo String con el consumo por kilometro
     */
    @Override
    public void solicitarCalculo(String distancia, String precio, String consumo) {
        if(distancia==null||precio==null||consumo==null){
            formateador.setMensajeError("Por favor complete todos los campos antes de calcular el resultado.");
        }else{
            try {

                double precioXKm = modelo.calculaFactorPrecioKilometro(Double.parseDouble(precio),Double.parseDouble(consumo));
                double precioTrayecto=modelo.calculaPrecioDistancia(precioXKm,Double.parseDouble(distancia));
                formateador.setResultado(precioTrayecto);

            }catch(Exception e){
                formateador.setMensajeError("Por favor, introduzca valores numéricos en los campos");
            }

        }
    }
}
