package com.example.juanalonsoastruga.cargascontrol.Controlador.CUConsultarEstadisticaCombustible;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.CodigoCombustibleVehiculo;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTODatosDerivadosCombustible;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTOVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Formateador.CUConsultarEstadisticaCombustible.InterfaceFConsultarEstadisticaCombustible;
import com.example.juanalonsoastruga.cargascontrol.Modelo.CUConsultarEstadisticaCombustible.InterfaceMConsultarEstadisticaCombustible;
import com.example.juanalonsoastruga.cargascontrol.Modelo.CUConsultarEstadisticaCombustible.ModeloConsultarEstadisticaCombustible;

import java.util.Calendar;

/**
 * Clase ControladorConsultarEstadisticaCombustible, representa al controlador del CU Consultar Estadisticas del
 * Consumo de Combustible.
 *
 */
public class ControladorConsultarEstadisticaCombustible implements InterfaceCConsultarEstadisticaCombustible {

    private InterfaceFConsultarEstadisticaCombustible formateador;
    private InterfaceMConsultarEstadisticaCombustible modelo;

    /**
     * Constructor de clase.
     *
     * @param formateador InterfaceFConsultarEstadisticaCombustible
     */
    public ControladorConsultarEstadisticaCombustible(InterfaceFConsultarEstadisticaCombustible formateador){
        setFormateador(formateador);
        setModel(new ModeloConsultarEstadisticaCombustible());
    }

    /**
     * Metodo setControlador, establece el formateador asociado a este controlador.
     *
     * @param formateador Objeto InterfaceFConsultarEstadisticaCombustible
     * @throws IllegalArgumentException si el argumento es null
     */
    public void setFormateador(InterfaceFConsultarEstadisticaCombustible formateador){
        if(formateador==null){
            throw new IllegalArgumentException();
        }else{
            this.formateador =formateador;
        }
    }

    /**
     * Metodo setModel, establece el modelo asociado a este controlador.
     *
     * @param modelo Objeto InterfaceMConsultarEstadisticaCombustible
     * @throws IllegalArgumentException si el argumento es null
     */
    public void setModel(InterfaceMConsultarEstadisticaCombustible modelo){
        if(modelo==null){
            throw new IllegalArgumentException();
        }else{
            this.modelo=modelo;
        }
    }

    /**
     * Metodo comienzoCU, despierta al controlador para el comienzo del caso de uso.
     *
     * @param matricula La matricula del vehiculo cuys datos se consultan
     * @param id    El id del combustible a consultar
     */
    @Override
    public void comienzoCU(String matricula, int id) {
        DTOVehiculo vehiculo = modelo.getVehiculo(matricula);
        if(vehiculo!=null) {
            String combustible;
            Calendar cal = Calendar.getInstance();
            if (id == (CodigoCombustibleVehiculo.CPRIM.getId())) {
                combustible = vehiculo.getCombustiblePrimario();
            } else {
                combustible = vehiculo.getCombustibleSecundario();
            }
            DTODatosDerivadosCombustible dtoCombustible = modelo.getDatosDerivadosCombustible(matricula, cal.get(Calendar.YEAR), cal.get(Calendar.MONTH), combustible);

            dtoCombustible = calcularDatosDerivados(dtoCombustible);

            formateador.setDatosDerivadosCombustible(dtoCombustible);
        }
    }

    private DTODatosDerivadosCombustible calcularDatosDerivados(DTODatosDerivadosCombustible dtoCombustible) {

        dtoCombustible.setMediaCombustibleAnualRecargado(dtoCombustible.getSumaCombustibleRecargadoPorAnio()/
                                                         dtoCombustible.getNumeroRecargasCombustiblePorAnio());

        dtoCombustible.setMediaCombustibleMensualRecargado(dtoCombustible.getSumaCombustibleRecargadoPorMes()/
                                                         dtoCombustible.getNumeroRecargasCombustiblePorMes());

        dtoCombustible.setMediaCombustibleXKmAnual(dtoCombustible.getSumaCombustibleXKmAnual()/
                                                   dtoCombustible.getNumeroRecargasCombustiblePorAnio());

        dtoCombustible.setMediaCombustibleXKmMensual(dtoCombustible.getSumaCombustibleXKmMensual()/
                                                     dtoCombustible.getNumeroRecargasCombustiblePorMes());

        return dtoCombustible;
    }
}
