package com.example.juanalonsoastruga.cargascontrol.Controlador.CUConsultarEstadisticaCostesCombustible;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.CodigoCombustibleVehiculo;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTODatosDerivadosCostesCombustible;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTOVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Modelo.CUConsultarEstadisticaCostesCombustible.InterfaceMConsultarEstadisticaCosteCombustible;
import com.example.juanalonsoastruga.cargascontrol.Modelo.CUConsultarEstadisticaCostesCombustible.ModeloConsultarEstadisticaCosteCombustible;
import com.example.juanalonsoastruga.cargascontrol.Formateador.CUConsultarEstadisticaCostesCombustible.InterfaceFConsultarEstadisticaCosteCombustible;

import java.util.Calendar;

/**
 * Clase ControladorConsultarEstadisticaCosteCombustible, representa el controlador del Caso de Uso Consultar Estadisticas Coste de Combustible
 *
 */
public class ControladorConsultarEstadisticaCosteCombustible implements InterfaceCConsultarEstadisticaCosteCombustible {

    private InterfaceFConsultarEstadisticaCosteCombustible formateador;
    private InterfaceMConsultarEstadisticaCosteCombustible modelo;

    /**
     * Constructor de clase.
     *
     * @param formateador InterfaceFConsultarEstadisticaCosteCombustible
     */
    public ControladorConsultarEstadisticaCosteCombustible(InterfaceFConsultarEstadisticaCosteCombustible formateador){
        setFormateador(formateador);
        setModel(new ModeloConsultarEstadisticaCosteCombustible());
    }

    /**
     * Metodo setControlador, establece el formateador asociado.
     *
     * @param formateador Objeto InterfaceFConsultarEstadisticaCosteCombustible
     * @throws IllegalArgumentException si el formateador es null
     */
    public void setFormateador(InterfaceFConsultarEstadisticaCosteCombustible formateador){
        if(formateador==null){
            throw new IllegalArgumentException();
        }else{
            this.formateador =formateador;
        }
    }
    /**
     * Metodo setModel, establece el modelo asociado.
     *
     * @param modelo Objeto InterfaceMConsultarEstadisticaCosteCombustible
     * @throws IllegalArgumentException si el modelo es null
     */
    public void setModel(InterfaceMConsultarEstadisticaCosteCombustible modelo){
        if(modelo==null){
            throw new IllegalArgumentException();
        }else{
            this.modelo=modelo;
        }
    }

    /**
     * Metodo comienzaCU, comienza la ejecución del CU despertando al controlador.
     *
     * @param matricula String con la matricula del vehiculo cuya informacion se va a consultar
     * @throws IllegalArgumentException si la matricula es null
     */
    @Override
    public void comienzaCU(String matricula, int id) {
        DTOVehiculo vehiculo = modelo.getVehiculo(matricula);
        if(vehiculo!=null) {
            String combustible;
            Calendar cal = Calendar.getInstance();
            if (id == (CodigoCombustibleVehiculo.CPRIM.getId())) {
                combustible = vehiculo.getCombustiblePrimario();
            } else {
                combustible = vehiculo.getCombustibleSecundario();
            }
            DTODatosDerivadosCostesCombustible dtoCostes = modelo.getDatosDerivadosCostesCombustible(matricula, cal.get(Calendar.YEAR), cal.get(Calendar.MONTH), combustible);

            dtoCostes = calcularCostesDatosDerivados(dtoCostes);

            formateador.setDatosDerivadosCosteCombustible(dtoCostes);
        }
    }

    private DTODatosDerivadosCostesCombustible calcularCostesDatosDerivados(DTODatosDerivadosCostesCombustible dtoCostes) {
        dtoCostes.setMediaCosteRecargaMensual(dtoCostes.getSumaCosteRecargaMensual()/dtoCostes.getNumeroRecargasCombustiblePorMes());
        dtoCostes.setMediaCosteRecargaAnual(dtoCostes.getSumaCosteRecargaAnual()/dtoCostes.getNumeroRecargasCombustiblePorAnio());
        dtoCostes.setMediaCosteXRecargaMensual(dtoCostes.getSumaCosteXRecargaMensual()/dtoCostes.getNumeroRecargasCombustiblePorMes());
        dtoCostes.setMediaCosteXRecargaAnual(dtoCostes.getSumaCosteXRecargaAnual()/dtoCostes.getNumeroRecargasCombustiblePorAnio());

        return dtoCostes;
    }


}
