package com.example.juanalonsoastruga.cargascontrol.Controlador.CUConsultarEstadisticaVehiculo;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTODatosDerivadosVehiculoTotales;
import com.example.juanalonsoastruga.cargascontrol.Modelo.CUConsultarEstadisticaVehiculo.InterfaceMConsultarEstadisticaVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Modelo.CUConsultarEstadisticaVehiculo.ModeloConsultarEstadisticaVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Formateador.CUConsultarEstadisticaVehiculo.InterfaceFConsultarEstadisticaVehiculo;

/**
 * Clase ControladorConsultarEstadisticaVehiculo, controlador en el CU  Consultar Datos Estadisticos del Vehiculo
 *
 */
public class ControladorConsultarEstadisticaVehiculo implements InterfaceCConsultarEstadisticaVehiculo {

    private InterfaceFConsultarEstadisticaVehiculo formateador;
    private InterfaceMConsultarEstadisticaVehiculo modelo;

    /**
     * Constructor de clase.
     *
     * @param formateador Objeto InterfaceFConsultarEstadisticaVehiculo, el formateador asociada a este controlador
     */
    public ControladorConsultarEstadisticaVehiculo(InterfaceFConsultarEstadisticaVehiculo formateador){

        setFormateador(formateador);
        setModel(new ModeloConsultarEstadisticaVehiculo());
    }

    /**
     * Metodo setControlador, establece la formateador asociada.
     *
     * @param formateador Objeto InterfaceFConsultarEstadisticaVehiculo
     * @throws IllegalArgumentException si la formateador es null
     */
    public void setFormateador(InterfaceFConsultarEstadisticaVehiculo formateador){
        if(formateador==null){
            throw new IllegalArgumentException();
        }else{
            this.formateador =formateador;
        }
    }
    /**
     * Metodo setModel, establece el modelo asociado.
     *
     * @param modelo Objeto InterfaceMConsultarEstadisticaVehiculo
     * @throws IllegalArgumentException si el modelo es null
     */
    public void setModel(InterfaceMConsultarEstadisticaVehiculo modelo){
        if(modelo==null){
            throw new IllegalArgumentException();
        }else{
            this.modelo=modelo;
        }
    }

    /**
     * Metodo comienzaCU, comienza la ejecución del CU despertando al controlador.
     *
     * @param matricula String con la matricula del vehiculo cuya informacion se va a consultar
     * @throws IllegalArgumentException si la matricula es null
     */
    @Override
    public void comienzaCU(String matricula) {
        if(matricula==null){
            throw new IllegalArgumentException();
        }else {
            DTODatosDerivadosVehiculoTotales datosEstadisticosVehiculo = modelo.obtenDatosEstadisticosVehiculo(matricula);
            formateador.setDatosDerivados(datosEstadisticosVehiculo);
        }
    }
}
