package com.example.juanalonsoastruga.cargascontrol.Controlador.CUConsultarInformacionGasolinera;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTOGasolinera;
import com.example.juanalonsoastruga.cargascontrol.Formateador.CUConsultarInformacionGasolinera.InterfaceFConsultarInformacionGasolinera;
import com.example.juanalonsoastruga.cargascontrol.Modelo.CUConsultarInformacionGasolinera.InterfaceMConsultarInformacionGasolinera;
import com.example.juanalonsoastruga.cargascontrol.Modelo.CUConsultarInformacionGasolinera.ModeloConsultarInformacionGasolinera;

/**
 * Clase ControladorConsultarInformacionGasolinera, controlador del CU Consultar Informacion de Gasolinera
 *
 */
public class ControladorConsultarInformacionGasolinera implements InterfaceCConsultarInformacionGasolinera {

    private InterfaceFConsultarInformacionGasolinera formateador;
    private InterfaceMConsultarInformacionGasolinera model;

    /**
     *Constructor de clase.
     *
     */
    public ControladorConsultarInformacionGasolinera(InterfaceFConsultarInformacionGasolinera formateador){
        setFormateador(formateador);
        setModel(new ModeloConsultarInformacionGasolinera());
    }

    /**
     *Metodo setControlador, asocia un formateador a este controlador.
     *
     *@param formateador Objeto InterfaceFConsultarInformacionGasolinera representando al formateador
     *@throws IllegalArgumentException si el argumento es nulo
     */
    @Override
    public void setFormateador(InterfaceFConsultarInformacionGasolinera formateador){
        if(formateador ==null){
            throw new IllegalArgumentException();
        }else{
            this.formateador = formateador;
        }

    }

    /**
     * Metodo getControlador, devuelve el formateador asociado a este controlador.
     *
     * @return objeto InterfaceFConsultarInformacionGasolinera
     */
    @Override
    public InterfaceFConsultarInformacionGasolinera getFormateador(){
        return formateador;
    }

    /**
     *Metodo setModel, asocia un modelo a este controlador.
     *
     *@param model Objeto InterfaceMConsultarInformacionGasolinera representando al modelo
     *@throws IllegalArgumentException si el argumento es nulo
     */
    @Override
    public void setModel(InterfaceMConsultarInformacionGasolinera model){
        if(model==null){
            throw new IllegalArgumentException();
        }else{
            this.model=model;
        }

    }

    /**
     * Metodo getModel, devuelve el modelo asociado a este controlador.
     *
     * @return objeto InterfaceMConsultarInformacionGasolinera
     */
    @Override
    public InterfaceMConsultarInformacionGasolinera getModel(){
        return model;
    }


    /**
     * Metodo comienzaCU, despierta al controlador al comenzar la funcionalidad del CU.
     *
     * @param latitud double con la latitud
     * @param longitud double con la longitud
     * @param direccion String con la direccion de la gasolinera
     */
    @Override
    public void comienzaCU(double latitud,double longitud,String direccion) {
        DTOGasolinera tmp=model.getDatosGasolinera(latitud,longitud,direccion);
        formateador.setDatosGasolinera(tmp);
    }

}