package com.example.juanalonsoastruga.cargascontrol.Controlador.CUConsultarInformacionRecarga;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTORecarga;
import com.example.juanalonsoastruga.cargascontrol.Formateador.CUConsultarInformacionRecarga.InterfaceFConsultarInformacionRecarga;
import com.example.juanalonsoastruga.cargascontrol.Modelo.CUConsultarInformacionRecarga.InterfaceMConsultarInformacionRecarga;
import com.example.juanalonsoastruga.cargascontrol.Modelo.CUConsultarInformacionRecarga.ModeloConsultarInformacionRecarga;

import java.util.Date;

/**
 * Clase ControladorConsultarInformacionRecarga, representa al controlador del CU Consultar Informacion de Recarga
 *
 */
public class ControladorConsultarInformacionRecarga implements InterfaceCConsultarInformacionRecarga {

    private InterfaceFConsultarInformacionRecarga formateador;
    private InterfaceMConsultarInformacionRecarga modelo;

    /**
     * Constructor de clase.
     *
     * @param formateador Un objeto InterfaceFConsultarInformacionRecarga como formateador asociada al controlador
     */
    public ControladorConsultarInformacionRecarga(InterfaceFConsultarInformacionRecarga formateador){
        setFormateador(formateador);
        setModel(new ModeloConsultarInformacionRecarga());
    }

    /**
     * Metodo getControlador, devuelve el formateador asociada a este controlador.
     *
     * @return Un objeto InterfaceFConsultarInformacionRecarga
     */
    public InterfaceFConsultarInformacionRecarga getFormateador() {
        return formateador;
    }

    /**
     * Metodo setControlador, establece el formateador asociado a este controlador.
     *
     * @param vista  Un objeto InterfaceFConsultarInformacionRecarga
     * @throws  IllegalArgumentException si la formateador proporcionada es null
     */
    public void setFormateador(InterfaceFConsultarInformacionRecarga vista) {
        if(vista==null){
            throw new IllegalArgumentException();
        }else {
            this.formateador = vista;
        }
    }

    /**
     * Metodo getModel, devuelve el modelo asociado a este controlador.
     *
     * @return  Un objeto InterfaceMConsultarInformacionRecarga
     */
    public InterfaceMConsultarInformacionRecarga getModel() {
        return modelo;
    }

    /**
     * Metodo setModel, asocia el modelo especificado a este controlador.
     *
     * @param modelo   Un objeto InterfaceMConsultarInformacionRecarga
     */
    public void setModel(InterfaceMConsultarInformacionRecarga modelo) {
        if(modelo==null){
            throw new IllegalArgumentException();
        }else {
            this.modelo = modelo;
        }
    }

    /**
     * Metodo comienzoCU, avisa al controlador del comienzo del Caso de Uso.
     *
     * @param matricula Un String con la matricula del vehiculo a consultar
     * @param fecha Un objeto Date con la fecha de la recarga a consultar
     * @param combustible   Un String con el nombre del combustible que se usó en la recarga
     * @throws IllegalArgumentException si alguno de los tres argumentos es null
     */
    @Override
    public void comienzoCU(String matricula, Date fecha, String combustible) {
        if(matricula==null||fecha==null||combustible==null){
            throw new IllegalArgumentException();
        }else{
            DTORecarga recarga = modelo.getRecarga(matricula,fecha,combustible);
            if(recarga!=null){
                formateador.setRecarga(recarga);
            }
        }
    }
    /**
     * Metodo registrarIDRecargaConsultada, despierta al controlador como parte del CU Registrar Ultima Recarga Consultada,
     * indica al modelo asociado que registre el id de la ultima recarga consultada.
     *
     * @param matricula String con la matricula del correspondiente vehiculo al que se le realizo la recarga
     * @param fecha Long con la fecha
     * @param combustible String con el combustible del vehiculo sobre el que se hizo la recarga
     */
    @Override
    public void registrarIDRecargaConsultada(String matricula, Long fecha, String combustible) {
        modelo.registraIDRecarga(matricula,fecha,combustible);
    }
}
