package com.example.juanalonsoastruga.cargascontrol.Controlador.CUConsultarInformacionVehiculo;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTOVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Formateador.CUConsultarInformacionVehiculo.InterfaceFConsultarInformacion;
import com.example.juanalonsoastruga.cargascontrol.Modelo.CUConsultarInformacionVehiculo.InterfaceMConsultarInformacion;
import com.example.juanalonsoastruga.cargascontrol.Modelo.CUConsultarInformacionVehiculo.ModeloConsultarInformacionVehiculo;

/**
 * Clase ControladorConsultarInformacionVehiculo, presentador del CU Consultar Informacion de Vehiculo
 *
 */
public class ControladorConsultarInformacionVehiculo implements InterfaceCConsultarInformacion {
    private InterfaceFConsultarInformacion formateador;
    private InterfaceMConsultarInformacion modelo;

    /**
     * Constructor de clase.
     *
     * @param formateador Objeto InterfaceFConsultarInformacion
     */
    public ControladorConsultarInformacionVehiculo(InterfaceFConsultarInformacion formateador){
        setFormateador(formateador);
        setModel(new ModeloConsultarInformacionVehiculo());
    }

    /**
     * Metodo setVista, establece la formateador asociada al presentador.
     *
     * @param v Objeto InterfaceFConsultarInformacion
     */
    @Override
    public void setFormateador(InterfaceFConsultarInformacion v) {
        if(v==null){
            throw new IllegalArgumentException();
        }else{
            this.formateador =v;
        }
    }
    /**
     * Metodo getVista, devuelve la formateador asociada al presentador.
     *
     * @return  Objeto InterfaceFConsultarInformacion
     */
    @Override
    public InterfaceFConsultarInformacion getFormateador(){
        return formateador;
    }

    /**
     * Metodo getModel, devuelve el modelo asociado al presentador
     *
     * @return  InterfaceMConsultarInformacion
     */
    @Override
    public InterfaceMConsultarInformacion getModel(){
        return modelo;
    }
    /**
     * Metodo setModel, establece el modelo asociada al presentador.
     *
     * @param m Objeto InterfaceMConsultarInformacion
     */
    @Override
    public void setModel( InterfaceMConsultarInformacion m) {
        if(m==null){
            throw new IllegalArgumentException();
        }else{
            this.modelo=m;
        }
    }

    /**
     * Metodo registrarIDVehiculoConsultado,llamado como parte del CU Registrar Vehiculo Consultado, el presentador
     * es despertado para llevar a cabo las acciones de registro del identificador del vehiculo  consultado.
     *
     * @param matricula String con la matricula del vehiculo consultado.
     */
    @Override
    public void registrarIDVehiculoConsultado(String matricula) {
       modelo.registrarIDVehiculoConsultado(matricula);
    }

    /**
     * Metodo activacionCU, despierta al presentador al comenzar la funcionalidad del CU
     *
     * @param matricula La matricula del vehiculo cuya informacion consultar
     */
    @Override
    public void activacionCU(String matricula) {
        DTOVehiculo v=modelo.consultarVehiculo(matricula);
        formateador.setVehiculo(v);
    }
}
