package com.example.juanalonsoastruga.cargascontrol.Controlador.CUConsultarPosicionMapa;

import com.example.juanalonsoastruga.cargascontrol.Formateador.CUConsultarPosicionMapa.InterfaceFConsultarPosicionMapa;
import com.example.juanalonsoastruga.cargascontrol.Modelo.CUConsultarPosicionMapa.InterfaceMConsultarPosicionMapa;
import com.example.juanalonsoastruga.cargascontrol.Modelo.CUConsultarPosicionMapa.ModeloConsultarPosicionMapa;

import java.util.Hashtable;

/**
 * Clase ControladorConsultarPosicionMapa, representa al controlador en el CU Consultar Posicion Actual en el Mapa
 *
 */
public class ControladorConsultarPosicionMapa implements InterfaceCConsultarPosicionMapa{


    private InterfaceFConsultarPosicionMapa formateador;
    private InterfaceMConsultarPosicionMapa modelo;


    /**
     * Constructor de clase.
     *
     * @param formateador Objeto InterfaceFConsultarPosicionMapa
     */
    public ControladorConsultarPosicionMapa (InterfaceFConsultarPosicionMapa formateador){
        setFormateador(formateador);
        setModelo(new ModeloConsultarPosicionMapa());
    }

    /**
     * Metodo setModelo, establece el modelo asociado a este controlador.
     *
     * @param modelo    Objeto InterfaceMConsultarPosicionMapa
     * @throws IllegalArgumentException si el argumento es nulo
     */
    @Override
    public void setModelo(InterfaceMConsultarPosicionMapa modelo){
        if(modelo==null){
            throw new IllegalArgumentException();
        }else{
            this.modelo=modelo;
        }
    }
    /**
     * Metodo getModelo, devuelve el modelo asociado as este controlador.
     *
     * @return objeto InterfaceMConsultarPosicionMapa
     */
    public InterfaceMConsultarPosicionMapa getModelo() {
        return modelo;
    }

    /**
     * Metodo solicitarActualizacion, despierta al controlador ante el suceso del evento de actualizacion
     * de mapa para la posicon actual.
     *
     */
    @Override
    public void solicitarActualizacion() {
        Hashtable posicion=modelo.consultarPosicionActual();
        formateador.setLatitud(Double.parseDouble(posicion.get("latitud")+""));
        formateador.setLongitud(Double.parseDouble(posicion.get("longitud")+""));
    }

    /**
     * Metodo setFormateador, establece el formateador asociado a este controlador.
     *
     * @param formateador InterfaceFConsultarPosicionMapa
     * @throws IllegalArgumentException si el arguemnto es nulo
     */
    @Override
    public void setFormateador(InterfaceFConsultarPosicionMapa formateador){
        if(formateador==null){
            throw new IllegalArgumentException();
        }else{
            this.formateador=formateador;
        }
    }

    /**
     * Metodo getFormateador, devuelve el formateador asociado as este controlador.
     *
     * @return objeto InterfaceFConsultarPosicionMapa
     */
    @Override
    public InterfaceFConsultarPosicionMapa getFormateador(){
        return formateador;
    }

}
