package com.example.juanalonsoastruga.cargascontrol.Controlador.CUConsultarUltimoVehiculo;

import com.example.juanalonsoastruga.cargascontrol.Formateador.CUConsultarUltimoVehiculo.InterfaceFConsultarUltimoVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Modelo.CUConsultarUltimoVehiculo.InterfaceMConsultarUltimoVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Modelo.CUConsultarUltimoVehiculo.ModeloConsultarUltimoVehiculo;

/**
 * Clase ControladorConsultarUltimoVehiculo, representa al controlador en el CU Recuperar Id del Vehiculo de la Ultima Consulta.
 *
 */
public class ControladorConsultarUltimoVehiculo implements InterfaceCConsultarUltimoVehiculo {

    private InterfaceFConsultarUltimoVehiculo formateador;
    private InterfaceMConsultarUltimoVehiculo model;

    /**
     * Contructor de clase.
     *
     * @param formateador objeto InterfaceFConsultarUltimoVehiculo
     */
    public ControladorConsultarUltimoVehiculo(InterfaceFConsultarUltimoVehiculo formateador){

        setFormateador(formateador);
        setModel(new ModeloConsultarUltimoVehiculo());
    }



    /**
     * Metodo setModel, metodo setModel, establece el modelo asociado a aeste controlador.
     *
     * @param model objeto InterfaceMConsultarUltimoVehiculo
     * @throws IllegalArgumentException si el argumento es nulo
     */
    @Override
    public void setModel(InterfaceMConsultarUltimoVehiculo model) {
        if(model==null){
            throw new IllegalArgumentException();
        }else{
            this.model=model;
        }
    }

    /**
     * Metodo getModel, devuelve el modelo asociado a este controlador.
     *
     * @return  Un objeto InterfaceMConsultarUltimoVehiculo
     */
    @Override
    public InterfaceMConsultarUltimoVehiculo getModel(){
        return model;
    }

    /**
     * Metodo existeVehiculo, responde a la exitencia o no del vehiculo especificado.
     *
     * @param idVehiculo    String con el identificador del vehiculo cuya existencia se contempla
     * @return  true si exite, false en caso contrario
     */
    @Override
    public boolean existeVehiculo(String idVehiculo) {
        return model.existeVehiculo(idVehiculo);
    }

    /**
     * Metodo borrarFichero, borra el fichero de registro asociado.
     *
     */
    @Override
    public void borrarFichero() {
        model.borrarFichero();
    }

    /**
     * Metodo setControlador, establece el formateador asociada a este controlador.
     *
     * @param formateador objeto InterfaceFConsultarUltimoVehiculo
     * @throws IllegalArgumentException si el argumento es nulo
     */
    @Override
    public void setFormateador(InterfaceFConsultarUltimoVehiculo formateador) {
        if(formateador==null){
            throw new IllegalArgumentException();
        }else{
            this.formateador =formateador;
        }

    }

    /**
     * Metodo getControlador, devuelve el formateador asociado a este controlador.
     *
     * @return Un objeto InterfaceFConsultarUltimoVehiculo
     */
    @Override
    public InterfaceFConsultarUltimoVehiculo getFormateador(){
        return formateador;
    }

    /**
     * Metodo eventoInicio, metodo para informar del evento al controlador que
     * de comienzo a la ejecucion del CU.
     *
     */
    @Override
    public void eventoInicio() {
       formateador.setIdVehiculo(model.recuperaIdUltimoVehiculoConsultado());
    }
}
