package com.example.juanalonsoastruga.cargascontrol.Controlador.CUGraficarConsumoCombustible;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.CodigoCombustibleVehiculo;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.CodigoPeriocidadGrafica;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTORecarga;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTOVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Modelo.CUGraficarConsumoCombustible.InterfaceMGraficarConsumoCombustible;
import com.example.juanalonsoastruga.cargascontrol.Modelo.CUGraficarConsumoCombustible.ModeloGraficarConsumoCombustible;
import com.example.juanalonsoastruga.cargascontrol.Formateador.CUGraficarConsumoCombustible.InterfaceFGraficarConsumoCombustible;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

/**
 * Clase ControladorGraficarConsumoCombustible, representa al Controlador en el CU Graficar Consumo Combustible
 *
 */
public class ControladorGraficarConsumoCombustible implements InterfaceCGraficarConsumoCombustible {

    private InterfaceFGraficarConsumoCombustible formateador;
    private InterfaceMGraficarConsumoCombustible model;
    private CodigoPeriocidadGrafica periodicidad;

    /**
     * Constructor de clase.
     *
     * @param formateador  Un Objeto InterfaceFGraficarConsumoCombustible representando al formateador asociada
     */
   public ControladorGraficarConsumoCombustible(InterfaceFGraficarConsumoCombustible formateador){
       setFormateador(formateador);
       setModel(new ModeloGraficarConsumoCombustible());
   }

    /**
     * Metodo setControlador, establece el formateador asociada al controlador.
     *
     * @param formateador    Un Objeto InterfaceFGraficarConsumoCombustible representando al formateador asociado
     * @throws IllegalArgumentException si el argumento es null
     */
   public void setFormateador(InterfaceFGraficarConsumoCombustible formateador){
       if(formateador ==null){
           throw new IllegalArgumentException();
       }else {
           this.formateador = formateador;
       }
   }
    /**
     * Metodo setModel, establece el modelo asociado al controlador.
     *
     * @param model    Un Objeto InterfaceMGraficarConsumoCombustible representando el modelo asociado
     * @throws IllegalArgumentException si el argumento es null
     */
    public void setModel (InterfaceMGraficarConsumoCombustible model){
        if(model==null){
            throw new IllegalArgumentException();
        }else {
            this.model = model;
        }
    } /**
     * Metodo getModel, devuelve el modelo asociada al controlador.
     *
     * @return Un Objeto InterfaceMGraficarConsumoCombustible
     */

    public InterfaceMGraficarConsumoCombustible getModel(){
       return model;
    }
    /**
     * Metodo getControlador, devuelve el formateador asociada al controlador.
     *
     * @return Un Objeto InterfaceFGraficarConsumoCombustible
     */
   public InterfaceFGraficarConsumoCombustible getFormateador(){
       return formateador;
   }


    /**
     * Metodo comienzaCU, despierta al controlador para iniciar la ejecución del Caso de Uso.
     *
     * @param periodicidad  Un CodigoPeriocidadGrafica con la periodicidad establecida
     */
    @Override
    public void comienzaCU(CodigoPeriocidadGrafica periodicidad) {
        this.periodicidad=periodicidad;
        CodigoCombustibleVehiculo codCombustible= formateador.getCodigoCombustible();
        DTOVehiculo vehiculo=model.getVehiculo(formateador.getMatricula());
        List<DTORecarga> recargas;
        if(periodicidad.getDescripcion().equals("MM")){
            Calendar calIni = Calendar.getInstance();
            calIni.setTime(new Date());
            calIni.set(Calendar.DAY_OF_MONTH,1);
            calIni.set(Calendar.HOUR_OF_DAY,0);
            calIni.set(Calendar.MINUTE,0);
            calIni.set(Calendar.SECOND,0);
            int numDias = calIni.getActualMaximum(Calendar.DAY_OF_MONTH);
            Calendar calFin = Calendar.getInstance();
            calFin.set(Calendar.DAY_OF_MONTH,numDias);
            calFin.set(Calendar.HOUR_OF_DAY,23);
            calFin.set(Calendar.MINUTE,59);
            calFin.set(Calendar.SECOND,59);
            Date dateIni,dateFin;
            dateIni=calIni.getTime();
            dateFin=calFin.getTime();
            if(codCombustible.getDescripcion().equals("CPRIM")) {
                recargas = model.getRecargasMensualesCombustible(vehiculo.getMatricula(), vehiculo.getCombustiblePrimario(), dateIni, dateFin);
            }else{
                recargas = model.getRecargasMensualesCombustible(vehiculo.getMatricula(), vehiculo.getCombustibleSecundario(), dateIni, dateFin);

            }
            if(recargas==null){
                recargas=new ArrayList<>(0);
            }
            recargas=establecePeriocidadMensual(recargas,numDias);
            setDatosVista(recargas);
        }else{
            Calendar calIni = Calendar.getInstance();
            calIni.setTime(new Date());
            calIni.set(Calendar.DAY_OF_MONTH,1);
            calIni.set(Calendar.HOUR_OF_DAY,0);
            calIni.set(Calendar.MINUTE,0);
            calIni.set(Calendar.SECOND,0);
            calIni.set(Calendar.MONTH,0);
            Calendar calFin = Calendar.getInstance();
            int numMeses = calFin.getActualMaximum(Calendar.MONTH);
            calFin.set(Calendar.HOUR_OF_DAY,23);
            calFin.set(Calendar.MINUTE,59);
            calFin.set(Calendar.SECOND,59);
            calFin.set(Calendar.MONTH,numMeses);
            calFin.set(Calendar.DAY_OF_MONTH,calFin.getActualMaximum(Calendar.DAY_OF_MONTH));
            Date dateIni,dateFin;
            dateIni=calIni.getTime();
            dateFin=calFin.getTime();
            if(codCombustible.getDescripcion().equals("CPRIM")) {
                recargas = model.getRecargasAnualesCombustible(vehiculo.getMatricula(), vehiculo.getCombustiblePrimario(), dateIni, dateFin);
            }else{
                recargas = model.getRecargasAnualesCombustible(vehiculo.getMatricula(), vehiculo.getCombustibleSecundario(), dateIni, dateFin);
            }
            if(recargas==null){
                recargas=new ArrayList<>(0);
            }
            recargas=establecePeriocidadAnual(recargas,dateIni,dateFin);
            setDatosVista(recargas);
        }

    }

    /**
     * Metodo setDatosVista, proporciona al formateador asociada los datos necesarios a graficar.
     *
     * @param recargas  Un Objeto List<DTORecarga>
     */
    @Override
    public void setDatosVista(List<DTORecarga> recargas){
        formateador.preparaDatosVisualizacion(recargas);
    }

    private List<DTORecarga> establecePeriocidadAnual(List<DTORecarga> r,Date dateIni, Date dateFin) {
        DTORecarga tmp;
        List<DTORecarga> copia=new ArrayList<>(0);
        Calendar cal = Calendar.getInstance();
        Calendar calComparacion = Calendar.getInstance();
        int posicion = 0;
        List<DTORecarga> tmpArray ;
        for(int i=0;i<r.size();i++){
            tmp=r.get(i);
            if(tmp!=null&&i+1<r.size()){
                tmpArray = r.subList(i+1, r.size());
                for (int j = 0; j < tmpArray.size(); j++) {
                    cal.setTime(tmp.getFecha());
                    calComparacion.setTime(tmpArray.get(j).getFecha());
                    if (cal.get(Calendar.MONTH) == calComparacion.get(Calendar.MONTH)) {
                        tmp.setMedicion(tmp.getMedicion() + tmpArray.get(j).getMedicion());
                    }
                }
            }
        }

        copia=new ArrayList<>(0);
        int mesIni=0;
        int numMes=12;
        for (int i=0;i<r.size();i++){
            tmp=r.get(i);
            if(tmp!=null){
                cal.setTime(tmp.getFecha());
                for(int j=mesIni;j<numMes;j++){
                    if(j<cal.get(Calendar.MONTH)){
                        copia.add(null);
                    }else{
                        if(j==cal.get(Calendar.MONTH)){
                            copia.add(tmp);
                            mesIni=j+1;
                        }
                    }
                }
            }
        }
        return copia;
    }

    private List<DTORecarga> establecePeriocidadMensual(List<DTORecarga>r,int numDiasFin){
        DTORecarga tmp;
        List<DTORecarga> copia=new ArrayList<>(0);
        Calendar cal = Calendar.getInstance();
        Calendar calComparacion = Calendar.getInstance();
        int posicion = 0;
        int fechaIni = 1;
        List<DTORecarga> tmpArray ;
        for(int i=0;i<r.size();i++){
            tmp=r.get(i);
            if(tmp!=null&&i+1<r.size()){
                tmpArray = r.subList(i+1, r.size());
                for (int j = 0; j < tmpArray.size(); j++) {
                    cal.setTime(tmp.getFecha());
                    calComparacion.setTime(tmpArray.get(j).getFecha());
                    if (cal.get(Calendar.DAY_OF_MONTH) == calComparacion.get(Calendar.DAY_OF_MONTH)) {
                        tmp.setMedicion(tmp.getMedicion() + tmpArray.get(j).getMedicion());
                    }
                }
            }
        }



        copia=new ArrayList<>(0);
        for (int i=0;i<r.size();i++){
            tmp=r.get(i);
            if(tmp!=null) {
                cal.setTime(tmp.getFecha());

                for (int j = fechaIni; j < numDiasFin + 1; j++) {
                    if (j < cal.get(Calendar.DAY_OF_MONTH)) {
                        copia.add(null);
                    } else {
                        if (j == cal.get(Calendar.DAY_OF_MONTH)) {
                            copia.add(tmp);
                            fechaIni = j + 1;

                            break;
                        }
                    }
                }
            }
        }

        return copia;
    }



}
