package com.example.juanalonsoastruga.cargascontrol.Controlador.CUGraficarConsumoCombustibleXKm;

import android.util.Log;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.CodigoCombustibleVehiculo;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.CodigoPeriocidadGrafica;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTORecarga;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTOVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Modelo.CUGraficarConsumoCombustibleXKm.InterfaceMGraficarConsumoCombustibleKm;
import com.example.juanalonsoastruga.cargascontrol.Modelo.CUGraficarConsumoCombustibleXKm.ModeloGraficarConsumoCombustibleKm;
import com.example.juanalonsoastruga.cargascontrol.Formateador.CUGraficarConsumoCombustibleXKm.InterfaceFGraficarConsumoCombustibleKm;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.Hashtable;
import java.util.List;
/**
 * Clase ControladorGraficarConsumoCombustibleKm, representa al Controlador en el CU Graficar Consumo Combustible por Km recorrido
 *
 */
public class ControladorGraficarConsumoCombustibleKm implements InterfaceCGraficarConsumoCombustibleKm {
    private InterfaceFGraficarConsumoCombustibleKm formateador;
    private InterfaceMGraficarConsumoCombustibleKm model;

    /**
     * Constructor de clase.
     *
     * @param formateador  Un Objeto InterfaceFGraficarConsumoCombustibleKm representando al formateador asociado
     */
    public ControladorGraficarConsumoCombustibleKm(InterfaceFGraficarConsumoCombustibleKm formateador){
        setFormateador(formateador);
        setModel(new ModeloGraficarConsumoCombustibleKm());
    }

    /**
     * Metodo setControlador, establece el formateador asociada al controlador.
     *
     * @param formateador    Un Objeto InterfaceFGraficarConsumoCombustibleKm representando el formateador asociado
     * @throws IllegalArgumentException si el argumento es null
     */
    public void setFormateador(InterfaceFGraficarConsumoCombustibleKm formateador){
        if(formateador ==null){
            throw new IllegalArgumentException();
        }else {
            this.formateador = formateador;
        }
    }

    /**
     * Metodo setModel, establece el modelo asociado al controlador.
     *
     * @param model    Un Objeto InterfaceMGraficarConsumoCombustibleKm representando el modelo asociado
     * @throws IllegalArgumentException si el argumento es null
     */
    public void setModel (InterfaceMGraficarConsumoCombustibleKm model){
        if(model==null){
            throw new IllegalArgumentException();
        }else {
            this.model = model;
        }
    }

    /**
     * Metodo getModel, devuelve el modelo asociada al controlador.
     *
     * @return Un Objeto InterfaceMGraficarConsumoCombustibleKm
     */
    public InterfaceMGraficarConsumoCombustibleKm getModel(){
        return model;
    }

    /**
     * Metodo getControlador, devuelve el formateador asociada al controlador.
     *
     * @return Un Objeto InterfaceFGraficarConsumoCombustibleKm
     */
    public InterfaceFGraficarConsumoCombustibleKm getFormateador(){
        return formateador;
    }


    /**
     * Metodo comienzaCU, despierta al controlador para iniciar la ejecución del Caso de Uso.
     *
     * @param periodicidad  Un String con la periodicidad establecida
     */
    @Override
    public void comienzaCU(CodigoPeriocidadGrafica periodicidad) {

        CodigoCombustibleVehiculo codCombustible= formateador.getCodigoCombustible();
        DTOVehiculo vehiculo=model.getVehiculo(formateador.getMatricula());
        double kmInicio=0;
        List<DTORecarga> recargas;
        double capacidadTanque=0;
        if(periodicidad.getDescripcion().equals("MM")){
            Calendar calIni = Calendar.getInstance();
            calIni.setTime(new Date());
            calIni.set(Calendar.DAY_OF_MONTH,1);
            calIni.set(Calendar.HOUR_OF_DAY,0);
            calIni.set(Calendar.MINUTE,0);
            calIni.set(Calendar.SECOND,0);


            int numDias = calIni.getActualMaximum(Calendar.DAY_OF_MONTH);
            Calendar calFin = Calendar.getInstance();
            calFin.set(Calendar.HOUR_OF_DAY,23);
            calFin.set(Calendar.MINUTE,59);
            calFin.set(Calendar.SECOND,59);
            calFin.set(Calendar.DAY_OF_MONTH,numDias);
            Date dateIni,dateFin;
            dateIni=calIni.getTime();
            dateFin=calFin.getTime();

            DTORecarga tmp;
            if(codCombustible.getDescripcion().equals("CPRIM")) {
                tmp = model.getRecargaMasLejanaAntesDe(formateador.getMatricula(), vehiculo.getCombustiblePrimario(), dateIni);
                capacidadTanque= vehiculo.getCapTankPrimario();
            }else{
                tmp = model.getRecargaMasLejanaAntesDe(formateador.getMatricula(), vehiculo.getCombustibleSecundario(), dateIni);
                capacidadTanque=vehiculo.getCapTankSecundario();
            }
            if(tmp==null){
                kmInicio=vehiculo.getKilometraje();
            }else{
                kmInicio=(tmp.getKm()==null?0.0:tmp.getKm());
            }
            Log.d("TAG","KM INICIO "+kmInicio);
            if(codCombustible.getDescripcion().equals("CPRIM")) {
                recargas = model.getRecargasMensualesCombustible(vehiculo.getMatricula(), vehiculo.getCombustiblePrimario(), dateIni, dateFin);
             }else{
                recargas = model.getRecargasMensualesCombustible(vehiculo.getMatricula(), vehiculo.getCombustibleSecundario(), dateIni, dateFin);
             }
            if(recargas==null){
                recargas=new ArrayList<>(0);
            }
            Log.d("TAG","NUMERO RECARGAS "+recargas.size());
            Log.d("TAG","Combustible "+capacidadTanque);
            recargas=establecePeriocidadMensual(recargas,numDias,kmInicio, capacidadTanque);
            Hashtable hash;
            hash =getResultadosRepresentables(recargas);
            formateador.preparaDatosVisualizacion(hash,recargas.size());
        }else{
            Calendar calIni = Calendar.getInstance();
            calIni.setTime(new Date());
            calIni.set(Calendar.DAY_OF_MONTH,1);
            calIni.set(Calendar.MONTH,0);
            Calendar calFin = Calendar.getInstance();
            int numMeses = calFin.getActualMaximum(Calendar.MONTH);
            calFin.set(Calendar.MONTH,numMeses);
            calFin.set(Calendar.DAY_OF_MONTH,calFin.getActualMaximum(Calendar.DAY_OF_MONTH));
            Date dateIni,dateFin;
            dateIni=calIni.getTime();
            dateFin=calFin.getTime();
            DTORecarga tmp;
            if(codCombustible.getDescripcion().equals("CPRIM")) {
                tmp = model.getRecargaMasLejanaAntesDe(formateador.getMatricula(), vehiculo.getCombustiblePrimario(), dateIni);
                capacidadTanque= vehiculo.getCapTankPrimario();
            }else{
                tmp = model.getRecargaMasLejanaAntesDe(formateador.getMatricula(), vehiculo.getCombustibleSecundario(), dateIni);
                capacidadTanque= vehiculo.getCapTankSecundario();
            }
            if(tmp==null){
                kmInicio=vehiculo.getKilometraje();
            }else{
                kmInicio=(tmp.getKm()==null?0.0:tmp.getKm());
            }
            if(codCombustible.getDescripcion().equals("CPRIM")) {
                recargas = model.getRecargasAnualesCombustible(vehiculo.getMatricula(), vehiculo.getCombustiblePrimario(), dateIni, dateFin);
            }else{
                recargas = model.getRecargasAnualesCombustible(vehiculo.getMatricula(), vehiculo.getCombustibleSecundario(), dateIni, dateFin);
            }
            if(recargas==null){
                recargas=new ArrayList<>(0);
            }
            Log.d("TAG","Combustible "+capacidadTanque);
            recargas=establecePeriocidadAnual(recargas,dateIni,dateFin,kmInicio,capacidadTanque);
            Hashtable hash;
            hash =getResultadosRepresentables(recargas);
            formateador.preparaDatosVisualizacion(hash,recargas.size());
        }

    }


    private Hashtable<Integer,Double> getResultadosRepresentables(List<DTORecarga> recargas){
        Hashtable hash;
        hash = new <Integer,Double>Hashtable();
        Double medicion;
        Double km;
        double division=0;

        for(int i=0;i<recargas.size();i++){
            try {
                medicion = ( recargas.get(i).getMedicion());
                km= ( recargas.get(i).getKm());
                division=medicion/km;
            }catch(NullPointerException e){
                medicion= 0.0;
                km= 0.0;
            }
            if(medicion==0||km==0){
                division=0;
            }

            hash.put(i+1, (division) );
        }
        return hash;
    }


    private List<DTORecarga> establecePeriocidadMensual(List<DTORecarga>r,int numDiasFin,double kmARestar,double capacidadTanque){
        DTORecarga tmp=null;
        List<DTORecarga> copia=new ArrayList<>(0);


        List<DTORecarga> tmpArray ;

        Calendar cal = Calendar.getInstance();
        Calendar calComparacion = Calendar.getInstance();

        double valInicialKm=0;

        int fechaIni = 1;
        double kM=0;

        Log.d("TAG","tamaño array "+r.size()+" val km inicial "+kmARestar);
        double combustibleGastado=0;


        for(int i=0;i<r.size();i++){
            combustibleGastado=capacidadTanque-r.get(i).getMedicion();
            if(combustibleGastado==0){
                combustibleGastado=capacidadTanque;
            }
            r.get(i).setMedicion(combustibleGastado);
            Log.d("TAG","combustibleGastado "+combustibleGastado);
        }







        for(int i=0;i<r.size();i++){
            tmp=r.get(i);
            if(tmp!=null&&i+1<r.size()){
                tmpArray = r.subList(i+1, r.size());
                for (int j = 0; j < tmpArray.size(); j++) {
                    cal.setTime(tmp.getFecha());
                    calComparacion.setTime(tmpArray.get(j).getFecha());
                    if (cal.get(Calendar.DAY_OF_MONTH) == calComparacion.get(Calendar.DAY_OF_MONTH)) {

                        tmp.setMedicion(tmp.getMedicion()+tmpArray.get(j).getMedicion() );
                    }
                }
            }
            if(r.size()==1){
                tmp=r.get(0) ;
                combustibleGastado=capacidadTanque-tmp.getMedicion();
            }
        }
        int penultimoDia=0;
        int ultimoDia=0;

        for(int i=0;i<r.size();i++){
            tmp=r.get(i);
            if(tmp!=null&&i+1<r.size()) {

                kM=tmp.getKm();
                tmp.setKm(tmp.getKm()-kmARestar);
                kmARestar=kM;
                tmpArray = r.subList(i+1, r.size());
                Log.d("TAG","km tmp "+tmp.getKm());
                cal.setTime(tmp.getFecha());
                ultimoDia=cal.get(Calendar.DAY_OF_MONTH);
                for (int j = 0; j < tmpArray.size(); j++) {
                    calComparacion.setTime(tmpArray.get(j).getFecha());
                    penultimoDia=calComparacion.get(Calendar.DAY_OF_MONTH);
                    if (cal.get(Calendar.DAY_OF_MONTH) == calComparacion.get(Calendar.DAY_OF_MONTH)) {
                        Log.d("TAG","km j "+ tmpArray.get(j).getKm());
                        tmp.setKm(tmp.getKm() + tmpArray.get(j).getKm()-kmARestar);
                        Log.d("TAG","km acumulados "+ tmp.getKm() );
                        kmARestar=tmpArray.get(j).getKm();
                        r.set(i+j+1,null);

                    }
                }
            }
            if(!(i+1<r.size())&&(penultimoDia!=ultimoDia)){
                r.get(r.size()-1).setKm(r.get(r.size()-1).getKm()-kmARestar);
            }
            if(r.size()==1){
                r.get(0).setKm(r.get(0).getKm()-kmARestar);
            }
        }


        //RELLENA LOS DIAS SIN REGISTRO CON NULL


        for(int i=0;i<r.size();i++){
            tmp=r.get(i);
            if(tmp!=null)
                Log.d("TAG","MSG1 "+tmp.getFecha().toGMTString()+" "+tmp.getMedicion()+" "+tmp.getKm());
        }

        copia=r;
       // r=copia;
        copia=new ArrayList<>(0);
        for (int i=0;i<r.size();i++){
            tmp=r.get(i);
            if(tmp!=null) {
                cal.setTime(tmp.getFecha());
                for (int j = fechaIni; j < numDiasFin + 1; j++) {
                    if (j < cal.get(Calendar.DAY_OF_MONTH)) {
                        copia.add(null);
                    } else {
                        if (j == cal.get(Calendar.DAY_OF_MONTH)) {
                            copia.add(tmp);
                            fechaIni = j + 1;
                            break;
                        }
                    }
                }
            }
        }
        return copia;
    }
    private List<DTORecarga> establecePeriocidadAnual(List<DTORecarga> r,Date dateIni, Date dateFin,double kmARestar,double capacidadTanque) {
        DTORecarga tmp=null;
        List<DTORecarga> copia=new ArrayList<>(0);


        List<DTORecarga> tmpArray ;
        double combustibleGastado, combustibleGastadoTmp;
        Calendar cal = Calendar.getInstance();
        Calendar calComparacion = Calendar.getInstance();

        double valInicialKm=0;

        int fechaIni = 1;
        double kM=0;

        Log.d("TAG","tamaño array "+r.size()+" val km inicial "+valInicialKm);


        for(int i=0;i<r.size();i++){
            combustibleGastado=capacidadTanque-r.get(i).getMedicion();
            if(combustibleGastado==0){
                combustibleGastado=capacidadTanque;
            }
            r.get(i).setMedicion(combustibleGastado);
            Log.d("TAG","combustibleGastado "+combustibleGastado);
        }

        for(int i=0;i<r.size();i++){
            tmp=r.get(i);
            if(tmp!=null&&i+1<r.size()){
                tmpArray = r.subList(i+1, r.size());
                for (int j = 0; j < tmpArray.size(); j++) {
                    cal.setTime(tmp.getFecha());
                    calComparacion.setTime(tmpArray.get(j).getFecha());
                    if (cal.get(Calendar.MONTH) == calComparacion.get(Calendar.MONTH)) {


                        tmp.setMedicion(tmp.getMedicion()+tmpArray.get(j).getMedicion() );

                    }
                }
            }
        }
        int penultimoMes=0;
        int ultimoMes=0;

        for(int i=0;i<r.size();i++){
            tmp=r.get(i);
            if(tmp!=null&&i+1<r.size()) {

                kM=tmp.getKm();
                tmp.setKm(tmp.getKm()-kmARestar);
                kmARestar=kM;
                tmpArray = r.subList(i+1, r.size());
                Log.d("TAG","km tmp "+tmp.getKm());
                cal.setTime(tmp.getFecha());
                ultimoMes=cal.get(Calendar.MONTH);
                for (int j = 0; j < tmpArray.size(); j++) {
                    calComparacion.setTime(tmpArray.get(j).getFecha());
                    penultimoMes=calComparacion.get(Calendar.MONTH);
                    if (cal.get(Calendar.MONTH) == calComparacion.get(Calendar.MONTH)) {
                        Log.d("TAG","km j "+ tmpArray.get(j).getKm());
                        tmp.setKm(tmp.getKm() + tmpArray.get(j).getKm()-kmARestar);
                        Log.d("TAG","km acumulados "+ tmp.getKm() );
                        kmARestar=tmpArray.get(j).getKm();
                        r.set(i+j+1,null);

                    }
                }
            }
            if(!(i+1<r.size())&&(penultimoMes!=ultimoMes)){
                r.get(r.size()-1).setKm(r.get(r.size()-1).getKm()-kmARestar);
            }
            if(r.size()==1){
                r.get(0).setKm(r.get(0).getKm()-kmARestar);
            }
        }

        for(int i=0;i<r.size();i++){
            tmp=r.get(i);
            if(tmp!=null)
                Log.d("TAG","MSG1 "+tmp.getFecha().toGMTString()+" "+tmp.getMedicion()+" "+tmp.getKm());
        }

        //RELLENA LOS DIAS SIN REGISTRO CON NULL


        copia=new ArrayList<>(0);
        int mesIni=0;
        int numMes=12;
        for (int i=0;i<r.size();i++){
            tmp=r.get(i);
            if(tmp!=null) {
                cal.setTime(tmp.getFecha());
                for (int j = mesIni; j < numMes; j++) {
                    if (j < cal.get(Calendar.MONTH)) {
                        copia.add(null);
                    } else {
                        if (j == cal.get(Calendar.MONTH)) {
                            copia.add(tmp);
                            mesIni = j + 1;
                        }
                    }
                }
            }
        }
        return copia;
    }

}
