package com.example.juanalonsoastruga.cargascontrol.Controlador.CUGraficarKm;

import android.util.Log;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.CodigoCombustibleVehiculo;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.CodigoPeriocidadGrafica;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTORecarga;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTOVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Modelo.CUGraficarKm.InterfaceMGraficarKm;
import com.example.juanalonsoastruga.cargascontrol.Modelo.CUGraficarKm.ModeloGraficarKm;
import com.example.juanalonsoastruga.cargascontrol.Formateador.CUGraficarKm.InterfaceFGraficarKm;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
/**
 * Clase ControladorGraficarKm, representa al Controlador en el CU Graficar Kilometros recorridos
 *
 */
public class ControladorGraficarKm implements InterfaceCGraficarKm {

    private InterfaceFGraficarKm formateador;
    private InterfaceMGraficarKm model;

    /**
     * Constructor de clase.
     *
     * @param formateador  Un Objeto InterfaceFGraficarKm representando la vista asociada
     */
    public ControladorGraficarKm(InterfaceFGraficarKm formateador){
        setFormateador(formateador);
        setModel(new ModeloGraficarKm());
    }


    /**
     * Metodo setControlador, establece el formateador asociado al controlador.
     *
     * @param formateador    Un Objeto InterfaceFGraficarKm representando al formateador asociado
     * @throws IllegalArgumentException si el argumento es null
     */
    public void setFormateador(InterfaceFGraficarKm formateador){
        if(formateador==null){
            throw new IllegalArgumentException();
        }else {
            this.formateador = formateador;
        }
    }


    /**
     * Metodo setModel, establece el modelo asociada al controlador.
     *
     * @param model    Un Objeto InterfaceMGraficarKm representando el modelo asociada
     * @throws IllegalArgumentException si el argumento es null
     */
    public void setModel (InterfaceMGraficarKm model){
        if(model==null){
            throw new IllegalArgumentException();
        }else {
            this.model = model;
        }
    }


    /**
     * Metodo getModel, devuelve el modelo asociada al presentador.
     *
     * @return Un Objeto InterfaceMGraficarKm
     */
    public InterfaceMGraficarKm getModel(){
        return model;
    }


    /**
     * Metodo getControlador, devuelve la vista asociada al presentador.
     *
     * @return Un Objeto InterfaceFGraficarKm
     */
    public InterfaceFGraficarKm getFormateador(){
        return formateador;
    }


    /**
     * Metodo comienzaCU, despierta al controlador para iniciar la ejecución del Caso de Uso.
     *
     * @param periodicidad  Un CodigoPeriocidadGrafica con la periodicidad establecida
     */
    @Override
    public void comienzaCU(CodigoPeriocidadGrafica periodicidad) {

        double kmInicio=0;

        CodigoCombustibleVehiculo codCombustible= formateador.getCodigoCombustible();
        DTOVehiculo vehiculo=model.getVehiculo(formateador.getMatricula());
        List<DTORecarga> recargas;
        if(periodicidad.getDescripcion().equals("MM")){
            Calendar calIni = Calendar.getInstance();
            calIni.setTime(new Date());
            calIni.set(Calendar.DAY_OF_MONTH,1);
            calIni.set(Calendar.HOUR_OF_DAY,0);
            calIni.set(Calendar.MINUTE,0);
            calIni.set(Calendar.SECOND,0);
            int numDias = calIni.getActualMaximum(Calendar.DAY_OF_MONTH);
            Calendar calFin = Calendar.getInstance();
            calFin.set(Calendar.DAY_OF_MONTH,numDias);
            calFin.set(Calendar.HOUR_OF_DAY,23);
            calFin.set(Calendar.MINUTE,59);
            calFin.set(Calendar.SECOND,59);
            Date dateIni,dateFin;
            dateIni=calIni.getTime();
            dateFin=calFin.getTime();
            DTORecarga tmp;


            if(codCombustible.getDescripcion().equals("CPRIM")) {
                recargas = model.getRecargasMensualesCombustible(vehiculo.getMatricula(), vehiculo.getCombustiblePrimario(), dateIni, dateFin);
                tmp =model.getUltimaRecargaCombustibleAntesDe(formateador.getMatricula(),dateIni,vehiculo.getCombustiblePrimario());
            }else{
                recargas = model.getRecargasMensualesCombustible(vehiculo.getMatricula(), vehiculo.getCombustibleSecundario(), dateIni, dateFin);
                tmp =model.getUltimaRecargaCombustibleAntesDe(formateador.getMatricula(),dateIni, vehiculo.getCombustibleSecundario());

            }
            if(tmp==null){
                kmInicio=vehiculo.getKilometraje();
            }else{
                kmInicio=(tmp.getKm()==null?0.0:tmp.getKm());
            }

            if(recargas==null){
                recargas=new ArrayList<>(0);
            }
            recargas=establecePeriocidadMensual(recargas,numDias,kmInicio);
            formateador.preparaDatosVisualizacion(recargas);
        }else{
            Calendar calIni = Calendar.getInstance();
            calIni.setTime(new Date());
            calIni.set(Calendar.DAY_OF_MONTH,1);
            calIni.set(Calendar.HOUR_OF_DAY,0);
            calIni.set(Calendar.MINUTE,0);
            calIni.set(Calendar.SECOND,0);
            calIni.set(Calendar.MONTH,0);
            Calendar calFin = Calendar.getInstance();
            int numMeses = calFin.getActualMaximum(Calendar.MONTH);

            calFin.set(Calendar.MONTH,numMeses);
            calFin.set(Calendar.DAY_OF_MONTH,calFin.getActualMaximum(Calendar.DAY_OF_MONTH));
            calFin.set(Calendar.HOUR_OF_DAY,23);
            calFin.set(Calendar.MINUTE,59);
            calFin.set(Calendar.SECOND,59);
            Date dateIni,dateFin;
            dateIni=calIni.getTime();
            dateFin=calFin.getTime();
            DTORecarga tmp;


            if(codCombustible.getDescripcion().equals("CPRIM")) {
                recargas = model.getRecargasAnualesCombustible(vehiculo.getMatricula(), vehiculo.getCombustiblePrimario(), dateIni, dateFin);
                tmp =model.getUltimaRecargaCombustibleAntesDe(formateador.getMatricula(),dateIni, vehiculo.getCombustiblePrimario());
            }else{
                recargas = model.getRecargasAnualesCombustible(vehiculo.getMatricula(), vehiculo.getCombustibleSecundario(), dateIni, dateFin);
                tmp =model.getUltimaRecargaCombustibleAntesDe(formateador.getMatricula(),dateIni, vehiculo.getCombustibleSecundario());
            }
            if(tmp==null){
                kmInicio=vehiculo.getKilometraje();
            }else{
                kmInicio=(tmp.getKm()==null?0.0:tmp.getKm());
            }
            if(recargas==null){
                recargas=new ArrayList<>(0);
            }
            recargas=establecePeriocidadAnual(recargas,dateIni,dateFin,kmInicio);
            formateador.preparaDatosVisualizacion(recargas);
        }
    }


    private List<DTORecarga> establecePeriocidadAnual(List<DTORecarga> r,Date dateIni, Date dateFin,double kmARestar) {
        DTORecarga tmp=null;
        List<DTORecarga> copia=new ArrayList<>(0);
        Calendar cal = Calendar.getInstance();
        Calendar calComparacion = Calendar.getInstance();

        List<DTORecarga> tmpArray ;
        Log.d("TAG","km a restar "+kmARestar);
        double kM=0;

        int penultimoMes=0;
        int ultimoMes=0;

        for(int i=0;i<r.size();i++){
            tmp=r.get(i);
            if(tmp!=null&&i+1<r.size()) {

                kM=tmp.getKm();
                tmp.setKm(tmp.getKm()-kmARestar);
                kmARestar=kM;
                tmpArray = r.subList(i+1, r.size());
                Log.d("TAG","km tmp "+tmp.getKm());
                cal.setTime(tmp.getFecha());
                ultimoMes=cal.get(Calendar.MONTH);
                for (int j = 0; j < tmpArray.size(); j++) {
                    calComparacion.setTime(tmpArray.get(j).getFecha());
                    penultimoMes=calComparacion.get(Calendar.MONTH);
                    if (cal.get(Calendar.MONTH) == calComparacion.get(Calendar.MONTH)) {
                        Log.d("TAG","km j "+ tmpArray.get(j).getKm());
                        tmp.setKm(tmp.getKm() + tmpArray.get(j).getKm()-kmARestar);
                        Log.d("TAG","km acumulados "+ tmp.getKm() );
                        kmARestar=tmpArray.get(j).getKm();
                        r.set(i+j+1,null);

                    }
                }
            }
            if(!(i+1<r.size())&&(penultimoMes!=ultimoMes)){
                r.get(r.size()-1).setKm(r.get(r.size()-1).getKm()-kmARestar);
            }
            if(r.size()==1){
                r.get(0).setKm(r.get(0).getKm()-kmARestar);
            }
        }

        //RELLENA LOS DIAS SIN REGISTRO CON NULL
        for(int i=0;i<r.size();i++){
            try {
                Log.d("TAG", "ENTRADA " + r.get(i).getFecha().toGMTString() + " km " + r.get(i).getKm());
            }catch (Exception e){}        }

        copia=new ArrayList<>(0);
        int mesIni=0;
        int numMes=12;
        for (int i=0;i<r.size();i++){
            tmp=r.get(i);
            if(tmp!=null) {
                cal.setTime(tmp.getFecha());
                for (int j = mesIni; j < numMes; j++) {
                    if (j < cal.get(Calendar.MONTH)) {
                        copia.add(null);
                    } else {
                        if (j == cal.get(Calendar.MONTH)) {
                            copia.add(tmp);
                            mesIni = j + 1;
                        }
                    }
                }
            }
        }
        return copia;
    }

    private List<DTORecarga> establecePeriocidadMensual(List<DTORecarga>r,int numDiasFin,double kmARestar){
        DTORecarga tmp=null;
        List<DTORecarga> copia=new ArrayList<>(0);

        Log.d("TAG","km a restar "+kmARestar);
        List<DTORecarga> tmpArray ;

        Calendar cal = Calendar.getInstance();
        Calendar calComparacion = Calendar.getInstance();
       Log.d("TAG","tamaño inicial "+r.size());
        double valInicialKm=0;
        int penultimoDia=0;
        int ultimoDia=0;
        int fechaIni = 1;
        double kM=0;
        for(int i=0;i<r.size();i++){
            tmp=r.get(i);
            if(tmp!=null&&i+1<r.size()) {

                kM=tmp.getKm();
                tmp.setKm(tmp.getKm()-kmARestar);
                kmARestar=kM;
                tmpArray = r.subList(i+1, r.size());
                Log.d("TAG","km tmp "+tmp.getKm());
                cal.setTime(tmp.getFecha());
                ultimoDia=cal.get(Calendar.DAY_OF_MONTH);
                for (int j = 0; j < tmpArray.size(); j++) {
                    calComparacion.setTime(tmpArray.get(j).getFecha());
                    penultimoDia=calComparacion.get(Calendar.DAY_OF_MONTH);
                    if (cal.get(Calendar.DAY_OF_MONTH) == calComparacion.get(Calendar.DAY_OF_MONTH)) {
                        Log.d("TAG","km j "+ tmpArray.get(j).getKm());
                        tmp.setKm(tmp.getKm() + tmpArray.get(j).getKm()-kmARestar);
                        Log.d("TAG","km acumulados "+ tmp.getKm() );
                        kmARestar=tmpArray.get(j).getKm();
                        r.set(i+j+1,null);

                    }
                }
            }
            if(!(i+1<r.size())&&(penultimoDia!=ultimoDia)){
                r.get(r.size()-1).setKm(r.get(r.size()-1).getKm()-kmARestar);
            }
            if(r.size()==1){
                r.get(0).setKm(r.get(0).getKm()-kmARestar);
            }
        }


        //RELLENA LOS DIAS SIN REGISTRO CON NULL
        for(int i=0;i<r.size();i++){
            try {
                Log.d("TAG", "ENTRADA " + r.get(i).getFecha().toGMTString() + " km " + r.get(i).getKm());
            }catch (Exception e){}
        }

        copia=new ArrayList<>(0);
        for (int i=0;i<r.size();i++){
            tmp=r.get(i);
            if(tmp!=null) {
                cal.setTime(tmp.getFecha());
                for (int j = fechaIni; j < numDiasFin + 1; j++) {
                    if (j < cal.get(Calendar.DAY_OF_MONTH)) {
                        copia.add(null);
                    } else {
                        if (j == cal.get(Calendar.DAY_OF_MONTH)) {
                            copia.add(tmp);
                            fechaIni = j + 1;
                        }
                    }
                }
            }
        }
        return copia;
    }
}
