package com.example.juanalonsoastruga.cargascontrol.Controlador.CUModificarInformacionVehiculo;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTOCombustible;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTOVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Formateador.CUModificarInformacionVehiculo.InterfaceFModificarInformacionVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Modelo.CUModificarInformacionVehiculo.InterfaceMModificarInformacionVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Modelo.CUModificarInformacionVehiculo.ModeloModificarInformacionVehiculo;

import java.util.List;

/**
 * ControladorModificarInformacionVehiculo, Presentador del CU ModificarInformacionVehiculo
 *
 */
public class ControladorModificarInformacionVehiculo implements InterfaceCModificarInformacionVehiculo {
    private InterfaceFModificarInformacionVehiculo formateador;
    private InterfaceMModificarInformacionVehiculo modelo;

    /**
     * Constructor de clase
     *
     * @param formateador La formateador InterfaceFModificarInformacionVehiculo asociada al controlador
     */
    public ControladorModificarInformacionVehiculo(InterfaceFModificarInformacionVehiculo formateador){
        setFormateador(formateador);
        setModel(new ModeloModificarInformacionVehiculo());
    }

    /**
     * Establece el formateador asociado
     *
     * @param formateador Objeto InterfaceVModificarInformacionVehiculo
     */
    @Override
    public void setFormateador(InterfaceFModificarInformacionVehiculo formateador){
        if(formateador==null){
            throw new IllegalArgumentException();
        }else{
            this.formateador =formateador;
        }
    }

    /**
     * Metodo getControlador devuelve la formateador asociada al controlador.
     *
     * @return  Objeto InterfaceFModificarInformacionVehiculo
     */
    @Override
    public InterfaceFModificarInformacionVehiculo getFormateador(){
        return formateador;
    }

    /**
     * Metodo setModel, establece el modelo asociado al controlador.
     *
     * @param m Objeto InterfaceMModificarInformacionVehiculo
     */
    @Override
    public void setModel(InterfaceMModificarInformacionVehiculo m){
        if(m==null){
            throw new IllegalArgumentException();
        }else{
            modelo=m;
        }
    }

    /**
     * Metodo getModel, devuelve el modelo asociado al controlador.
     *
     * @return  Objeto InterfaceMModificarInformacionVehiculo
     */
    @Override
    public InterfaceMModificarInformacionVehiculo getModel(){
        return modelo;
    }

    /**
     * Metodo comienzoCU, despierta al controlador para el comienzo de la ejecución del CU.
     *
     * @param matricula String representando a la matricula del vehiculo a modificar.
     */
    @Override
    public void comienzoCU(String matricula) {
        List<DTOCombustible> combustibles= modelo.getCombustibles();
        formateador.setInitInfo(combustibles,modelo.getVehiculo(matricula));
    }

    /**
     * Metood pulsacionBotonModificarDatos, avisa al controlador del evento de confirmacion
     * de la modificacion del CU.
     *
     * @return  true si exito, false en caso contrario
     */
    @Override
    public boolean pulsacionBotonModificarDatos() {
        DTOVehiculo vehiculo= getDatosFormulario();

        if(vehiculo.getNombre()!=null&&vehiculo.getCombustiblePrimario()!=null){

            modelo.actualizaVehiculo(vehiculo);
            formateador.setRegistroCompletado();
            return true;
        }else{
            formateador.setRegistroNoCompleto();
            return false;
        }


    }

    /**
     * Metodo rellenarCampoIncorrectamente, ordena a la formateador mostrar un mensaje de error, en caso de rellenar de forma
     * no adecuado un campo del registro.
     *
     * @param campo String con el codigo de campo
     * @return  true si operación completada, false en caso contrario
     */
    @Override
    public boolean rellenarCampoIncorrectamente(String campo){
        String mensaje="";
        switch(campo){
            case "año":
                mensaje="Campo Año incorrectamente completado, reinténtelo por favor";
                break;
            case "km":
                mensaje="Campo Kilometraje incorrectamente completado,\n reinténtelo por favor";
                break;
            case "CTP":
                mensaje="Campo Capacidad de Carga de Combustible Principal  incorrectamente\n completado,reinténtelo por favor";
                break;
            case "CTS":
                mensaje="Campo Capacidad de Carga de Combustible Secundario  incorrectamente\n completado,reinténtelo por favor";
                break;
        }
        formateador.setMensajeError(mensaje);
        return true;
    }

    private DTOVehiculo getDatosFormulario() {
        DTOVehiculo vehiculo = formateador.getRegistro();

        return  vehiculo;
    }
}
