package com.example.juanalonsoastruga.cargascontrol.Controlador.CUMostrarListadoVehiculos;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTOVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Formateador.CUMostrarListadoVehiculos.InterfaceFMostrarListado;
import com.example.juanalonsoastruga.cargascontrol.Modelo.CUMostrarListadoVehiculos.InterfaceMMostrarListado;
import com.example.juanalonsoastruga.cargascontrol.Modelo.CUMostrarListadoVehiculos.ModeloMostrarListadoVehiculos;

import java.util.List;

/**
 * Clase ControladorMostrarListadoVehiculos, actua como controlador, dentro del patron MVP, del CU
 * Mostrar Listado de Vehiculos Registrados.
 *
 */
public class ControladorMostrarListadoVehiculos implements InterfaceCMostrarListado {

    private static final String TAG ="ControladorMostrarListadoVehiculos" ;
    private InterfaceMMostrarListado modelo;
    private InterfaceFMostrarListado formateador;

    /**
     * Constructor de la clase.
     *
     * @param formateador objeto InterfaceFMostrarListado la formateador a asociar al controlador
     */
    public ControladorMostrarListadoVehiculos(InterfaceFMostrarListado formateador) {
        //Establece formateador
        setFormateador(formateador);
        setModelo( new ModeloMostrarListadoVehiculos() );

    }

    /**
     * Metodo activacionCU, inicia la ejecucion del controlador.
     *
     */
    @Override
    public void inicio(){

        getListadoVehiculos();
    }

    /**
     * Metodo setControlador, establece la formateador asociada al controlador.
     *
     * @param formateador El objeto InterfaceFMostrarListado, asociado al controlador
     */
    @Override
    public void setFormateador(InterfaceFMostrarListado formateador){
        if(formateador==null){
            throw new IllegalArgumentException();
        }else{
            this.formateador = formateador;
        }
    }
    /**
     * Metodo getControlador, devuelve la formateador asociada al controlador.
     *
     * @return El objeto InterfaceFMostrarListado, asociado al controlador
     */
    @Override
    public InterfaceFMostrarListado getFormateador() {
        return formateador;
    }

    /**
     * Metodo setModelo, establece el modelo asociado al controlador.
     *
     * @param m El objeto InterfaceMMostrarListado, asociado al controlador
     */
    @Override
    public void setModelo(InterfaceMMostrarListado m){
        if(m==null){
            throw new IllegalArgumentException();
        }else{
            modelo=m;
        }

    }
    /**
     * Metodo getModel, establece el modelo asociado al controlador.
     *
     * @return   El objeto InterfaceMMostrarListado, asociado al controlador
     */
    @Override
    public InterfaceMMostrarListado getModel() {
        return modelo;
    }

    /**
     * Metodo getListadoVehiculos, establece en la formateador la infromacion de vehiculos
     * registrados.
     *
     */
    @Override
    public void getListadoVehiculos() {
        List<DTOVehiculo> ids = recuperaIdsListadoVehiculos();
        formateador.setIdsvehiculos(ids);
    }


    private List<DTOVehiculo> recuperaIdsListadoVehiculos(){
        //Recupera Datos del Modelo

        List<DTOVehiculo> ids = modelo.getListadoIdVehiculos();

        return ids;
    }





}
