package com.example.juanalonsoastruga.cargascontrol.Controlador.CURegistrarRecarga;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTODatosDerivadosCombustible;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTODatosDerivadosCostesCombustible;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTODatosDerivadosVehiculoTotales;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTORecarga;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTOVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Modelo.CURegistrarRecarga.InterfaceMRegistrarRecarga;
import com.example.juanalonsoastruga.cargascontrol.Modelo.CURegistrarRecarga.ModeloRegistrarRecarga;
import com.example.juanalonsoastruga.cargascontrol.Formateador.CURegistrarRecarga.InterfaceFRegistrarRecarga;

import java.util.Calendar;
import java.util.Date;

/**
 * Clase ControladorRegistrarRecarrga, representa al Controlador del CU Registrar Recarga
 *
 */
public class ControladorRegistrarRecarga implements InterfaceCRegistrarRecarga {

    private InterfaceFRegistrarRecarga formateador;
    private InterfaceMRegistrarRecarga model;

    /**
     * Constructor de Clase
     *
     * @param formateador El Objeto InterfaceFRegistrarRecarga representando al formateador asociada
     */
    public ControladorRegistrarRecarga(InterfaceFRegistrarRecarga formateador){
        setFormateador(formateador);
        setModel(new ModeloRegistrarRecarga());
    }

    /**
     * Método setControlador, establece el formateador asociada al controlador.
     *
     * @param formateador Un objeto InterfaceFRegistrarRecarga representando al formateador
     */
    @Override
    public void setFormateador(InterfaceFRegistrarRecarga formateador){
        if(formateador==null){
            throw new IllegalArgumentException();
        }else{
            this.formateador =formateador;
        }
    }

    /**
     * Metodo setModel, establece el Modelo asociado al Presentador del CU Registrar Recarga.
     *
     * @param model Un objeto InterfaceMRegistrarRecarga
     */
    @Override
    public void setModel(InterfaceMRegistrarRecarga model){
        if(model==null){
            throw new IllegalArgumentException();
        }else{
            this.model=model;
        }
    }

    /**
     * Metodo getControlador, devuelve una instancia de formateador asociada al controlador.
     *
     * @return  Un Objeto InterfaceFRegistrarRecarga
     */
    @Override
    public InterfaceFRegistrarRecarga getFormateador() {
        return formateador;
    }

    /**
     * Metodo getModel, devuelve una instancia del Modelo asociado al controlador.
     *
     * @return  Un Objeto InterfaceMRegistrarRecarga
     */
    @Override
    public InterfaceMRegistrarRecarga getModel() {
        return model;
    }

    /**
     * Metodo comienzoCU, despierta al controlador del CU Registrar Recarga al comienzo del mismo.
     *
     */
    @Override
    public void comienzoCU() {
        formateador.setDinamicData();
    }
    /**
     * Metodo registroSinFecha, indica al formateador que el registro no contiene un campo de fecha completado
     *
     */
    @Override
    public void registroSinFecha() {
        formateador.setSinFecha();
    }
    /**
     * Metodo registroSinTiempo, indica al formateador que el registro no contiene los campos de horas y minutos completado
     *
     */
    @Override
    public void registroSinTiempo() {
        formateador.setSinTiempo();
    }
    /**
     * Metodo registrarRecarga, indica al controlador que se deve registrar los datos recogidos como una recarga.
     *
     * @param recarga Un Objeto DTORecarga con los datos a registrar
     * @throw IllegalArgumentException si la recarga o la matricula en recarga no es valida
     */
    @Override
    public void registrarRecarga(DTORecarga recarga) {
        if(recarga==null||recarga.getMatricula()==null||recarga.getMatricula().equals("")){

            throw new IllegalArgumentException();

        }
        else{
            Date actualDate= new Date();

            DTOVehiculo v = model.getVehiculo(recarga.getMatricula());
            String str;


            if(
                recarga.getCombustible()!=null&&recarga.getCombustible()!=""&&
                recarga.getKm()!=null&&(recarga.getKm()+"").charAt(0)!='.'&&
                recarga.getFecha()!=null&&
                recarga.getCoste()!=null&&recarga.getCoste().toString().charAt(0)!='.'&&recarga.getCoste()!=0&&
                recarga.getMedicion()!=null&&recarga.getMedicion().toString().charAt(0)!='.'&&recarga.getMedicion()!=0&&
                actualDate.getTime()>=recarga.getFecha().getTime()&&
                formateador.isCamposRegistroCompletos()){
                //COMPROBACION DE LOGICA

                double capacidad=0;

                if(recarga.getCombustible().equals(v.getCombustiblePrimario())){
                    capacidad=v.getCapTankPrimario();
                }else{
                    capacidad=v.getCapTankSecundario();
                }

                if(recarga.getMedicion()<=capacidad) {

                    if (recarga.getKm() == 0.0) {
                        if (recarga.getMedicion() != 0.0) {
                            str = "No es posible recorrer kilometros sin consumir combustible, revise los datos por favor";
                            formateador.setMsgError(str);
                            formateador.setRegistroCompletado(false);
                        }
                    } else {

                        DTORecarga ultimaRecargaRegistradaCombustible = model.getUltimaRecargaRegistradaMismoCombustible(recarga.getMatricula(), recarga.getCombustible());
                        DTORecarga ultimaRecargaRegistrada = model.getUltimaRecargaRegistrada(recarga.getMatricula(), actualDate);
                        if (ultimaRecargaRegistrada != null) {

                            if ((ultimaRecargaRegistrada.getKm() >= recarga.getKm() &
                                    ultimaRecargaRegistrada.getFecha().getTime() <= recarga.getFecha().getTime())||
                                    (ultimaRecargaRegistrada.getKm() <= recarga.getKm() &
                                            ultimaRecargaRegistrada.getFecha().getTime() >= recarga.getFecha().getTime())
                            ) {
                                str = "Existen Recargas Registradas Previamente para este vehículo \n con un kilometraje inconsistente al declarado en esta";
                                formateador.setMsgError(str);
                                formateador.setRegistroCompletado(false);
                            } else {


                                if ((ultimaRecargaRegistradaCombustible != null && (
                                        ultimaRecargaRegistradaCombustible.getKm() <= recarga.getKm() &
                                                ultimaRecargaRegistradaCombustible.getFecha().getTime() >= recarga.getFecha().getTime())
                                ) || (v != null && (v.getKilometraje() > recarga.getKm())
                                )) {
                                    str = "Existen Recargas Registradas Previamente para este vehículo \n con un kilometraje mayor que el declarado en esta";
                                    formateador.setMsgError(str);
                                    formateador.setRegistroCompletado(false);
                                } else {
                                    double tmpKmC=0;
                                    double tmpKm=0;
                                    if(ultimaRecargaRegistradaCombustible != null){
                                        tmpKmC=ultimaRecargaRegistradaCombustible.getKm();
                                    }
                                    if(ultimaRecargaRegistrada!=null){
                                        tmpKm=ultimaRecargaRegistrada.getKm();
                                    }
                                    completarRegistro(recarga,capacidad,v,tmpKmC,tmpKm);
                                    formateador.setRegistroCompletado(true);
                                }
                            }
                        } else {
                            double tmpKmC=0;
                            double tmpKm=0;
                            if(ultimaRecargaRegistradaCombustible != null){
                                tmpKmC=ultimaRecargaRegistradaCombustible.getKm();
                            }
                            if(ultimaRecargaRegistrada!=null){
                                tmpKm=ultimaRecargaRegistrada.getKm();
                            }
                            completarRegistro(recarga,capacidad,v,tmpKmC,tmpKm);
                            formateador.setRegistroCompletado(true);
                        }
                    }
                }else{
                    str = "No puede darse una recarga con mayor capacidad de la declarada en el tanque";
                    formateador.setMsgError(str);
                    formateador.setRegistroCompletado(false);
                }

            }else{
                str="No se han rellenado correctamente algunos campos.\n Los  obligatotios son ( fecha, hora, coste, combustible recargado)";
                formateador.setMsgError(str);
                formateador.setRegistroCompletado(false);
            }

        }
    }

    private void completarRegistro(DTORecarga r,double capacidad,DTOVehiculo v,double kmUltimaRecargaCombustible,double kmUltimaRecarga){
        model.registraRecarga(r);
        Calendar c = Calendar.getInstance();
        String combustible=r.getCombustible();



        setDatosDerivadosCombustible(r,c,combustible,capacidad,v,kmUltimaRecargaCombustible);

        setDatosDerivadosCosteCombustible(r,c,combustible);

        setDatosDerivadosVehiculo(r,c,v,kmUltimaRecarga);
    }

    private void setDatosDerivadosCosteCombustible(DTORecarga r, Calendar c, String combustible) {
        String matricula=r.getMatricula();
        int year=c.get(Calendar.YEAR);
        int month=c.get(Calendar.MONTH);
        //month=month+1;
        DTODatosDerivadosCostesCombustible ddv = model.getDDVCostesCombustible(matricula,year,month,combustible);

        if(ddv==null) {
            //No hay datos para anio y mes seleccionados
            if (model.isPrimerRegistroDatosDerivadosCosteCombustible(matricula, combustible)) {
                //Registro Nuevo
                ddv= new DTODatosDerivadosCostesCombustible(matricula,year,month,combustible);


                double costeXComb=r.getCoste()/r.getMedicion();

                ddv.setNumeroRecargasCombustiblePorMes(1);
                ddv.setNumeroRecargasCombustiblePorAnio(1);

                ddv.setMaximoCosteRecargaAnual(r.getCoste());
                ddv.setMinimoCosteRecargaAnual(r.getCoste());
                ddv.setMaximoCosteRecargaMensual(r.getCoste());
                ddv.setMinimoCosteRecargaMensual(r.getCoste());

                ddv.setSumaCosteRecargaMensual(r.getCoste());
                ddv.setSumaCosteRecargaAnual(r.getCoste());


                ddv.setMinimoCosteXRecargaMensual(costeXComb);
                ddv.setMinimoCosteXRecargaAnual(costeXComb);
                ddv.setMaximoCosteXRecargaMensual(costeXComb);
                ddv.setMaximoCosteXRecargaAnual(costeXComb);

                ddv.setSumaCosteXRecargaAnual(costeXComb);
                ddv.setSumaCosteXRecargaMensual(costeXComb);

                model.addDatosDerivadosCosteCombustible(ddv);
            }else{
                DTODatosDerivadosCostesCombustible ddvAnterior=model.getLastRegistroDatosDerivadosCostesCombustible(matricula,combustible);
                //Ya hay otros registros
                //El mes han cambiado
                if(ddvAnterior.getAnio().equals(year)) {
                    //El anio es el mismo y el mes es nuevo

                    ddv= new DTODatosDerivadosCostesCombustible(matricula,year,month,combustible);

                    double costeXComb=r.getCoste()/r.getMedicion();
                    double maximoCosteAnual=r.getCoste()<ddvAnterior.getMaximoCosteRecargaAnual()?ddvAnterior.getMaximoCosteRecargaAnual():r.getCoste();
                    double minimoCosteAnual=r.getCoste()>ddvAnterior.getMinimoCosteRecargaAnual()?ddvAnterior.getMinimoCosteRecargaAnual():r.getCoste();

                    double maximoCosteXRecargaAnual=(r.getCoste()/r.getMedicion())<ddvAnterior.getMaximoCosteXRecargaAnual()?ddvAnterior.getMaximoCosteXRecargaAnual():(r.getCoste()/r.getMedicion());
                    double minimoCosteXRecargaAnual=(r.getCoste()/r.getMedicion())>ddvAnterior.getMinimoCosteXRecargaAnual()?ddvAnterior.getMinimoCosteXRecargaAnual():(r.getCoste()/r.getMedicion());


                    double maximoCosteMensual=r.getCoste();
                    double minimoCosteMensual=r.getCoste();

                    double maximoCosteXRecargaMensual=(costeXComb);
                    double minimoCosteXRecargaMensual=(costeXComb);

                    ddv.setNumeroRecargasCombustiblePorMes(1);
                    ddv.setNumeroRecargasCombustiblePorAnio(ddvAnterior.getNumeroRecargasCombustiblePorAnio()+1);

                    ddv.setMaximoCosteRecargaAnual(maximoCosteAnual);
                    ddv.setMinimoCosteRecargaAnual(minimoCosteAnual);
                    ddv.setMaximoCosteRecargaMensual(maximoCosteMensual);
                    ddv.setMinimoCosteRecargaMensual(minimoCosteMensual);

                    ddv.setSumaCosteRecargaMensual(r.getCoste());
                    ddv.setSumaCosteRecargaAnual(ddvAnterior.getSumaCosteRecargaAnual()+r.getCoste());

                    ddv.setMinimoCosteXRecargaMensual(minimoCosteXRecargaMensual);
                    ddv.setMaximoCosteXRecargaMensual(maximoCosteXRecargaMensual);
                    ddv.setMinimoCosteXRecargaAnual(minimoCosteXRecargaAnual);
                    ddv.setMaximoCosteXRecargaAnual(maximoCosteXRecargaAnual);

                    ddv.setSumaCosteXRecargaAnual(ddvAnterior.getSumaCosteXRecargaAnual()+(costeXComb));
                    ddv.setSumaCosteXRecargaMensual((costeXComb));

                    model.addDatosDerivadosCosteCombustible(ddv);

                }else{
                //El mes y el anio han cambiado

                    ddv= new DTODatosDerivadosCostesCombustible(matricula,year,month,combustible);


                    double costeXComb=r.getCoste()/r.getMedicion();

                    double maximoCosteAnual=r.getCoste();
                    double minimoCosteAnual=r.getCoste();

                    double maximoCosteMensual=r.getCoste();
                    double minimoCosteMensual=r.getCoste();
                    double maximoCosteXRecargaAnual=costeXComb;
                    double minimoCosteXRecargaAnual=costeXComb;

                    double maximoCosteXRecargaMensual=costeXComb;
                    double minimoCosteXRecargaMensual=costeXComb;

                    ddv.setNumeroRecargasCombustiblePorMes(1);
                    ddv.setNumeroRecargasCombustiblePorAnio(1);

                    ddv.setMaximoCosteRecargaAnual(maximoCosteAnual);
                    ddv.setMinimoCosteRecargaAnual(minimoCosteAnual);
                    ddv.setMaximoCosteRecargaMensual(maximoCosteMensual);
                    ddv.setMinimoCosteRecargaMensual(minimoCosteMensual);

                    ddv.setSumaCosteRecargaMensual(r.getCoste());
                    ddv.setSumaCosteRecargaAnual(r.getCoste());



                    ddv.setMinimoCosteXRecargaMensual(minimoCosteXRecargaMensual);
                    ddv.setMaximoCosteXRecargaMensual(maximoCosteXRecargaMensual);
                    ddv.setMinimoCosteXRecargaAnual(minimoCosteXRecargaAnual);
                    ddv.setMaximoCosteXRecargaAnual(maximoCosteXRecargaAnual);

                    ddv.setSumaCosteXRecargaAnual((costeXComb));
                    ddv.setSumaCosteXRecargaMensual((costeXComb));

                    model.addDatosDerivadosCosteCombustible(ddv);

                }
            }
        }else{
            //Hay un registro ya existente


            double maximoCosteAnual=r.getCoste()<ddv.getMaximoCosteRecargaAnual()?ddv.getMaximoCosteRecargaAnual():r.getCoste();
            double minimoCosteAnual=r.getCoste()>ddv.getMinimoCosteRecargaAnual()?ddv.getMinimoCosteRecargaAnual():r.getCoste();

            double maximoCosteMensual=r.getCoste()<ddv.getMaximoCosteRecargaMensual()?ddv.getMaximoCosteRecargaMensual():r.getCoste();
            double minimoCosteMensual=r.getCoste()>ddv.getMinimoCosteRecargaMensual()?ddv.getMinimoCosteRecargaMensual():r.getCoste();


            double maximoCosteXRecargaAnual=(r.getCoste()/r.getMedicion())<ddv.getMaximoCosteXRecargaAnual()?ddv.getMaximoCosteXRecargaAnual():(r.getCoste()/r.getMedicion());
            double minimoCosteXRecargaAnual=(r.getCoste()/r.getMedicion())>ddv.getMinimoCosteXRecargaAnual()?ddv.getMinimoCosteXRecargaAnual():(r.getCoste()/r.getMedicion());

            double maximoCosteXRecargaMensual=(r.getCoste()/r.getMedicion())<ddv.getMaximoCosteXRecargaMensual()?ddv.getMaximoCosteXRecargaMensual():(r.getCoste()/r.getMedicion());
            double minimoCosteXRecargaMensual=(r.getCoste()/r.getMedicion())>ddv.getMinimoCosteXRecargaMensual()?ddv.getMinimoCosteXRecargaMensual():(r.getCoste()/r.getMedicion());

            ddv.setNumeroRecargasCombustiblePorMes(ddv.getNumeroRecargasCombustiblePorMes()+1);
            ddv.setNumeroRecargasCombustiblePorAnio(ddv.getNumeroRecargasCombustiblePorAnio()+1);

            ddv.setMaximoCosteRecargaAnual(maximoCosteAnual);
            ddv.setMinimoCosteRecargaAnual(minimoCosteAnual);
            ddv.setMaximoCosteRecargaMensual(maximoCosteMensual);
            ddv.setMinimoCosteRecargaMensual(minimoCosteMensual);

            ddv.setSumaCosteRecargaMensual(ddv.getSumaCosteRecargaMensual()+r.getCoste());
            ddv.setSumaCosteRecargaAnual(ddv.getSumaCosteRecargaAnual()+r.getCoste());



            ddv.setMinimoCosteXRecargaMensual(minimoCosteXRecargaMensual);
            ddv.setMaximoCosteXRecargaMensual(maximoCosteXRecargaMensual);
            ddv.setMinimoCosteXRecargaAnual(minimoCosteXRecargaAnual);
            ddv.setMaximoCosteXRecargaAnual(maximoCosteXRecargaAnual);

            ddv.setSumaCosteXRecargaAnual(ddv.getSumaCosteXRecargaAnual()+(r.getCoste()/r.getMedicion()));
            ddv.setSumaCosteXRecargaMensual(ddv.getSumaCosteXRecargaMensual()+(r.getCoste()/r.getMedicion()));

            model.actualizaDatosDerivadosCosteCombustible(ddv);
        }

    }

    private void setDatosDerivadosCombustible(DTORecarga r, Calendar c, String combustible,double capacidad,DTOVehiculo v,double kmUltimaRecargaCombustible) {
        String matricula=r.getMatricula();
        int year=c.get(Calendar.YEAR);
        int month=c.get(Calendar.MONTH);
        double km=kmUltimaRecargaCombustible;
        //month=month+1;
        DTODatosDerivadosCombustible ddv = model.getDDVCombustible(matricula,year,month,combustible);
        if(ddv==null){
            //No hay datos para anio y mes seleccionados
            if(model.isPrimerRegistroDatosDerivadosCombustible(matricula,combustible)) {
                //Registro Nuevo

                km=v.getKilometraje();

                double cmbXKm;
                if(capacidad-r.getMedicion()!=0) {
                    cmbXKm=(capacidad - r.getMedicion()) / (r.getKm() - km);
                }
                else{
                    cmbXKm=(capacidad) / (r.getKm() - km);
                }
                ddv= new DTODatosDerivadosCombustible(matricula,year,month,combustible);

                ddv.setNumeroRecargasCombustiblePorMes(1);
                ddv.setNumeroRecargasCombustiblePorAnio(1);

                ddv.setMaximoCombustibleAnualRecargado(r.getMedicion());
                ddv.setMinimoCombustibleAnualRecargado(r.getMedicion());
                ddv.setMaximoCombustibleMensualRecargado(r.getMedicion());
                ddv.setMinimoCombustibleMensualRecargado(r.getMedicion());

                ddv.setMaximoCombustibleXKmMensual(cmbXKm);
                ddv.setMinimoCombustibleXKmMensual(cmbXKm);
                ddv.setMaximoCombustibleXKmAnual(cmbXKm);
                ddv.setMinimoCombustibleXKmAnual(cmbXKm);

                ddv.setSumaCombustibleRecargadoPorMes(r.getMedicion());
                ddv.setSumaCombustibleRecargadoPorAnio(r.getMedicion());
                ddv.setSumaCombustibleXKmMensual(cmbXKm);
                ddv.setSumaCombustibleXKmAnual(cmbXKm);

                model.addDatosDerivadosCombustible(ddv);
            }else{


                DTODatosDerivadosCombustible ddvAnterior=model.getLastRegistroDatosDerivadosCombustible(matricula,combustible);
                //Ya hay otros registros
                //O el anio o el mes han cambiado
                if(ddvAnterior.getAnio().equals(year)) {
                    //El anio es el mismo y el mes es nuevo

                    double cmbXKm;
                    if(capacidad-r.getMedicion()!=0) {
                        cmbXKm=(capacidad - r.getMedicion()) / (r.getKm() - km);
                    }
                    else{
                        cmbXKm=(capacidad) / (r.getKm() - km);
                    }


                    double maximoCAnual=r.getMedicion()<ddvAnterior.getMaximoCombustibleAnualRecargado()?ddvAnterior.getMaximoCombustibleAnualRecargado():r.getMedicion();
                    double minimoCAnual=r.getMedicion()>ddvAnterior.getMinimoCombustibleAnualRecargado()?ddvAnterior.getMinimoCombustibleAnualRecargado():r.getMedicion();

                    double maximoCMensual=r.getMedicion();
                    double minimoCMensual=r.getMedicion();

                    double maximoCXKmAnual=(cmbXKm)<ddvAnterior.getMaximoCombustibleXKmAnual()?ddvAnterior.getMaximoCombustibleXKmAnual():(cmbXKm);
                    double minimoCXKmAnual=(cmbXKm)>ddvAnterior.getMinimoCombustibleXKmAnual()?ddvAnterior.getMinimoCombustibleXKmAnual():(cmbXKm);

                    double maximoCXKmMensual=(cmbXKm);
                    double minimoCXKmMensual=(cmbXKm);

                    ddv= new DTODatosDerivadosCombustible(matricula,year,month,combustible);

                    ddv.setNumeroRecargasCombustiblePorMes(1);
                    ddv.setNumeroRecargasCombustiblePorAnio(ddvAnterior.getNumeroRecargasCombustiblePorAnio()+1);

                    ddv.setMaximoCombustibleAnualRecargado(maximoCAnual);
                    ddv.setMinimoCombustibleAnualRecargado(minimoCAnual);
                    ddv.setMaximoCombustibleMensualRecargado(maximoCMensual);
                    ddv.setMinimoCombustibleMensualRecargado(minimoCMensual);

                    ddv.setMaximoCombustibleXKmMensual(maximoCXKmMensual);
                    ddv.setMinimoCombustibleXKmMensual(minimoCXKmMensual);
                    ddv.setMaximoCombustibleXKmAnual(maximoCXKmAnual);
                    ddv.setMinimoCombustibleXKmAnual(minimoCXKmAnual);

                    ddv.setSumaCombustibleRecargadoPorMes(r.getMedicion());
                    ddv.setSumaCombustibleRecargadoPorAnio(r.getMedicion()+ddvAnterior.getSumaCombustibleRecargadoPorAnio());
                    ddv.setSumaCombustibleXKmMensual(cmbXKm);
                    ddv.setSumaCombustibleXKmAnual(cmbXKm+ddvAnterior.getSumaCombustibleXKmAnual());

                    model.addDatosDerivadosCombustible(ddv);

                }else{
                    //Nuevo anio y nuevo mes
                    double cmbXKm;
                    if(capacidad-r.getMedicion()!=0) {
                        cmbXKm=(capacidad - r.getMedicion()) / (r.getKm() - km);
                    }
                    else{
                        cmbXKm=(capacidad) / (r.getKm() - km);
                    }

                    ddv= new DTODatosDerivadosCombustible(matricula,year,month,combustible);

                    ddv.setNumeroRecargasCombustiblePorMes(1);
                    ddv.setNumeroRecargasCombustiblePorAnio(1);

                    ddv.setMaximoCombustibleAnualRecargado(r.getMedicion());
                    ddv.setMinimoCombustibleAnualRecargado(r.getMedicion());
                    ddv.setMaximoCombustibleMensualRecargado(r.getMedicion());
                    ddv.setMinimoCombustibleMensualRecargado(r.getMedicion());

                    ddv.setMaximoCombustibleXKmMensual(cmbXKm);
                    ddv.setMinimoCombustibleXKmMensual(cmbXKm);
                    ddv.setMaximoCombustibleXKmAnual(cmbXKm);
                    ddv.setMinimoCombustibleXKmAnual(cmbXKm);

                    ddv.setSumaCombustibleRecargadoPorMes(r.getMedicion());
                    ddv.setSumaCombustibleRecargadoPorAnio(r.getMedicion());
                    ddv.setSumaCombustibleXKmMensual(cmbXKm);
                    ddv.setSumaCombustibleXKmAnual(cmbXKm);

                    model.addDatosDerivadosCombustible(ddv);

                }
            }
        }else{
            //Hay un registro ya existente

            double cmbXKm;
            if(capacidad-r.getMedicion()!=0) {
                cmbXKm=(capacidad - r.getMedicion()) / (r.getKm() - km);
            }
            else{
                cmbXKm=(capacidad) / (r.getKm() - km);
            }



            double maximoCAnual=r.getMedicion()<ddv.getMaximoCombustibleAnualRecargado()?ddv.getMaximoCombustibleAnualRecargado():r.getMedicion();
            double minimoCAnual=r.getMedicion()>ddv.getMinimoCombustibleAnualRecargado()?ddv.getMinimoCombustibleAnualRecargado():r.getMedicion();

            double maximoCMensual=r.getMedicion()<ddv.getMaximoCombustibleMensualRecargado()?ddv.getMaximoCombustibleMensualRecargado():r.getMedicion();
            double minimoCMensual=r.getMedicion()>ddv.getMinimoCombustibleMensualRecargado()?ddv.getMinimoCombustibleMensualRecargado():r.getMedicion();

            double maximoCXKmAnual=(cmbXKm)<ddv.getMaximoCombustibleXKmAnual()?ddv.getMaximoCombustibleXKmAnual():(cmbXKm);
            double minimoCXKmAnual=(cmbXKm)>ddv.getMinimoCombustibleXKmAnual()?ddv.getMinimoCombustibleXKmAnual():(cmbXKm);

            double maximoCXKmMensual=(cmbXKm)<ddv.getMaximoCombustibleXKmMensual()?ddv.getMaximoCombustibleXKmMensual():(cmbXKm);
            double minimoCXKmMensual=(cmbXKm)>ddv.getMinimoCombustibleXKmMensual()?ddv.getMinimoCombustibleXKmMensual():(cmbXKm);



            ddv.setNumeroRecargasCombustiblePorMes(ddv.getNumeroRecargasCombustiblePorMes()+1);
            ddv.setNumeroRecargasCombustiblePorAnio(ddv.getNumeroRecargasCombustiblePorAnio()+1);

            ddv.setMaximoCombustibleAnualRecargado(maximoCAnual);
            ddv.setMinimoCombustibleAnualRecargado(minimoCAnual);
            ddv.setMaximoCombustibleMensualRecargado(maximoCMensual);
            ddv.setMinimoCombustibleMensualRecargado(minimoCMensual);

            ddv.setMaximoCombustibleXKmMensual(maximoCXKmMensual);
            ddv.setMinimoCombustibleXKmMensual(minimoCXKmMensual);
            ddv.setMaximoCombustibleXKmAnual(maximoCXKmAnual);
            ddv.setMinimoCombustibleXKmAnual(minimoCXKmAnual);

            ddv.setSumaCombustibleRecargadoPorMes(r.getMedicion()+ddv.getSumaCombustibleRecargadoPorMes());
            ddv.setSumaCombustibleRecargadoPorAnio(r.getMedicion()+ddv.getSumaCombustibleRecargadoPorAnio());
            ddv.setSumaCombustibleXKmMensual((cmbXKm)+ddv.getSumaCombustibleXKmMensual());
            ddv.setSumaCombustibleXKmAnual((cmbXKm)+ddv.getSumaCombustibleXKmAnual());

            model.actualizaDatosDerivadosCombustible(ddv);
        }
    }

    private void setDatosDerivadosVehiculo(DTORecarga r, Calendar c,DTOVehiculo v,double kmUltimaRecargaCombustible){
        String matricula=r.getMatricula();
        int year=c.get(Calendar.YEAR);
        int month=c.get(Calendar.MONTH);
        //month=month+1;
        double km=kmUltimaRecargaCombustible;
        double coste=r.getCoste();
        DTODatosDerivadosVehiculoTotales ddv=model.getDDVehiculo(matricula,year,month);
        if(ddv==null){
            //No hay registros para ese anio y mes
            if(model.isPrimerRegistroDatosDerivadosVehiculo(matricula)) {
                //Registro nuevo
                ddv = new DTODatosDerivadosVehiculoTotales(matricula, year, month);
                ddv.setNumeroRecargasMes(1);
                ddv.setNumeroRecargasAnio(1);
                ddv.setKmRecargasTotalesPorMes(r.getKm()-v.getKilometraje());
                ddv.setKmRecargasTotalesPorAnio(r.getKm()-v.getKilometraje());
                ddv.setCosteRecargasTotalesPorMes(coste);
                ddv.setCosteRecargasTotalesPorAnio(coste);
                model.addDatosDerivadosVehiculo(ddv);
            }else{
                //Ya hay otros registros
                //O el anio o el mes han cambiado
                DTODatosDerivadosVehiculoTotales ddvAnterior=model.getLastRegistroDatosDerivadosVehiculo(matricula);
                if(ddvAnterior.getAnio().equals(year)){
                    //El anio es el mismo y el mes es nuevo
                    ddv = new DTODatosDerivadosVehiculoTotales(matricula, year, month);
                    ddv.setNumeroRecargasMes(1);
                    ddv.setNumeroRecargasAnio(ddvAnterior.getNumeroRecargasAnio()+1);
                    ddv.setKmRecargasTotalesPorMes(r.getKm()-km);
                    ddv.setKmRecargasTotalesPorAnio(ddvAnterior.getKmRecargasTotalesPorAnio()+r.getKm()-km);
                    ddv.setCosteRecargasTotalesPorMes(coste);
                    ddv.setCosteRecargasTotalesPorAnio(ddvAnterior.getCosteRecargasTotalesPorAnio()+coste);
                    model.addDatosDerivadosVehiculo(ddv);
                }else{
                    //Nuevo anio y nuevo mes
                    ddv = new DTODatosDerivadosVehiculoTotales(matricula, year, month);
                    ddv.setNumeroRecargasMes(1);
                    ddv.setNumeroRecargasAnio(1);
                    ddv.setKmRecargasTotalesPorMes(r.getKm()-km);
                    ddv.setKmRecargasTotalesPorAnio(r.getKm()-km);
                    ddv.setCosteRecargasTotalesPorMes(coste);
                    ddv.setCosteRecargasTotalesPorAnio(coste);
                    model.addDatosDerivadosVehiculo(ddv);
                }
            }
        }else{
            //mismo mes y anio

            ddv.setNumeroRecargasMes(ddv.getNumeroRecargasMes()+1);
            ddv.setNumeroRecargasAnio(ddv.getNumeroRecargasAnio()+1);
            ddv.setKmRecargasTotalesPorMes(ddv.getKmRecargasTotalesPorMes()+r.getKm()-km);
            ddv.setKmRecargasTotalesPorAnio(ddv.getKmRecargasTotalesPorAnio()+r.getKm()-km);
            ddv.setCosteRecargasTotalesPorMes(ddv.getCosteRecargasTotalesPorMes()+coste);
            ddv.setCosteRecargasTotalesPorAnio(ddv.getCosteRecargasTotalesPorAnio()+coste);
            model.actualizaDatosDerivadosVehiculo(ddv);
        }
    }
}
