package com.example.juanalonsoastruga.cargascontrol.Controlador.CURegistrarVehiculo;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTOCombustible;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTOVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Modelo.CURegistrarVehiculo.InterfaceMRegistrarVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Modelo.CURegistrarVehiculo.ModeloRegistroVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Formateador.CURegistrarVehiculo.InterfaceFRegistroVehiculo;

import java.util.List;

/**
 * Clase ControladorRegistroVehiculo, representa al controlador del CU Registro de Vehiculo
 *
 */
public class ControladorRegistroVehiculo implements InterfaceCRegistroVehiculo {
    private static final String TAG = "ControladorRegistroVehiculo";
    private InterfaceFRegistroVehiculo formateador;
    private InterfaceMRegistrarVehiculo modelo;

    /**
     * Constructor de clase.
     *
     * @param formateador Objeto InterfaceFRegistroVehiculo
     */
    public ControladorRegistroVehiculo(InterfaceFRegistroVehiculo formateador){
        setFormateador(formateador);
        setModel(new ModeloRegistroVehiculo());
    }

    /**
     * Establece la formateador asociada
     *
     * @param formateador Objeto InterfaceFRegistroVehiculo
     */
    @Override
    public void setFormateador(InterfaceFRegistroVehiculo formateador){
        if(formateador==null){
            throw new IllegalArgumentException();
        }else {
            this.formateador = formateador;
        }
    }
    /**
     * Metodo getControlador devuelve la formateador asociada al controlador.
     *
     * @return  Objeto InterfaceFRegistroVehiculo
     */
    @Override
    public InterfaceFRegistroVehiculo getFormateador(){
        return formateador;
    }
    /**
     * Metodo setModel, establece el modelo asociado al controlador.
     *
     * @param m Objeto InterfaceMRegistrarVehiculo
     */
    @Override
    public void setModel(InterfaceMRegistrarVehiculo m){
        if(m==null){
            throw new IllegalArgumentException();
        }else {
            this.modelo= m;
        }
    }
    /**
     * Metodo getModel, devuelve el modelo asociado al controlador.
     *
     * @return  Objeto InterfaceMRegistrarVehiculo
     */
    @Override
    public InterfaceMRegistrarVehiculo getModel(){
        return modelo;
    }

    /**
     * Metodo eventoComienzo, permite al formateador avisar al controlador del comienzo del CU
     *
     */
    @Override
    public void eventoComienzo() {
        List<DTOCombustible> listado= modelo.getCombustibles();
        formateador.setSpinnersCombustible(listado);
    }

    /**
     * Metodo solicitarGuardado, permite avisar al controlador del comienzo de la operacion de guardado.
     *
     * @return  true si exito, false en caso contrario
     */
    @Override
    public boolean solicitarGuardado() {
        //Comprueba campos basicos no son null
            DTOVehiculo vehiculo = formateador.getRegistro();

            if(vehiculo.getMatricula()!=null&&vehiculo.getMatricula()!=""&&vehiculo.getNombre()!=null&&vehiculo.getNombre()!=""&&vehiculo.getCombustiblePrimario()!=null&&vehiculo.getCombustiblePrimario()!=""){


                boolean comprobacion=modelo.isVehiculoRegistrado(vehiculo);

                if((vehiculo.getCombustiblePrimario()!=null&vehiculo.getCapTankPrimario()==0)||
                        (vehiculo.getCombustibleSecundario()!=null&vehiculo.getCapTankSecundario()==0)){
                    formateador.setMensajeError("No se puede omitir el dato de capacidad de tanque  para el combustible declarado");
                    formateador.setRegistroCompletado(false);
                }
                else{


                    if(vehiculo.getCombustiblePrimario()!=null&&vehiculo.getCombustibleSecundario()!=null&&
                        vehiculo.getCombustiblePrimario().equals(vehiculo.getCombustibleSecundario())){

                        formateador.setMensajeError("El vehiculo no puede tener igual \n  combustible primario y secundario");
                        formateador.setRegistroCompletado(false);
                    }else{
                        if(!comprobacion) {

                            modelo.registraVehiculo(vehiculo);
                            formateador.setRegistroCompletado(true);
                        }else{
                            formateador.setMensajeError("Vehiculo con igual matricula ya ha sido registrado");
                            formateador.setRegistroCompletado(false);
                        }
                    }
                }

            }else{
                //Si son null Toast indicando que no se puede hacer y que se completen
                formateador.setMensajeError("No se han rellenado campos obligatotios \n (nombre, matricula, combustible primario)");
                formateador.setRegistroCompletado(false);
            }
        return true;
    }

    /**
     * Metodo rellenarCampoIncorrectamente, avisa al controlador que un campo del registro del vehiculo
     * es incorrecto.
     *
     * @param cmp   Objeto String indica el codigo del campo incorrecto
     * @return      true si exito, false en caso contrario
     */
    @Override
    public boolean rellenarCampoIncorrectamente(String cmp) {
        String mensaje="";
        switch(cmp){
            case "año":
                mensaje="Campo Año incorrectamente completado, reinténtelo por favor";
                break;
            case "km":
                mensaje="Campo Kilometraje incorrectamente completado,\n reinténtelo por favor";
                break;
            case "CTP":
                mensaje="Campo Capacidad de Carga de Combustible Principal  incorrectamente\n completado,reinténtelo por favor";
                break;
            case "CTS":
                mensaje="Campo Capacidad de Carga de Combustible Secundario incorrectamente\n  completado,reinténtelo por favor";
                break;
        }
        formateador.setMensajeError(mensaje);
        return true;
    }
}
