package com.example.juanalonsoastruga.cargascontrol.Datos.CombustibleProvider;

import com.example.juanalonsoastruga.cargascontrol.Datos.Entidades.Combustible;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import java.util.ArrayList;
import java.util.List;

/**
 * Clase CombustibleHandler, trata el parseo del documento de combustibles inciales.
 *
 */
public class CombustibleHandler extends DefaultHandler {
    private List<Combustible> combustibles;
    private Combustible combustibleActual;
    private StringBuilder sbText;

    public Boolean parsingError = false;

    /**
     * Metodo getCombustibles, devuelve un listado de Combustibles.
     *
     * @return Un Objeto List<Combustible>
     */
    public List<Combustible> getCombustibles(){
        return combustibles;
    }

    /**
     * Metodo startDocument, incializa los argumentos del parseador al comienzo del documento XML.
     *
     * @throws SAXException
     */
    @Override
    public void startDocument() throws SAXException {

        super.startDocument();

        combustibles = new ArrayList<Combustible>();
        sbText = new StringBuilder();
    }
    /**
     * Metodo startElement, incializa los argumentos del elemento actual al comienzo de una etiqueta XML.
     *
     * @throws SAXException
     */
    @Override
    public void startElement(String uri, String localId, String name, Attributes attributes) throws SAXException {

        super.startElement(uri, localId, name, attributes);

        if (localId.equals("combustible")) {
            combustibleActual = new Combustible(0, "", "");
        }
    }

    /**
     * Metodo characters, acumula los caracteres leidos del documento.
     *
     * @param ch    El array char[] de characteres leidos
     * @param start  in con el punto de comienzo de lectura
     * @param length   in con la longitud de la cadena a leer
     * @throws SAXException
     */
    @Override
    public void characters(char[] ch, int start, int length)
            throws SAXException {

        super.characters(ch, start, length);

        if (this.combustibleActual != null)
            sbText.append(ch, start, length);
    }

    /**
     * Metodo endElement, determina que hacer al llegar a un cierre de etiqueta en el documento XML.
     *
     * @param uri
     * @param localName
     * @param id
     * @throws SAXException
     */
    @Override
    public void endElement(String uri, String localName, String id)
            throws SAXException {

        super.endElement(uri, localName, id);
        double value;
        String campo;
        if (this.combustibleActual != null) {

            if (localName.equals("id")) {
                combustibleActual.setId(Integer.parseInt((sbText.toString()).trim()));
            } else if (localName.equals("nombre")) {
                campo =  (sbText.toString().trim());
                combustibleActual.setTipo(campo);
            } else if (localName.equals("unidades")) {
                campo = (sbText.toString().trim());
                combustibleActual.setUnidades(campo);
            } else if (localName.equals("combustible")) {
                combustibles.add(combustibleActual);
            }else{
            // NOT CORRECT STRUCTURE
            parsingError = true;
            }

            sbText.setLength(0);
        }
    }

}
