package com.example.juanalonsoastruga.cargascontrol.Datos;

import androidx.room.Room;
import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.util.Log;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTOCombustible;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTODatosDerivadosCombustible;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTODatosDerivadosCostesCombustible;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTODatosDerivadosVehiculoTotales;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTOGasolinera;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTORecarga;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTOVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Datos.CombustibleProvider.CombustibleParser;
import com.example.juanalonsoastruga.cargascontrol.Datos.GasolinerasProvider.InterfaceGasolinerasProvider;
import com.example.juanalonsoastruga.cargascontrol.Datos.Entidades.Combustible;
import com.example.juanalonsoastruga.cargascontrol.Datos.Entidades.DatosDerivadosCombustible;
import com.example.juanalonsoastruga.cargascontrol.Datos.Entidades.DatosDerivadosCostesCombustible;
import com.example.juanalonsoastruga.cargascontrol.Datos.Entidades.DatosDerivadosVehiculoTotales;
import com.example.juanalonsoastruga.cargascontrol.Datos.Entidades.Recarga;
import com.example.juanalonsoastruga.cargascontrol.Datos.Entidades.Vehiculo;
import com.example.juanalonsoastruga.cargascontrol.Datos.LocationProvider.InterfaceLocationProvider;
import com.example.juanalonsoastruga.cargascontrol.Datos.UltimaConsultaProvider.InterfaceUltimaConsultaProvider;
import com.example.juanalonsoastruga.cargascontrol.Datos.database.AppDataBase;
import com.example.juanalonsoastruga.cargascontrol.R;

import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.Hashtable;
import java.util.List;


/**
 * Clase FachadaDatos, implementa la Fachada de Datos de la aplicacion.
 *
 */
public class FachadaDatos implements InterfaceFachadaDatos {


    private AppDataBase db;
    protected static FachadaDatos instancia;
    private Context appContext;
    private InterfaceLocationProvider locationProvider;
    private InterfaceGasolinerasProvider gasolinerasProvider;
    private InterfaceUltimaConsultaProvider ultimaConsultaProvider;
    private final String TAG = "FachadaDatos";
    private double latitudMin,latitudMax,longitudMin,longitudMax;
   /**
     * Constructor de clase.
     */
    protected FachadaDatos() {

        this.appContext = null;
        this.locationProvider =null;
    }

    /**
     * Metodo instancia, devuelve una instancia de la clase.
     *
     * @return Objeto FachadaDatos
     */
    public static FachadaDatos instancia() {
        if (instancia == null) {
            instancia = new FachadaDatos();

        }
        return instancia;
    }


    /**
     * Metodo setContext, establece el Contexto para la BD.
     *
     * @param appContext El Conexto de la aplicacion
     */
    public void setContext(Context appContext) {
        instancia.appContext = appContext;
        db = createDatabase();
    }

    /**
     * Metodo setLocationProvider, establece el proveedor de localizacion GPS para la fachada.
     *
     * @param provider Objeto InterfaceLocationProvider
     */
    public void setLocationProvider(InterfaceLocationProvider provider) {
        instancia.locationProvider =provider;

    }

    /**
     * Metodo setUltimaConsultaProvider, establece el proveedor de identificadores para la ultima consulta
     * de vehiculo y recarga.
     *
     * @param ultimaConsultaProvider    Objeto InterfaceUltimaConsultaProvider
     */
    public void setUltimaConsultaProvider(InterfaceUltimaConsultaProvider ultimaConsultaProvider) {
        this.ultimaConsultaProvider = ultimaConsultaProvider;
    }

    /**
     * Metodo setEstacionesRecargaProvider, establece el proveedor de estaciones de recarga de combustible
     * nacionales para la fachada.
     *
     * @param provider Objeto InterfaceGasolinerasProvider
     */
    public void setEstacionesRecargaProvider(InterfaceGasolinerasProvider provider){
        instancia.gasolinerasProvider=provider;
    }

    private AppDataBase createDatabase() {
        db = Room.databaseBuilder(appContext, AppDataBase.class, "GasCar").fallbackToDestructiveMigration()
                .allowMainThreadQueries().build();
        return db;
    }

    /**
     * Metodo pobladoInicial, puebla la base de datos con la informacion inicial necesaria.
     *
     */
    public void pobladoInicial() {
        //Tambien hay que hacer un poblado inicial en el caso de que no exista de los tipos de datos

        InputStream is = this.appContext.getResources().openRawResource(R.raw.pobladoinicial);

        CombustibleParser parser = new CombustibleParser(is);

        List<Combustible> combustibles = parser.parse();

        setCombustibles(combustibles);

    }

    /**
     * Metodo getLocalizacionActual, devuelve una estructura de datos con latitud y longitud actuales.
     *
     * @param localizacion Hashtable
     * @return Hashtable con latitud y longitud como claves
     */
    @Override
    public Hashtable getLocalizacionActual(Hashtable localizacion) {
        return this.locationProvider.getLocalizacionActual(localizacion);
    }

    /**
     * Metodo getEstacionesRecargaNacionales, realiza la descarga de la informacion de estaciones nacionales
     * de recarga de combustible a través de una fuente externa.
     *
     * @param coordenadas Hashtable con latitud y longitud de la posicion actual
     */
    @Override
    public List<DTOGasolinera> getEstacionesRecargaNacionales(Hashtable coordenadas) {


        //Como referencia 1 grado de latitud o longitud = 111,1 km
        //que, teniendo un rango de +/- 20 km se traduce en +/- 0,18 para latitud y longitud
        //he escojido esos 20 km arbitrariamente, podía haber sido otra cantidad pero la cifra parecia
        // razonable en caso de estar con reserva en el tanque de combustible.
        double longitud=(double)(coordenadas.get("longitud"));
        double latitud=(double)(coordenadas.get("latitud"));
        double calculoKmaGD=(20)/(111.1);

        Log.d("CALCULO","latitud "+latitud+" longitud "+longitud);
        latitudMin = (latitud - calculoKmaGD);
        latitudMax = (latitud + calculoKmaGD);
        Log.d("CALCULO","latitudMax "+latitudMax+" latitudMin "+latitudMin);
        longitudMin= (longitud-calculoKmaGD);
        longitudMax= (longitud+calculoKmaGD);
        Log.d("CALCULO","longitudMax "+longitudMax+" longitudMin "+longitudMin);

        List<DTOGasolinera> gasolineras=gasolinerasProvider.descargaListado(latitudMin,latitudMax,longitudMin,longitudMax,latitud,longitud);

        return gasolineras;
    }

    @Override
    public DTOGasolinera getDTOGasolinera(double latitud, double longitud,String direccion) {
        return gasolinerasProvider.descargaGasolinera(latitud,longitud,direccion);
    }


    /**
     * Metodo registraIDUltimoVehiculo, registra el identificador del ultimo recurso consultado.
     *
     * @param matricula String con la matricula del ultimo vehiculo registrado
     */
    @Override
    public void registraIDUltimoVehiculo(String matricula) {
      ultimaConsultaProvider.setUltimoVeiculo(matricula);
    }


    @Override
    public Hashtable getIdUltimaRecarga(){
       return ultimaConsultaProvider.getIdUltimaRecarga();
    }

    @Override
    public String getIdUltimoVehiculo(){
        return ultimaConsultaProvider.getUltimoVehiculo();
    }

    @Override
    public void borrarFicheroVehiculo() {
        ultimaConsultaProvider.borraFicheroVehiculo();
    }

    @Override
    public void borrarFicheroRecarga() {
        ultimaConsultaProvider.borrarFicheroRecarga();
    }


    /**
     * Metodo registraIDUltimaRecarga, registra el identificador de la ultima recarga consultada.
     *
     * @param fecha Long con la fecha de la recarga
     * @param matricula String con la matricula del ultimo vehiculo registrado
     * @param combustible String con el combustible de la recarga
     */
    @Override
    public void registraIDUltimaRecarga(Long fecha,String matricula,String combustible) {
       ultimaConsultaProvider.setUltimaRecarga(matricula,combustible,fecha);
    }


    /**
     * Metodo isReady, cuestiona si la BD ha sido poblada al solicitarActualizacionMapa de la aplicacion.
     *
     * @return  true si asi es, false en caso contrario
     */
    @Override
    public boolean isReady() {
        if(db.getCombustibleDAO().getCombustibles().size()>0){
            return true;
        }else{
            return false;
        }
    }

    /*Combustible*/

    /**
     * Metodo getCombustibles, devuelve un listado de los combustibles registrados en el sistema
     *
     * @return Un Objeto List<DTOCombustible>
     */
    @Override
    public List<DTOCombustible> getCombustibles() {
        List<Combustible> combustibles= db.getCombustibleDAO().getCombustibles();
        ArrayList<DTOCombustible> listado =new ArrayList<DTOCombustible>(0);
        Combustible combustible;
        DTOCombustible dtoCombustible;
        for(int i=0;i<combustibles.size();i++){
            combustible=combustibles.get(i);
            dtoCombustible = new DTOCombustible(combustible.getId(),combustible.getTipo(),combustible.getUnidades());
            listado.add(dtoCombustible);
        }
        return listado;
    }


    private void setCombustibles(List<Combustible> combustibles){
        int i=0;
        while(i<combustibles.size()){
            db.getCombustibleDAO().addCombustible(combustibles.get(i));
            i++;
        }
    }

    private Integer stringToIdCombustible(String combustible){
        List<Combustible> combustibles= db.getCombustibleDAO().getCombustibles();
        Combustible c=null;
        if(combustible.contains("(")){
            combustible=combustible.substring(0,combustible.indexOf("("));
            combustible=combustible.trim();
        }
        for(int i=0;i<combustibles.size();i++){

            if(combustibles.get(i).getTipo().equals(combustible)){
                c=combustibles.get(i);
            }
        }
        Integer id;
        try {
            id=c.getId();
        }catch(NullPointerException excp){
            id=null;
        }
        return id;
    }

    private String idToCombustibleString(Integer combustible){
        List<Combustible> combustibles= db.getCombustibleDAO().getCombustibles();
        Combustible c=null;
        for(int i=0;i<combustibles.size();i++){
            if(combustibles.get(i).getId().equals(combustible)){
                c=combustibles.get(i);
            }
        }
        String id;
        try {
            id=c.getTipo()+" "+"("+c.getUnidades()+")";
        }catch(NullPointerException excp){
            id=null;
        }
        return id;
    }


    /*Vehiculo*/


    /**
     * Metodo getVehiculo, devuelve un DTOVehiculo con los datos del vehiculo cuya matricula se ha especificado.
     *
     * @param matricula String con la matricula del vehiculo
     * @return  Un DTOVehiculo
     */
    @Override
    public DTOVehiculo getVehiculo(String matricula) {

        Vehiculo v = db.getVehiculoDAO().getVehiculo(matricula);

        DTOVehiculo dto = getDTOVehiculo(v);

        return dto;

    }

    /**
     * Metodo getVehiculos, devuelve un listado de los vehiculos registrados en el sistema.
     *
     * @return  Un Objeto List<DTOVehiculo>
     */
    @Override
    public List<DTOVehiculo> getVehiculos() {

        List<Vehiculo> vehiculos=db.getVehiculoDAO().getVehiculos();
        List<Combustible> combustibles=db.getCombustibleDAO().getCombustibles();
        List<DTOVehiculo> res = new ArrayList<DTOVehiculo>(0);
        int i=0,j=0;
        DTOVehiculo tmp;Vehiculo v;Combustible c;
        String matricula, nombre;
        while(i<vehiculos.size()){
            v=vehiculos.get(i);
            matricula=v.getMatricula();
            nombre=v.getNombre();
            tmp=new DTOVehiculo(matricula,nombre);
            tmp.setAño(v.getAño());
            tmp.setMarca(v.getMarca());
            tmp.setModelo(v.getModelo());
            tmp.setKilometraje(v.getKilometraje());
            while(j<combustibles.size()){
                c=combustibles.get(j);
                if(c.getId().equals(v.getCombustiblePrimario())){
                    tmp.setCombustiblePrimario(c.getTipo()+" "+"("+c.getUnidades()+")");
                }
                if(c.getId().equals(v.getCombustibleSecundario())){
                    tmp.setCombustibleSecundario(c.getTipo()+" "+"("+c.getUnidades()+")");
                }
                j++;
            }
            tmp.setCapTankPrimario(v.getCapTankPrimario());
            tmp.setCapTankSecundario(v.getCapTankSecundario());


            byte[] byteArray= v.getFoto();

            Bitmap bm = BitmapFactory.decodeByteArray(byteArray, 0 ,byteArray.length);
            tmp.setFoto(bm);
            i++;
        }
        return res;
    }

    /**
     * Metodo actualizaVehiculo, actualiza la informacion de un vehiculo ya registrado.
     *
     * @param vehiculo  Un Objeto DTOVehiculo con la nueva informacion
     * @return true si exito, false en caso contrario
     */
    @Override
    public boolean actualizaVehiculo(DTOVehiculo vehiculo) {

        Vehiculo v = getEntityVehiculo(vehiculo);

        db.getVehiculoDAO().actualizaVehiculo(v);

        return true;
    }

    /**
     * Metodo isVehiculoRegistrado, cuestiona si un determinado vehiculo esta registrado.
     *
     * @param vehiculo  Un Objeto DTOVehiculo
     * @return true si esta registrado, false en caso contrario
     */
    @Override
    public boolean isVehiculoRegistrado(DTOVehiculo vehiculo){
        Vehiculo v=null;
        boolean res=false;
        try{
            v = db.getVehiculoDAO().getVehiculo(vehiculo.getMatricula());
            if(v!=null){
                res= true;
            }
            return res;
        }catch(Exception e){
            return res;
        }
    }

    /**
     * Metodo isVehiculoRegistrado, cuestiona si un determinado vehiculo esta registrado.
     *
     * @param matricula  Un String con el identificador univoco del vehiculo
     * @return true si esta registrado, false en caso contrario
     */
    @Override
    public boolean isVehiculoRegistrado(String matricula){
        Vehiculo v=null;
        boolean res=false;
        try{
            v = db.getVehiculoDAO().getVehiculo(matricula);
            if(v!=null){
                res= true;
            }
            return res;
        }catch(Exception e){
            return res;
        }
    }

    /**
     * Metodo setVehiculo, registra un nuevo vehiculo.
     *
     * @param vehiculo  Un DTOVehiculo con la informacion a registrar
     * @return  true en caso de exito en la operacion, false en caso contrario
     */
    @Override
    public boolean setVehiculo(DTOVehiculo vehiculo) {

        Vehiculo v = getEntityVehiculo(vehiculo);

        db.getVehiculoDAO().addVehiculo(v);

        return true;
    }

    /**
     * Metodo getIdentificadoresVehiculos, devuelve un listado de identificadores de los vehiculos registrados.
     *
     * @return  Un Objeto List<DTOVehiculo>
     */
    @Override
    public List<DTOVehiculo> getIdentificadoresVehiculos(){
        List<Vehiculo> vehiculos=db.getVehiculoDAO().getVehiculos();
        if(vehiculos==null)
            Log.d(TAG,"Listado Nulo");
        List<DTOVehiculo> res = new ArrayList<DTOVehiculo>(0);
        int i=0;
        Vehiculo v;
        DTOVehiculo dto;
        while(i<vehiculos.size()){
            v=vehiculos.get(i);
            dto = new DTOVehiculo(v.getMatricula(),v.getNombre());
            res.add(dto);
            i++;
        }
        return res;
    }

    /**
     * Metodo borradoVehiculo, borra un determinado vehiculo registrado.
     *
     * @param matricula Un String con la matricula del vehiculo a borrar
     * @return true si exito en al operacion, false en caso contrario
     */
    @Override
    public boolean borradoVehiculo(String matricula) {

        Vehiculo v=db.getVehiculoDAO().getVehiculo(matricula);

        db.getVehiculoDAO().borraVehiculo(v);

        return true;
    }


    private  Vehiculo getEntityVehiculo(DTOVehiculo vehiculo){

        Integer cSec;

        try {
            cSec = stringToIdCombustible(vehiculo.getCombustibleSecundario());
        }catch(NullPointerException excp){
            cSec=null;
        }

        byte[] byteArray;

        ByteArrayOutputStream stream = new ByteArrayOutputStream();
        Bitmap bmp = vehiculo.getFoto();
        if(bmp!=null){
            bmp.compress(Bitmap.CompressFormat.PNG, 100, stream);
            byteArray = stream.toByteArray();
            bmp.recycle();
        }else{
            byteArray=null;
        }

        Vehiculo v = new Vehiculo(vehiculo.getMatricula(),vehiculo.getNombre(),stringToIdCombustible(vehiculo.getCombustiblePrimario()));

        if(vehiculo.getMarca()!=null){
            v.setMarca(vehiculo.getMarca());

        }
        if(vehiculo.getModelo()!=null){
            v.setModelo(vehiculo.getModelo());

        }
        if(vehiculo.getAño()!=null){
            v.setAño(vehiculo.getAño());

        }
        if(((Double)vehiculo.getKilometraje())!= null&&vehiculo.getKilometraje()!=0){
            v.setKilometraje(vehiculo.getKilometraje());

        }

        if(cSec!=null&&cSec!=0){
            v.setCombustibleSecundario(cSec);

            if(vehiculo.getCapTankSecundario()!=0){
                v.setCapTankSecundario(vehiculo.getCapTankSecundario());

            }
        }
        if((Double)vehiculo.getCapTankPrimario()!=0){

            v.setCapTankPrimario((Double)vehiculo.getCapTankPrimario());
        }
        if(byteArray!=null){

            v.setFoto(byteArray);
        }
        return v;

    }

    private DTOVehiculo getDTOVehiculo(Vehiculo v){

        Combustible c;
        DTOVehiculo dto = new DTOVehiculo();
        dto.setMatricula(v.getMatricula());
        dto.setNombre(v.getNombre());
        dto.setMarca(v.getMarca());
        dto.setModelo(v.getModelo());
        if(v.getAño()==null){
            Calendar cal= Calendar.getInstance();
            int year= cal.get(Calendar.YEAR);
            dto.setAño(year);
        }else{
            dto.setAño(v.getAño());
        }
        if(v.getKilometraje()==null){
            dto.setKilometraje(0.0);
        }else{  dto.setKilometraje(v.getKilometraje());}

        dto.setCombustiblePrimario(idToCombustibleString(v.getCombustiblePrimario()));

        dto.setCombustibleSecundario(idToCombustibleString(v.getCombustibleSecundario()));

        if(v.getCapTankPrimario()==null){dto.setCapTankPrimario(0);}else{dto.setCapTankPrimario(v.getCapTankPrimario());}
        if(v.getCapTankSecundario()==null){dto.setCapTankSecundario(0);}else{dto.setCapTankSecundario(v.getCapTankSecundario());}
        Bitmap bitmap;
        try {
            bitmap = BitmapFactory.decodeByteArray(v.getFoto(), 0, v.getFoto().length);
        }catch(NullPointerException excp){
            bitmap=null;
        }
        if(bitmap!=null){
            dto.setFoto(bitmap);
        }
        return dto;
    }


    /*Recargas*/

    /**
     * Metodo getListadoRecargas, devuelve un listado de las recargas asociadas a un vehiculo.
     *
     * @param matricula Un String con la amtricula del vehiculo.
     * @return  Un Objeto List<DTORecarga>
     */
    @Override
    public List<DTORecarga> getListadoRecargas(String matricula) {
        List<Recarga> recargas= db.getRecargaDAO().getRecargas(matricula);
        ArrayList<DTORecarga> resultado=new ArrayList<>(0);
        DTORecarga recarga=null;
        Recarga tmp;

        for(int i=0;i<recargas.size();i++){

            tmp=recargas.get(i);

            recarga=getDTORecarga(tmp);

            resultado.add(recarga);
        }
        return resultado;
    }

    /**
     * Metodo getRecarga, devuelve el conjunto de informacion asociado a una recarga.
     *
     * @param matricula Un String con la matricula de un vehiculo
     * @param fecha  Un Objeto Date con la fecha de la recarga
     * @param combustible Un String con el combustible del vehiculo sobre el que realizar la busqueda
     * @return Un Objeto DTORecarga
     */
    @Override
    public DTORecarga getRecarga(String matricula, Date fecha, String combustible) {

        Recarga r = db.getRecargaDAO().getRecarga(matricula,stringToIdCombustible(combustible),fecha.getTime());

        DTORecarga dto=getDTORecarga(r);

        return dto;
    }

    /**
     * Metodo setRecarga, registra el conjunto de datos de una recarga.
     *
     * @param recarga Un OBjeto DTORecarga  con los datos a registrar
     */
    @Override
    public void setRecarga(DTORecarga recarga) {

        Recarga r = getEntityRecarga(recarga);

        db.getRecargaDAO().addRecarga(r);
    }

    /**
     * Metodo getRecargasEnPeriodoPorCombustibleyVehiculo, devuelve un listado de recargas de combustible del vehiculo especificado
     * que se produjeron entre las fechas de solicitarActualizacionMapa y fin de mes especificadas.
     *
     * @param matricula Un String con la matricula de un vehiculo
     * @param combustible  Un String con el combustible del vehiculo sobre el que realizar la busqueda
     * @param primerDia  Un Objeto Date con la fecha de solicitarActualizacionMapa a partir de la que se hace la busqueda
     * @param ultimoDia  Un Objeto Date con la fecha limite a partir de la que se hace la busqueda
     * @return Un Objeto List<DTORecarga>
     */
    @Override
    public List<DTORecarga> getRecargasEnPeriodoPorCombustibleyVehiculo(String matricula, String combustible, Date primerDia, Date ultimoDia) {



       List<Recarga> recargas=db.getRecargaDAO().getRecargasEntreFechasforCombustible(matricula,stringToIdCombustible(combustible),primerDia.getTime(),ultimoDia.getTime());

       ArrayList<DTORecarga> r=new ArrayList<>(0);

       Recarga tmp=null;

       DTORecarga dto=null;

       for(int i=0;i<recargas.size();i++){

           tmp=recargas.get(i);

           dto=getDTORecarga(tmp);

           r.add(dto);
       }
        return r;
    }

    /**
     * Metodo getLastrecargaBefore, devuelve la ultima recarga asociada al combustible de un vehiculo antes de la fecha especificada.
     *
     * @param matricula Un String con la matricula de un vehiculo
     * @param dateIni   Un Objeto Date con la fecha limite a partir de la que se hace la busqueda
     * @param combustible   Un String con el combustible del vehiculo sobre el que realizar la busqueda
     * @return  Un Objeto DTORecarga
     */
    @Override
    public DTORecarga getLastRecargaBefore(String matricula,Date dateIni,String combustible) {

        Recarga tmp=db.getRecargaDAO().getLastRecargaBeforeDate(matricula,dateIni.getTime(),stringToIdCombustible(combustible));

        DTORecarga dto=getDTORecarga(tmp);

        return dto;
    }

    /**
     * Metodo getLastRecargaPorCombustible, devuelve la ultima recarga asociada al combustible de un vehiculo.
     *
     * @param matricula Un String con la matricula de un vehiculo
     * @param combustible   Un String con el combustible del vehiculo sobre el que realizar la busqueda
     * @return  Un Objeto DTORecarga
     */
    @Override
    public DTORecarga getLastRecargaPorCombustible(String matricula, String combustible) {

        Recarga tmp=db.getRecargaDAO().getLastRecargaPorCombustible(matricula,stringToIdCombustible(combustible));

        DTORecarga dto=getDTORecarga(tmp);

        return dto;
    }

    /**
     * Metodo getLastRecarga, devuelve la ultima recarga asociada a  un vehiculo.
     *
     * @param matricula Un String con la matricula de un vehiculo
     * @param date Un Date con la fecha a partir de la que buscar
     * @return  Un Objeto DTORecarga
     */
    @Override
    public DTORecarga getLastRecargaBefore(String matricula,Date date) {

        Recarga tmp=db.getRecargaDAO().getLastRecarga(matricula,date.getTime());

        DTORecarga dto=getDTORecarga(tmp);

        return dto;
    }


    private Recarga getEntityRecarga(DTORecarga recarga){
        Recarga r = new Recarga(recarga.getFecha(),recarga.getMatricula(),stringToIdCombustible(recarga.getCombustible()));
        r.setKilometraje(recarga.getKm());
        r.setCoste(recarga.getCoste());
        r.setCombustibleRecargado(recarga.getMedicion());
        ByteArrayOutputStream stream = new ByteArrayOutputStream();
        Bitmap bmp = recarga.getFoto();
        bmp.compress(Bitmap.CompressFormat.PNG, 100, stream);
        byte[] byteArray = stream.toByteArray();
        bmp.recycle();
        if(byteArray!=null){
            r.setFoto(byteArray);
        }
        return r;
    }
    private DTORecarga getDTORecarga( Recarga tmp){

        DTORecarga dto;

        dto = new DTORecarga();
        try {

            dto.setFecha(tmp.getFecha());
            dto.setCombustible(idToCombustibleString(tmp.getCombustible()));
            double coste;
            if (tmp.getCoste() == null) {
                coste = 0.0;
            } else {
                coste = tmp.getCoste();
            }
            dto.setCoste(coste);
            double medicion;
            if (tmp.getCombustibleRecargado() == null) {
                medicion = 0.0;
            } else {
                medicion = tmp.getCombustibleRecargado();
            }
            dto.setMedicion(medicion);
            double km;
            if (tmp.getKilometraje() == null) {
                km = 0.0;
            } else {
                km = tmp.getKilometraje();
            }
            dto.setKm(km);
            Bitmap bitmap;
            try {
                BitmapFactory.Options options = new BitmapFactory.Options();
                options.inSampleSize = 2;
                //bitmap = BitmapFactory.decodeByteArray(tmp.getFoto(),options);
                bitmap = BitmapFactory.decodeByteArray(tmp.getFoto(), 0, tmp.getFoto().length,options);
            } catch (NullPointerException excp) {
                bitmap = null;
            }
            dto.setFoto(bitmap);


        }catch(NullPointerException npe){
            dto=null;
        }
        return dto;
    }

    /*Datos Derivados Vehiculo*/

    /**
     * Metodo getDDerivadosVehiculo, devuelve el conjunto de datos derivados de recargas asociados a un cierto vehiculo.
     *
     * @param matricula Un String con la matricula del vehiculo
     * @param anio  Un Integer con el año asociado a las recargas del vehiculo
     * @param mes   Un Integer con el mes asociado a las recargas del vehiculo
     * @return Un Objeto DTODatosDerivadosVehiculoTotales
     */
    @Override
    public DTODatosDerivadosVehiculoTotales getDDerivadosVehiculo(String matricula, int anio, int mes) {

        DatosDerivadosVehiculoTotales d = db.getDatDervVehiculoTotalDAO().getRegistro(matricula,anio,mes);

        DTODatosDerivadosVehiculoTotales registro=getDTODatosDerivadosVehiculoTotales(d);

        return registro;

    }
    /**
     * Metodo getUltimoRegistroDDerivadosVehiculo, devuelve el conjunto de datos dderivados de recargas asociados
     * a un vehiculo.
     *
     * @param matricula     Un String con la matricula del vehiculo
     * @return  Un Objeto DTODatosDerivadosVehiculoTotales
     */
    @Override
    public DTODatosDerivadosVehiculoTotales getUltimoRegistroDDerivadosVehiculo(String matricula) {

        DatosDerivadosVehiculoTotales d = db.getDatDervVehiculoTotalDAO().getUltimoRegistro(matricula);

        DTODatosDerivadosVehiculoTotales registro= getDTODatosDerivadosVehiculoTotales(d);

        return registro;

    }

    /**
     * Metodo registraDatosDerivadosVehiculo, registra el conjunto de datos derivados de recargas asociados
     * a un vehiculo.
     *
     * @param ddv   Un Objeto DTODatosDerivadosVehiculoTotales
     */
    @Override
    public void registraDatosDerivadosVehiculo(DTODatosDerivadosVehiculoTotales ddv) {

        DatosDerivadosVehiculoTotales d = getEntityDatosDerivadosVehiculoTotales(ddv);

        db.getDatDervVehiculoTotalDAO().addRegistro(d);
    }

    /**
     * Metodo actualizaDatosDerivadosVehiculo, actualiza el conjunto de datos derivados de recargas sobre el
     * asociados a un vehiculo.
     *
     * @param ddv   Un Objeto DTODatosDerivadosVehiculoTotales, con el conjunto de datos que se actualizaran
     */
    @Override
    public void actualizaDatosDerivadosVehiculo(DTODatosDerivadosVehiculoTotales ddv) {

        DatosDerivadosVehiculoTotales d = getEntityDatosDerivadosVehiculoTotales(ddv);

        db.getDatDervVehiculoTotalDAO().actualizaRegistro(d);
    }

    private DatosDerivadosVehiculoTotales getEntityDatosDerivadosVehiculoTotales(DTODatosDerivadosVehiculoTotales ddv){

        DatosDerivadosVehiculoTotales d=null;
        if(ddv!=null) {
            d = new DatosDerivadosVehiculoTotales(ddv.getMatricula(), ddv.getAnio(), ddv.getMes());
            d.setNumeroRecargasTotalesPorMes(ddv.getNumeroRecargasMes());
            d.setNumeroRecargasTotalesPorAnio(ddv.getNumeroRecargasAnio());
            d.setKmRecargasTotalesPorMes(ddv.getKmRecargasTotalesPorMes());
            d.setKmRecargasTotalesPorAnio(ddv.getKmRecargasTotalesPorAnio());
            d.setCosteRecargasTotalesPorMes(ddv.getCosteRecargasTotalesPorMes());
            d.setCosteRecargasTotalesPorAnio(ddv.getCosteRecargasTotalesPorAnio());
        }
        return d;
    }

    private DTODatosDerivadosVehiculoTotales getDTODatosDerivadosVehiculoTotales(DatosDerivadosVehiculoTotales d){

        DTODatosDerivadosVehiculoTotales registro=null;
        if(d!=null) {
            registro = new DTODatosDerivadosVehiculoTotales(d.getMatriculaVehiculo(), d.getAnio(), d.getMes());
            registro.setNumeroRecargasAnio(d.getNumeroRecargasTotalesPorAnio());
            registro.setNumeroRecargasMes(d.getNumeroRecargasTotalesPorMes());
            registro.setKmRecargasTotalesPorMes(d.getKmRecargasTotalesPorMes());
            registro.setKmRecargasTotalesPorAnio(d.getKmRecargasTotalesPorAnio());
            registro.setCosteRecargasTotalesPorMes(d.getCosteRecargasTotalesPorMes());
            registro.setCosteRecargasTotalesPorAnio(d.getCosteRecargasTotalesPorAnio());
        }
        return registro;
    }

    /*Datos Derivados Combustible*/

    /**
     * Metodo getDDerivadosCombustible, devuelve el conjunto de datos derivados asociados a las recargas de un combustible
     * del vehiculo especificado.
     *
     * @param matricula Un String con la matricula del vehiculo
     * @param year  Un Integer con el año asociado a la busqueda
     * @param month Un Integer con el mes asociado a la busqueda
     * @param combustible   Un String con el combustible del vehiculo
     * @return  Un Objeto DTODatosDerivadosCombustible
     */
    @Override
    public DTODatosDerivadosCombustible getDDerivadosCombustible(String matricula, int year, int month, String combustible) {

        DatosDerivadosCombustible d= db.getDatDervCombustibleDAO().getRegistro(matricula,year,month,stringToIdCombustible(combustible));

        DTODatosDerivadosCombustible dto =  getDTODatosDerivadosCombustible(d);

        return dto;
    }

    /**
     * Metodo getUltimoRegistroDDerivadosCombustible, devuelve el conjunto de datos derivados de recargas al combustible
     * especificado.
     *
     * @param matricula     Un String con la matricula del vehiculo
     * @param combustible   Un String con el combustible del vehiculo
     * @return  Un Objeto DTODatosDerivadosCombustible
     */
    @Override
    public DTODatosDerivadosCombustible getUltimoRegistroDDerivadosCombustible(String matricula, String combustible) {

        DatosDerivadosCombustible d= db.getDatDervCombustibleDAO().getUltimoRegistro(matricula, stringToIdCombustible(combustible));

        DTODatosDerivadosCombustible dto =  getDTODatosDerivadosCombustible(d);

        return dto;
    }

    /**
     * Metodo registraDatosDerivadosCombustible, registra el conjunto de datos derivados de recargas al combustible
     * especificado.
     *
     * @param ddv   Un Objeto DTODatosDerivadosCombustible
     */
    @Override
    public void registraDatosDerivadosCombustible(DTODatosDerivadosCombustible ddv) {

        DatosDerivadosCombustible d = getEntityDatosDerivadosCombustible(ddv);

        db.getDatDervCombustibleDAO().addRegistro(d);
    }

    /**
     * Metodo actualizaDatosDerivadosCombustible, actualiza el conjunto de datos derivados de recargas sobre el
     * combustible especificado de un vehiculo.
     *
     * @param ddv   Un Objeto DTODatosDerivadosCombustible, con el conjunto de datos que se actualizaran
     */
    @Override
    public void actualizaDatosDerivadosCombustible(DTODatosDerivadosCombustible ddv) {

        DatosDerivadosCombustible d =getEntityDatosDerivadosCombustible(ddv);

        db.getDatDervCombustibleDAO().actualizaRegistro(d);
    }

    private DatosDerivadosCombustible getEntityDatosDerivadosCombustible(DTODatosDerivadosCombustible ddv){
        DatosDerivadosCombustible d=null;
        if(ddv!=null) {
            d = new DatosDerivadosCombustible(ddv.getMatriculaVehiculo(),ddv.getMes() , ddv.getAnio(), stringToIdCombustible(ddv.getCombustible()));

            d.setNumeroRecargasCombustiblePorMes(ddv.getNumeroRecargasCombustiblePorMes());
            d.setNumeroRecargasCombustiblePorAnio(ddv.getNumeroRecargasCombustiblePorAnio());

            d.setMaximoCombustibleMensualRecargado(ddv.getMaximoCombustibleMensualRecargado());
            d.setMaximoCombustibleXKmMensual(ddv.getMaximoCombustibleXKmMensual());
            d.setMinimoCombustibleMensualRecargado(ddv.getMinimoCombustibleMensualRecargado());
            d.setMinimoCombustibleXKmMensual(ddv.getMinimoCombustibleXKmMensual());

            d.setMaximoCombustibleAnualRecargado(ddv.getMaximoCombustibleAnualRecargado());
            d.setMaximoCombustibleXKmAnual(ddv.getMaximoCombustibleXKmAnual());
            d.setMinimoCombustibleAnualRecargado(ddv.getMinimoCombustibleAnualRecargado());
            d.setMinimoCombustibleXKmAnual(ddv.getMinimoCombustibleXKmAnual());

            d.setSumaCombustibleRecargadoPorMes(ddv.getSumaCombustibleRecargadoPorMes());
            d.setSumaCombustibleRecargadoPorAnio(ddv.getSumaCombustibleRecargadoPorAnio());
            d.setSumaCombustibleXKmMensual(ddv.getSumaCombustibleXKmMensual());
            d.setSumaCombustibleXKmAnual(ddv.getSumaCombustibleXKmAnual());
        }
        return d;
    }

    private DTODatosDerivadosCombustible getDTODatosDerivadosCombustible(DatosDerivadosCombustible d){
        DTODatosDerivadosCombustible dto=null;
        if(d!=null) {
            dto = new DTODatosDerivadosCombustible(d.getMatriculaVehiculo(), d.getAnio(), d.getMes(), idToCombustibleString(d.getCombustible()));

            dto.setNumeroRecargasCombustiblePorMes(d.getNumeroRecargasCombustiblePorMes());
            dto.setNumeroRecargasCombustiblePorAnio(d.getNumeroRecargasCombustiblePorAnio());

            dto.setMaximoCombustibleMensualRecargado(d.getMaximoCombustibleMensualRecargado());
            dto.setMaximoCombustibleXKmMensual(d.getMaximoCombustibleXKmMensual());
            dto.setMinimoCombustibleMensualRecargado(d.getMinimoCombustibleMensualRecargado());
            dto.setMinimoCombustibleXKmMensual(d.getMinimoCombustibleXKmMensual());

            dto.setMaximoCombustibleAnualRecargado(d.getMaximoCombustibleAnualRecargado());
            dto.setMaximoCombustibleXKmAnual(d.getMaximoCombustibleXKmAnual());
            dto.setMinimoCombustibleAnualRecargado(d.getMinimoCombustibleAnualRecargado());
            dto.setMinimoCombustibleXKmAnual(d.getMinimoCombustibleXKmAnual());

            dto.setSumaCombustibleRecargadoPorMes(d.getSumaCombustibleRecargadoPorMes());
            dto.setSumaCombustibleRecargadoPorAnio(d.getSumaCombustibleRecargadoPorAnio());
            dto.setSumaCombustibleXKmMensual(d.getSumaCombustibleXKmMensual());
            dto.setSumaCombustibleXKmAnual(d.getSumaCombustibleXKmAnual());
        }
        return dto;
    }



    /*Datos Derivados Coste Combustible*/

    /**
     * Metodo getDDerivadosCosteCombustible, devuelve el conjunto de datos derivados de recargas de
     * costes de combustible para un vehiculo y combustible especificos.
     *
     * @param matricula Un String con la matricula del vehiculo sobre el que buscar
     * @param year  Un Integer con el anio sobre el que buscar
     * @param month Un Integer con el mes sobre  el que buscar
     * @param combustible   Un String con el combustible sobre el que buscar
     * @return Un Objeto DTODatosDerivadosCostesCombustible
     */
    @Override
    public DTODatosDerivadosCostesCombustible getDDerivadosCosteCombustible(String matricula, int year, int month, String combustible) {

        DatosDerivadosCostesCombustible datos= db.getDatDervCostesCombustibleDAO().getRegistro(matricula,year,month, stringToIdCombustible(combustible));



        DTODatosDerivadosCostesCombustible dto= getDTODatosDerivadosCostesCombustible(datos);

        return dto;
    }


    /**
     * Metodo getUltimoRegistroDDerivadosCosteCombustible, devuelve el ultimo registro de datos derivados del coste
     * de combustible para un vehiuclo especifico.
     *
     * @param matricula Un String con la matricula del  vehicuo
     * @param combustible   Un String con el combustible del vehiculo sobre el que se realiza la búsqueda
     * @return  Un Objeto DTODatosDerivadosCostesCombustible o null si no existe
     */
    @Override
    public DTODatosDerivadosCostesCombustible getUltimoRegistroDDerivadosCosteCombustible(String matricula, String combustible) {

        DatosDerivadosCostesCombustible datos= db.getDatDervCostesCombustibleDAO().getUltimoRegistro(matricula, stringToIdCombustible(combustible));



        DTODatosDerivadosCostesCombustible dto= getDTODatosDerivadosCostesCombustible(datos);

        return dto;
    }
    /**
     * Metodo actualizaDatosDerivadosCosteCombustible, actualiza los datos derivados de coste del combustible.
     *
     * @param ddv   Un Objeto DTODatosDerivadosCostesCombustible
     */
    @Override
    public void actualizaDatosDerivadosCosteCombustible(DTODatosDerivadosCostesCombustible ddv) {

        DatosDerivadosCostesCombustible datos= getEntityDatosDerivadosCosteCombustible(ddv);



        db.getDatDervCostesCombustibleDAO().actualizaRegistro(datos);
    }

    /**
     * Metodo registrarDatosDerivadosCosteCombustible, registra los datos derivados de coste del combustible.
     *
     * @param ddv   Un Objeto DTODatosDerivadosCostesCombustible
     */
    @Override
    public void registraDatosDerivadosCosteCombustible(DTODatosDerivadosCostesCombustible ddv) {

        DatosDerivadosCostesCombustible datos= getEntityDatosDerivadosCosteCombustible(ddv);

        db.getDatDervCostesCombustibleDAO().addRegistro(datos);
    }

    private DatosDerivadosCostesCombustible getEntityDatosDerivadosCosteCombustible( DTODatosDerivadosCostesCombustible datos){

        DatosDerivadosCostesCombustible d=null;
        if(datos!=null) {
            d = new DatosDerivadosCostesCombustible(datos.getMatriculaVehiculo(),datos.getMes() , datos.getAnio(), stringToIdCombustible(datos.getCombustible()));

            d.setNumeroRecargasCombustiblePorMes(datos.getNumeroRecargasCombustiblePorMes());
            d.setNumeroRecargasCombustiblePorAnio(datos.getNumeroRecargasCombustiblePorAnio());

            d.setMaximoCosteRecargaMensual(datos.getMaximoCosteRecargaMensual());
            d.setMaximoCosteRecargaAnual(datos.getMaximoCosteRecargaAnual());
            d.setMinimoCosteRecargaMensual(datos.getMinimoCosteRecargaMensual());
            d.setMinimoCosteRecargaAnual(datos.getMinimoCosteRecargaAnual());
            d.setSumaCosteRecargaMensual(datos.getSumaCosteRecargaMensual());
            d.setSumaCosteRecargaAnual(datos.getSumaCosteRecargaAnual());


            d.setMaximoCosteXRecargaMensual(datos.getMaximoCosteXRecargaMensual());
            d.setMaximoCosteXRecargaAnual(datos.getMaximoCosteXRecargaAnual());
            d.setMinimoCosteXRecargaMensual(datos.getMinimoCosteXRecargaMensual());
            d.setMinimoCosteXRecargaAnual(datos.getMinimoCosteXRecargaAnual());
            d.setSumaCosteXRecargaMensual(datos.getSumaCosteXRecargaMensual());
            d.setSumaCosteXRecargaAnual(datos.getSumaCosteXRecargaAnual());
        }
        return d;

    }

    private DTODatosDerivadosCostesCombustible getDTODatosDerivadosCostesCombustible(DatosDerivadosCostesCombustible datos){

        DTODatosDerivadosCostesCombustible dto=null;
        if(datos!=null) {
            dto = new DTODatosDerivadosCostesCombustible(datos.getMatriculaVehiculo(), datos.getAnio(), datos.getMes(), idToCombustibleString(datos.getCombustible()));

            dto.setNumeroRecargasCombustiblePorMes(datos.getNumeroRecargasCombustiblePorMes());
            dto.setNumeroRecargasCombustiblePorAnio(datos.getNumeroRecargasCombustiblePorAnio());

            dto.setMaximoCosteRecargaMensual(datos.getMaximoCosteRecargaMensual());
            dto.setMaximoCosteRecargaAnual(datos.getMaximoCosteRecargaAnual());
            dto.setMinimoCosteRecargaMensual(datos.getMinimoCosteRecargaMensual());
            dto.setMinimoCosteRecargaAnual(datos.getMinimoCosteRecargaAnual());
            dto.setSumaCosteRecargaMensual(datos.getSumaCosteRecargaMensual());
            dto.setSumaCosteRecargaAnual(datos.getSumaCosteRecargaAnual());


            dto.setMaximoCosteXRecargaMensual(datos.getMaximoCosteXRecargaMensual());
            dto.setMaximoCosteXRecargaAnual(datos.getMaximoCosteXRecargaAnual());
            dto.setMinimoCosteXRecargaMensual(datos.getMinimoCosteXRecargaMensual());
            dto.setMinimoCosteXRecargaAnual(datos.getMinimoCosteXRecargaAnual());
            dto.setSumaCosteXRecargaMensual(datos.getSumaCosteXRecargaMensual());
            dto.setSumaCosteXRecargaAnual(datos.getSumaCosteXRecargaAnual());
        }
        return dto;
    }




}
