package com.example.juanalonsoastruga.cargascontrol.Datos.GasolinerasProvider;

import android.util.Log;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTOGasolinera;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import java.util.ArrayList;
import java.util.List;

public class GasolinerasHandler extends DefaultHandler {
    private StringBuilder sbText;
    private List<DTOGasolinera> gasolineras;
    private DTOGasolinera gasolineraActual;
    private double minLatitud;
    private double maxLatitud;
    private double minLongitud;
    private double maxLongitud;
    private final double latitud;
    private final double longitud;
    private  int maximoNumeroResultados;
    private int cont;
    public Boolean parsingError = false;
    private boolean cumpleRequisitos =true;
    private boolean etiquetaApertura=false;
    private double value;
    private String direccion="";
    private String horario="";
    private String localidad="";
    private String campo;

    private double etiquetalatitud=0;

    public GasolinerasHandler(double minLatitud, double maxLatitud, double minLongitud, double maxLongitud, double latitud, double longitud, int count){
        super();
        this.minLatitud=minLatitud;
        this.maxLatitud=maxLatitud;
        this.minLongitud=minLongitud;
        this.maxLongitud=maxLongitud;
        this.latitud=latitud;
        this.longitud=longitud;
        this.maximoNumeroResultados =count;
        this.gasolineras=new ArrayList<DTOGasolinera>(0);
        cont=0;
    }

    public List<DTOGasolinera> getGasolineras(){return gasolineras;}

    @Override
    public void startDocument() throws SAXException {

        super.startDocument();
        sbText = new StringBuilder();
    }

    @Override
    public void startElement(String uri, String localId, String name, Attributes attributes) throws SAXException {

        super.startElement(uri, localId, name, attributes);

        if (localId.equals("EESS")) {
            etiquetaApertura=true;
        }
    }

    @Override
    public void characters(char[] ch, int start, int length)
            throws SAXException {

        super.characters(ch, start, length);

        if (etiquetaApertura)
            sbText.append(ch, start, length);
    }

    /**
     * Metodo endElement, llamado cada vez que el parser encuentra una etiqueta de cierre.
     *
     * @param uri
     * @param localName
     * @param id
     * @throws SAXException
     */
    @Override
    public void endElement(String uri, String localName, String id)
            throws SAXException {

        super.endElement(uri, localName, id);



        if (etiquetaApertura) {

            if (localName.equals("Direccion")) {
                campo =  (sbText.toString().trim());
                direccion=campo;
            } else if (localName.equals("Horario")) {
                campo =  (sbText.toString().trim());
                horario=campo;
            } else if (localName.equals("Latitud")) {
                campo = (sbText.toString().trim());

                 try {
                    campo=campo.replace(',','.');
                    double cantidad = Double.parseDouble(campo);
                    if (minLatitud <= cantidad && maxLatitud >= cantidad) {
                        cumpleRequisitos = cumpleRequisitos & true;
                        etiquetalatitud = cantidad;
                    } else {
                        cumpleRequisitos = cumpleRequisitos & false;
                        direccion="";
                        horario="";
                        etiquetalatitud=0;
                        localidad="";
                    }
                 }catch (NumberFormatException nfe){
                     cumpleRequisitos = cumpleRequisitos & false;
                     Log.d("ERROR","contiene: "+campo);
                 }

            } else if (cumpleRequisitos&localName.equals("Localidad")) {
                campo = (sbText.toString().trim());
                localidad=campo;
            } else if (cumpleRequisitos&localName.equals("Longitud")) {
                campo = (sbText.toString().trim());
                campo=campo.replace(',','.');
                double cantidad=Double.parseDouble(campo);
                if(minLongitud<=cantidad&&maxLongitud>=cantidad){
                    cumpleRequisitos =cumpleRequisitos&true;
                    gasolineraActual=new DTOGasolinera();
                    gasolineraActual.setDireccion(direccion);
                    gasolineraActual.setHorario(horario);
                    gasolineraActual.setLatitud(etiquetalatitud);
                    gasolineraActual.setLocalidad(localidad);
                    gasolineraActual.setLongitud(cantidad);

                    gasolineraActual.setKm(convierteAKm(gasolineraActual.getLatitud(),gasolineraActual.getLongitud()));

                }else{
                    cumpleRequisitos =cumpleRequisitos&false;
                    gasolineraActual=null;
                }
                direccion="";
                horario="";
                etiquetalatitud=0;
                localidad="";

            } else if (cumpleRequisitos&localName.equals("Municipio")) {
                campo = (sbText.toString().trim());
                gasolineraActual.setMunicipio(campo);
            } else if (cumpleRequisitos&localName.equals("Biodiesel")) {
                campo = (sbText.toString().trim());
                if(campo.equals("true")){
                    gasolineraActual.setBiodiesel();
                }
            } else if (cumpleRequisitos&localName.equals("Bioetanol")) {
                campo = (sbText.toString().trim());
                if(campo.equals("true")){
                    gasolineraActual.setBioetanol();
                }
            } else if (cumpleRequisitos&localName.equals("GasNaturalComprimido")) {
                campo = (sbText.toString().trim());
                if(campo.equals("true")){
                    gasolineraActual.setGasNaturalComprimido();
                }
            } else if (cumpleRequisitos&localName.equals("GasNaturalLicuado")) {
                campo = (sbText.toString().trim());
                if(campo.equals("true")){
                    gasolineraActual.setGasNaturalLicuado();
                }
            } else if (cumpleRequisitos&localName.equals("Gaseslicuadosdelpetroleo")) {
                campo = (sbText.toString().trim());
                if(campo.equals("true")){
                    gasolineraActual.setGasLicuadoPetroleo();
                }
            } else if (cumpleRequisitos&localName.equals("GasoleoA")) {
                campo = (sbText.toString().trim());
                if(campo.equals("true")){
                    gasolineraActual.setGasoleoA();
                }
            } else if (cumpleRequisitos&localName.equals("GasoleoB")) {
                campo = (sbText.toString().trim());
                if(campo.equals("true")){
                    gasolineraActual.setGasoleoB();
                }
            } else if (cumpleRequisitos&localName.equals("Gasolina95Proteccion")) {
                campo = (sbText.toString().trim());
                if(campo.equals("true")){
                    gasolineraActual.setGasolina95();
                }
            } else if (cumpleRequisitos&localName.equals("Gasolina98")) {
                campo = (sbText.toString().trim());
                if(campo.equals("true")){
                    gasolineraActual.setGasolina98();
                }
            } else if (cumpleRequisitos&localName.equals("NuevoGasoleoA")) {
                campo = (sbText.toString().trim());
                if(campo.equals("true")){
                    gasolineraActual.setNuevoGasoleoA();
                }
            } else if (cumpleRequisitos&localName.equals("Provincia")) {
                campo = (sbText.toString().trim());
                gasolineraActual.setProvincia(campo);
            } else if (cumpleRequisitos&localName.equals("Rotulo")) {
                campo = (sbText.toString().trim());
                gasolineraActual.setRotulo(campo);
            } else if (localName.equals("EESS")) {
                etiquetaApertura=false;
                if(cumpleRequisitos) {
                    addElemento(gasolineraActual);
                    System.out.println("PUNTO VALIDO "+gasolineraActual.getDireccion()+" km "+gasolineraActual.getKm());
                }
                gasolineraActual=null;
                cumpleRequisitos=true;
            }else{
                // Error
                parsingError = true;
            }

            sbText.setLength(0);

        }
    }
    private void addElemento(DTOGasolinera gasolinera){
        //Hay que tener en cuenta que como espania esta atravesado por el meridiano de
        //Greenwich puede tener coordenadas con longitudes positivas y negativas pero solo latitudes positivas
        //ya que el ecuador no nos atraviesa.

        DTOGasolinera tmp;

        if(gasolineras.size()<maximoNumeroResultados){

            for(int i=0;gasolinera!=null&&i<gasolineras.size();i++){
                tmp=gasolineras.get(i);
                System.out.println("COMPARACION RELLENO"+gasolinera.getDireccion()+" km "+gasolinera.getKm());
                if(Double.compare(gasolinera.getKm(),tmp.getKm())<0){
                    gasolineras.add(i,gasolinera);

                    System.out.println("RELLENO CORRECTO"+gasolinera.getDireccion()+" km "+gasolinera.getKm());
                    System.out.println("RELLENADO CORRECTO"+tmp.getDireccion()+" km "+tmp.getKm());
                    gasolinera=null;
                }

            }

            if(gasolinera!=null){
                    //O bien todos son menores en cuyo caso este va al final por ser el mayor o bien el array esta vacio
                    gasolineras.add(gasolinera);
                    System.out.println("PUNTO VALIDO  COMPARACION RELLENO CORRECTO");
            }


        //Este else controlara que sea igual al tamanio, no mayor
        }else{
            sustituyeAlMayor(gasolinera);
        }
    }

    private void sustituyeAlMayor(DTOGasolinera gasolinera){
        DTOGasolinera tmp;

        for(int i=0;gasolinera!=null&&i<gasolineras.size();i++){
            tmp=gasolineras.get(i);
            System.out.println("PUNTO VALIDO  COMPARACION"+gasolinera.getDireccion()+" km "+gasolinera.getKm());
            if(Double.compare(gasolinera.getKm(),tmp.getKm())<0){

                System.out.println("SUSTITUCION CORRECTO "+gasolinera.getDireccion()+" km "+gasolinera.getKm());
                System.out.println("SUSTITUIDO CORRECTO  i: "+i+" "+tmp.getDireccion()+" km "+tmp.getKm());
                gasolineras.set(i,gasolinera);
                gasolinera=null;
            }

        }
    }

    private double convierteAKm(double latitud1,double longitud1){
       //Para hacer esto se debe usar la formula del haversine para poder tener en cuenta el error
       //introducido por la curvatura de la Tierra al calcular la distancia entre dos puntos.

        //KM del radio de la Tierra en el  ecuador
        int radio = 6371;

        double c1=calcularDistanciaEnRadianes(latitud,latitud1);
        double c2=calcularDistanciaEnRadianes(longitud,longitud1);

        //haversin(distancia entre puntos/ radio esfera) = haversin(diferencia de latitudes)+cos(latitud1)*cos(latitud2)
        //*haversin(diferencia de longitudes)



        double a = Math.sin(c1 / 2) * Math.sin(c1 / 2)
                + Math.cos(Math.toRadians(latitud)) * Math.cos(Math.toRadians(latitud1))
                * Math.sin(c2 / 2) * Math.sin(c2 / 2);
        double c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1 - a));
        double distance = radio * c ;
        return distance;
    }

    private double calcularDistanciaEnRadianes(double p1, double p2){

        if(p1>p2){
            return Math.toRadians(p1-p2);
        }
        else{
            return Math.toRadians(p2-p1);
        }

    }
}
