package com.example.juanalonsoastruga.cargascontrol.Formateador.CUBorrarVehiculo;

import com.example.juanalonsoastruga.cargascontrol.Controlador.CUBorrarVehiculo.InterfaceCBorradoVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Controlador.CUBorrarVehiculo.ControladorBorradoVehiculo;

/**
 * Clase FormateadorBorradoVehiculo, representa al formateador en el patron MVP del CU Borrado de Vehiculo
 *
 */
public class FormateadorBorradoVehiculo implements InterfaceFBorradoVehiculoImpl, InterfaceFBorradoVehiculo {

    private InterfaceCBorradoVehiculo formateador;
    private String matricula;
    private boolean dialogoBorrado;

    /**
     * Constructor de clase.
     *
     */
    public FormateadorBorradoVehiculo(){
        setControlador(  new ControladorBorradoVehiculo(this));
        dialogoBorrado=false;
    }


    /**
     * Metodo comienzoCU, inicia la ejecucion del CU.
     *
     * @param matricula String representando a la matricula sobre la que borrar el contenido
     */
    @Override
    public void init(String matricula){

        setMatricula(matricula);
        formateador.botonBorrarSeleccionado();
    }

    /**
     * Metodo setMatriculaVehiculo, establece dado un String la matricula del vehiculo a borrar
     *
     * @param matricula Un string con la matricula
     */
    public void setMatricula(String matricula){
        if(matricula==null){
            throw new IllegalArgumentException();
        }else {
            this.matricula = matricula;
        }
    }

    /**
     * Metodo setControler, establece el controlador asociado al formateador.
     *
     * @param controlador Un objeto que implemente InterfaceCMostrarListado
     */
    @Override
    public void setControlador(InterfaceCBorradoVehiculo controlador){
        if(controlador==null){
            throw new IllegalArgumentException();
        }else{
            formateador =controlador;
        }
    }

    /**
     * Metodo getControler, devuelve el controlador asociado al formateador.
     *
     * @return  Un objeto InterfaceCBorradoVehiculo
     */
    @Override
    public InterfaceCBorradoVehiculo getControlador(){
        return formateador;
    }

    /**
     * Metodo muestraDialogoConfirmacionBorrado, obliga a mostrar un dialogo
     * de confirmacion de borrado en la vista.
     *
     * @return true en caso de completarse con exito, false en caso contrario
     */
    @Override
    public boolean muestraDialogoConfirmacionBorrado() {
        dialogoBorrado=true;
        return true;
    }

    /**
     * Metodo getConfirmacionBorrado, devuelve el estado de la operacion de
     * borrado.
     *
     * @return  true en caso de producirse, false en caso contrario
     */
    @Override
    public boolean getConfirmacionBorrado() {
        if( dialogoBorrado==true){
            dialogoBorrado=false;
            return true;
        }else{
            return false;
        }
    }

    /**
     * Metodo eliminaAccionBorrado, cancela la operacion de borrado.
     *
     */
    @Override
    public boolean eliminaAccionBorrado() {
        matricula=null;
        return true;
    }
    /**
     * Metodo confirmarBorrado, confirma la operacion de borrado.
     *
     */
    @Override
    public boolean confirmarBorrado(){
        formateador.borradoConfirmado(getMatricula());
        return true;
    }

    /**
     * Metodo getMatriculaVehiculo, devuelve ls matricula del vehiculo a borrar.
     *
     * @return  Un objeto  String representacion de la matricula
     */
    public String getMatricula() {
        return matricula;
    }
}
