package com.example.juanalonsoastruga.cargascontrol.Formateador.CUCalcularCosteTrayecto;



import com.example.juanalonsoastruga.cargascontrol.Controlador.CUCalcularCosteTrayecto.ControladorCalcularCosteTrayecto;
import com.example.juanalonsoastruga.cargascontrol.Controlador.CUCalcularCosteTrayecto.InterfaceCCalcularCosteTrayecto;

import java.math.RoundingMode;
import java.text.DecimalFormat;

/**
 * FormateadorCalcularCosteTrayecto, representa al formateador del  CU   Calcular Coste del Trayecto
 *
 */
public class FormateadorCalcularCosteTrayecto implements InterfaceFCalcularCosteTrayecto, InterfaceFCalcularCosteTrayectoImpl {
   private InterfaceCCalcularCosteTrayecto controlador;
   private String mensajeError;
   private double resultado;

    /**
     * Constructor de clase
     *
     */
    public FormateadorCalcularCosteTrayecto(){
        setControlador(new ControladorCalcularCosteTrayecto(this));
    }

    /**
     * Metodo setControlador, establece el controlador asociado a este formateador.
     *
     * @param controlador   Objeto InterfaceCCalcularCosteTrayecto
     * @throws IllegalArgumentException si el argumento es nulo
     */
    @Override
    public void setControlador(InterfaceCCalcularCosteTrayecto controlador){
        if(controlador==null){
            throw new IllegalArgumentException();
        }else{
            this.controlador=controlador;
        }
    }

    /**
     * Metodo getControlador, devuelve el controlador asociado a este formateador.
     *
     * @return  Objeto InterfaceCCalcularCosteTrayecto
     */
    @Override
    public InterfaceCCalcularCosteTrayecto getControlador() {
        return controlador;
    }

    /**
     * Metodo solicitudCalculo, avisa del evento de solicitud de un nuevo calculo de coste de trayecto.
     *
     * @param distancia String
     * @param precio String
     * @param consumo String
     */
    @Override
    public void solicitudCalculo(String distancia, String precio, String consumo) {
        controlador.solicitarCalculo(distancia,precio,consumo);
    }

    /**
     * Metodo getMensajeError, devuelve el mensaje de error adecuado a la situacion.
     *
     * @return String con el mensaje
     */
    @Override
    public String getMensajeError() {
        return mensajeError;
    }

    /**
     * Metodo setMensajeError, establece el mensaje de error asociado al formateador.
     *
     * @param mensajeError String con el mensaje
     */
    @Override
    public void setMensajeError(String mensajeError) {
        this.mensajeError = mensajeError;
    }

    /**
     * Metodo getResultado, devuelve el resultado del calculo solicitado.
     *
     * @return  String con el resultado.
     */
    @Override
    public String getResultado() {
        DecimalFormat formato = new DecimalFormat("#.##");
        formato.setRoundingMode(RoundingMode.CEILING);
        return formato.format(this.resultado).replace('.',',');
    }

    /**
     * Metodo setResultado, pone a disposicion del formateador el resultado del calculo.
     *
     * @param resultado double con el resultado
     */
    @Override
    public void setResultado(double resultado) {
        this.resultado = resultado;
    }
}
