package com.example.juanalonsoastruga.cargascontrol.Formateador.CUConsultarEstadisticaCostesCombustible;


import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTODatosDerivadosCostesCombustible;
import com.example.juanalonsoastruga.cargascontrol.Controlador.CUConsultarEstadisticaCostesCombustible.ControladorConsultarEstadisticaCosteCombustible;
import com.example.juanalonsoastruga.cargascontrol.Controlador.CUConsultarEstadisticaCostesCombustible.InterfaceCConsultarEstadisticaCosteCombustible;

import java.math.RoundingMode;
import java.text.DecimalFormat;

/**
 * Clase FormateadorConsultarEstadisticaCosteCombustible, representa al formateador en el CU Consultar Estadistica Coste de Combustible.
 *
 */
public class FormateadorConsultarEstadisticaCosteCombustible implements InterfaceFConsultarEstadisticaCosteCombustible, InterfaceFConsultarEstadisticaCosteCombustibleImpl {

    private InterfaceCConsultarEstadisticaCosteCombustible controlador;
    private DTODatosDerivadosCostesCombustible datos;

    /**
     * Constructor de clase
     *
     */
    public FormateadorConsultarEstadisticaCosteCombustible(){
        setControlador(new ControladorConsultarEstadisticaCosteCombustible(this));
    }

    /**
     * Metodo setControler, establece el controlador asociado a este formateador.
     *
     * @param controlador InterfaceCConsultarEstadisticaCosteCombustible
     * @throws  IllegalArgumentException si el argumento es null
     */
    public void setControlador(InterfaceCConsultarEstadisticaCosteCombustible controlador){
        if(controlador==null){
            throw new IllegalArgumentException();
        }else{
            this.controlador =controlador;
        }
    }

    /**
     * Metodo comienzaCU, despierta al controlador para el comienzo del Caso de Uso.
     *
     * @param matricula String con la matricula del vehiculo cuyos datos se consultan
     * @param id    int con el id del combustible del vehiculo a consultar
     */
    @Override
    public void comienzaCU(String matricula, int id) {
        controlador.comienzaCU(matricula,id);
    }

    /**
     * Metodo getMatricula, devuelve la matricula asociada al vehiculo consultado.
     *
     * @throws IllegalStateException si no se han asignado datos derivados anteriormente
     * @return String con la matricula
     */
    @Override
    public String getMatriculaVehiculo() {
        if(datos==null){
            throw new IllegalStateException();
        }else {
            return datos.getMatriculaVehiculo();
        }
    }
    /**
     * Metodo getMes, devuelve el numero de mes de la ultima recarga de la que se extrayeron datos derivados.
     *
     * @throws IllegalStateException si no se han asignado datos derivados anteriormente
     * @return  int con el numero de mes (1 a 12)
     */
    @Override
    public int getMes() {
        if(datos==null){
            throw new IllegalStateException();
        }else {
            return (datos.getMes() + 1);
        }
    }
    /**
     * Metodo getAnio, devuelve el anio de la ultima recarga de la que se extrayeron datos derivados.
     *
     * @throws IllegalStateException si no se han asignado datos derivados anteriormente
     * @return  int con el numero de anio
     */
    @Override
    public int getAnio() {
        if(datos==null){
            throw new IllegalStateException();
        }else {
            return datos.getAnio();}
    }
    /**
     * Metodo getNumerosRecargasMes, devuelve el numero de recargas del ultimo mes .
     *
     * @throws IllegalStateException si no se han asignado datos derivados anteriormente
     * @return int con el numero
     */
    @Override
    public int getNumeroRecargasMes() {
        if(datos==null){
            throw new IllegalStateException();
        }else {
            return datos.getNumeroRecargasCombustiblePorMes();
        }
    }
    /**
     * Metodo getNumerosRecargasAnio, devuelve el numero de recargas del ultimo anio.
     *
     * @throws IllegalStateException si no se han asignado datos derivados anteriormente
     * @return int con el numero
     */
    @Override
    public int getNumeroRecargasAnio() {
        if(datos==null){
            throw new IllegalStateException();
        }else {
            return datos.getNumeroRecargasCombustiblePorAnio();
        }
    }

    /**
     * Metodo getCosteRecargasMes, devuelve el coste de recargas en el ultimo mes.
     *
     *  @throws IllegalStateException si no se han asignado datos derivados anteriormente
     * @return String con la cifra formateada
     */
    @Override
    public String getCosteRecargasMes() {
        if(datos==null){
            throw new IllegalStateException();
        }else {
            double valor = datos.getSumaCosteRecargaMensual();
            DecimalFormat formato = new DecimalFormat("#.##");
            formato.setRoundingMode(RoundingMode.CEILING);
            return formato.format(valor).replace('.',',');
        }
    }
    /**
     * Metodo getCosteRecargasAnio, devuelve el coste de recargas en el ultimo anio.
     *
     * @throws IllegalStateException si no se han asignado datos derivados anteriormente
     * @return String con la cifra formateada
     */
    @Override
    public String getCosteRecargasAnio() {
        if(datos==null){
            throw new IllegalStateException();
        }else {
            double valor= datos.getSumaCosteRecargaAnual();
            DecimalFormat formato = new DecimalFormat("#.##");
            formato.setRoundingMode(RoundingMode.CEILING);
            return formato.format(valor).replace('.',',');
        }
    }

    /**
     * Metodo getMediaCosteRecargasMes, devuelve la media del coste de recargas realizadas durante el mes.
     *
     * @throws IllegalStateException si no se han asignado datos derivados anteriormente
     * @return String con la cifra formateada
     */
    @Override
    public String getMediaCosteRecargasMes() {
        if(datos==null){
            throw new IllegalStateException();
        }else {
            double valor = datos.getMediaCosteRecargaMensual();
            DecimalFormat formato = new DecimalFormat("#.##");
            formato.setRoundingMode(RoundingMode.CEILING);
            return formato.format(valor).replace('.',',');
        }
    }
    /**
     * Metodo getMediaCosteRecargasAnio, devuelve la media del coste de recargas realizadas durante el anio.
     *
     * @throws IllegalStateException si no se han asignado datos derivados anteriormente
     * @return String con la cifra formateada
     */
    @Override
    public String getMediaCosteRecargasAnio() {
        if(datos==null){
            throw new IllegalStateException();
        }else {
            double valor = datos.getMediaCosteRecargaAnual();
            DecimalFormat formato = new DecimalFormat("#.##");
            formato.setRoundingMode(RoundingMode.CEILING);
            return formato.format(valor).replace('.',',');
        }
    }

    /**
     * Metodo getMinimoCosteRecargasMes, devuelve el minimo coste de recargas en el ultimo mes
     *
     * @throws IllegalStateException si no se han asignado datos derivados anteriormente
     * @return String con la cifra formateada
     */
    @Override
    public String getMinimoCosteRecargasMes() {
        if(datos==null){
            throw new IllegalStateException();
        }else {
            double valor = datos.getMinimoCosteRecargaMensual();
            DecimalFormat formato = new DecimalFormat("#.##");
            formato.setRoundingMode(RoundingMode.CEILING);
            return formato.format(valor).replace('.',',');
        }
    }
    /**
     * Metodo getMaximoCosteRecargasMes, devuelve el maximo coste de recargas en el ultimo mes
     *
     * @throws IllegalStateException si no se han asignado datos derivados anteriormente
     * @return String con la cifra formateada
     */
    @Override
    public String getMaximoCosteRecargasMes() {
        if(datos==null){
            throw new IllegalStateException();
        }else {
            double valor = datos.getMaximoCosteRecargaMensual();
            DecimalFormat formato = new DecimalFormat("#.##");
            formato.setRoundingMode(RoundingMode.CEILING);
            return formato.format(valor).replace('.',',');
        }
    }
    /**
     * Metodo getMinimoCosteRecargasAnio, devuelve el minimo coste de recargas en el ultimo anio
     *
     * @throws IllegalStateException si no se han asignado datos derivados anteriormente
     * @return String con la cifra formateada
     */
    @Override
    public String getMinimoCosteRecargasAnio() {
        if(datos==null){
            throw new IllegalStateException();
        }else {
            double valor = datos.getMinimoCosteRecargaAnual();
            DecimalFormat formato = new DecimalFormat("#.##");
            formato.setRoundingMode(RoundingMode.CEILING);
            return formato.format(valor).replace('.',',');
        }
    }
    /**
     * Metodo getMaximoCosteRecargasAnio, devuelve el maximo coste de recargas en el ultimo anio
     *
     * @throws IllegalStateException si no se han asignado datos derivados anteriormente
     * @return String con la cifra formateada
     */
    @Override
    public String getMaximoCosteRecargasAnio() {
        if(datos==null){
            throw new IllegalStateException();
        }else {
            double valor = datos.getMaximoCosteRecargaAnual();
            DecimalFormat formato = new DecimalFormat("#.##");
            formato.setRoundingMode(RoundingMode.CEILING);
            return formato.format(valor).replace('.',',');
        }
    }

    /**
     * Metodo getMediaCosteCombustiblesXUnidadMes, devuelve la media coste por combustible recargado
     * en el ultimo mes.
     *
     * @throws IllegalStateException si no se han asignado datos derivados anteriormente
     * @return String con la cifra formateada
     */
    @Override
    public String getMediaCosteCombustiblesXUnidadMes() {
        if(datos==null){
            throw new IllegalStateException();
        }else {
            double valor = datos.getMediaCosteXRecargaMensual();
            DecimalFormat formato = new DecimalFormat("#.##");
            formato.setRoundingMode(RoundingMode.CEILING);
            return formato.format(valor).replace('.',',');
        }
    }
    /**
     * Metodo  getMediaCosteCombustiblesXUnidadAnio, devuelve la media coste por combustible recargado
     * en el ultimo anio.
     *
     * @throws IllegalStateException si no se han asignado datos derivados anteriormente
     * @return String con la cifra formateada
     */
    @Override
    public String getMediaCosteCombustiblesXUnidadAnio() {
        if(datos==null){
            throw new IllegalStateException();
        }else {
            double valor = datos.getMediaCosteXRecargaAnual();
            DecimalFormat formato = new DecimalFormat("#.##");
            formato.setRoundingMode(RoundingMode.CEILING);
            return formato.format(valor).replace('.',',');
        }
    }

    /**
     * Metodo  getMinimoCosteCombustiblesXUnidadMes, devuelve el minimo coste por combustible recargado
     * en el ultimo mes.
     *
     * @throws IllegalStateException si no se han asignado datos derivados anteriormente
     * @return String con la cifra formateada
     */
    @Override
    public String getMinimoCosteCombustiblesXUnidadMes() {
        if(datos==null){
            throw new IllegalStateException();
        }else {
            double valor= datos.getMinimoCosteXRecargaMensual();
            DecimalFormat formato = new DecimalFormat("#.##");
            formato.setRoundingMode(RoundingMode.CEILING);
            return formato.format(valor).replace('.',',');
        }
    }
    /**
     * Metodo  getMaximoCosteCombustiblesXUnidadMes, devuelve el maximo coste por combustible recargado
     * en el ultimo mes.
     *
     * @throws IllegalStateException si no se han asignado datos derivados anteriormente
     * @return String con la cifra formateada
     */
    @Override
    public String getMaximoCosteCombustiblesXUnidadMes() {
        if(datos==null){
            throw new IllegalStateException();
        }else {
            double valor= datos.getMaximoCosteXRecargaMensual();
            DecimalFormat formato = new DecimalFormat("#.##");
            formato.setRoundingMode(RoundingMode.CEILING);
            return formato.format(valor).replace('.',',');
        }
    }
    /**
     * Metodo  getMinimoCosteCombustiblesXUnidadAnio, devuelve el minimo coste por combustible recargado
     * en el ultimo anio.
     *
     * @throws IllegalStateException si no se han asignado datos derivados anteriormente
     * @return String con la cifra formateada
     */
    @Override
    public String getMinimoCosteCombustiblesXUnidadAnio() {
        if(datos==null){
            throw new IllegalStateException();
        }else {
            double valor= datos.getMinimoCosteXRecargaAnual();
            DecimalFormat formato = new DecimalFormat("#.##");
            formato.setRoundingMode(RoundingMode.CEILING);
            return formato.format(valor).replace('.',',');
        }
    }
    /**
     * Metodo  getMaximoCosteCombustiblesXUnidadAnio, devuelve el maximo coste por combustible recargado
     * en el ultimo anio.
     *
     * @throws IllegalStateException si no se han asignado datos derivados anteriormente
     * @return String con la cifra formateada
     */
    @Override
    public String getMaximoCosteCombustiblesXUnidadAnio() {
        if(datos==null){
            throw new IllegalStateException();
        }else {
            double valor = datos.getMaximoCosteXRecargaAnual();
            DecimalFormat formato = new DecimalFormat("#.##");
            formato.setRoundingMode(RoundingMode.CEILING);
            return formato.format(valor).replace('.',',');
        }
    }

    /**
     * Metodo setDatosDerivadosCosteCombustible, asocia los datos derivados de costes de combustible con la
     * formateador.
     *
     * @param dtoCoste DTODatosDerivadosCostesCombustible
     */
    @Override
    public void setDatosDerivadosCosteCombustible(DTODatosDerivadosCostesCombustible dtoCoste) {
        this.datos=dtoCoste;
    }
}
