package com.example.juanalonsoastruga.cargascontrol.Formateador.CUConsultarEstadisticaVehiculo;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTODatosDerivadosVehiculoTotales;
import com.example.juanalonsoastruga.cargascontrol.Controlador.CUConsultarEstadisticaVehiculo.InterfaceCConsultarEstadisticaVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Controlador.CUConsultarEstadisticaVehiculo.ControladorConsultarEstadisticaVehiculo;

import java.math.RoundingMode;
import java.text.DecimalFormat;

/**
 * Clase FormateadorConsultarEstadisticaVehiculo, representa al formateador en el CU  Consultar Datos Estadisticos del Vehiculo
 *
 */
public class FormateadorConsultarEstadisticaVehiculo implements InterfaceFConsultarEstadisticaVehiculo, InterfaceFConsultarEstadisticaVehiculoImpl {

   private InterfaceCConsultarEstadisticaVehiculo controlador;
   private DTODatosDerivadosVehiculoTotales  datosDerivados;
    /**
     * Constructor de clase.
     *
     */
    public FormateadorConsultarEstadisticaVehiculo(){
        setControlador(new ControladorConsultarEstadisticaVehiculo(this));
    }

    /**
     * Metodo setControler, establece el controlador asociado a esta formateador.
     *
     * @param controlador Objeto InterfaceCConsultarEstadisticaVehiculo
     * @throws IllegalArgumentException si el argumento es null
     */
    public void setControlador(InterfaceCConsultarEstadisticaVehiculo controlador){
        if(controlador==null){
            throw new IllegalArgumentException();
        }else{
            this.controlador =controlador;
        }
    }

    /**
     * Metodo comienzaCU, dada la matricula del vehiculo a consultar, comienza la ejecucion del caso de uso.
     *
     * @param matricula Un String con la matricula
     * @throws IllegalArgumentException si el argumento es null
     */
    @Override
    public void comienzaCU(String matricula) {
        if(matricula==null){
            throw new IllegalArgumentException();
        }else {
            controlador.comienzaCU(matricula);
        }
    }

    /**
     * Metodo getMatricula, devuelve la matricula del vehiculo cuyos datos se consultan.
     *
     * @return Objeto String con la matricula
     * @throws IllegalStateException si se intenta sin datos derivados asociados
     */
    @Override
    public String getMatricula() {
        if(
            datosDerivados==null){throw new IllegalStateException();
        }
        else{
            return datosDerivados.getMatricula();
        }

    }

    /**
     * Metodo getMes, devuelve el numero de mes ultimo en el que se hizo un registro.
     *
     * @return  Un String con el mes
     * @throws IllegalStateException si no hay datos asociados
     */
    @Override
    public String getMes() {
        if(
            datosDerivados==null){throw new IllegalStateException();
        }
        else{
            return ( datosDerivados.getMes() + 1)+"";
        }
    }
    /**
     * Metodo getAnio, devuelve el numero de anio ultimo en el que se hizo un registro.
     *
     * @return  Un String con el anio
     * @throws IllegalStateException si no hay datos asociados
     */
    @Override
    public String getAnio() {
        if(
            datosDerivados==null){throw new IllegalStateException();
        }
        else{
            return datosDerivados.getAnio()+"";
        }
    }

    /**
     * Metodo getNRMes, devuelve el numero de recargas realizadas en el mes del ultimo registro.
     *
     * @return String con la cifra
     * @throws IllegalStateException si no hay datos asociados
     */
    @Override
    public String getNRMes() {
        if(
            datosDerivados==null){throw new IllegalStateException();
        }
        else{
            return datosDerivados.getNumeroRecargasMes()+"";
        }
    }
    /**
     * Metodo getNRAnio, devuelve el numero de recargas realizadas en el anio del ultimo registro.
     *
     * @return String con la cifra
     * @throws IllegalStateException si no hay datos asociados
     */
    @Override
    public String getNRAnio() {
        if(
            datosDerivados==null){throw new IllegalStateException();
        }
        else{
            return datosDerivados.getNumeroRecargasAnio()+"";
        }
    }

    /**
     * Metodo getCosteRecargasMes, devuelve el coste de las recargas del mes de la ultima recarga.
     *
     * @return  String con el coste
     *  @throws IllegalStateException si no hay datos asociados
     */
    @Override
    public String getCosteRecargasMes() {
        if(
            datosDerivados==null){throw new IllegalStateException();
        }
        else{
            double valor=  datosDerivados.getCosteRecargasTotalesPorMes();
            DecimalFormat formato = new DecimalFormat("#.##");
            formato.setRoundingMode(RoundingMode.CEILING);
            return formato.format(valor).replace('.',',');

        }
    }

    /**
     * Metodo getCosteRecargasAnio, devuelve el coste acumulado de las recargas del añio del ultimo registro.
     *
     * @return  String con el coste
     *  @throws IllegalStateException si no hay datos asociados
     */
    @Override
    public String getCosteRecargasAnio() {
        if(
            datosDerivados==null){throw new IllegalStateException();
        }
        else{
            double valor=   datosDerivados.getCosteRecargasTotalesPorAnio();
            DecimalFormat formato = new DecimalFormat("#.##");
            formato.setRoundingMode(RoundingMode.CEILING);
            return formato.format(valor).replace('.',',');

        }
    }

    /**
     * Metodo getKmRecargasMes, devuelve el numero de km registrados en recargas del ultimo mes.
     *
     * @return  Un String con la cantidad
     *  @throws IllegalStateException si no hay datos asociados
     */
    @Override
    public String getKmRecargasMes() {
        if(
            datosDerivados==null){throw new IllegalStateException();
        }
        else{

            double valor=   datosDerivados.getKmRecargasTotalesPorMes();
            DecimalFormat formato = new DecimalFormat("#.##");
            formato.setRoundingMode(RoundingMode.CEILING);
            return formato.format(valor).replace('.',',');
        }
    }
    /**
     * Metodo getKmRecargasAnio, devuelve el numero de km registrados en recargas del ultimo anio.
     *
     * @return  Un String con la cantidad
     *  @throws IllegalStateException si no hay datos asociados
     */
    @Override
    public String getKmRecargasAnio() {
        if(
            datosDerivados==null){throw new IllegalStateException();
        }
        else{

            double valor=   datosDerivados.getKmRecargasTotalesPorAnio();
            DecimalFormat formato = new DecimalFormat("#.##");
            formato.setRoundingMode(RoundingMode.CEILING);
            return formato.format(valor).replace('.',',');
        }
    }

    /**
     * Metodo setDatosderivados, establece el conjunto de datos derivados que el formateador debe tener a su disposicion.
     *
     * @param datosEstadisticosVehiculo DTODatosDerivadosVehiculoTotales
     * @throws IllegalStateException si no hay datos asociados
     */
    @Override
    public void setDatosDerivados(DTODatosDerivadosVehiculoTotales datosEstadisticosVehiculo) {
        if(datosEstadisticosVehiculo==null){
            throw new IllegalArgumentException();
        }else{
            this.datosDerivados=datosEstadisticosVehiculo;
        }
    }
}
