package com.example.juanalonsoastruga.cargascontrol.Formateador.CUConsultarInformacionGasolinera;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTOGasolinera;
import com.example.juanalonsoastruga.cargascontrol.Controlador.CUConsultarInformacionGasolinera.InterfaceCConsultarInformacionGasolinera;
import com.example.juanalonsoastruga.cargascontrol.Controlador.CUConsultarInformacionGasolinera.ControladorConsultarInformacionGasolinera;

/**
 *Clase FormateadorConsultarInformacionGasolinera, representa al formateador en el CU Consultar Informacion Gasolinera.
 *
 */
public class FormateadorConsultarInformacionGasolinera implements InterfaceFConsultarInformacionGasolinera, InterfaceFConsultarInformacionGasolineraImpl {

    private InterfaceCConsultarInformacionGasolinera controlador;
    private DTOGasolinera dto;

    /**
     *Constructor de clase.
     *
     */
    public FormateadorConsultarInformacionGasolinera(){
        setControlador(new ControladorConsultarInformacionGasolinera(this));
    }

    /**
     *Metodo setControler, establece el controlador asociado a este formateador.
     *
     *@param controlador, el controlador  InterfaceCConsultarInformacionGasolinera a asociar
     *@throws IllegalArgumentException si el argumento es nulo
     */
    @Override
    public void setControlador(InterfaceCConsultarInformacionGasolinera controlador){
        if(controlador ==null){
            throw new IllegalArgumentException();
        }else{
            this.controlador = controlador;
        }
    }

    /**
     * Metodo getControler, devuelve el controlador asociado a este formateador.
     *
     * @return objeto InterfaceCConsultarInformacionGasolinera
     */
    @Override
    public InterfaceCConsultarInformacionGasolinera getControlador(){
        return controlador;
    }

    /**
     *Metodo setDatosGasolinera, establece los datos asociados a la gasolinera consultada.
     *
     *@param dto Objeto DTOGasolinera con los datos
     * @throws IllegalArgumentException si el argumento es nulo
     */
    @Override
    public void setDatosGasolinera(DTOGasolinera dto){

        if(dto==null){
            throw new IllegalArgumentException();
        }else {
            this.dto = dto;
        }
    }


    /**
     *Metodo init, da comienzo al caso de uso diciendo a la vista que despierte al controlador.
     *
     *@param latitud double con la latitud
     *@param longitud double con la longitud
     *@param direccion String con la direccion de la estacion de recarga
     */
    @Override
    public void init(double latitud,double longitud,String direccion){
        controlador.comienzaCU(latitud,longitud,direccion);
    }

    /**
     *Metodo getRotulo, devuelve el rotulo de la gasolinera contenido en el conjunto de datos de la misma.
     *
     *@throws IllegalStateException si no se ha asociado a la vista el conjunto de datos de la gasolinera
     *@return String con el texto
     */
    @Override
    public String getRotulo(){
        if(this.dto==null){
			throw new IllegalStateException();
        }else{
            return dto.getRotulo();
        }
    }

    /**
     *Metodo getProvincia, devuelve la provincia de la gasolinera contenida en el conjunto de datos de la misma.
     *
     *@throws IllegalStateException si no se ha asociado a la vista el conjunto de datos de la gasolinera
     *@return String con el texto
     */
    @Override
    public String getProvincia(){
        if(this.dto==null){
			throw new IllegalStateException();
        }else{
            return dto.getProvincia();
        }
    }

    /**
     *Metodo getMunicipio, devuelve el municipio de la gasolinera contenido en el conjunto de datos de la misma.
     *
     *@throws IllegalStateException si no se ha asociado a la vista el conjunto de datos de la gasolinera
     *@return String con el texto
     */
    @Override
    public String getMunicipio(){
        if(this.dto==null){
			throw new IllegalStateException();
        }else{
            return dto.getMunicipio();
        }
    }

    /**
     *Metodo getLocalidad, devuelve la localidad de la gasolinera contenido en el conjunto de datos de la misma.
     *
     *@throws IllegalStateException si no se ha asociado a la vista el conjunto de datos de la gasolinera
     *@return String con el texto
     */
    @Override
    public String getLocalidad(){
        if(this.dto==null){
			throw new IllegalStateException();
        }else{
            return dto.getLocalidad();
        }
    }

    /**
     *Metodo getDireccion, devuelve la direccion de la gasolinera contenida en el conjunto de datos de la misma.
     *
     *@throws IllegalStateException si no se ha asociado a la vista el conjunto de datos de la gasolinera
     *@return String con el texto
     */
    @Override
    public String getDireccion(){
        if(this.dto==null){
			throw new IllegalStateException();
        }else{
            return dto.getDireccion();
        }
    }

    /**
     *Metodo getHorario, devuelve el horario de la gasolinera contenida en el conjunto de datos de la misma.
     *
     *@throws IllegalStateException si no se ha asociado a la vista el conjunto de datos de la gasolinera
     *@return String con el texto
     */
    @Override
    public String getHorario(){
        if(this.dto==null){
			throw new IllegalStateException();
        }else{
            return dto.getHorario();
        }
    }

    /**
     *Metodo getLatitud, devuelve la latitud de la gasolinera contenida en el conjunto de datos de la misma.
     *
     *@throws IllegalStateException si no se ha asociado a la vista el conjunto de datos de la gasolinera
     *@return String con el texto
     */
    @Override
    public String getLatitud(){
        if(this.dto==null){
			throw new IllegalStateException();
        }else{
            return dto.getLatitud()+"";
        }
    }

    /**
     *Metodo getLongitud, devuelve la longitud de la gasolinera contenida en el conjunto de datos de la misma.
     *
     *@throws IllegalStateException si no se ha asociado a la vista el conjunto de datos de la gasolinera
     *@return String con el texto
     */
    @Override
    public String getLongitud(){
        if(this.dto==null){
			throw new IllegalStateException();
        }else{
            return dto.getLongitud()+"";
        }
    }

    /**
     *Metodo getBiodiesel, resuelve si la gasolinera es un posible proveedor de combustible (Biodiesel).
     *
     *@throws IllegalStateException si no se ha asociado a la vista el conjunto de datos de la gasolinera
     *@return String con el texto (Si o No)
     */
    @Override
    public String getBiodiesel(){
        if(this.dto==null){
			throw new IllegalStateException();
        }else{
            return dto.getBiodiesel()?"SI":"NO";
        }
    }

    /**
     *Metodo getBioetanol, resuelve si la gasolinera es un posible proveedor de combustible (Bioetanol).
     *
     *@throws IllegalStateException si no se ha asociado a la vista el conjunto de datos de la gasolinera
     *@return String con el texto (Si o No)
     */
    @Override
    public String getBioetanol(){
        if(this.dto==null){
			throw new IllegalStateException();
        }else{
            return dto.getBioetanol()?"SI":"NO";
        }
    }

    /**
     *Metodo getGasNaturalComprimido, resuelve si la gasolinera es un posible proveedor de combustible (GNC).
     *
     *@throws IllegalStateException si no se ha asociado a la vista el conjunto de datos de la gasolinera
     *@return String con el texto (Si o No)
     */
    @Override
    public String getGasNaturalComprimido(){
        if(this.dto==null){
			throw new IllegalStateException();
        }else{
            return dto.getGasNaturalComprimido()?"SI":"NO";
        }
    }

    /**
     *Metodo getGasNaturalLicuado, resuelve si la gasolinera es un posible proveedor de combustible (GNL).
     *
     *@throws IllegalStateException si no se ha asociado a la vista el conjunto de datos de la gasolinera
     *@return String con el texto (Si o No)
     */
    @Override
    public String getGasNaturalLicuado(){
        if(this.dto==null){
			throw new IllegalStateException();
        }else{
            return dto.getGasNaturalLicuado()?"SI":"NO";
        }
    }

    /**
     *Metodo getGasLicuadoPetroleo, resuelve si la gasolinera es un posible proveedor de combustible (GLP).
     *
     *@throws IllegalStateException si no se ha asociado a la vista el conjunto de datos de la gasolinera
     *@return String con el texto (Si o No)
     */
    @Override
    public String getGasLicuadoPetroleo(){
        if(this.dto==null){
			throw new IllegalStateException();
        }else{
            return dto.getGasLicuadoPetroleo()?"SI":"NO";
        }
    }

    /**
     *Metodo getGasoleoA, resuelve si la gasolinera es un posible proveedor de combustible (Gasoleo A).
     *
     *@throws IllegalStateException si no se ha asociado a la vista el conjunto de datos de la gasolinera
     *@return String con el texto (Si o No)
     */
    @Override
    public String getGasoleoA(){
        if(this.dto==null){
			throw new IllegalStateException();
        }else{
            return dto.getGasoleoA()?"SI":"NO";
        }
    }

    /**
     *Metodo getGasoleoB, resuelve si la gasolinera es un posible proveedor de combustible (Gasoleo B).
     *
     *@throws IllegalStateException si no se ha asociado a la vista el conjunto de datos de la gasolinera
     *@return String con el texto (Si o No)
     */
    @Override
    public String getGasoleoB(){
        if(this.dto==null){
			throw new IllegalStateException();
        }else{
            return dto.getGasoleoB()?"SI":"NO";
        }
    }

    /**
     *Metodo getGasolina95, resuelve si la gasolinera es un posible proveedor de combustible (Gasolina 95).
     *
     *@throws IllegalStateException si no se ha asociado a la vista el conjunto de datos de la gasolinera
     *@return String con el texto (Si o No)
     */
    @Override
    public String getGasolina95(){
        if(this.dto==null){
			throw new IllegalStateException();
        }else{
            return dto.getGasolina95()?"SI":"NO";
        }
    }

    /**
     *Metodo getGasolina98, resuelve si la gasolinera es un posible proveedor de combustible (Gasolina 98).
     *
     *@throws IllegalStateException si no se ha asociado a la vista el conjunto de datos de la gasolinera
     *@return String con el texto (Si o No)
     */
    @Override
    public String getGasolina98(){
        if(this.dto==null){
			throw new IllegalStateException();
        }else{
            return dto.getGasolina98()?"SI":"NO";
        }
    }

    /**
     *Metodo getNuevoGasoleoA, resuelve si la gasolinera es un posible proveedor de combustible (Nuevo Gasoleo A).
     *
     *@throws IllegalStateException si no se ha asociado a la vista el conjunto de datos de la gasolinera
     *@return String con el texto (Si o No)
     */
    @Override
    public String getNuevoGasoleoA(){
        if(this.dto==null){
			throw new IllegalStateException();
        }else{
            return dto.getNuevoGasoleoA()?"SI":"NO";
        }
    }
}