package com.example.juanalonsoastruga.cargascontrol.Formateador.CUConsultarInformacionRecarga;

import android.graphics.Bitmap;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTORecarga;
import com.example.juanalonsoastruga.cargascontrol.Controlador.CUConsultarInformacionRecarga.InterfaceCConsultarInformacionRecarga;
import com.example.juanalonsoastruga.cargascontrol.Controlador.CUConsultarInformacionRecarga.ControladorConsultarInformacionRecarga;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * Clase  FormateadorConsultarInformacionRecarga, representa el Formateador del CU Consultar Informacion Recarga
 *
 */
public class FormateadorConsultarInformacionRecarga implements InterfaceFConsultarInformacionRecargaImpl, InterfaceFConsultarInformacionRecarga {

    private InterfaceCConsultarInformacionRecarga controlador;

    private DTORecarga recarga;

    /**
     * Constructor de clase
     *
     */
    public FormateadorConsultarInformacionRecarga(){
        setControlador(new ControladorConsultarInformacionRecarga(this));
    }

    /**
     * Metodo getControler, devuelve el controlador asociado a este formateador.
     *
     * @return  Objeto InterfaceCConsultarInformacionRecarga
     *
     */
    public InterfaceCConsultarInformacionRecarga getControlador() {
            return controlador;
    }
    /**
     * Metodo setControler, establece el controlador asociado a esta formateador.
     *
     * @param controlador   Objeto InterfaceCConsultarInformacionRecarga
     * @throws  IllegalArgumentException
     *
     */
    public void setControlador(InterfaceCConsultarInformacionRecarga controlador) {
        if(controlador ==null){
            throw new IllegalArgumentException();
        }else {
            this.controlador = controlador;
        }
    }

    /**
     * Metodo init, indica al formateador que despierte al controaldor asociado para comenzar el caso de uso.
     *
     * @param matricula Un String con la matricula del vehiculo cuyos datos consultar
     * @param fecha Un Date con la fecha de la recarga del vehículo a consultar
     * @param combustible   Un String con el combustible usado en la recarga del vehículo
     */
    @Override
    public void init(String matricula, Date fecha, String combustible) {
        controlador.comienzoCU(matricula,fecha,combustible);
    }

    /**
     * Metodo getFecha, devuelve la fecha en la que se efectuó la recarga.
     *
     * @return  Un String con la representación Dia/Mes/Ano Hora:Minuto de la fecha
     */
    @Override
    public String getFecha() {
        String pattern = "dd/MM/yyyy HH:mm";
        DateFormat df = new SimpleDateFormat(pattern);
        Date fecha=recarga.getFecha();
        String fechaAsString = df.format(fecha);
        return fechaAsString;
    }

    /**
     * Metodo getKm, devuelve el kilometraje del coche al realizar la recarga.
     *
     * @return  Un double con el kilometraje
     */
    @Override
    public double getKm() {
        return recarga.getKm();
    }

    /**
     * Metodo getCoste, devuelve el coste integro de la recarga.
     *
     * @return  Un double con el coste
     */
    @Override
    public double getCoste() {
        return recarga.getCoste();
    }

    /**
     * Metodo getMedicion, devuelve la cantidad de conmbustible recargado.
     *
     * @return  Un double con la medicion de la recarga
     */
    @Override
    public double getMedicion() {
        return recarga.getMedicion();
    }

    /**
     * Metodo getMatricula, devuelve un String con la matricula del vehiculo.
     *
     * @return  Un String con la matricula del vehiculo
     */
    @Override
    public String getMatricula() {
        return recarga.getMatricula();
    }

    /**
     * Metodo getCombustible, devuelve el combustible sobre el que se ha hecho la recarga.
     *
     * @return  Un String con el nombre del combustible
     */
    @Override
    public String getCombustible() {
        return recarga.getCombustible();
    }

    /**
     * Metodo getFoto, devuelve un bitmap de la foto del ticket de recarga.
     *
     * @return  Un objeto Bitmap
     */
    @Override
    public Bitmap getFoto() {
        return recarga.getFoto();
    }

    /**
     * Metodo registrarUltimaRecargaConsultada, despierta al controlador como parte del CU Registrar Ultima Recarga Consultada.
     *
     * @param matricula String con la matricula del correspondiente vehiculo al que se le realizo la recarga
     * @param fecha Long con la fecha
     * @param combustible String con el combustible del vehiculo sobre el que se hizo la recarga
     */
    @Override
    public void registrarUltimaRecargaConsultada(String matricula, Long fecha, String combustible) {
        controlador.registrarIDRecargaConsultada(matricula,fecha,combustible);
    }

    /**
     * Metodo setRecarga, establece el contenedor de datos dinámicos de la recarga.
     *
     * @param recarga   Un Objeto DTORecarga
     */
    @Override
    public void setRecarga(DTORecarga recarga) {
        if(recarga==null){
            throw new IllegalArgumentException();
        }else {
            this.recarga = recarga;
        }

    }
}
