package com.example.juanalonsoastruga.cargascontrol.Formateador.CUConsultarInformacionVehiculo;

import android.graphics.Bitmap;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTOVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Controlador.CUConsultarInformacionVehiculo.ControladorConsultarInformacionVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Controlador.CUConsultarInformacionVehiculo.InterfaceCConsultarInformacion;

import java.math.RoundingMode;
import java.text.DecimalFormat;

/**
 * Clase FormateadorConsultarInformacionVehiculo, hace de vista en el CU Consultar Informacion de Vehiculo
 *
 */
public class FormateadorConsultarInformacionVehiculo implements InterfaceFConsultarInformacion, InterfaceFConsultarInformacionImpl {

    private InterfaceCConsultarInformacion controler;
    private DTOVehiculo dto;

    /**
     * Constructor de clase
     *
     */
    public FormateadorConsultarInformacionVehiculo(){
        setControler(new ControladorConsultarInformacionVehiculo(this));
    }


    /**
     * Metodo init, indica a la vista que debe avisar al controler asociado para el comienzo del caso de uso.
     *
     * @param matricula UN String con la matricula del vehiculo sobre el que se realiza el CU
     */
    @Override
    public void init(String matricula) {

        controler.activacionCU(matricula);
    }

    /**
     * Metodo setControler, establece el controler para esta vista deacuerdo a MVP
     *
     * @param controler Un objeto InterfaceCConsultarInformacion, que haga de controler
     */
    @Override
    public void setControler(InterfaceCConsultarInformacion controler){
        if(controler==null){
            throw new IllegalArgumentException();
        }else{
            this.controler =controler;
        }
    }

    /**
     * Metodo getControler, devuelve el controler asociado a esta vista
     *
     * @return Un objeto InterfaceCConsultarInformacion
     */
    @Override
    public InterfaceCConsultarInformacion getControler(){
        return controler;
    }


    /**
     * Metodo getVehiculo, devuelve el conjunto dee datos de vehiculo en consulta
     *
     * @return  Un objeto DTOVehiculo
     */

    public DTOVehiculo  getVehiculo(){
        return dto;
    }

    /**
     * Establece el vehiculo sobre el que consultar informacion
     *
     * @param v Un objeto DTOVehiculo
     */
    @Override
    public void setVehiculo(DTOVehiculo v) {
        if(v==null){
            throw new IllegalArgumentException();
        }else{
            this.dto=v;
        }

    }

    /**
     * Devuelve la matricula del  vehiculo en consulta
     *
     * @return  Un objeto CharSequence
     */
    @Override
    public CharSequence getMatricula() {
        return dto.getMatricula();
    }
    /**
     * Devuelve el nombre del  vehiculo en consulta
     *
     * @return  Un objeto CharSequence
     */
    @Override
    public CharSequence getNombre() {
        return dto.getNombre();
    }
    /**
     * Devuelve el modelo del vehiculo en consulta
     *
     * @return  Un objeto CharSequence
     */
    @Override
    public CharSequence getModelo() {
        return dto.getModelo();
    }
    /**
     * Devuelve la marca del vehiculo en consulta
     *
     * @return  Un objeto CharSequence
     */
    @Override
    public CharSequence getMarca() {
        return dto.getMarca();
    }
    /**
     * Devuelve el año del vehiculo en consulta
     *
     * @return  Un objeto CharSequence
     */
    @Override
    public CharSequence getAño() {
        return dto.getAño().toString();
    }
    /**
     * Devuelve los km del vehiculo en consulta
     *
     * @return  Un objeto CharSequence
     */
    @Override
    public CharSequence getKm() {
        double valor = dto.getKilometraje();
        DecimalFormat formato = new DecimalFormat("#.##");
        formato.setRoundingMode(RoundingMode.CEILING);
        return formato.format(valor).replace('.',',');

    }
    /**
     * Devuelve el combustible primario del vehiculo en consulta
     *
     * @return  Un objeto CharSequence
     */
    @Override
    public CharSequence getCPrimario() {
        return dto.getCombustiblePrimario();
    }
    /**
     * Devuelve la capacidad del tanque de combustible primario del vehiculo en consulta
     *
     * @return  Un objeto CharSequence
     */
    @Override
    public CharSequence getCTPrimario() {
        double valor = dto.getCapTankPrimario();
        DecimalFormat formato = new DecimalFormat("#.##");
        formato.setRoundingMode(RoundingMode.CEILING);
        return formato.format(valor).replace('.',',');

    }
    /**
     * Devuelve el combustible secundario del vehiculo en consulta
     *
     * @return  Un objeto CharSequence
     */
    @Override
    public CharSequence getCSecundario() {
        return dto.getCombustibleSecundario();
    }
    /**
     * Devuelve la capacidad del tanque de combustible secundario del vehiculo en consulta
     *
     * @return  Un objeto Double
     */
    @Override
    public CharSequence getCTSecundario() {

        double valor = dto.getCapTankSecundario();
        DecimalFormat formato = new DecimalFormat("#.##");
        formato.setRoundingMode(RoundingMode.CEILING);
        return formato.format(valor).replace('.',',');
    }
    /**
     * Devuelve un vector de pixeles de la foto del vehiculo
     *
     * @return  Un objeto Bitmap
     */
    @Override
    public Bitmap getFoto() {
        return dto.getFoto();
    }


    /**
     * Metodo registrarUltimoVehiculoConsultado, avisaal controler del comienzo del CU Registrar Ultimo Vehiculo Consultado.
     *
     * @param matricula Un String con la matricula del vehiculo consultado
     */
    @Override
    public void registrarUltimoVehiculoConsultado(String matricula) {
        controler.registrarIDVehiculoConsultado(matricula);
    }
}
