package com.example.juanalonsoastruga.cargascontrol.Formateador.CUConsultarPosicionMapa;

import com.example.juanalonsoastruga.cargascontrol.Controlador.CUConsultarPosicionMapa.ControladorConsultarPosicionMapa;
import com.example.juanalonsoastruga.cargascontrol.Controlador.CUConsultarPosicionMapa.InterfaceCConsultarPosicionMapa;

import java.util.Hashtable;

/**
 * Clase FormateadorConsultarPosicionMapa, representa al formateador del caso de uso Consultar Posicion Actual en Mapa
 *
 */
public class FormateadorConsultarPosicionMapa implements InterfaceFConsultarPosicionMapaImpl, InterfaceFConsultarPosicionMapa{

    private Hashtable posicionActual;
    private InterfaceCConsultarPosicionMapa controlador;
    private double latitud;
    private double longitud;

    /**
     * Constructor de clase.
     *
     */
    public FormateadorConsultarPosicionMapa(){
        setControlador(new ControladorConsultarPosicionMapa(this));
    }

    /**
     * Metodo setControlador, asocia un controlador a la instancia del formateador.
     *
     * @param controlador Objeto InterfaceCConsultarPosicionMapa
     * @throws  IllegalArgumentException si el argumento es nulo
     */
    @Override
    public void setControlador(InterfaceCConsultarPosicionMapa controlador){

        if(controlador==null){
            throw new IllegalArgumentException();
        }else{
            this.controlador=controlador;
        }

    }

    /**
     * Metodo getControlador, devuelve el controlador asociado a este formateador.
     *
     * @return Objeto InterfaceCConsultarPosicionMapa
     */
    @Override
    public InterfaceCConsultarPosicionMapa getControlador(){
        return this.controlador;
    }

    /**
     * Metodo setLatitud, establece la latitud para el posicionamiento en el mapa.
     *
     * @param latitud   double con el valor de la latitud
     */
    @Override
    public void setLatitud(double latitud) {
        this.latitud=latitud;
    }
    /**
     * Metodo setLongitud, establece la longitud para el posicionamiento en el mapa.
     *
     * @param longitud  double con el valor de la longitud
     */
    @Override
    public void setLongitud(double longitud) {
        this.longitud=longitud;
    }

    /**
     * Metodo solicitarActualizacionMapa, llamado para avisar al controlador asociado de el suceso
     * del evento asociado a la actualizacion de un mapa para la posicion actual.
     *
     */
    @Override
    public void solicitarActualizacionMapa() {
        controlador.solicitarActualizacion();
    }

    /**
     * Metodo getLatitudActual, devuelve un double con la latitud en grados de la posicion
     * actual.
     *
     * @return double con la latitud en grados
     */
    @Override
    public double getLatitudActual() {
        return latitud;
    }
    /**
     * Metodo getLongitudActual, devuelve un double con la longitud en grados de la posicion
     * actual.
     *
     * @return double con la longitud en grados
     */
    @Override
    public double getLongitudActual() {
        return longitud;
    }


}
