package com.example.juanalonsoastruga.cargascontrol.Formateador.CUConsultarUltimoVehiculo;

import com.example.juanalonsoastruga.cargascontrol.Controlador.CUConsultarUltimoVehiculo.ControladorConsultarUltimoVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Controlador.CUConsultarUltimoVehiculo.InterfaceCConsultarUltimoVehiculo;

/**
 * Clase FormateadorConsultarUltimoVehiculo, representa al formateador en el CU Consultar Ultimo Vehiculo.
 *
 */
public class FormateadorConsultarUltimoVehiculo implements InterfaceFConsultarUltimoVehiculo, InterfaceFConsultarUltimoVehiculoImpl {
    private InterfaceCConsultarUltimoVehiculo controlador;
    private String idVehiculo;

    /**
     * Constructor de clase.
     *
     */
    public FormateadorConsultarUltimoVehiculo(){

        setControlador(new ControladorConsultarUltimoVehiculo(this));
    }

    /**
     * Metodo setControler, establece el controlador asociado al formateador.
     *
     * @param controlador Objeto InterfaceCConsultarUltimoVehiculo
     * @throws IllegalArgumentException si el argumento es nulo
     */
    @Override
    public void setControlador(InterfaceCConsultarUltimoVehiculo controlador) {
        if(controlador ==null){
            throw new IllegalArgumentException();
        }else{
            this.controlador = controlador;
        }
    }


    /**
     * Metodo getControler, devuelve el controlador asociado a este formateador.
     *
     * @return objeto InterfaceCConsultarUltimoVehiculo
     */
    @Override
    public InterfaceCConsultarUltimoVehiculo getControlador(){
        return controlador;
    }



    /**
     * Metodo eventoInicio, informa al controlador del evento de comienzo del CU.
     *
     *
     */
    @Override
    public void eventoInicio() {
            controlador.eventoInicio();
    }


    /**
     * Metodo setIdVehiculo, establece el id del vehiculo para el formateador.
     *
     * @param id String con el id
     */
    @Override
    public void setIdVehiculo(String id){
        idVehiculo=id;
    }

    /**
     * Metodo getIdVehiculo, devuelve el id formateado del ultimo vehiculo consultado.
     *
     *
     * @return String con la matricula del ultimo vehiculo consultado
     */
    @Override
    public String getIdVehiculo() {


            return idVehiculo;

    }
    /**
     * Metodo existeVehiculo, responde a la exitencia o no del vehiculo especificado.
     *
     * @param idVehiculo    String con el identificador del vehiculo cuya existencia se contempla
     * @return  true si exite, false en caso contrario
     */
    @Override
    public boolean existeVehiculo(String idVehiculo) {
        return controlador.existeVehiculo(idVehiculo);
    }
    /**
     * Metodo borraFichero, ordena al controlador que borre el fichero de registro.
     *
     *
     */
    @Override
    public void borraFichero() {
        controlador.borrarFichero();
    }
}
