package com.example.juanalonsoastruga.cargascontrol.Formateador.CUGraficarConsumoCombustible;

import android.util.Log;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.CodigoCombustibleVehiculo;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.CodigoPeriocidadGrafica;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTORecarga;
import com.example.juanalonsoastruga.cargascontrol.Controlador.CUGraficarConsumoCombustible.InterfaceCGraficarConsumoCombustible;
import com.example.juanalonsoastruga.cargascontrol.Controlador.CUGraficarConsumoCombustible.ControladorGraficarConsumoCombustible;
import com.github.mikephil.charting.data.Entry;
import com.github.mikephil.charting.data.LineDataSet;

import java.util.ArrayList;
import java.util.List;

/**
 * Clase FormateadorGraficarConsumoCombustible, representa el formateador en el CU Graficar Consumo de Combustible.
 *
 */
public class FormateadorGraficarConsumoCombustible implements InterfaceFGraficarConsumoCombustible, InterfaceFGraficarConsumoCombustibleImpl {

    private InterfaceCGraficarConsumoCombustible controlador;
    private String matricula;
    private CodigoCombustibleVehiculo codCombustible;
    private LineDataSet dataSet;


    /**
     * Constructor de Clase.
     *
     */
    public FormateadorGraficarConsumoCombustible(){
        setControlador(new ControladorGraficarConsumoCombustible(this));
    }

    /**
     * Metodo setControlador, establece el controlador asociado a este formateador.
     *
     * @param controlador Un Objeto InterfaceCGraficarConsumoCombustible
     * @throws IllegalArgumentException si el argumento es null
     */
    public void setControlador(InterfaceCGraficarConsumoCombustible controlador){
        if(controlador ==null){
            throw new IllegalArgumentException();
        }else{
            this.controlador = controlador;
        }
    }

    /**
     * Metodo getControlador, devuelve el controlador asociado a este formateador.
     *
     * @return  Un Objeto InterfaceCGraficarConsumoCombustible
     */
    public InterfaceCGraficarConsumoCombustible getControlador(){
        return this.controlador;
    }

    /**
     * Metodo comienzaCU, indica al formateador que despierte al controlador asociado para el comienzo
     * del caso de uso.
     *
     * @param matricula Un String con la matricula del vehiculo sobre el que se consulta
     * @param codCombustible   Un CodigoCombustibleVehiculo con el codigo de combustible Primario o Secundario
     * @param periodicidad     Un CodigoPeriocidadGrafica con la periocidad a mostrar en la gráfica
     */
    @Override
    public void comienzaCU(String matricula, CodigoCombustibleVehiculo codCombustible, CodigoPeriocidadGrafica periodicidad) {
        Log.d("FormateadorGraficarConsumoCombustible","matricula "+matricula);
        this.matricula=matricula;
        this.codCombustible=codCombustible;
        controlador.comienzaCU(periodicidad);
    }

    /**
     * Metodo getMatricula, devuelve la matricula del vehiculo asociado.
     *
     * @return  Un String con la matricula
     */
    @Override
    public String getMatricula() {
        return matricula;
    }

    /**
     * Metodo getCodigoCombustible, devuelve el codigo de combustible (Primario o Secundario)
     * que se consultará en la gráfica.
     *
     * @return  Un CodigoCombustibleVehiculo con el codigo
     */
    @Override
    public CodigoCombustibleVehiculo getCodigoCombustible() {
        return codCombustible;
    }

    /**
     * Metodo preparaDatosVisualizacion, dado un listado de recargas crea un conjunto de datos
     * de consumos de combustibles deacuerdo a ese listado.
     *
     * @param recargas  Un Objeto List<DTORecarga> con el listado de recargas
     */
    @Override
    public void preparaDatosVisualizacion(List<DTORecarga> recargas) {
        if(recargas==null){
            throw new IllegalArgumentException();
        }else{
            List<Entry> entradasGrafica = new ArrayList<>(0);
            Float medicion;

            for (int i = 0; i < recargas.size(); i++) {
                try {
                    medicion = (float) ((double) recargas.get(i).getMedicion());

                } catch (NullPointerException e) {
                    medicion = (float) 0.0;
                }
                if(medicion!=0.0) {
                    entradasGrafica.add(new Entry((float) (i + 1), medicion));
                }
            }
            dataSet = new LineDataSet(entradasGrafica, "");

        }
    }

    /**
     * Metodo getDataSet, devuelve un listado de entradas representables en una gráfica lineal como
     * los consumos de combustibles realizados.
     *
     * @return  Un Objeto LineDataSet
     */
    @Override
    public LineDataSet getDataSet() {
        return dataSet;
    }
}
