package com.example.juanalonsoastruga.cargascontrol.Formateador.CUGraficarConsumoCombustibleXKm;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.CodigoCombustibleVehiculo;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.CodigoPeriocidadGrafica;
import com.example.juanalonsoastruga.cargascontrol.Controlador.CUGraficarConsumoCombustibleXKm.InterfaceCGraficarConsumoCombustibleKm;
import com.example.juanalonsoastruga.cargascontrol.Controlador.CUGraficarConsumoCombustibleXKm.ControladorGraficarConsumoCombustibleKm;
import com.github.mikephil.charting.data.Entry;
import com.github.mikephil.charting.data.LineDataSet;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;

/**
 * Clase FormateadorGraficarConsumoCombustibleKm, representa al formateador en el CU Graficar Consumo de Combustible por Km recorrido
 *
 */
public class FormateadorGraficarConsumoCombustibleKm implements InterfaceFGraficarConsumoCombustibleKmImpl, InterfaceFGraficarConsumoCombustibleKm {
    private InterfaceCGraficarConsumoCombustibleKm controlador;
    private String matricula;
    private CodigoCombustibleVehiculo codCombustible;
    private LineDataSet dataSet;
    private double km;

    /**
     * Constructor de clase.
     *
     */
    public FormateadorGraficarConsumoCombustibleKm(){
        setControlador(new ControladorGraficarConsumoCombustibleKm(this));
    }

    /**
     * Metodo setControlador, establece el controlador asociado a este formateador.
     *
     * @param controlador Un Objeto InterfaceCGraficarConsumoCombustibleKm
     * @throws IllegalArgumentException si el argumento es null
     */
    public void setControlador(InterfaceCGraficarConsumoCombustibleKm controlador){
        if(controlador ==null){
            throw new IllegalArgumentException();
        }else{
            this.controlador = controlador;
        }
    }
    /**
     * Metodo getControlador, devuelve el controlador asociado a este formateador.
     *
     * @return  Un Objeto InterfaceCGraficarConsumoCombustibleKm
     */
    public InterfaceCGraficarConsumoCombustibleKm getControlador(){
        return this.controlador;
    }


    /**
     * Metodo comienzaCU, indica a la vista que despierte al controlador asociado para el comienzo
     * del caso de uso.
     *
     * @param matricula Un String con la matricula del vehiculo sobre el que se consulta
     * @param codCombustible   Un String con el nombre del codigo de combustible Primario o Secundario
     * @param periodicidad     Un CodigoPeriocidadGrafica con la periocidad a mostrar en la gráfica
     */
    @Override
    public void comienzaCU(String matricula, CodigoCombustibleVehiculo codCombustible, CodigoPeriocidadGrafica periodicidad) {
        if(matricula==null||codCombustible==null||periodicidad==null){
            throw new IllegalArgumentException();
        }else{
            this.matricula=matricula;
            this.codCombustible=codCombustible;
            controlador.comienzaCU(periodicidad);
        }

    }
    /**
     * Metodo getMatricula, devuelve la matricula del vehiculo asociado.
     *
     * @return  Un String con la matricula
     */
    @Override
    public String getMatricula() {
        return matricula;
    }
    /**
     * Metodo getCodigoCombustible, devuelve el codigo de combustible (Primario o Secundario)
     * que se consultará en la gráfica.
     *
     * @return  Un CodigoCombustibleVehiculo con el codigo
     */
    @Override
    public CodigoCombustibleVehiculo getCodigoCombustible() {
        return codCombustible;
    }
    /**
     * Metodo preparaDatosVisualizacion, dado un listado de recargas crea un conjunto de datos
     * de consumos de combustibles deacuerdo a ese listado.
     *
     * @param recargas  Un Objeto List<DTORecarga> con el listado de recargas
     * @throws IllegalArgumentException si el argumento proporcionado es null
     */
    @Override
    public void preparaDatosVisualizacion(Hashtable recargas,int size) {
        if(recargas==null){
            throw new IllegalArgumentException();
        }else{
            List<Entry>  entradasGrafica=new ArrayList<>(0);
            Float medicion;
            Float km;

            float division=0;


            for(int i=0;i<size;i++){
                try {
                    medicion=(float)((double)recargas.get(i+1));
                    if(medicion!=0.0) {
                        entradasGrafica.add(new Entry((float) (i + 1), medicion));
                    }
                }catch(NullPointerException npe){
                    entradasGrafica.add(new Entry(i+1,(float)(0) ));
                }
            }
            dataSet = new LineDataSet(entradasGrafica,"");
        }

    }

    /**
     * Metodo getDataSet, devuelve un listado de entradas representables en una gráfica lineal como
     * los consumos de combustibles realizados.
     *
     * @return  Un Objeto LineDataSet
     */
    @Override
    public LineDataSet getDataSet() {
        return dataSet;
    }
}
