package com.example.juanalonsoastruga.cargascontrol.Formateador.CUGraficarCosteCombustibleXUnidadRecarga;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.CodigoCombustibleVehiculo;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.CodigoPeriocidadGrafica;
import com.example.juanalonsoastruga.cargascontrol.Controlador.CUGraficarCosteCombustibleXUnidadRecarga.ControladorGraficarCosteCombustibleUnidadRecarga;
import com.example.juanalonsoastruga.cargascontrol.Controlador.CUGraficarCosteCombustibleXUnidadRecarga.InterfaceCGraficarCosteCombustibleUnidadRecarga;
import com.github.mikephil.charting.data.Entry;
import com.github.mikephil.charting.data.LineDataSet;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;
/**
 * Clase FormateadorGraficarCosteCombustibleUnidadRecarga, representa al formateador en el CU Graficar Coste por Kilometro
 *
 */
public class FormateadorGraficarCosteCombustibleUnidadRecarga implements InterfaceFGraficarCosteCombustibleUnidadRecarga, InterfaceFGraficarCosteCombustibleUnidadRecargaImpl {
    private InterfaceCGraficarCosteCombustibleUnidadRecarga controlador;
    private String matricula;
    private CodigoCombustibleVehiculo codCombustible;
    private LineDataSet dataSet;
    private double km;
    /**
     * Constructor de clase.
     *
     */
    public FormateadorGraficarCosteCombustibleUnidadRecarga(){
        setControlador(new ControladorGraficarCosteCombustibleUnidadRecarga(this));
    }

    /**
     * Metodo setControler, establece el controlador asociado a este formateador.
     *
     * @param controlador Un Objeto InterfaceCGraficarCosteCombustibleUnidadRecarga
     * @throws IllegalArgumentException si el argumento es null
     */
    public void setControlador(InterfaceCGraficarCosteCombustibleUnidadRecarga controlador){
        if(controlador ==null){
            throw new IllegalArgumentException();
        }else{
            this.controlador = controlador;
        }
    }

    /**
     * Metodo getControler, devuelve el controlador asociado a esta formateador.
     *
     * @return  Un Objeto InterfaceCGraficarCosteCombustibleUnidadRecarga
     */
    public InterfaceCGraficarCosteCombustibleUnidadRecarga getControlador(){
        return this.controlador;
    }
    /**
     * Metodo comienzaCU, indica al formateador que despierte al controlador asociado para el comienzo
     * del caso de uso.
     *
     * @param matricula Un String con la matricula del vehiculo sobre el que se consulta
     * @param codCombustible   Un CodigoCombustibleVehiculo con el nombre del codigo de combustible Primario o Secundario
     * @param periodicidad     Un CodigoPeriocidadGrafica con la periocidad a mostrar en la gráfica
     */
    @Override
    public void comienzaCU(String matricula, CodigoCombustibleVehiculo codCombustible, CodigoPeriocidadGrafica periodicidad) {
        this.matricula=matricula;
        this.codCombustible=codCombustible;
        controlador.comienzaCU(periodicidad);
    }
    /**
     * Metodo getMatricula, devuelve la matricula del vehiculo asociado.
     *
     * @return  Un String con la matricula
     */
    @Override
    public String getMatricula() {
        return matricula;
    }
    /**
     * Metodo getCodigoCombustible, devuelve el codigo de combustible (Primario o Secundario)
     * que se consultará en la gráfica.
     *
     * @return  Un CodigoCombustibleVehiculo con el codigo
     */
    @Override
    public CodigoCombustibleVehiculo getCodigoCombustible() {
        return codCombustible;
    }
    /**
     * Metodo preparaDatosVisualizacion, dado un listado de recargas crea un conjunto de datos
     * de consumos de combustibles deacuerdo a ese listado.
     *
     * @param recargas  Un Objeto Hashtable  con el listado de recargas
     * @throws IllegalArgumentException si el argumento proporcionado es null
     */
    @Override
    public void preparaDatosVisualizacion(Hashtable recargas,int size) {
        if(recargas==null){
            throw new IllegalArgumentException();
        }else {
            List<Entry> entradasGrafica = new ArrayList<>(0);
            Float medicion;
            Float km;
            float division = 0;
            for (int i = 0; i < size; i++) {
                try {
                    medicion = (float) ((double) recargas.get(i+1));

                } catch (NullPointerException e) {

                    medicion = (float) 0.0;

                }
                if(medicion!=0.0) {
                    entradasGrafica.add(new Entry((float) (i + 1), medicion));
                }
            }
            dataSet = new LineDataSet(entradasGrafica, "");
        }
    }


    /**
     * Metodo getDataSet, devuelve el conjunto de datos listos para su representacion grafica.
     *
     * @return  Un Objeto LineDataSet
     */
    @Override
    public LineDataSet getDataSet() {
        return dataSet;
    }
}
