package com.example.juanalonsoastruga.cargascontrol.Formateador.CUGraficarKm;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.CodigoCombustibleVehiculo;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.CodigoPeriocidadGrafica;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTORecarga;
import com.example.juanalonsoastruga.cargascontrol.Controlador.CUGraficarKm.ControladorGraficarKm;
import com.example.juanalonsoastruga.cargascontrol.Controlador.CUGraficarKm.InterfaceCGraficarKm;
import com.github.mikephil.charting.data.Entry;
import com.github.mikephil.charting.data.LineDataSet;

import java.util.ArrayList;
import java.util.List;
/**
 * Clase FormateadorGraficarKm, representa al formateador en el CU Graficar Kilometros Recorridos
 *
 */
public class FormateadorGraficarKm implements InterfaceFGraficarKm, InterfaceFGraficarKmImpl {
    private InterfaceCGraficarKm controlador;
    private String matricula;
    private CodigoCombustibleVehiculo codCombustible;
    private LineDataSet dataSet;
    /**
     * Constructor de clase.
     *
     */
    public FormateadorGraficarKm(){
        setControlador(new ControladorGraficarKm(this));
    }
    /**
     * Metodo setControler, establece el controlador asociado a esta formateador.
     *
     * @param controlador Un Objeto InterfaceCGraficarKm
     * @throws IllegalArgumentException si el argumento es null
     */

    public void setControlador(InterfaceCGraficarKm controlador){
        if(controlador ==null){
            throw new IllegalArgumentException();
        }else{
            this.controlador = controlador;
        }
    }
    /**
     * Metodo getControler, devuelve el controlador asociado a este formateador.
     *
     * @return  Un Objeto InterfaceCGraficarKm
     */
    public InterfaceCGraficarKm getControlador(){
        return this.controlador;
    }
    /**
     * Metodo comienzaCU, indica al formateador que despierte al controladoe asociado para el comienzo
     * del caso de uso.
     *
     * @param matricula Un String con la matricula del vehiculo sobre el que se consulta
     * @param codCombustible   Un CodigoCombustibleVehiculo con el nombre del codigo de combustible Primario o Secundario
     * @param periodicidad     Un CodigoPeriocidadGrafica con la periocidad a mostrar en la gráfica
     */
    @Override
    public void comienzaCU(String matricula, CodigoCombustibleVehiculo codCombustible, CodigoPeriocidadGrafica periodicidad) {
        this.matricula=matricula;
        this.codCombustible=codCombustible;
        controlador.comienzaCU(periodicidad);
    }
    /**
     * Metodo getMatricula, devuelve la matricula del vehiculo asociado.
     *
     * @return  Un String con la matricula
     */
    @Override
    public String getMatricula() {
        return matricula;
    }
    /**
     * Metodo getCodigoCombustible, devuelve el codigo de combustible (Primario o Secundario)
     * que se consultará en la gráfica.
     *
     * @return  Un CodigoCombustibleVehiculo con el codigo
     */
    @Override
    public CodigoCombustibleVehiculo getCodigoCombustible() {
        return codCombustible;
    }
    /**
     * Metodo preparaDatosVisualizacion, dado un listado de recargas crea un conjunto de datos
     * de consumos de combustibles deacuerdo a ese listado.
     *
     * @param recargas  Un Objeto List<DTORecarga> con el listado de recargas
     * @throws IllegalArgumentException si el argumento proporcionado es null
     */
    @Override
    public void preparaDatosVisualizacion(List<DTORecarga> recargas) {
        if(recargas==null){
            throw new IllegalArgumentException();
        }else {
            List<Entry> entradasGrafica = new ArrayList<>(0);
            Float medicion;
            //entradasGrafica.add(new Entry((float)0.0,(float)0.0));
            for (int i = 0; i < recargas.size(); i++) {
                try {
                    medicion = (float) ((double) recargas.get(i).getKm());

                } catch (NullPointerException e) {
                    medicion = (float) 0.0;
                }
                if(medicion!=0.0) {
                    entradasGrafica.add(new Entry((float) (i + 1), medicion));
                }
            }
            dataSet = new LineDataSet(entradasGrafica, "");
        }
    }
    /**
     * Metodo getDataSet, devuelve el conjunto de datos listos para su representacion grafica.
     *
     * @return  Un Objeto LineDataSet
     */
    @Override
    public LineDataSet getDataSet() {
        return dataSet;
    }
}
