package com.example.juanalonsoastruga.cargascontrol.Formateador.CUModificarInformacionVehiculo;

import android.graphics.Bitmap;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTOCombustible;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTOVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Controlador.CUModificarInformacionVehiculo.InterfaceCModificarInformacionVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Controlador.CUModificarInformacionVehiculo.ControladorModificarInformacionVehiculo;

import java.util.ArrayList;
import java.util.List;

/**
 * Clase FormateadorModificarInformacionVehiculo, Formateador del CU Modificar Informacion de Vehiculo
 *
 */
public class FormateadorModificarInformacionVehiculo implements InterfaceFModificarInformacionVehiculo, InterfaceFModificarInformacionVehiculoImpl {
    private InterfaceCModificarInformacionVehiculo controlador;
    private List<String> combustibles;
    private DTOVehiculo registro;
    private  boolean registroCompleto;
    private String mensajeError;

    /**
     * Constructor de clase.
     *
     */
    public FormateadorModificarInformacionVehiculo(){
        setControlador(new ControladorModificarInformacionVehiculo(this));
        registro= new DTOVehiculo();
        registroCompleto=false;
        mensajeError=null;
    }

    /**
     * Metodo setControlador, establece el controlador deacuerdo al patron MVP
     *
     * @param controlador Objeto InterfaceCModificarInformacionVehiculo
     */
    @Override
    public void setControlador(InterfaceCModificarInformacionVehiculo controlador){
        if(controlador==null){
            throw new IllegalArgumentException();
        }else{
            this.controlador =controlador;
        }
    }

    /**
     * Metodo getControlador, devuelve un objeto representado el controlador asociado a esta formateador.
     *
     * @return  Objeto InterfaceCModificarInformacionVehiculo
     */
    @Override
    public InterfaceCModificarInformacionVehiculo getControlador(){
        return controlador;
    }

    /**
     * Metodo comienzoCU, inicia la ejecucion del caso de uso
     *
     * @param matricula String representando la matricula del vehiculo a modificar
     */
    @Override
    public void init(String matricula) {
        controlador.comienzoCU(matricula);

    }

    /**
     * Metodo setInitInfo, establece en la vista la informacion necesaria para iniciar la ejecucion del CU
     *
     * @param c Objeto List<DTOCombustible> , representando un listado de combustibles disponibles
     * @param v Objeto DTOVehiculo, con los datos del vehiculo a modificar
     * @return true si exito, false en caso contrario
     */
    @Override
    public boolean setInitInfo(List<DTOCombustible> c,DTOVehiculo v ) {
        if(c==null||v==null){
            throw new IllegalArgumentException();
        }else {
            this.combustibles = new ArrayList<String>(0);
            registro = v;
            for (int i = 0; i < c.size(); i++) {
                combustibles.add(c.get(i).getTipo());
            }
        }return true;
    }

    /**
     * Metodo getRegistro, devuelve un objeto representando la informacion dinamica del vehiculo que se
     * esta modificando.
     *
     * @return  Objeto  DTOVehiculo
     */
    @Override
    public DTOVehiculo getRegistro() {
        return registro;
    }

    /**
     * Metodo setRegistroCompletado, informa a la vista que la operacion de modificacion se ha completado.
     *
     * @return true si exito, false en caso contrario
     */
    @Override
    public boolean setRegistroCompletado() {
        registroCompleto=true;
        return true;
    }

    /**
     * Metodo setMensajeError, establece un mensaje de error en la vista
     *
     * @param mensaje   El String con el mensaje
     */
    @Override
    public boolean setMensajeError(String mensaje) {
        this.mensajeError=mensaje;
        return true;
    }



    /**
     * Metodo, informa a la vista de que el registro no se ha completado
     *
     */
    @Override
    public boolean setRegistroNoCompleto() {
        registroCompleto=false;
        return true;
    }

    /**
     * Metodo pulsacionBotonModificar, informa del comienzo del evento de modificacion de informacion
     * del vehiculo.
     *
     */
    @Override
    public void pulsacionBotonModificar() {
        controlador.pulsacionBotonModificarDatos();
    }

    /**
     * Devuelve un listado de combustibles necesarios para la vista.
     *
     * @return  Objeto  List<String>
     */
    @Override
    public List<String> getInfoCombustible(){
        return combustibles;
    }

    /**
     * Metodo getNombre, devuelve el nombre del vehiculo
     *
     * @return  Objeto String
     */
    @Override
    public String getNombre() {
        return registro.getNombre();
    }
    /**
     * Devuelve la matricula del  vehiculo en consulta
     *
     * @return  Un objeto String
     */
    @Override
    public String getMatricula() {
        return registro.getMatricula();
    }
    /**
     * Devuelve la marca del vehiculo en consulta
     *
     * @return  Un objeto String
     */
    @Override
    public String getMarca() {
        return registro.getMarca();
    }
    /**
     * Devuelve el modelo del vehiculo en consulta
     *
     * @return  Un objeto String
     */
    @Override
    public String getModelo() {
        return registro.getModelo();
    }
    /**
     * Devuelve el año del vehiculo en consulta
     *
     * @return  Un objeto Integer
     */
    @Override
    public int getAnio() {
        return registro.getAño();
    }
    /**
     * Devuelve los km del vehiculo en consulta
     *
     * @return  Un objeto Double
     */
    @Override
    public double getKm() {
        return registro.getKilometraje();
    }
    /**
     * Devuelve el combustible primario del vehiculo en consulta
     *
     * @return  Un objeto String
     */
    @Override
    public String getCPrim() {
        return registro.getCombustiblePrimario();
    }
    /**
     * Devuelve el combustible secundario del vehiculo en consulta
     *
     * @return  Un objeto CharSequence
     */
    @Override
    public String getCSec() {
        return registro.getCombustibleSecundario();
    }
    /**
     * Devuelve la capacidad del tanque de combustible secundario del vehiculo en consulta
     *
     * @return  Un objeto Double
     */
    @Override
    public double getCapSec() {
        return registro.getCapTankSecundario();
    }
    /**
     * Devuelve la capacidad del tanque de combustible primario del vehiculo en consulta
     *
     * @return  Un objeto Double
     */
    @Override
    public double getCapPrim() {
        return registro.getCapTankPrimario();
    }
    /**
     * Devuelve un vector de pixeles de la foto del vehiculo
     *
     * @return  Un objeto Bitmap
     */
    @Override
    public Bitmap getFoto() {
        return registro.getFoto();
    }

    /**
     * Metodo actualizaNombre, establece el nombre del vehiculo
     *
     * @param nombre  Objeto String
     */
    @Override
    public void actualizaNombre(String nombre) {
        registro.setNombre(nombre);
    }
    /**
     * Metodo actualizaMarca, establece la marca del vehiculo
     *
     * @param marca  Objeto String
     */
    @Override
    public void actualizaMarca(String marca) {
        registro.setMarca(marca);
    }
    /**
     * Metodo actualizaModelo, establece el modelo del vehiculo
     *
     * @param modelo  Objeto String
     */
    @Override
    public void actualizaModelo(String modelo) {
        registro.setModelo(modelo);
    }
    /**
     * Metodo actualizaAno, establece el ano del vehiculo
     *
     * @param año  Objeto int
     */
    @Override
    public void actualizaAno(int año) {
        registro.setAño(año);
    }
    /**
     * Metodo actualizaKm, establece los km del vehiculo
     *
     * @param km  Objeto double
     */
    @Override
    public void actualizaKm(double km) {
        registro.setKilometraje(km);
    }
    /**
     * Metodo actualizaCPrimario, establece el Combustible Primario del vehiculo
     *
     * @param CPrim Objeto String
     */
    @Override
    public void actualizaCPrimario(String CPrim) {
        registro.setCombustiblePrimario(CPrim);
    }
    /**
     * Metodo actualizaCapTP, establece la capacidad del tanque de Combustible Primario del vehiculo
     *
     * @param capacidad Objeto double
     */
    @Override
    public void actualizaCapTP(double capacidad) {
        registro.setCapTankPrimario(capacidad);
    }
    /**
     * Metodo actualizaCSecundario, establece el Combustible Secundario del vehiculo
     *
     * @param CSec Objeto String
     */
    @Override
    public void actualizaCSecundario(String CSec) {
        registro.setCombustibleSecundario(CSec);
    }
    /**
     * Metodo actualizaCSecundario, establece la capacidad del tanque de Combustible Secundario del vehiculo
     *
     * @param capacidad Objeto double
     */
    @Override
    public void actualizaCapTS(double capacidad) {
        registro.setCapTankSecundario(capacidad);
    }
    /**
     * Metodo actualizaFoto, establece la foto del vehiculo
     *
     * @param bitmap Objeto Bitmap
     */
    @Override
    public void actualizaFoto(Bitmap bitmap) {
        registro.setFoto(bitmap);
    }

    /**
     * Informa de si la operación de registrar la modificacion se ha completado
     *
     * @return  true en caso de completarse, false en caso contrario
     */
    @Override
    public boolean isRegistroCompletado() {

        return registroCompleto;
    }

    /**
     * Metodo getMsgError, devuelve el mensaje de error establecido en el formateador.
     *
     * @return  Un String con el mensaje
     */
    @Override
    public String getMensajeError() {
        String msj=this.mensajeError;
        this.mensajeError=null;

        return msj;
    }

    /**Metodo, avisa al formateador de la existencia de un campo invalido
     *
     *
     * @param cmp   Un string con el codgio de campo
     */
    @Override
    public void deteccionCampoInvalido(String cmp) {
        controlador.rellenarCampoIncorrectamente(cmp);
    }


}
