package com.example.juanalonsoastruga.cargascontrol.Formateador.CUMostrarListadoGasolineras;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTOGasolinera;
import com.example.juanalonsoastruga.cargascontrol.Controlador.CUMostrarListadoGasolineras.InterfaceCMostrarListadoGasolineras;
import com.example.juanalonsoastruga.cargascontrol.Controlador.CUMostrarListadoGasolineras.ControladorMostrarListadoGasolineras;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.List;

/**
 * Clase FormateadorMostrarListadoGasolineras, formateador del caso de uso Mostrar Listado Gasolineras.
 *
 */
public class FormateadorMostrarListadoGasolineras implements InterfaceFMostrarListadoGasolineras, InterfaceFMostrarListadoGasolineraImpl {

    private InterfaceCMostrarListadoGasolineras controlador;
    private List<PostDataGasolinera> estaciones;
    private String latitudActual;
    private String longitudActual;
    /**
     * Constructor de clase.
     *
     */
    public FormateadorMostrarListadoGasolineras(){
        setControlador(new ControladorMostrarListadoGasolineras(this));
    }

    /**
     * Metodo setControler, establece el controlador a asociar a este formateador.
     *
     * @param controlador objeto InterfaceCMostrarListadoGasolineras
     * @throws IllegalArgumentException si el argumento es nulo
     */
    @Override
    public void setControlador(InterfaceCMostrarListadoGasolineras controlador){
        if(controlador ==null){
            throw new IllegalArgumentException();
        }else{
            this.controlador = controlador;
        }
    }

    /**
     * Metodo setLatitudActual, establece la latitud actual del dispositivo.
     *
     * @param latitud   String con la latitud
     */
    @Override
    public void setLatitudActual(double latitud) {
        this.latitudActual=latitud+"";
    }
    /**
     * Metodo setLongitudActual, establece la longitud actual del dispositivo.
     *
     * @param longitud   String con la longitud
     */
    @Override
    public void setLongitudActual(double longitud) {
        this.longitudActual=longitud+"";
    }

    /**
     * Metodo getLatitudActual devuelve la latitud actual del dispositivo.
     *
     * @return String con la latitud
     */
    @Override
    public String getLatitudActual() {
        return latitudActual;
    }
    /**
     * Metodo getLongitudActual devuelve la longitud actual del dispositivo.
     *
     * @return String con la longitud
     */
    @Override
    public String getLongitudActual() {
        return longitudActual;
    }

    /**
     * Metodo getControler, devuelve el controlador asociado al formateador.
     *
     * @return objeto InterfaceCMostrarListadoGasolineras
     */
    @Override
    public InterfaceCMostrarListadoGasolineras getControlador(){
        return controlador;
    }

    /**
     * Metodo init, evento que transimitir al controlador para el comienzo de la ejecucion del CU.
     *
     */
    @Override
    public void init() {
        controlador.inicio();
    }

    /**
     * Metodo getListadoEstaciones, devuelve un listado de las estaciones cercanas a la posición geográfica actual.
     *
     * @return Objeto List
     */
    @Override
    public List<PostDataGasolinera> getListadoEstaciones() {
        return estaciones;
    }

    /**
     * Metodo setGasolineras, establece el contendor de datos de gasolineras cercanas en esta clase.
     *
     * @param gasolineras objeto List<DTOGasolinera>
     * @throws IllegalArgumentException si el argumento es nulo
     */
    @Override
    public void setGasolineras(List<DTOGasolinera> gasolineras) {
        if(gasolineras==null){
            throw new IllegalArgumentException();
        }else {
            estaciones = new ArrayList<PostDataGasolinera>(0);
            PostDataGasolinera tmpPD;
            DTOGasolinera tmp;
            for (int i = 0; i < gasolineras.size(); i++) {
                tmp = gasolineras.get(i);
                tmpPD = new PostDataGasolinera(tmp.getRotulo(), tmp.getKm() + "", tmp.getDireccion(), tmp.getLocalidad(), tmp.getLatitud(), tmp.getLongitud());
                estaciones.add(tmpPD);
            }
        }
    }
}
