package com.example.juanalonsoastruga.cargascontrol.Formateador.CURegistrarRecarga;

import android.graphics.Bitmap;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTORecarga;
import com.example.juanalonsoastruga.cargascontrol.Controlador.CURegistrarRecarga.InterfaceCRegistrarRecarga;
import com.example.juanalonsoastruga.cargascontrol.Controlador.CURegistrarRecarga.ControladorRegistrarRecarga;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

/**
 * Clase FormateadorRegistrarRecarga, implementa el Formateador del CU Registrar Recarga.
 *
 */
public class FormateadorRegistrarRecarga implements InterfaceFRegistrarRecarga, InterfaceFRegistrarRecargaImpl {

    private InterfaceCRegistrarRecarga controlador;
    private DTORecarga recarga;
    private boolean sinFechaEnRegistro=false;
    private boolean sinTiempoEnRegistro=false;
    private boolean registroCompletado=false;
    private boolean camposCompletos=true;
    private String msgError;

    /**
     * Constructor de Clase.
     *
     */
    public FormateadorRegistrarRecarga(){
        setControlador(new ControladorRegistrarRecarga(this));
    }

    /**
     * Metodo setControler, establece el Controlador del CU Registrar Recarga
     *
     * @param controlador Un Objeto que implemente InterfaceCRegistrarRecarga
     * @throws  IllegalArgumentException si el objeto proporcionado es null
     */
    @Override
    public void setControlador(InterfaceCRegistrarRecarga controlador){
        if(controlador==null){
            throw new IllegalArgumentException();
        }else {
            this.controlador = controlador;
        }

    }

    /**
     * Metodo getControler devuelve el Controlador asociado en el CU Registrar Recarga
     *
     * @return  Un Objeto que implemente InterfaceCRegistrarRecarga
     */
    @Override
    public InterfaceCRegistrarRecarga getControlador(){
        return controlador;
    }


    /**
     * Metodo comienzoCU, indica a la vista que despierte al Controladdr para el comienzo del CU.
     *
     *
     */
    @Override
    public void comienzoCU() {
        controlador.comienzoCU();

    }

    /**
     * Metodo getRecarga, devuelve la recarga registrada hasta el momento
     *
     * @return Un objeto DTORecarga
     */
    @Override
    public DTORecarga getRecarga(){
        return recarga;
    }

    /**
     * Metodo registrarSinFecha, informa al controlador de un registro de recarga sin especificar fecha.
     *
     *
     */
    @Override
    public void registrarSinFecha() {
        controlador.registroSinFecha();

    }

    /**
     * Metodo registrarSinTiempo, informa al controlador de un registro de recarga sin especificar hora y minutos
     *
     */
    @Override
    public void registrarSinTiempo() {
        controlador.registroSinTiempo();
    }

    /**
     * Metodo setDinamicData, establece un contenedor desde el que se cargan los datos para el registro.
     *
     */
    @Override
    public void setDinamicData( ) {

            this.recarga =new DTORecarga();

    }

    /**
     * Metodo setSinFecha, indica al formateador que el contenedor con los datos de la recarga no contiene fecha.
     *
     */
    @Override
    public void setSinFecha() {
        sinFechaEnRegistro=true;
    }
    /**
     * Metodo setSinTiempo, indica al formateador que el contenedor con los datos de la recarga no contiene hora y minutos.
     *
     */
    @Override
    public void setSinTiempo() {
        sinTiempoEnRegistro=true;
    }

    /**
     * Metodo isCamposRegistroCompletos, informa de si los campos del contenedor de la recarga están completos.
     *
     * @return  True si es cierto, false en caso contrario
     */
    @Override
    public boolean isCamposRegistroCompletos() {
        return camposCompletos;
    }

    /**
     * Metodo setRegistroCompletado, indica al formateador si el registro de la recarga se ha producido con éxito o no.
     *
     * @param result True si ha tenido éxito, false en caso contrario
     */
    @Override
    public void setRegistroCompletado(boolean result) {
        if(result==true){
            registroCompletado=true;
        }else{
            registroCompletado=false;
        }
    }

    /**
     * Metodo setMsgError, indica al formateador el mensaje de error a mostrar posteriormente.
     *
     * @param message   Un String representando el mensaje
     */
    @Override
    public void setMsgError(String message) {
        msgError=message;
    }

    /**
     * Metodo getMsgError, devuelve un String con el mensaje de error establecido.
     *
     * @return  Un String representando el mensaje
     */
    @Override
    public String getMsgError() {
        return msgError;
    }

    /**
     * Metodo setConFecha, indica a la vista que en el registro de recarga se ha indicado la fecha.
     *
     */
    @Override
    public void setConFecha() {
        sinFechaEnRegistro=false;
    }
    /**
     * Metodo setConTiempo, indica a la vista que en el registro de recarga se ha indicado la hora y minutos.
     *
     */
    @Override
    public void setConTiempo() {
        sinTiempoEnRegistro=false;
    }
    /**
     * Metodo isSinFecha, indica a la vista que en el registro de recarga no se ha indicado la fecha.
     *
     */

    @Override
    public boolean isSinFecha(){
        return sinFechaEnRegistro;
    }

    /**
     * Metodo isSinTiempo, indica a la vista que en el registro de recarga no se ha indicado horas y minutos.
     *
     */
    @Override
    public boolean isSinTiempo(){
        return sinTiempoEnRegistro;
    }

    /**
     * Metodo registroConCamposIncompletos, indica a la vista que en el registro de recarga se han omitido campos obligatorios.
     *
     */
    @Override
    public void registroConCamposIncompletos() {
        camposCompletos=false;
    }

    /**
     * Metodo registroConCamposIncompletos, indica a la vista que en el registro de recarga  no se han omitido campos obligatorios.
     *
     */
    @Override
    public void registroConCamposCompletos() {
        camposCompletos=true;
    }

    /**
     * Metodo isRegistroCorrecto, indica si el registro de la recarga se ha realizado correctamente.
     *
     * @return  True si éxito, false en caso contrario
     */
    @Override
    public boolean isRegistroCorrecto(){
        if(!(sinTiempoEnRegistro)&!(sinFechaEnRegistro)){

            return registroCompletado;
        }else{
            return false;
        }
    }

    /**
     * Metodo registrarMatricula, carga en el contenedor de datos dinámicos la matricula del vehículo
     * cuya recarga se está registrando.
     *
     * @param matricula Un String con la matricula del vehículo
     * @throws IllegalArgumentException Si argumento es null
     */
    @Override
    public void registrarMatricula(String matricula) {

        if(matricula==null){
            throw new IllegalArgumentException();
        }else{
            recarga.setMatricula(matricula);
        }
    }

    /**
     * Método registrarFechayTiempo, carga la fecha y horas en el contenedor de datos dinámicos para la recarga
     * a registrar en curso.
     *
     * @param ano   Un objeto int con el año
     * @param mes   Un objeto int con el mes
     * @param dia   Un objeto int con el dia
     * @param hora  Un objeto int con las horas
     * @param minuto Un objeto int con los minutos
     * @throws IllegalArgumentException Si algun argumento es null
     */
    @Override
    public void registrarFechayTiempo(Integer ano, Integer mes, Integer dia, Integer hora, Integer minuto)  {
        if(ano==null||mes==null||dia==null||hora==null||minuto==null){
            throw new IllegalArgumentException();
        }
        else{
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm");
            String d= ano+"-"+mes+"-"+dia+" "+hora+":"+minuto;
            Date fechaConHora=null;
            Calendar cal=Calendar.getInstance();

            try {
              fechaConHora = sdf.parse(d);
              cal.setTime(fechaConHora);
            } catch (ParseException e) {
                e.printStackTrace();
            }


            recarga.setFecha(fechaConHora);
        }
    }

    /**
     * Metodo registrarKm, carga en el contenedor de datos dinámicos el kilometraje del vehículo
     * en el momento de la recarga.
     *
     * @param km    Un objeto double con los km a registrar
     * @throws IllegalArgumentException Si argumento es null
     */
    @Override
    public void registrarKm(Double km) {
        if(km==null){
            throw new IllegalArgumentException();
        }else {
            recarga.setKm(km);
        }
    }

    /**
     * Metodo registrarCoste, carga en el contenedor de datos dinámicos el coste de la recarga.
     *
     * @param coste  Un objeto double con el coste total a registrar
     * @throws IllegalArgumentException Si argumento es null
     */
    @Override
    public void registrarCoste(Double coste) {
        if(coste==null){
            throw new IllegalArgumentException();
        }else {
            recarga.setCoste(coste);
        }
    }
    /**
     * Metodo registrarCombustible, carga en el contenedor de datos dinámicos el tipo de combustible
     * recargado.
     *
     * @param combustible  Un objeto String con el tipo de combustible
     * @throws IllegalArgumentException Si argumento es null
     */
    @Override
    public void registrarCombustible(String combustible) {
        if(combustible==null){
            throw new IllegalArgumentException();
        }else {
            recarga.setCombustible(combustible);
        }
    }
    /**
     * Metodo registrarRecarga, carga en el contenedor de datos dinámicos la cantidad de combustible
     * recargado.
     *
     * @param medicion  Un objeto double con la cantidad recargada
     * @throws IllegalArgumentException Si argumento es null
     */
    @Override
    public void registrarRecarga(Double medicion) {
        if(medicion==null){
            throw new IllegalArgumentException();
        }else {
            recarga.setMedicion(medicion);
        }
    }
    /**
     * Metodo registraFoto, carga en el contenedor de datos dinámicos la foto del ticket asociado a la
     * recarga.
     *
     * @param bitmap  Un objeto Bitmap con la imagen
     */
    @Override
    public void registraFoto(Bitmap bitmap) {
        recarga.setFoto(bitmap);
    }

    /**
     * Método solicitarRegistro, avisa al controlador del evento de solicitud de registro de recarga de los
     * datos introducidos.
     *
     */
    @Override
    public void solicitarRegistro() {
        controlador.registrarRecarga(recarga);
    }
}
