package com.example.juanalonsoastruga.cargascontrol.Formateador.CURegistrarVehiculo;

import android.graphics.Bitmap;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTOCombustible;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTOVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Controlador.CURegistrarVehiculo.ControladorRegistroVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Controlador.CURegistrarVehiculo.InterfaceCRegistroVehiculo;

import java.util.ArrayList;
import java.util.List;

/**
 * Clase FormateadorRegistroVehiculo, representa al formateador en el CU Registro Vehiculo
 *
 */
public class FormateadorRegistroVehiculo implements InterfaceFRegistroVehiculo, InterfaceFRegistroVehiculoImpl {
    private InterfaceCRegistroVehiculo presentador;
    private List<DTOCombustible> listadoCombustible;
    private DTOVehiculo registro;
    private boolean registroCompletado;
    private String mensajeError;

    /**
     * Constructor de clase.
     *
     */
    public FormateadorRegistroVehiculo(){
        setControlador( new ControladorRegistroVehiculo(this));
    }

    /**
     * Metodo setControler, establece el controlador asociado a esta formateador
     *
     * @param controlador Objeto InterfaceCRegistroVehiculo
     */
    @Override
    public void setControlador(InterfaceCRegistroVehiculo controlador) {
        if(controlador==null){
            throw new IllegalArgumentException();
        }else{
            this.presentador =controlador;}
    }

    /**
     * Metodo getControler, devuelve el controlador asociado a este formateador.
     *
     * @return  Objeto InterfaceCRegistroVehiculo
     */
    @Override
    public InterfaceCRegistroVehiculo getControlador(){
        return presentador;
    }

    /**
     * Metodo eventoComienzo, da comienzo al CU
     *
     */
    @Override
    public void init(){
        presentador.eventoComienzo();
        registro = new DTOVehiculo();
        registroCompletado=false;
    }


    /**
     * Metodo getInfoSpinners, solicita a la vista el listado de cobustibles disponibles
     *
     * @return  Un listado  de combustibles representados por Strings
     */
    @Override
    public List<String> getInfoSpinners() {
        ArrayList<String>listado= new ArrayList<String>(0);
        DTOCombustible combustible;
        for(int i=0;i<listadoCombustible.size();i++){
            combustible=listadoCombustible.get(i);
            listado.add(combustible.getTipo()+" "+"("+combustible.getUnidades()+")");
        }
        return listado;
    }
    /**
     * Metodo setSpinnersCombustible, establece el listado de combustibles para el registro
     *
     * @param listado   Objeto List<DTOCombustible>
     */
    @Override
    public void setSpinnersCombustible(List<DTOCombustible> listado) {
        this.listadoCombustible=listado;
    }

    /**
     * Metodo solicitarGuardado, indica al presentador el evento de solicitarActualizacionMapa de la operacion de registro
     */
    @Override
    public void solicitarGuardado() {
        presentador.solicitarGuardado();
    }


    /**
     * Metodo actualizaNombre, establece el nombre del vehiculo
     *
     * @param text  Objeto String
     */
    @Override
    public void actualizaNombre(CharSequence text) {
        registro.setNombre((String)text);
    }
    /**
     * Metodo actualizaMatricula, establece la matricula del vehiculo
     *
     * @param text  Objeto String
     */
    @Override
    public void actualizaMatricula(CharSequence text) {
        registro.setMatricula((String)text);
    }

    /**
     * Metodo actualizaMarca, establece la marca del vehiculo
     *
     * @param text  Objeto String
     */
    @Override
    public void actualizaMarca(CharSequence text) {
        registro.setMarca((String)text);
    }
    /**
     * Metodo actualizaModelo, establece el modelo del vehiculo
     *
     * @param text  Objeto String
     */
    @Override
    public void actualizaModelo(CharSequence text) {
        registro.setModelo((String)text);
    }
    /**
     * Metodo actualizaAno, establece el ano del vehiculo
     *
     * @param anio  Objeto int
     */
    @Override
    public void actualizaAno(int anio) {
        registro.setAño(anio);
    }
    /**
     * Metodo actualizaKm, establece los km del vehiculo
     *
     * @param km  Objeto double
     */
    @Override
    public void actualizaKm(double km) {
        registro.setKilometraje(km);
    }
    /**
     * Metodo actualizaCPrimario, establece el Combustible Primario del vehiculo
     *
     * @param CPrim Objeto String
     */
    @Override
    public void actualizaCPrimario(String CPrim) {
        registro.setCombustiblePrimario(CPrim);
    }
    /**
     * Metodo actualizaCapTP, establece la capacidad del tanque de Combustible Primario del vehiculo
     *
     * @param capacidad Objeto double
     */
    @Override
    public void actualizaCapTP(double capacidad) {
        registro.setCapTankPrimario(capacidad);
    }
    /**
     * Metodo actualizaCSecundario, establece el Combustible Secundario del vehiculo
     *
     * @param CSec Objeto String
     */
    @Override
    public void actualizaCSecundario(String CSec) {
        registro.setCombustibleSecundario(CSec);
    }
    /**
     * Metodo actualizaCSecundario, establece la capacidad del tanque de Combustible Secundario del vehiculo
     *
     * @param capacidad Objeto double
     */
    @Override
    public void actualizaCapTS(double capacidad) {
        registro.setCapTankSecundario(capacidad);
    }
    /**
     * Metodo actualizaFoto, establece la foto del vehiculo
     *
     * @param bitmap Objeto Bitmap
     */
    @Override
    public void actualizaFoto(Bitmap bitmap) {
        registro.setFoto(bitmap);
    }


    /**
     * Metodo getRegistro devuelve una copia de la informacion recogida en el registro
     *
     * @return  Objeto DTOVehiculo
     */
    @Override
    public DTOVehiculo getRegistro() {
        return registro;
    }

    /**
     * Metodo isRegistroCompletado, determina si la operacion de registro se ha completado correctamente
     *
     * @return  true si exito, false en caso contrario
     */
    @Override
    public boolean isRegistroCompletado() {
        return registroCompletado;
    }

    /**
     * Metodo deteccionCampoInvalido, indica a la vista que existen campos rellenados incorrectamnete en
     * el registro.
     *
     * @param cmp   Un Objeto String
     */
    @Override
    public void deteccionCampoInvalido(String cmp) {
        presentador.rellenarCampoIncorrectamente(cmp);
    }

    /**
     * Metodo getMsgError, devuelve el mensaje del error a representar en la vista.
     *
     * @return  Un Objeto String con el cuerpo del mensaje
     */
    @Override
    public String getMensajeError() {
        String msj=this.mensajeError;
        this.mensajeError=null;

        return msj;
    }

    /**
     * Metodo setRegistroCompletado, informa a la vista de que el registro se ha completado.
     *
     * @param registroCompletado    true o false segun se haya completado exitosamente o no
     * @return true en caso de exito, false en caso contrario
     */
    @Override
    public boolean setRegistroCompletado(boolean registroCompletado) {
        this.registroCompletado = registroCompletado;
        return true;
    }

    /**
     * Indica a la vista que debe mostrar un mensaje de error.
     *
     * @param mensaje Un String con el cuerpo del mensaje
     */
    @Override
    public boolean setMensajeError(String mensaje) {
        this.mensajeError=mensaje;
        return true;
    }


}
