package com.example.juanalonsoastruga.cargascontrol.Modelo.CUConsultarEstadisticaCombustible;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTODatosDerivadosCombustible;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTOVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Datos.FachadaDatos;
import com.example.juanalonsoastruga.cargascontrol.Datos.InterfaceFachadaDatos;

/**
 * Clase ModeloConsultarEstadisticaCombustible, representa al modelo del CU Consultar Estadisticas de Consumo de Combustible.
 *
 */
public class ModeloConsultarEstadisticaCombustible implements InterfaceMConsultarEstadisticaCombustible {

    private InterfaceFachadaDatos datos;

    /**
     * Constructor de clase.
     *
     */
    public ModeloConsultarEstadisticaCombustible(){
        setRepo();
    }

    /**
     * Metodo setRepo, establece un repositorio de datos asociado.
     *
     */
    public void setRepo(){
        datos= FachadaDatos.instancia();
    }

    /**
     * Metodo setRepo, asocia el repositorio de datos proporcionado al modelo.
     *
     * @throws IllegalArgumentException
     * @param datos InterfaceFachadaDatos
     */
    public void setRepo(InterfaceFachadaDatos datos){
        if(datos==null){
            throw new IllegalArgumentException();
        }else{
            this.datos=datos;
        }
    }

    /**
     * Metodo getDatos, devuelve la instancia asociada del repositorio de datos.
     *
     * @returnInterfaceFachadaDatos
     */
    @Override
    public InterfaceFachadaDatos getDatos() {
        return datos;
    }

    /**
     * Metodo getVehiculo, devuelve el vehiculo cuya matricula se especifica.
     *
     * @param matricula String
     * @return DTOVehiculo
     */
    @Override
    public DTOVehiculo getVehiculo(String matricula) {
        return datos.getVehiculo(matricula);
    }

    /**
     * Metodo getDatosDerivadosCombustible, devuelve el conjunto de datos derivados asociados al consumo
     * de combustible.
     *
     * @param matricula La matricula del vehiculo cuyos datos se consultan
     * @param year  El anio a consultar sus registros
     * @param month El mes a consultar sus registros
     * @param combustible  El combustible del vehiculo sobre el que se consulta
     * @return DTODatosDerivadosCombustible
     */
    @Override
    public DTODatosDerivadosCombustible getDatosDerivadosCombustible(String matricula,int year, int month, String combustible) {
        return datos.getDDerivadosCombustible(matricula,year,month,combustible);
    }
}
