package com.example.juanalonsoastruga.cargascontrol.Modelo.CUConsultarEstadisticaCostesCombustible;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTODatosDerivadosCostesCombustible;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTOVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Datos.FachadaDatos;
import com.example.juanalonsoastruga.cargascontrol.Datos.InterfaceFachadaDatos;

/**
 * Clase ModeloConsultarEstadisticaCosteCombustible, representa al modelo del CU Consultar Estadistica Coste Combustible
 *
 */
public class ModeloConsultarEstadisticaCosteCombustible implements InterfaceMConsultarEstadisticaCosteCombustible {

    private InterfaceFachadaDatos datos;

    /**
     * Constructor de clase.
     *
     */
    public ModeloConsultarEstadisticaCosteCombustible(){
        setRepo();
    }

    /**
     * Metodo setRepo, establece un repositorio de datos asociado.
     *
     */
    public void setRepo(){
        datos= FachadaDatos.instancia();
    }
    /**
     * Metodo setRepo, asocia el repositorio de datos proporcionado al modelo.
     *
     * @throws IllegalArgumentException
     * @param datos InterfaceFachadaDatos
     */
    public void setRepo(InterfaceFachadaDatos datos){
        if(datos==null){
            throw new IllegalArgumentException();
        }else{
            this.datos=datos;
        }
    }

    /**
     * Metodo getDatos, devuelve la instancia asociada del repositorio de datos.
     *
     * @returnInterfaceFachadaDatos
     */
    @Override
    public InterfaceFachadaDatos getDatos() {
        return datos;
    }

    /**
     * Metodo getVehiculo, devuelve el vehiculo cuya matricula se especifica.
     *
     * @param matricula String
     * @return DTOVehiculo
     */
    @Override
    public DTOVehiculo getVehiculo(String matricula) {
        return datos.getVehiculo(matricula);
    }

    @Override
    public DTODatosDerivadosCostesCombustible getDatosDerivadosCostesCombustible(String matricula, int year, int month, String combustible) {
        return datos.getDDerivadosCosteCombustible(matricula,year,month,combustible);
    }
}
