package com.example.juanalonsoastruga.cargascontrol.Modelo.CUConsultarEstadisticaVehiculo;


import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTODatosDerivadosVehiculoTotales;
import com.example.juanalonsoastruga.cargascontrol.Datos.FachadaDatos;
import com.example.juanalonsoastruga.cargascontrol.Datos.InterfaceFachadaDatos;

import java.util.Calendar;
import java.util.List;

/**
 * Clase ModeloConsultarEstadisticaVehiculo, representa al modelo en el CUConsultar Estadisticas Generales del Vehiculo.
 *
 */
public class ModeloConsultarEstadisticaVehiculo implements InterfaceMConsultarEstadisticaVehiculo {

    private InterfaceFachadaDatos datos;

    /**
     * Constructor de clase.
     *
     */
    public ModeloConsultarEstadisticaVehiculo(){
        setRepo();
    }

    /**
     * Metodo setRepo, establece un repositorio de datos para este modelo.
     *
     */
    public void setRepo(){
        datos= FachadaDatos.instancia();
    }
    /**
     * Metodo setRepo, establece un repositorio de datos proporcionado para este modelo.
     *
     * @param datos Un Objeto InterfaceFachadaDatos
     * @throws IllegalArgumentException si el argumento es null
     */
    public void setRepo(InterfaceFachadaDatos datos){
        if(datos==null){
            throw new IllegalArgumentException();
        }else{
            this.datos=datos;
        }
    }

    /**
     * Metodo obtenDatosEstadisticosVehiculo, obtiene el conjunto de datos estadisticos generales asociados a
     * el vehiculo cuya matricula se especifica.
     *
     * @param matricula String con la matricula
     * @return DTODatosDerivadosVehiculoTotales
     */
    @Override
    public DTODatosDerivadosVehiculoTotales obtenDatosEstadisticosVehiculo(String matricula) {
        Calendar cal = Calendar.getInstance();
        return datos.getDDerivadosVehiculo(matricula,cal.get(Calendar.YEAR),cal.get(Calendar.MONTH));
    }
}
