package com.example.juanalonsoastruga.cargascontrol.Modelo.CUConsultarInformacionGasolinera;


import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTOGasolinera;
import com.example.juanalonsoastruga.cargascontrol.Datos.FachadaDatos;
import com.example.juanalonsoastruga.cargascontrol.Datos.InterfaceFachadaDatos;

/**
 *Clase ModeloConsultarInformacionGasolinera, representa al modelo del CU Consultar Informacion Gasolinera.
 *
 *
 */
public class ModeloConsultarInformacionGasolinera implements InterfaceMConsultarInformacionGasolinera {

    private InterfaceFachadaDatos datos;


    /**
     *Constructor de clase.
     *
     */
    public ModeloConsultarInformacionGasolinera(){
        setRepo();
    }
    /**
     * Metodo setRepo, establee el repositorio de datos del modelo
     *
     * @param d El objeto que representa al repositorio de datos
     * @throws IllegalArgumentException si el argumento es null
     */
    @Override
    public void setRepo(InterfaceFachadaDatos d){
        if(d==null){
           throw new IllegalArgumentException();
        }else{
            this.datos=d;
        }
    }
    /**
     * Metodo setRepo, establee un repositorio de datos del modelo
     *
     */
    @Override
    public void setRepo(){
        this.datos= FachadaDatos.instancia();
    }

    /**
     * Metodo getRepo, devuelve un objeto representacion del repositorio de datos
     *
     * @return El objeto representacion del repositorio de datos
     */
    @Override
    public InterfaceFachadaDatos getRepo(){
        return datos;
    }

    /**
     * Metodo getDatosGasolinera, devuelve el conjunto de datos asociados a una gasolinera cuya
     * direccion se especifica.
     *
     * @param direccion String con la direccion de la gasolinera
     * @param latitud double latitud
     * @param longitud double longitud
     * @return Un objeto DTOGasolinera representacion de los datos asociados a la gasolinera
     */
    @Override
    public DTOGasolinera getDatosGasolinera(double latitud,double longitud,String direccion){
        return datos.getDTOGasolinera(latitud,longitud,direccion);
    }
}
