package com.example.juanalonsoastruga.cargascontrol.Modelo.CUConsultarInformacionRecarga;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTORecarga;
import com.example.juanalonsoastruga.cargascontrol.Datos.FachadaDatos;
import com.example.juanalonsoastruga.cargascontrol.Datos.InterfaceFachadaDatos;

import java.util.Date;

/**
 * Clase ModeloConsultarInformacionRecarga , representa al Modelo en el CU Consultar Informacion de
 * recarga.
 *
 */
public class ModeloConsultarInformacionRecarga implements InterfaceMConsultarInformacionRecarga {


    private InterfaceFachadaDatos datos;

    /**
     * Constructor de clase
     *
     */
    public ModeloConsultarInformacionRecarga(){

        setRepo();
    }

    /**
     * Metodo setRepo, porporciona al modelo una instancia del repositorio de datos.
     *
     */
    @Override
    public void setRepo(){
        datos= FachadaDatos.instancia();
    }
    /**
     * Metodo setRepo, asocia al modelo una instancia del repositorio de datos.
     *
     * @param d Un Objeto InterfaceFachadaDatos representando la instancia del reposiotrio de datos
     */
    @Override
    public void setRepo(InterfaceFachadaDatos d){
        if(d==null){
            throw new IllegalArgumentException();
        }else{
            this.datos=d;
        }
    }

    /**
     * Metodo getRepo, devuelve el repositorio de datos asociado al modelo.
     *
     * @return  Un Objeto InterfaceFachadaDatos
     */
    @Override
    public InterfaceFachadaDatos getRepo(){

            return datos;
    }
    /**
     * Metodo registraIDRecarga, despierta al presentador como parte del CU Registrar Ultima Recarga Consultada,
     * indica al modelo que registre el id de la ultima recarga consultada.
     *
     * @param matricula String con la matricula del correspondiente vehiculo al que se le realizo la recarga
     * @param fecha Long con la fecha
     * @param combustible String con el combustible del vehiculo sobre el que se hizo la recarga
     */
    @Override
    public void registraIDRecarga(String matricula, Long fecha, String combustible) {
        datos.registraIDUltimaRecarga(fecha,matricula,combustible);
    }

    /**
     * Metodo getRecarga, devuelve la informacion de la recarga que se correcponda a esa fecha
     * y hora, matricula y combustible.
     *
     * @param matricula Un String con la matricula
     * @param fecha Un Objeto Date con la fecha y hora
     * @param combustible   Un String con el combustible
     * @throws IllegalArgumentException si alguno de los argumentos es null
     * @return  Un objeto DTORecarga
     */
    @Override
    public DTORecarga getRecarga(String matricula, Date fecha, String combustible) {
        if(matricula==null||fecha==null||combustible==null){
            throw new IllegalArgumentException();
        }else {
            return datos.getRecarga(matricula, fecha, combustible);
        }
    }
}
