package com.example.juanalonsoastruga.cargascontrol.Modelo.CUConsultarInformacionVehiculo;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTOVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Datos.FachadaDatos;
import com.example.juanalonsoastruga.cargascontrol.Datos.InterfaceFachadaDatos;

/**
 * Clase ModeloConsultarInformacionVehiculo, modelo del CU Consultar Informacion de Vehiculo
 *
 */
public class ModeloConsultarInformacionVehiculo implements InterfaceMConsultarInformacion {
    private InterfaceFachadaDatos datos;
    public ModeloConsultarInformacionVehiculo(){
        setRepo();
    }
    /**
     * Metodo setRepo, establee el repositorio de datos del modelo
     *
     * @param d El objeto que representa al repositorio de datos
     */
    @Override
    public void setRepo(InterfaceFachadaDatos d){
        if(d==null){
            this.datos= FachadaDatos.instancia();
        }else{
            this.datos=d;
        }
    }
    /**
     * Metodo setRepo, establee un repositorio de datos del modelo
     *
     */
    @Override
    public void setRepo(){
        this.datos= FachadaDatos.instancia();
    }
    /**
     * Metodo getRepo, devuelve un objeto representacion del repositorio de datos
     *
     * @return El objeto representacion del repositorio de datos
     */
    @Override
    public InterfaceFachadaDatos getRepo(){
        return datos;
    }

    /**
     * Metodo registrarIDVehiculoConsultado, como parte del CU Registrar Ultimo Vehiculo Consultado,
     * se pide al modelo que registre el identificador del vehiculo consultado.
     *
     * @param matricula Un String con la matricula del vehiculo
     */
    @Override
    public void registrarIDVehiculoConsultado(String matricula) {
        datos.registraIDUltimoVehiculo(matricula);
    }

    /**
     * Metodo consultarVehiculo, consulta la informacion del vehiculo cuya matricula se proporciona
     *
     * @param matricula Un String con la matricula del vehiculo a consultar
     * @return  Un Objeto DTOVehiculo
     */
    @Override
    public DTOVehiculo consultarVehiculo(String matricula) {
        return datos.getVehiculo(matricula);
    }
}
