package com.example.juanalonsoastruga.cargascontrol.Modelo.CUConsultarUltimoVehiculo;

import com.example.juanalonsoastruga.cargascontrol.Datos.FachadaDatos;
import com.example.juanalonsoastruga.cargascontrol.Datos.InterfaceFachadaDatos;

/**
 * Clase ModeloConsultarUltimoVehiculo, representa al modelo del CU Consultar Ultimo Vehiculo.
 *
 */
public class ModeloConsultarUltimoVehiculo implements InterfaceMConsultarUltimoVehiculo {

    private InterfaceFachadaDatos datos;

    /**
     * Constructor de clase.
     *
     */
    public ModeloConsultarUltimoVehiculo(){
        setRepo();
    }

    /**
     * Metodo getRepo, devuelve el repositorio de datos asociado.
     *
     * @return InterfaceFachadaDatos
     */
    @Override
    public InterfaceFachadaDatos getRepo(){
        return datos;
    }

    /**
     * Metodo existeVehiculo, responde a la exitencia o no del vehiculo especificado.
     *
     * @param idVehiculo    String con el identificador del vehiculo cuya existencia se contempla
     * @return  true si exite, false en caso contrario
     */
    @Override
    public boolean existeVehiculo(String idVehiculo) {
        return datos.isVehiculoRegistrado(idVehiculo) ;
    }

    /**
     * Metodo borrarFichero, borra el fichero del vehiculo.
     *
     */
    @Override
    public void borrarFichero() {
        datos.borrarFicheroVehiculo();
    }

    /**
     * Metodo setRepo, establee el repositorio de datos del modelo.
     *
     * @param d El objeto que representa al repositorio de datos
     */
    @Override
    public void setRepo(InterfaceFachadaDatos d){
        if(d==null)
            datos= FachadaDatos.instancia();
        else
            datos=d;
    }

    /**
     * Metodo setRepo, establece un repositorio de datos del modelo.
     *
     */
    @Override
    public void setRepo(){

        datos= FachadaDatos.instancia();

    }

    /**
     * Metodo recuperaIdUltimoVehiculoConsultado.
     *
     * @return  String con el id del ultimo vehiculo consultado
     */
    @Override
    public String recuperaIdUltimoVehiculoConsultado() {
        return datos.getIdUltimoVehiculo();
    }
}
