package com.example.juanalonsoastruga.cargascontrol.Modelo.CUGraficarConsumoCombustibleXKm;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTORecarga;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTOVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Datos.FachadaDatos;
import com.example.juanalonsoastruga.cargascontrol.Datos.InterfaceFachadaDatos;

import java.util.Date;
import java.util.List;
/**
 * Clase ModeloGraficarConsumoCombustible, representa al Modelo del Caso de Uso Graficar Consumo de Combustible.
 *
 */
public class ModeloGraficarConsumoCombustibleKm implements InterfaceMGraficarConsumoCombustibleKm {

    private InterfaceFachadaDatos datos;
    /**
     * Constructor de Clase
     *
     */
    public ModeloGraficarConsumoCombustibleKm(){
        setRepo();
    }
    /**
     * Metodo setRepo, establece un repositorio de datos para este modelo.
     *
     */
    public void setRepo(){
        this.datos= FachadaDatos.instancia();
    }
    /**
     * Metodo setRepo, establece un repositorio de datos proporcionado para este modelo.
     *
     * @param datos Un Objeto InterfaceFachadaDatos
     * @throws IllegalArgumentException si el argumento es null
     */
    public void setRepo(InterfaceFachadaDatos datos){
        if(datos==null){
            throw new IllegalArgumentException();
        }else{
            this.datos=datos;
        }
    }
    /**
     * Devuelve la instancia del repositorio de datos asociado a este modelo.
     *
     * @return  Un Objeto InterfaceFachadaDatos
     */
    public InterfaceFachadaDatos getRepo(){
        return this.datos;
    }
    /**
     * Metodo getVehiculo, devuelve la informacion del vehiculo cuya matricula se ha especificado.
     *
     * @param matricula Un String con la matricula del vehiculo
     * @return  Un Objeto DTOVehiculo
     */
    @Override
    public DTOVehiculo getVehiculo(String matricula) {
        return datos.getVehiculo(matricula);
    }
    /**
     * Metodo getRecargasMensualesCombustible, devuelve un listado de recargas mensuales realizadas entre las fechas proporcionadas.
     *
     * @param matricula Un Objeto String con la matricula del vehiculo a los que corresponden esas recargas
     * @param combustible   Un Objeto String con el combustible recargado en el conjutno de recargas
     * @param primerDiaMes  Un Objeto Date con la fecha del primer dia del mes a consultar
     * @param ultimoDiaMes  Un Objeto Date con la fecha del ultimo dia del mes a consultar
     * @return  Un Objeto List<DTORecarga>
     */
    @Override
    public List<DTORecarga> getRecargasMensualesCombustible(String matricula, String combustible, Date primerDiaMes,Date ultimoDiaMes) {
        List<DTORecarga> recargas= datos.getRecargasEnPeriodoPorCombustibleyVehiculo(matricula,combustible,primerDiaMes,ultimoDiaMes);
        return recargas;
    }
    /**
     * Metodo getRecargasAnualesCombustible, devuelve un listado de recargas anuales realizadas entre las fechas proporcionadas.
     *
     * @param matricula Un Objeto String con la matricula del vehiculo a los que corresponden esas recargas
     * @param combustible   Un Objeto String con el combustible recargado en el conjutno de recargas
     * @param dateIni  Un Objeto Date con la fecha del primer dia del anio a consultar
     * @param dateFin  Un Objeto Date con la fecha del ultimo dia del anio a consultar
     * @return  Un Objeto List<DTORecarga>
     */
    @Override
    public List<DTORecarga> getRecargasAnualesCombustible(String matricula, String combustible, Date dateIni, Date dateFin) {
        List<DTORecarga> recargas= datos.getRecargasEnPeriodoPorCombustibleyVehiculo(matricula,combustible,dateIni,dateFin);
        return recargas;
    }

    /**
     * Metodo getRecargaMasLejanaAntesDe, proporciona la ultima recarga registrada para el vehiculo y combustible indicados antes de la fecha
     * requerida.
     *
     * @param matricula Un Objeto String con la matricula del vehiculo al que corresponde esta recarga
     * @param combustible  Un Objeto String con el combustible usado en la recarga
     * @param dateIni   Un Objeto Date con la fecha a partir de la que buscar la recarga
     * @return  Un Objeto DTORecarga
     */
    @Override
    public DTORecarga getRecargaMasLejanaAntesDe(String matricula, String combustible, Date dateIni) {
        return datos.getLastRecargaBefore(matricula,dateIni,combustible);
    }
}
