package com.example.juanalonsoastruga.cargascontrol.Modelo.CUModificarInformacionVehiculo;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTOCombustible;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTOVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Datos.FachadaDatos;
import com.example.juanalonsoastruga.cargascontrol.Datos.InterfaceFachadaDatos;

import java.util.List;

/**
 * Clase ModeloModificarInformacionVehiculo, se comporta como el modelo del
 * CU.
 */
public class ModeloModificarInformacionVehiculo implements InterfaceMModificarInformacionVehiculo {
    private InterfaceFachadaDatos datos;

    /**
     *  Constructor de clase
     *
     *
     */
    public ModeloModificarInformacionVehiculo(){
        setRepo();
    }

    /**
     * Metodo setRepo, establece el repositorio de datos para el modelo
     *
     * @param d Objeto InterfaceFachadaDatos
     */
    @Override
    public void setRepo(InterfaceFachadaDatos d){
        if(d==null){
            this.datos=FachadaDatos.instancia();
        }else{
            this.datos=d;
        }
    }

    /**
     * Metodo setRepo, establece un repositorio de datos para el modelo
     *
     */
    @Override
    public void setRepo(){

            this.datos=FachadaDatos.instancia();

    }

    /**
     * Devuelve un repositorio de datos asociado al modelo.
     *
     * @return  Objeto InterfaceFachadaDatos
     */
    public InterfaceFachadaDatos getRepo(){
        return datos;
    }


    /**
     * Metodo getCombustibles, devuelve un listado de combustibles disponibles.
     *
     * @return  Objeto List<DTOCombustible>
     */
    @Override
    public List<DTOCombustible> getCombustibles() {
        return datos.getCombustibles();
    }

    /**
     * Metodo getVehiculo, devuelve la informacion del vehiculo de la matricula especificada.
     *
     * @param matricula String representando la matricula del vehiculo
     * @return  Un objeto DTOVehiculo
     */
    @Override
    public DTOVehiculo getVehiculo(String matricula) {
        return datos.getVehiculo(matricula);
    }

    /**
     * Metodo actualizaVehiculo, modifica la inforacion del vehiculo especificado.
     *
     * @param vehiculo  Objeto DTOVehiculo especificado
     * @return  true si exito, false en caso contrario
     */
    @Override
    public boolean actualizaVehiculo(DTOVehiculo vehiculo) {
        datos.actualizaVehiculo(vehiculo);
        return true;
    }
}
