package com.example.juanalonsoastruga.cargascontrol.Modelo.CUMostrarListadoGasolineras;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTOGasolinera;
import com.example.juanalonsoastruga.cargascontrol.Datos.FachadaDatos;
import com.example.juanalonsoastruga.cargascontrol.Datos.InterfaceFachadaDatos;

import java.util.Hashtable;
import java.util.List;

/**
 * Clase ModeloMostrarListadoGasolineras, representa al modelo en el caso de uso Mostrar Listado de Gasolineras Cercanas.
 *
 */
public class ModeloMostrarListadoGasolineras implements InterfaceMMostrarListadoGasolineras {

    private InterfaceFachadaDatos datos;

    /**
     * Constructor de clase.
     *
     */
    public ModeloMostrarListadoGasolineras(){
        setRepo();
    }
    /**
     * Metodo setRepo, establee un repositorio de datos del modelo
     *
     */
    @Override
    public void setRepo(){
        datos= FachadaDatos.instancia();
    }
    /**
     * Metodo setRepo, establece el repositorio de datos del modelo.
     *
     * @param datos El objeto que representa al repositorio de datos
     */
    @Override
    public void setRepo(InterfaceFachadaDatos datos){
        if(datos==null){
            throw new IllegalArgumentException();
        }else{
            this.datos=datos;
        }
    }
    /**
     * Metodo getRepo, devuelve el repositorio de datos asociado.
     *
     * @return InterfaceFachadaDatos
     */
    @Override
    public InterfaceFachadaDatos getRepo(){

        return datos;

    }

    /**
     * Metodo getLocalizacion, devuelve la estructura de datos rellenada con la latitud y longitud geograficas actuales.
     *
     * @param localizacion Hashtable
     * @return Hashtable (latitud, longitud)
     */
    @Override
    public Hashtable getLocalizacion(Hashtable localizacion) {
        return datos.getLocalizacionActual(localizacion);
    }


    /**
     * Metodo descargaListadoGasolineras, devuelve un listado de gasolineras cercanas a la posicion geografica actual.
     *
     * @param coordenadas Hashtable con las coordenadas
     * @return  List<DTOGasolinera> con el listado de gasolineras
     */
    @Override
    public List<DTOGasolinera> descargaListadoGasolineras(Hashtable coordenadas) {
        return datos.getEstacionesRecargaNacionales(coordenadas);
    }
}
