package com.example.juanalonsoastruga.cargascontrol.Modelo.CURegistrarRecarga;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTODatosDerivadosCombustible;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTODatosDerivadosCostesCombustible;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTODatosDerivadosVehiculoTotales;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTORecarga;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTOVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Datos.FachadaDatos;
import com.example.juanalonsoastruga.cargascontrol.Datos.InterfaceFachadaDatos;

import java.util.Date;

/**
 * Clase ModeloRegistrarRecarga, representa el Modelo del CU RegistrarRecarga
 *
 */
public class ModeloRegistrarRecarga implements InterfaceMRegistrarRecarga {

    private InterfaceFachadaDatos datos;

    /**
     * Constructor de clase
     *
     */
    public ModeloRegistrarRecarga(){
        setRepo();
    }

    /**
     * Metodo setrepo, asigna una instancia del repositorio de datos.
     *
     */
    @Override
    public void setRepo(){
        datos= FachadaDatos.instancia();
    }

    /**
     * Metodo setRepo, dada una instancia del repositorio de datos, esta queda asignada al modelo.
     *
     * @param datos Un Objeto InterfaceFachadaDatos
     * @throws IllegalArgumentException di el objeto es null
     */
    @Override
    public void setRepo(InterfaceFachadaDatos datos){
        if(datos==null){
            throw new IllegalArgumentException();
        }else{
            this.datos=datos;
        }
    }

    /**
     * Metodo getRepo, devuelve una instancia que haya sido asociada al modelo.
     *
     * @return  Un objeto InterfaceFachadaDatos
     * @throws IllegalStateException si el metodo es llamado sin haber asociado una instancia de repositorio
     */
    @Override
    public InterfaceFachadaDatos getDatos() {
        if(this.datos==null){
            throw new IllegalStateException();
        }else {
            return datos;
        }
    }

    /**
     * Metodo  getDDVehiculo, devuelve el conjunto de datos derivados asociados a un vehiculo para un mes y anio particular.
     *
     * @param matricula Un String con la matricula del vehiculo
     * @param anio  Un Integer con el anio sobre el que se consulta
     * @param mes   Un Integer con el mes sobre el que se consulta
     * @return  Un objeto DTODatosDerivadosVehiculoTotales
     */
    @Override
    public DTODatosDerivadosVehiculoTotales getDDVehiculo(String matricula, int anio, int mes) {
        return datos.getDDerivadosVehiculo(matricula,anio,mes);
    }

    /**
     * Metodo isPrimerRegistroDatosDerivadosVehiculo, cuestiona si el vehiculo ya posee algún registro de datos derivados de recargas.
     *
     * @param matricula  Un String con la matricula del vehiculo
     * @return true en caso de que no lo posea, false en caso contrario
     */
    @Override
    public boolean isPrimerRegistroDatosDerivadosVehiculo(String matricula) {
        DTODatosDerivadosVehiculoTotales ultimo=datos.getUltimoRegistroDDerivadosVehiculo(matricula);
        if(ultimo==null){
            return true;
        }else{
            return false;
        }
    }

    /**
     * Metodo addDatosDerivadosVehiculo, registra un nuevo conjunto de datos derivados de recargas asociados al vehiculo.
     *
     * @param ddv   Objeto DTODatosDerivadosVehiculoTotales
     */
    @Override
    public void addDatosDerivadosVehiculo(DTODatosDerivadosVehiculoTotales ddv) {
        datos.registraDatosDerivadosVehiculo(ddv);
    }

    /**
     * Metodo getLastRegistroDatosDerivadosVehiculo, devuelve el ultimo registro del conjunto de datos de recargas asociadas al vehiculo.
     *
     * @param matricula Un String con la matricula del vehiculo
     * @return  Un objeto DTODatosDerivadosVehiculoTotales
     */
    @Override
    public DTODatosDerivadosVehiculoTotales getLastRegistroDatosDerivadosVehiculo(String matricula) {
        return   datos.getUltimoRegistroDDerivadosVehiculo(matricula);
    }

    /**
     * Metodo actualizaDatosDerivadosVehiculo, actualiza el conjunto de datos de recargas asociadas al vehiculo.
     *
     * @param ddv Un objeto DTODatosDerivadosVehiculoTotales
     */
    @Override
    public void actualizaDatosDerivadosVehiculo(DTODatosDerivadosVehiculoTotales ddv) {
        datos.actualizaDatosDerivadosVehiculo(ddv);
    }

    @Override
    public DTODatosDerivadosCombustible getDDVCombustible(String matricula, int year, int month, String combustible) {
        return datos.getDDerivadosCombustible(matricula,year,month,combustible);
    }

    @Override
    public boolean isPrimerRegistroDatosDerivadosCombustible(String matricula, String combustible) {
        DTODatosDerivadosCombustible ultimo=datos.getUltimoRegistroDDerivadosCombustible(matricula,combustible);
        if(ultimo==null){
            return true;
        }else{
            return false;
        }
    }

    @Override
    public boolean isPrimerRegistroDatosDerivadosCosteCombustible(String matricula, String combustible) {
        DTODatosDerivadosCostesCombustible ultimo=datos.getUltimoRegistroDDerivadosCosteCombustible(matricula,combustible);
        if(ultimo==null){
            return true;
        }else{
            return false;
        }
    }



    @Override
    public void addDatosDerivadosCombustible(DTODatosDerivadosCombustible ddv) {
        datos.registraDatosDerivadosCombustible(ddv);
    }

    @Override
    public DTODatosDerivadosCombustible getLastRegistroDatosDerivadosCombustible(String matricula,String combustible) {
        DTODatosDerivadosCombustible ultimo=datos.getUltimoRegistroDDerivadosCombustible(matricula,combustible);
        return ultimo;
    }

    @Override
    public void actualizaDatosDerivadosCombustible(DTODatosDerivadosCombustible ddv) {
        datos.actualizaDatosDerivadosCombustible(ddv);
    }

    @Override
    public DTODatosDerivadosCostesCombustible getDDVCostesCombustible(String matricula, int year, int month, String combustible) {
        DTODatosDerivadosCostesCombustible dto=datos.getDDerivadosCosteCombustible(matricula, year, month, combustible);
        return dto;
    }

    @Override
    public DTODatosDerivadosCostesCombustible getLastRegistroDatosDerivadosCostesCombustible(String matricula, String combustible) {
        DTODatosDerivadosCostesCombustible dto=datos.getUltimoRegistroDDerivadosCosteCombustible(matricula, combustible);
        return dto;
    }

    @Override
    public void actualizaDatosDerivadosCosteCombustible(DTODatosDerivadosCostesCombustible ddv) {
        datos.actualizaDatosDerivadosCosteCombustible(ddv);
    }

    @Override
    public void addDatosDerivadosCosteCombustible(DTODatosDerivadosCostesCombustible ddv) {
        datos.registraDatosDerivadosCosteCombustible(ddv);
    }

    /**
     * Metodo getRecargaMasLejanaAntesDe, devuelve la ultima recarga realizada antes de la fecha indicada.
     *
     * @param matricula Un String con la matricula del vehiculo cuya recarga se consulta
     * @param dateIni   Un Objeto Date con la fecha a partir de la que se busca la ultima recarga
     * @param combustible   Un String combustible con el nombre del combustible de la recarga
     * @return  Un Objeto DTORecarga con la recarga encontrada
     */
    @Override
    public DTORecarga getUltimaRecargaAntesDe(String matricula, Date dateIni,String combustible) {

        return datos.getLastRecargaBefore(matricula,dateIni,combustible);
    }
    /**
     * Metodo getUltimaRecargaRegistradaMismoCombustible, devuelve la ultima recarga realizada para ese combustible.
     *
     * @param matricula Un String con la matricula del vehiculo cuya recarga se consulta
     * @param combustible   Un String combustible con el nombre del combustible de la recarga
     * @return  Un Objeto DTORecarga con la recarga encontrada
     */
    @Override
    public DTORecarga getUltimaRecargaRegistradaMismoCombustible(String matricula,String combustible){
        return datos.getLastRecargaPorCombustible(matricula,combustible);
    }

    /**
     * Metodo getUltimaRecargaRegistrada, devuelve la ultima recarga realizada .
     *
     * @param matricula Un String con la matricula del vehiculo cuya recarga se consulta
     * @return  Un Objeto DTORecarga con la recarga encontrada
     */
    @Override
    public DTORecarga getUltimaRecargaRegistrada(String matricula,Date date){
        return datos.getLastRecargaBefore(matricula,date);
    }

    /**
     * Metodo registraRecarga, registra los datos proporcionados como una nueva recarga.
     *
     * @param recarga   Un Objeto DTORecarga con los datos de la recarga
     */
    @Override
    public void registraRecarga(DTORecarga recarga) {
        if(recarga==null){
            throw  new IllegalArgumentException();
        }else{
            datos.setRecarga(recarga);
        }
    }

    /**
     * Metodo getVehiculo, proporciona los datos del vehiculo cuya matricula se indica.
     *
     * @param matricula Un String con la matricula del vehiculo a consultar
     * @return  Un Objeto DTOVehiculo
     */
    @Override
    public DTOVehiculo getVehiculo(String matricula) {
            return datos.getVehiculo(matricula);
    }
}
