package com.example.juanalonsoastruga.cargascontrol.Modelo.CURegistrarVehiculo;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTOCombustible;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTOVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Datos.FachadaDatos;
import com.example.juanalonsoastruga.cargascontrol.Datos.InterfaceFachadaDatos;

import java.util.List;

/**
 * Clase ModeloRegistroVehiculo, actua de modelo en el CU Registrar Vehiculo
 *
 */
public class ModeloRegistroVehiculo implements InterfaceMRegistrarVehiculo {
    private InterfaceFachadaDatos datos;

    /**
     * Constructor de clase.
     *
     */
    public ModeloRegistroVehiculo(){
        setRepo();
    }

    /**
     * Metodo setRepo, establece el repositorio de datos para el modelo
     *
     * @param d Objeto InterfaceFachadaDatos
     */
    @Override
    public void setRepo(InterfaceFachadaDatos d){
        if(d==null){
            this.datos=FachadaDatos.instancia();
        }else{
            this.datos=d;
        }
    }
    /**
     * Metodo setRepo, establece un repositorio de datos para el modelo
     *
     */
    @Override
    public void setRepo(){

        this.datos=FachadaDatos.instancia();

    }

    /**
     * Devuelve un repositorio de datos asociado al modelo.
     *
     * @return  Objeto InterfaceFachadaDatos
     */
    public InterfaceFachadaDatos getRepo(){
        return datos;
    }

    /**
     * Metodo getCombustibles, devuelve un listado de los combustibles disponibles.
     *
     * @return  Objeto List<DTOCombustible>
     */
    @Override
    public List<DTOCombustible> getCombustibles() {
        return datos.getCombustibles();
    }

    /**
     * Metodo  registraVehiculo, registra un nuevo vehiculo.
     *
     * @param vehiculo  DTOVehiculo con los datos del vehiculo
     * @return  true si exito, false en caso contrario
     */
    @Override
    public boolean registraVehiculo(DTOVehiculo vehiculo) {
        datos.setVehiculo(vehiculo);
        return true;
    }

    /**
     * isVehiculoRegistrado, verifica si hay vehiculos registrados con la matricula del vehiculo especificado.
     *
     * @param vehiculo  Objeto DTOVehiculo
     * @return  true si existen, false en caso contrario
     */
    @Override
    public boolean isVehiculoRegistrado(DTOVehiculo vehiculo) {
        return datos.isVehiculoRegistrado(vehiculo);
    }
}
