package com.example.juanalonsoastruga.cargascontrol.Controlador.CUCalcularCosteTrayecto;

import com.example.juanalonsoastruga.cargascontrol.Formateador.CUCalcularCosteTrayecto.FormateadorCalcularCosteTrayecto;
import com.example.juanalonsoastruga.cargascontrol.Formateador.CUCalcularCosteTrayecto.InterfaceFCalcularCosteTrayecto;
import com.example.juanalonsoastruga.cargascontrol.Modelo.CUCalcularCosteTrayecto.InterfaceMCalcularCosteTrayecto;
import com.example.juanalonsoastruga.cargascontrol.Modelo.CUCalcularCosteTrayecto.ModeloCalcularCosteTrayecto;

import org.easymock.IAnswer;
import org.easymock.Mock;
import org.junit.Before;
import org.junit.Test;

import static org.easymock.EasyMock.anyDouble;
import static org.easymock.EasyMock.createMock;
import static org.easymock.EasyMock.expect;
import static org.easymock.EasyMock.expectLastCall;
import static org.easymock.EasyMock.isA;
import static org.easymock.EasyMock.replay;
import static org.easymock.EasyMock.verify;
import static org.junit.Assert.*;

public class ControladorCalcularCosteTrayectoTest {

    private ControladorCalcularCosteTrayecto control;

    @Mock
    private InterfaceFCalcularCosteTrayecto formateador;
    @Mock
    private InterfaceMCalcularCosteTrayecto modelo;

    @Before
    public void setUp(){
        formateador=createMock(FormateadorCalcularCosteTrayecto.class);
        modelo=createMock(ModeloCalcularCosteTrayecto.class);
        control= new ControladorCalcularCosteTrayecto(formateador);
    }


    @Test(expected = IllegalArgumentException.class)
    public void setFormateadorNull() {
        control.setFormateador(null);
    }

    @Test
    public void setFormateadorNotNull() {
        control.setFormateador(formateador);
        assertEquals(formateador,control.getFormateador());
    }


    @Test(expected = IllegalArgumentException.class)
    public void setModeloNull() {
        control.setModelo(null);
    }

    @Test
    public void setModelo() {
        control.setModelo(modelo);
        assertEquals(modelo,control.getModelo());
    }

    @Test
    public void solicitarCalculoDistanciaNull() {
        formateador.setMensajeError(isA(String.class));
        expectLastCall().andAnswer(new IAnswer<Object>() {
            @Override
            public Object answer() throws Throwable {
                return null;
            }
        });

        replay(formateador);

        control.setFormateador(formateador);

        control.solicitarCalculo(null,"","");

        verify(formateador);
    }

    @Test
    public void solicitarCalculoPrecioNull() {
        formateador.setMensajeError(isA(String.class));
        expectLastCall().andAnswer(new IAnswer<Object>() {
            @Override
            public Object answer() throws Throwable {
                return null;
            }
        });

        replay(formateador);

        control.setFormateador(formateador);

        control.solicitarCalculo("",null,"");

        verify(formateador);
    }
    @Test
    public void solicitarCalculoConsumoNull() {
        formateador.setMensajeError(isA(String.class));
        expectLastCall().andAnswer(new IAnswer<Object>() {
            @Override
            public Object answer() throws Throwable {
                return null;
            }
        });

        replay(formateador);

        control.setFormateador(formateador);

        control.solicitarCalculo("","",null);

        verify(formateador);
    }

    @Test
    public void solicitarCalculo() {
        expect(modelo.calculaFactorPrecioKilometro(0.0,0.0)).andReturn(0.0);
        expect(modelo.calculaPrecioDistancia(0.0,0.0)).andReturn(0.0);
        formateador.setResultado(anyDouble());
        expectLastCall().andAnswer(new IAnswer<Object>() {
            @Override
            public Object answer() throws Throwable {
                return null;
            }
        });
        replay(formateador);
        replay(modelo);

        control.setFormateador(formateador);
        control.setModelo(modelo);

        control.solicitarCalculo("0","0","0");

        verify(formateador);
        verify(modelo);
    }
}