package com.example.juanalonsoastruga.cargascontrol.Controlador.CUConsultarEstadisticaCombustible;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.CodigoCombustibleVehiculo;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTODatosDerivadosCombustible;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTOVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Formateador.CUConsultarEstadisticaCombustible.FormateadorConsultarEstadisticaCombustible;
import com.example.juanalonsoastruga.cargascontrol.Modelo.CUConsultarEstadisticaCombustible.InterfaceMConsultarEstadisticaCombustible;
import com.example.juanalonsoastruga.cargascontrol.Modelo.CUConsultarEstadisticaCombustible.ModeloConsultarEstadisticaCombustible;
import com.example.juanalonsoastruga.cargascontrol.Formateador.CUConsultarEstadisticaCombustible.InterfaceFConsultarEstadisticaCombustible;

import org.easymock.EasyMock;
import org.easymock.IAnswer;
import org.easymock.Mock;
import org.junit.Before;
import org.junit.Test;

import java.util.Calendar;

import static org.easymock.EasyMock.createMock;
import static org.easymock.EasyMock.expect;
import static org.easymock.EasyMock.isA;
import static org.easymock.EasyMock.replay;
import static org.easymock.EasyMock.verify;

public class ControladorConsultarEstadisticaCombustibleTest {

    private ControladorConsultarEstadisticaCombustible controlador;
    @Mock
    private InterfaceFConsultarEstadisticaCombustible formateador;
    @Mock
    private InterfaceMConsultarEstadisticaCombustible modelo;
    @Mock
    private DTOVehiculo dtoVehiculo;
    @Mock
    private DTODatosDerivadosCombustible dtoDatosDerivados;
    @Before
    public void setUp(){
        formateador =createMock(FormateadorConsultarEstadisticaCombustible.class);
        controlador = new ControladorConsultarEstadisticaCombustible(formateador);
        modelo=createMock(ModeloConsultarEstadisticaCombustible.class);
        dtoVehiculo=createMock(DTOVehiculo.class);
        dtoDatosDerivados=createMock(DTODatosDerivadosCombustible.class);
    }

    @Test(expected=IllegalArgumentException.class)
    public void setVistaNull(){
        controlador.setFormateador(null);
    }
    @Test
    public void setVistaNotNull(){
        controlador.setFormateador(formateador);
    }
    @Test(expected=IllegalArgumentException.class)
    public void setModeloNull(){
        controlador.setModel(null);
    }
    @Test
    public void setModeloNotNull(){
        controlador.setModel(modelo);
    }

    @Test
    public void comienzoCUVehiculoNull(){
        expect(modelo.getVehiculo(isA(String.class))).andReturn(null);
        replay(modelo);
        controlador.setModel(modelo);
        controlador.comienzoCU("",0);
        verify(modelo);
    }
    @Test
    public void comienzoCUVehiculoCPrimario(){
        Calendar  cal =Calendar.getInstance();

        expect(modelo.getVehiculo(isA(String.class))).andReturn(dtoVehiculo);
        expect(dtoVehiculo.getCombustiblePrimario()).andReturn("");
        expect(modelo.getDatosDerivadosCombustible("",cal.get(Calendar.YEAR),cal.get(Calendar.MONTH),"")).andReturn(dtoDatosDerivados);
        expect(dtoDatosDerivados.getSumaCombustibleRecargadoPorAnio()).andReturn(1.0);
        expect(dtoDatosDerivados.getNumeroRecargasCombustiblePorAnio()).andReturn(1);
        expect(dtoDatosDerivados.getSumaCombustibleRecargadoPorMes()).andReturn(1.0);
        expect(dtoDatosDerivados.getNumeroRecargasCombustiblePorMes()).andReturn(1);
        expect(dtoDatosDerivados.getSumaCombustibleXKmAnual()).andReturn(1.0);
        expect(dtoDatosDerivados.getNumeroRecargasCombustiblePorAnio()).andReturn(1);
        expect(dtoDatosDerivados.getSumaCombustibleXKmMensual()).andReturn(1.0);
        expect(dtoDatosDerivados.getNumeroRecargasCombustiblePorMes()).andReturn(1);

        dtoDatosDerivados.setMediaCombustibleAnualRecargado(isA(Double.class));
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        dtoDatosDerivados.setMediaCombustibleMensualRecargado(isA(Double.class));
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        dtoDatosDerivados.setMediaCombustibleXKmAnual(isA(Double.class));
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        dtoDatosDerivados.setMediaCombustibleXKmMensual(isA(Double.class));
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        replay(modelo);
        replay(dtoVehiculo);
        replay(dtoDatosDerivados);
        controlador.setModel(modelo);
        controlador.comienzoCU("",CodigoCombustibleVehiculo.CPRIM.getId());

        verify(modelo);
        verify(dtoVehiculo);
        verify(dtoDatosDerivados);
    }
    @Test
    public void comienzoCUVehiculoCSecundario(){
        Calendar  cal =Calendar.getInstance();

        expect(modelo.getVehiculo(isA(String.class))).andReturn(dtoVehiculo);

        expect(dtoVehiculo.getCombustibleSecundario()).andReturn("");
        expect(modelo.getDatosDerivadosCombustible("",cal.get(Calendar.YEAR),cal.get(Calendar.MONTH),"")).andReturn(dtoDatosDerivados);
        expect(dtoDatosDerivados.getSumaCombustibleRecargadoPorAnio()).andReturn(1.0);
        expect(dtoDatosDerivados.getNumeroRecargasCombustiblePorAnio()).andReturn(1);
        expect(dtoDatosDerivados.getSumaCombustibleRecargadoPorMes()).andReturn(1.0);
        expect(dtoDatosDerivados.getNumeroRecargasCombustiblePorMes()).andReturn(1);
        expect(dtoDatosDerivados.getSumaCombustibleXKmAnual()).andReturn(1.0);
        expect(dtoDatosDerivados.getNumeroRecargasCombustiblePorAnio()).andReturn(1);
        expect(dtoDatosDerivados.getSumaCombustibleXKmMensual()).andReturn(1.0);
        expect(dtoDatosDerivados.getNumeroRecargasCombustiblePorMes()).andReturn(1);

        dtoDatosDerivados.setMediaCombustibleAnualRecargado(isA(Double.class));
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        dtoDatosDerivados.setMediaCombustibleMensualRecargado(isA(Double.class));
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        dtoDatosDerivados.setMediaCombustibleXKmAnual(isA(Double.class));
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        dtoDatosDerivados.setMediaCombustibleXKmMensual(isA(Double.class));
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        replay(modelo);
        replay(dtoVehiculo);
        replay(dtoDatosDerivados);
        controlador.setModel(modelo);
        controlador.comienzoCU("",CodigoCombustibleVehiculo.CSEC.getId());

        verify(modelo);
        verify(dtoVehiculo);
        verify(dtoDatosDerivados);
    }

}