package com.example.juanalonsoastruga.cargascontrol.Controlador.CUConsultarEstadisticaCostesCombustible;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.CodigoCombustibleVehiculo;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTODatosDerivadosCostesCombustible;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTOVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Modelo.CUConsultarEstadisticaCostesCombustible.InterfaceMConsultarEstadisticaCosteCombustible;
import com.example.juanalonsoastruga.cargascontrol.Modelo.CUConsultarEstadisticaCostesCombustible.ModeloConsultarEstadisticaCosteCombustible;
import com.example.juanalonsoastruga.cargascontrol.Formateador.CUConsultarEstadisticaCostesCombustible.InterfaceFConsultarEstadisticaCosteCombustible;
import com.example.juanalonsoastruga.cargascontrol.Formateador.CUConsultarEstadisticaCostesCombustible.FormateadorConsultarEstadisticaCosteCombustible;

import org.easymock.EasyMock;
import org.easymock.IAnswer;
import org.easymock.Mock;
import org.junit.Before;
import org.junit.Test;

import java.util.Calendar;

import static org.easymock.EasyMock.createMock;
import static org.easymock.EasyMock.expect;
import static org.easymock.EasyMock.isA;
import static org.easymock.EasyMock.replay;
import static org.easymock.EasyMock.verify;

public class ControladorConsultarEstadisticaCosteCombustibleTest {

    @Mock
    private InterfaceFConsultarEstadisticaCosteCombustible formateador;
    @Mock
    private InterfaceMConsultarEstadisticaCosteCombustible modelo;
    @Mock
    private DTODatosDerivadosCostesCombustible dtoDatosDerivados;
    @Mock
    private DTOVehiculo dtoVehiculo;
    private ControladorConsultarEstadisticaCosteCombustible controlador;


    @Before
    public void setUp(){
        formateador =createMock(FormateadorConsultarEstadisticaCosteCombustible.class);
        controlador =new ControladorConsultarEstadisticaCosteCombustible(formateador);
        modelo =createMock(ModeloConsultarEstadisticaCosteCombustible.class);
        dtoDatosDerivados=createMock(DTODatosDerivadosCostesCombustible.class);
        dtoVehiculo=createMock(DTOVehiculo.class);
    }

    @Test(expected=IllegalArgumentException.class)
    public void setVistaNull(){
        controlador.setFormateador(null);
    }
    @Test
    public void setVistaNotNull(){
        controlador.setFormateador(formateador);
    }
    @Test(expected=IllegalArgumentException.class)
    public void setModeloNull(){
        controlador.setModel(null);
    }
    @Test
    public void setModeloNotNull(){
        controlador.setModel(modelo);
    }



    @Test
    public void comienzoCUVehiculoNull(){
        expect(modelo.getVehiculo(isA(String.class))).andReturn(null);
        replay(modelo);
        controlador.setModel(modelo);
        controlador.comienzaCU("",0);
        verify(modelo);
    }
    @Test
    public void comienzoCUVehiculoCPrimario(){
        Calendar cal =Calendar.getInstance();

        expect(modelo.getVehiculo(isA(String.class))).andReturn(dtoVehiculo);
        expect(dtoVehiculo.getCombustiblePrimario()).andReturn("");
        expect(modelo.getDatosDerivadosCostesCombustible("",cal.get(Calendar.YEAR),cal.get(Calendar.MONTH),"")).andReturn(dtoDatosDerivados);
        expect(dtoDatosDerivados.getSumaCosteRecargaAnual()).andReturn(1.0);
        expect(dtoDatosDerivados.getSumaCosteRecargaMensual()).andReturn(1.0);
        expect(dtoDatosDerivados.getSumaCosteXRecargaAnual()).andReturn(1.0);
        expect(dtoDatosDerivados.getSumaCosteXRecargaMensual()).andReturn(1.0);
        expect(dtoDatosDerivados.getNumeroRecargasCombustiblePorMes()).andReturn(1).times(2) ;
        expect(dtoDatosDerivados.getNumeroRecargasCombustiblePorAnio()).andReturn(1).times(2);

        dtoDatosDerivados.setMediaCosteRecargaMensual(1.0);
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        dtoDatosDerivados.setMediaCosteXRecargaMensual(1.0);
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        dtoDatosDerivados.setMediaCosteRecargaAnual(isA(Double.class));
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        dtoDatosDerivados.setMediaCosteXRecargaAnual(isA(Double.class));
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });

        replay(modelo);
        replay(dtoVehiculo);
        replay(dtoDatosDerivados);
        controlador.setModel(modelo);
        controlador.comienzaCU("", CodigoCombustibleVehiculo.CPRIM.getId());

        verify(modelo);
        verify(dtoVehiculo);
        verify(dtoDatosDerivados);
    }
    @Test
    public void comienzoCUVehiculoCSecundario(){
        Calendar cal =Calendar.getInstance();

        expect(modelo.getVehiculo(isA(String.class))).andReturn(dtoVehiculo);
        expect(dtoVehiculo.getCombustiblePrimario()).andReturn("");

        expect(modelo.getDatosDerivadosCostesCombustible("",cal.get(Calendar.YEAR),cal.get(Calendar.MONTH),"")).andReturn(dtoDatosDerivados);
        expect(dtoDatosDerivados.getSumaCosteRecargaAnual()).andReturn(1.0);
        expect(dtoDatosDerivados.getSumaCosteRecargaMensual()).andReturn(1.0);
        expect(dtoDatosDerivados.getSumaCosteXRecargaAnual()).andReturn(1.0);
        expect(dtoDatosDerivados.getSumaCosteXRecargaMensual()).andReturn(1.0);
        expect(dtoDatosDerivados.getNumeroRecargasCombustiblePorMes()).andReturn(1).times(2) ;
        expect(dtoDatosDerivados.getNumeroRecargasCombustiblePorAnio()).andReturn(1).times(2);

        dtoDatosDerivados.setMediaCosteRecargaMensual(1.0);
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        dtoDatosDerivados.setMediaCosteXRecargaMensual(1.0);
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        dtoDatosDerivados.setMediaCosteRecargaAnual(isA(Double.class));
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        dtoDatosDerivados.setMediaCosteXRecargaAnual(isA(Double.class));
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });

        replay(modelo);
        replay(dtoVehiculo);
        replay(dtoDatosDerivados);
        controlador.setModel(modelo);
        controlador.comienzaCU("", CodigoCombustibleVehiculo.CPRIM.getId());

        verify(modelo);
        verify(dtoVehiculo);
        verify(dtoDatosDerivados);
    }
}