package com.example.juanalonsoastruga.cargascontrol.Controlador.CUConsultarInformacionRecarga;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTORecarga;
import com.example.juanalonsoastruga.cargascontrol.Formateador.CUConsultarInformacionRecarga.FormateadorConsultarInformacionRecarga;
import com.example.juanalonsoastruga.cargascontrol.Modelo.CUConsultarInformacionRecarga.InterfaceMConsultarInformacionRecarga;
import com.example.juanalonsoastruga.cargascontrol.Modelo.CUConsultarInformacionRecarga.ModeloConsultarInformacionRecarga;
import com.example.juanalonsoastruga.cargascontrol.Formateador.CUConsultarInformacionRecarga.InterfaceFConsultarInformacionRecarga;

import org.easymock.EasyMock;
import org.easymock.IAnswer;
import org.easymock.Mock;
import org.junit.Before;
import org.junit.Test;

import java.util.Date;

import static org.easymock.EasyMock.createMock;
import static org.easymock.EasyMock.expect;
import static org.easymock.EasyMock.replay;
import static org.easymock.EasyMock.verify;
import static org.junit.Assert.*;

public class ControladorConsultarInformacionRecargaTest {

    private ControladorConsultarInformacionRecarga controlador;
    @Mock
    private InterfaceFConsultarInformacionRecarga formateador;
    @Mock
    private InterfaceMConsultarInformacionRecarga modelo;
    @Mock
    private Date fecha;
    @Mock
    private DTORecarga recarga;

    @Before
    public void setUp(){
        modelo=createMock(ModeloConsultarInformacionRecarga.class);
        formateador =createMock(FormateadorConsultarInformacionRecarga.class);
        fecha=createMock(Date.class);
        recarga= createMock(DTORecarga.class);
        controlador = new ControladorConsultarInformacionRecarga(formateador);
    }

    @Test(expected = IllegalArgumentException.class)
    public void setViewNull(){
        controlador.setFormateador(null);
    }

    @Test
    public void setViewCorrect(){
        controlador.setFormateador(formateador);
        assertNotNull(controlador.getFormateador());
    }

    @Test(expected = IllegalArgumentException.class)
    public void setModeloNull(){
        controlador.setModel(null);
    }

    @Test
    public void setModeloCorrect(){
        controlador.setModel(modelo);
        assertNotNull(controlador.getModel());
    }

    @Test(expected = IllegalArgumentException.class)
    public void ComienzoCUMatriculaNull(){
        controlador.comienzoCU(null,new Date(),"");
    }

    @Test(expected = IllegalArgumentException.class)
    public void ComienzoCUFechaNull(){
        controlador.comienzoCU("",null,"");
    }

    @Test(expected = IllegalArgumentException.class)
    public void ComienzoCUCombustibleNull(){
        controlador.comienzoCU("",new Date(),null);
    }

    @Test(expected = AssertionError.class)
    public void ComienzoCUSinRecargaReal(){

        controlador.setModel(modelo);
        controlador.setFormateador(formateador);

        expect(modelo.getRecarga("",fecha,"")).andReturn(null);

        replay(modelo);

        formateador.setRecarga(recarga);
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });

        controlador.comienzoCU("",fecha,"");
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {

                DTORecarga recarga = modelo.getRecarga("",fecha,"");
                if(recarga!=null){
                    formateador.setRecarga(recarga);
                }
                return null;
            }
        });
        controlador.comienzoCU("",fecha,"");

       verify(modelo);

    }

    @Test
    public void ComienzoCUConRecargaReal(){

        controlador.setModel(modelo);
        controlador.setFormateador(formateador);

        expect(modelo.getRecarga("",fecha,"")).andReturn(recarga).times(2);

        replay(modelo);

        formateador.setRecarga(recarga);
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });

        controlador.comienzoCU("",fecha,"");
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {

                recarga = modelo.getRecarga("",fecha,"");
                if(recarga!=null){
                    formateador.setRecarga(recarga);
                }
                return null;
            }
        });
        controlador.comienzoCU("",fecha,"");

        verify(modelo);

    }


}