package com.example.juanalonsoastruga.cargascontrol.Controlador.CUConsultarPosicionMapa;

import com.example.juanalonsoastruga.cargascontrol.Formateador.CUConsultarPosicionMapa.FormateadorConsultarPosicionMapa;
import com.example.juanalonsoastruga.cargascontrol.Formateador.CUConsultarPosicionMapa.InterfaceFConsultarPosicionMapa;
import com.example.juanalonsoastruga.cargascontrol.Modelo.CUConsultarPosicionMapa.InterfaceMConsultarPosicionMapa;
import com.example.juanalonsoastruga.cargascontrol.Modelo.CUConsultarPosicionMapa.ModeloConsultarPosicionMapa;

import org.easymock.IAnswer;
import org.easymock.Mock;
import org.junit.Before;
import org.junit.Test;

import java.util.Hashtable;

import static org.easymock.EasyMock.anyDouble;
import static org.easymock.EasyMock.createMock;
import static org.easymock.EasyMock.expect;
import static org.easymock.EasyMock.expectLastCall;
import static org.easymock.EasyMock.isA;
import static org.easymock.EasyMock.replay;
import static org.easymock.EasyMock.verify;
import static org.junit.Assert.*;

public class ControladorConsultarPosicionMapaTest {

    @Mock
    private InterfaceFConsultarPosicionMapa formateador;
    private ControladorConsultarPosicionMapa controlador;
    @Mock
    private InterfaceMConsultarPosicionMapa modelo;
    @Mock
    private Hashtable posicion;

    @Before
    public void setUp(){
        modelo=createMock(ModeloConsultarPosicionMapa.class);
        formateador=createMock(FormateadorConsultarPosicionMapa.class);
        controlador = new ControladorConsultarPosicionMapa(formateador);
        posicion=createMock(Hashtable.class);
    }


    @Test(expected = IllegalArgumentException.class)
    public void setFormateadorNull() {
        controlador.setFormateador(null);
    }

    @Test
    public void setFormateadorNotNull() {
        controlador.setFormateador(formateador);
        assertEquals(formateador,controlador.getFormateador());
    }


    @Test(expected = IllegalArgumentException.class)
    public void setModeloNull() {
        controlador.setModelo(null);
    }

    @Test
    public void setModelo() {
        controlador.setModelo(modelo);
        assertEquals(modelo,controlador.getModelo());
    }


    @Test
    public void solicitarActualizacion() {
        expect(modelo.consultarPosicionActual()).andReturn(posicion);
        expect(posicion.get("latitud")).andReturn(0);
        expect(posicion.get("longitud")).andReturn(0);
        formateador.setLatitud(anyDouble());
        expectLastCall().andAnswer(new IAnswer<Object>() {
            @Override
            public Object answer() throws Throwable {
                return null;
            }
        });
        formateador.setLongitud(anyDouble());
        expectLastCall().andAnswer(new IAnswer<Object>() {
            @Override
            public Object answer() throws Throwable {
                return null;
            }
        });
        replay(modelo);
        replay(formateador);
        replay(posicion);

        controlador.setModelo(modelo);
        controlador.setFormateador(formateador);
        controlador.solicitarActualizacion();

        verify(modelo);
        verify(formateador);
        verify(posicion);
    }




}