package com.example.juanalonsoastruga.cargascontrol.Controlador.CUGraficarConsumoCombustibleXKm;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.CodigoCombustibleVehiculo;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.CodigoPeriocidadGrafica;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTORecarga;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTOVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Formateador.CUGraficarConsumoCombustibleXKm.InterfaceFGraficarConsumoCombustibleKm;
import com.example.juanalonsoastruga.cargascontrol.Modelo.CUGraficarConsumoCombustibleXKm.InterfaceMGraficarConsumoCombustibleKm;
import com.example.juanalonsoastruga.cargascontrol.Modelo.CUGraficarConsumoCombustibleXKm.ModeloGraficarConsumoCombustibleKm;
import com.example.juanalonsoastruga.cargascontrol.Formateador.CUGraficarConsumoCombustibleXKm.FormateadorGraficarConsumoCombustibleKm;

import org.easymock.EasyMock;
import org.easymock.IAnswer;
import org.easymock.Mock;
import org.junit.Before;
import org.junit.Test;

import java.util.Date;
import java.util.Hashtable;
import java.util.List;

import static org.easymock.EasyMock.createMock;
import static org.easymock.EasyMock.expect;
import static org.easymock.EasyMock.expectLastCall;
import static org.easymock.EasyMock.isA;
import static org.easymock.EasyMock.replay;
import static org.easymock.EasyMock.verify;
import static org.junit.Assert.*;

public class ControladorGraficarConsumoCombustibleKmTest {
    private ControladorGraficarConsumoCombustibleKm controlador;

    @Mock
    private InterfaceFGraficarConsumoCombustibleKm formateador;
    @Mock
    private InterfaceMGraficarConsumoCombustibleKm modelo;
    @Mock
    private List<DTORecarga> recargas;
    @Mock
    private DTOVehiculo vehiculo;
    @Mock
    private Date fecha;
    @Mock
    private DTORecarga dto;
    @Mock
    private Hashtable hash;

    @Before
    public void setUp(){
        formateador =createMock(FormateadorGraficarConsumoCombustibleKm.class);
        controlador =new ControladorGraficarConsumoCombustibleKm(formateador);
        modelo=createMock(ModeloGraficarConsumoCombustibleKm.class);
        recargas=createMock(List.class);
        vehiculo=createMock(DTOVehiculo.class);
        fecha=createMock(Date.class);
        dto=createMock(DTORecarga.class);
        hash=createMock(Hashtable.class);
    }

    @Test(expected = IllegalArgumentException.class)
    public void setViewNull(){
        controlador.setFormateador(null);
    }

    @Test
    public void setViewCorrect(){
        controlador.setFormateador(formateador);
        assertNotNull(controlador.getFormateador());
    }

    @Test(expected = IllegalArgumentException.class)
    public void setModeloNull(){
        controlador.setModel(null);
    }

    @Test
    public void setModeloCorrect(){
        controlador.setModel(modelo);
        assertNotNull(controlador.getModel());
    }

    @Test
    public void comienzoCUPeriocidadMensualPrimario(){

        controlador.setFormateador(formateador);
        controlador.setModel(modelo);
        String arg="";

        expect(formateador.getCodigoCombustible()).andReturn(CodigoCombustibleVehiculo.CPRIM) ;
        expect(modelo.getVehiculo(arg)).andReturn(vehiculo);
        expect(formateador.getMatricula()).andReturn(arg).times(2) ;
        expect(vehiculo.getMatricula()).andReturn(arg);

        expect(vehiculo.getCombustiblePrimario()).andReturn(arg).times(2);
        expect(vehiculo.getCapTankPrimario()).andReturn(0.0);
        expect(modelo.getRecargaMasLejanaAntesDe(isA(String.class),isA(String.class),isA(Date.class))).andReturn(dto);
        expect(modelo.getRecargasMensualesCombustible(isA(String.class),isA(String.class),isA(Date.class),isA(Date.class))).andReturn(recargas);

        expect(recargas.size()).andReturn(0).anyTimes();


        formateador.preparaDatosVisualizacion(isA(Hashtable.class), EasyMock.anyInt());
        expectLastCall().andAnswer(new IAnswer<Object>() {
            @Override
            public Object answer() throws Throwable {
                return null;
            }
        });
        replay(vehiculo);
        replay(formateador);
        replay(modelo);
        replay(recargas);

        controlador.comienzaCU(CodigoPeriocidadGrafica.PERIOCIDAD_GRAFICA1);

        verify(vehiculo);
        verify(formateador);
        verify(modelo);
        verify(recargas);
    }

    @Test
    public void comienzoCUPeriocidadMensualSecundario(){

        controlador.setFormateador(formateador);
        controlador.setModel(modelo);
        String arg="";

        expect(formateador.getCodigoCombustible()).andReturn(CodigoCombustibleVehiculo.CSEC) ;
        expect(modelo.getVehiculo(arg)).andReturn(vehiculo);
        expect(formateador.getMatricula()).andReturn(arg).times(2) ;
        expect(vehiculo.getMatricula()).andReturn(arg);

        expect(vehiculo.getCombustibleSecundario()).andReturn(arg).times(2);
        expect(vehiculo.getCapTankSecundario()).andReturn(0.0);
        expect(modelo.getRecargaMasLejanaAntesDe(isA(String.class),isA(String.class),isA(Date.class))).andReturn(dto);
        expect(modelo.getRecargasMensualesCombustible(isA(String.class),isA(String.class),isA(Date.class),isA(Date.class))).andReturn(recargas);

        expect(recargas.size()).andReturn(0).anyTimes();

        formateador.preparaDatosVisualizacion(isA(Hashtable.class),EasyMock.anyInt());
        expectLastCall().andAnswer(new IAnswer<Object>() {
            @Override
            public Object answer() throws Throwable {
                return null;
            }
        });
        replay(vehiculo);
        replay(formateador);
        replay(modelo);
        replay(recargas);

        controlador.comienzaCU(CodigoPeriocidadGrafica.PERIOCIDAD_GRAFICA1);

        verify(vehiculo);
        verify(formateador);
        verify(modelo);
        verify(recargas);
    }

    @Test
    public void comienzoCUPeriocidadAnioPrimario(){

        controlador.setFormateador(formateador);
        controlador.setModel(modelo);
        String arg="";

        expect(formateador.getCodigoCombustible()).andReturn(CodigoCombustibleVehiculo.CPRIM) ;
        expect(modelo.getVehiculo(arg)).andReturn(vehiculo);
        expect(formateador.getMatricula()).andReturn(arg).times(2) ;
        expect(vehiculo.getMatricula()).andReturn(arg);

        expect(vehiculo.getCombustiblePrimario()).andReturn(arg).times(2);
        expect(vehiculo.getCapTankPrimario()).andReturn(0.0);
        expect(modelo.getRecargaMasLejanaAntesDe(isA(String.class),isA(String.class),isA(Date.class))).andReturn(dto);
        expect(modelo.getRecargasAnualesCombustible(isA(String.class),isA(String.class),isA(Date.class),isA(Date.class))).andReturn(recargas);

        expect(recargas.size()).andReturn(0).anyTimes();

        formateador.preparaDatosVisualizacion(isA(Hashtable.class),EasyMock.anyInt());
        expectLastCall().andAnswer(new IAnswer<Object>() {
            @Override
            public Object answer() throws Throwable {
                return null;
            }
        });
        replay(vehiculo);
        replay(formateador);
        replay(modelo);
        replay(recargas);

        controlador.comienzaCU(CodigoPeriocidadGrafica.PERIOCIDAD_GRAFICA2);

        verify(vehiculo);
        verify(formateador);
        verify(modelo);
        verify(recargas);
    }

    @Test
    public void comienzoCUPeriocidadAnioSecundario(){

        controlador.setFormateador(formateador);
        controlador.setModel(modelo);
        String arg="";

        expect(formateador.getCodigoCombustible()).andReturn(CodigoCombustibleVehiculo.CSEC) ;
        expect(modelo.getVehiculo(arg)).andReturn(vehiculo);
        expect(formateador.getMatricula()).andReturn(arg).times(2) ;
        expect(vehiculo.getMatricula()).andReturn(arg);
        expect(vehiculo.getCapTankSecundario()).andReturn(0.0);
        expect(vehiculo.getCombustibleSecundario()).andReturn(arg) .times(2);
        expect(modelo.getRecargaMasLejanaAntesDe(isA(String.class),isA(String.class),isA(Date.class))).andReturn(dto);
        expect(modelo.getRecargasAnualesCombustible(isA(String.class),isA(String.class),isA(Date.class),isA(Date.class))).andReturn(recargas);

        expect(recargas.size()).andReturn(0).anyTimes();

        formateador.preparaDatosVisualizacion(isA(Hashtable.class),EasyMock.anyInt());
        expectLastCall().andAnswer(new IAnswer<Object>() {
            @Override
            public Object answer() throws Throwable {
                return null;
            }
        });
        replay(vehiculo);
        replay(formateador);
        replay(modelo);
        replay(recargas);

        controlador.comienzaCU(CodigoPeriocidadGrafica.PERIOCIDAD_GRAFICA2);

        verify(vehiculo);
        verify(formateador);
        verify(modelo);
        verify(recargas);
    }
}