package com.example.juanalonsoastruga.cargascontrol.Controlador.CUModificarInformacionVehiculo;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTOCombustible;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTOVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Modelo.CUModificarInformacionVehiculo.InterfaceMModificarInformacionVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Modelo.CUModificarInformacionVehiculo.ModeloModificarInformacionVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Formateador.CUModificarInformacionVehiculo.InterfaceFModificarInformacionVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Formateador.CUModificarInformacionVehiculo.FormateadorModificarInformacionVehiculo;

import org.easymock.Mock;
import org.junit.Before;
import org.junit.Test;

import java.util.ArrayList;
import java.util.List;

import static org.easymock.EasyMock.createControl;
import static org.easymock.EasyMock.createMock;
import static org.easymock.EasyMock.expect;
import static org.easymock.EasyMock.replay;
import static org.easymock.EasyMock.verify;
import static org.junit.Assert.*;

public class ControladorModificarInformacionVehiculoTest {

    private ControladorModificarInformacionVehiculo controlador;

    @Mock
    private InterfaceFModificarInformacionVehiculo formateador;
    @Mock
    private InterfaceMModificarInformacionVehiculo model;
    @Mock
    private List<DTOCombustible> combustibles;
    @Mock
    private  DTOVehiculo vehiculo;

    @Before
    public void setUp(){
        formateador =createMock(FormateadorModificarInformacionVehiculo.class);
        model=createMock(ModeloModificarInformacionVehiculo.class);
        vehiculo=createMock(DTOVehiculo.class);
        combustibles= createMock(ArrayList.class );
        controlador = new ControladorModificarInformacionVehiculo(formateador);
    }
    @Test(expected = IllegalArgumentException.class)
    public void setViewNull(){
        controlador.setFormateador(null);
    }
    @Test
    public void setViewCorrect(){
        controlador.setFormateador(formateador);
        assertNotNull(controlador.getFormateador());
    }
    @Test(expected = IllegalArgumentException.class)
    public void setModeloNull(){
        controlador.setModel(null);
    }
    @Test
    public void setModeloCorrect(){
        controlador.setModel(model);
        assertNotNull(controlador.getModel());
    }
    @Test
    public void comienzoCU(){

        expect(model.getCombustibles()).andReturn(combustibles);
        expect(model.getVehiculo("")).andReturn(vehiculo);
        expect(formateador.setInitInfo(combustibles,vehiculo)).andReturn(true);
        replay(model);
        replay(formateador);
        controlador.setFormateador(formateador);
        controlador.setModel(model);
        controlador.comienzoCU("");
        verify(model);
        verify(formateador);
    }

    @Test
    public void pulsacionBotonModificacionIncorrectoNoNombre(){

        expect(formateador.getRegistro()).andReturn(vehiculo);
        expect(formateador.setRegistroNoCompleto()).andReturn(true);
        expect(vehiculo.getNombre()).andReturn(null);


        replay(formateador);
        replay(vehiculo);

        controlador.setFormateador(formateador);
        controlador.pulsacionBotonModificarDatos();

        verify(vehiculo);
        verify(formateador);

    }
    @Test
    public void pulsacionBotonModificacionIncorrectoNoCPrim(){

        expect(formateador.getRegistro()).andReturn(vehiculo);
        expect(formateador.setRegistroNoCompleto()).andReturn(true);
        expect(vehiculo.getNombre()).andReturn("");
        expect(vehiculo.getCombustiblePrimario()).andReturn(null);

        replay(formateador);
        replay(vehiculo);

        controlador.setFormateador(formateador);
        controlador.pulsacionBotonModificarDatos();

        verify(vehiculo);
        verify(formateador);

    }
    @Test
    public void pulsacionBotonModificacionCorrectoCamposObligatorios(){
        expect(formateador.getRegistro()).andReturn(vehiculo);
        expect(formateador.setRegistroCompletado()).andReturn(true);
        expect(model.actualizaVehiculo(vehiculo)).andReturn(true);
        expect(vehiculo.getNombre()).andReturn("");
        expect(vehiculo.getCombustiblePrimario()).andReturn("");
        replay(formateador);
        replay(model);
        replay(vehiculo);
        controlador.setFormateador(formateador);
        controlador.setModel(model);
        controlador.pulsacionBotonModificarDatos();
        verify(vehiculo);
        verify(formateador);
        verify(model);
    }


    @Test
    public void rellenarCampoIncorrectamenteAno(){

        String msj="Campo Año incorrectamente completado, reinténtelo por favor";
        expect(formateador.setMensajeError(msj)).andReturn(true);
        replay(formateador);
        controlador.setFormateador(formateador);
        controlador.rellenarCampoIncorrectamente("año");
        verify(formateador);

    }
    @Test
    public void rellenarCampoIncorrectamenteKM(){

        String msj="Campo Kilometraje incorrectamente completado,\n reinténtelo por favor";
        expect(formateador.setMensajeError(msj)).andReturn(true);
        replay(formateador);
        controlador.setFormateador(formateador);
        controlador.rellenarCampoIncorrectamente("km");
        verify(formateador);

    }
    @Test
    public void rellenarCampoIncorrectamenteCP(){

        String msj="Campo Capacidad de Carga de Combustible Principal  incorrectamente\n completado,reinténtelo por favor";
        expect(formateador.setMensajeError(msj)).andReturn(true);
        replay(formateador);
        controlador.setFormateador(formateador);
        controlador.rellenarCampoIncorrectamente("CTP");
        verify(formateador);

    }
    @Test
    public void rellenarCampoIncorrectamenteCS(){

        String msj="Campo Capacidad de Carga de Combustible Secundario  incorrectamente\n completado,reinténtelo por favor";
        expect(formateador.setMensajeError(msj)).andReturn(true);
        replay(formateador);
        controlador.setFormateador(formateador);
        controlador.rellenarCampoIncorrectamente("CTS");
        verify(formateador);

    }
}