package com.example.juanalonsoastruga.cargascontrol.Controlador.CURegistrarRecarga;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTODatosDerivadosCombustible;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTODatosDerivadosCostesCombustible;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTODatosDerivadosVehiculoTotales;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTORecarga;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTOVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Formateador.CURegistrarRecarga.FormateadorRegistrarRecarga;
import com.example.juanalonsoastruga.cargascontrol.Formateador.CURegistrarRecarga.InterfaceFRegistrarRecarga;
import com.example.juanalonsoastruga.cargascontrol.Modelo.CURegistrarRecarga.InterfaceMRegistrarRecarga;
import com.example.juanalonsoastruga.cargascontrol.Modelo.CURegistrarRecarga.ModeloRegistrarRecarga;

import org.easymock.EasyMock;
import org.easymock.IAnswer;
import org.easymock.Mock;
import org.junit.Before;
import org.junit.Test;

import java.util.Calendar;
import java.util.Date;

import static org.easymock.EasyMock.createControl;
import static org.easymock.EasyMock.createMock;
import static org.easymock.EasyMock.expect;
import static org.easymock.EasyMock.isA;
import static org.easymock.EasyMock.replay;
import static org.easymock.EasyMock.verify;
import static org.junit.Assert.*;

public class ControladorRegistrarRecargaTest {

    private ControladorRegistrarRecarga controlador;
    @Mock
    private InterfaceFRegistrarRecarga formateador;
    @Mock
    private InterfaceMRegistrarRecarga modelo;
    @Mock
    private DTORecarga recarga;
    @Mock
    private DTORecarga recarga2;
    @Mock
    private DTOVehiculo dtoVehiculo;
    @Mock
    private DTODatosDerivadosCombustible dtoDatosDerivadosCombustible;
    @Mock
    private DTODatosDerivadosCostesCombustible dtoDatosDerivadosCostesCombustible;
    @Mock
    private DTODatosDerivadosVehiculoTotales dtoDatosDerivadosVehiculo;

    @Before
    public void setUp(){
        modelo=createMock(ModeloRegistrarRecarga.class);
        formateador =createMock(FormateadorRegistrarRecarga.class);
        controlador =new ControladorRegistrarRecarga(formateador);
        recarga=createMock(DTORecarga.class);
        recarga2=createMock(DTORecarga.class);
        dtoVehiculo = createMock(DTOVehiculo.class);
        dtoDatosDerivadosCombustible=createMock(DTODatosDerivadosCombustible.class);
        dtoDatosDerivadosCostesCombustible=createMock(DTODatosDerivadosCostesCombustible.class);
        dtoDatosDerivadosVehiculo=createMock(DTODatosDerivadosVehiculoTotales.class);
    }

    @Test(expected = IllegalArgumentException.class)
    public void setNullVista(){
        controlador.setFormateador(null);
    }

    @Test
    public void setNotNullVista(){
        controlador.setFormateador(formateador);
        assertNotNull(controlador.getFormateador());
        assertEquals(controlador.getFormateador().getClass(), formateador.getClass());
    }

    @Test(expected = IllegalArgumentException.class)
    public void setNullModel(){
        controlador.setModel(null);
    }

    @Test
    public void setNotNullModel(){
        controlador.setModel(modelo);
        assertNotNull(controlador.getModel());
        assertEquals(controlador.getModel().getClass(),modelo.getClass());
    }

    @Test
    public void comprobarComienzoCU(){
        controlador.comienzoCU();
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                formateador.setDinamicData();
                EasyMock.expectLastCall().andAnswer(new IAnswer() {
                    public Object answer() {
                        return null;
                    }
                });
                EasyMock.replay(formateador);
                formateador.setDinamicData();
                verify(formateador);
                return null;
            }
        });
        controlador.comienzoCU();
    }


     @Test(expected = IllegalArgumentException.class)
    public void registrarRecargaConMatriculaNull(){
        expect(recarga.getMatricula()).andReturn("0");
        replay(recarga);
        controlador.registrarRecarga(null);
        verify(recarga);
    }
    @Test (expected = IllegalArgumentException.class)
    public void registrarRecargaSinMatricula(){
        expect(recarga.getMatricula()).andReturn("").times(2) ;
        replay(recarga);
        controlador.registrarRecarga(recarga);
        verify(recarga);
    }
    @Test
    public void registrarRecargaSinCombustible(){
        expect(recarga.getCombustible()).andReturn("").times(2);
        expect(recarga.getMatricula()).andReturn("0").times(3);
        expect(modelo.getVehiculo(isA(String.class))).andReturn(dtoVehiculo);
        replay(recarga);
        replay(modelo);
        formateador.setMsgError(isA(String.class)) ;
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        formateador.setRegistroCompletado(false);
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        replay(formateador);
        controlador.setFormateador(formateador);
        controlador.setModel(modelo);
        controlador.registrarRecarga(recarga);
        verify(recarga);
        verify(modelo);
        verify(formateador);
    }

    @Test
    public void registrarRecargaConCombustibleNull(){
        expect(recarga.getCombustible()).andReturn(null);
        expect(recarga.getMatricula()).andReturn("0").times(3);
        expect(modelo.getVehiculo(isA(String.class))).andReturn(dtoVehiculo);
        replay(recarga);
        replay(modelo);
        formateador.setMsgError(isA(String.class)) ;
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        formateador.setRegistroCompletado(false);
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        replay(formateador);
        controlador.setFormateador(formateador);
        controlador.setModel(modelo);
        controlador.registrarRecarga(recarga);
        verify(recarga);
        verify(modelo);
        verify(formateador);
    }

    @Test
    public void registrarRecargaConKmNull(){
        expect(recarga.getKm()).andReturn(null);
        expect(recarga.getCombustible()).andReturn("0").times(2);
        expect(recarga.getMatricula()).andReturn("0").times(3);
        expect(modelo.getVehiculo(isA(String.class))).andReturn(dtoVehiculo);
        replay(recarga);
        replay(modelo);
        formateador.setMsgError(isA(String.class)) ;
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        formateador.setRegistroCompletado(false);
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        replay(formateador);
        controlador.setFormateador(formateador);
        controlador.setModel(modelo);
        controlador.registrarRecarga(recarga);
        verify(recarga);
        verify(modelo);
        verify(formateador);
    }
    @Test
    public void registrarRecargaConFechaNull(){
        double km =0.0;
        expect(recarga.getFecha()).andReturn(null);
        expect(recarga.getKm()).andReturn(km).times(2);
        expect(recarga.getCombustible()).andReturn("0").times(2);
        expect(recarga.getMatricula()).andReturn("0").times(3);
        expect(modelo.getVehiculo(isA(String.class))).andReturn(dtoVehiculo);
        replay(recarga);
        replay(modelo);
        formateador.setMsgError(isA(String.class)) ;
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        formateador.setRegistroCompletado(false);
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        replay(formateador);
        controlador.setFormateador(formateador);
        controlador.setModel(modelo);
        controlador.registrarRecarga(recarga);
        verify(recarga);
        verify(modelo);
        verify(formateador);
    }
    @Test
    public void registrarRecargaConCosteNull(){
        double km =0.0;
        expect(recarga.getCoste()).andReturn(null);
        expect(recarga.getFecha()).andReturn(new Date());
        expect(recarga.getKm()).andReturn(km).times(2);
        expect(recarga.getCombustible()).andReturn("0").times(2);
        expect(recarga.getMatricula()).andReturn("0").times(3);
        expect(modelo.getVehiculo(isA(String.class))).andReturn(dtoVehiculo);
        replay(recarga);
        replay(modelo);
        formateador.setMsgError(isA(String.class)) ;
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        formateador.setRegistroCompletado(false);
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        replay(formateador);
        controlador.setFormateador(formateador);
        controlador.setModel(modelo);
        controlador.registrarRecarga(recarga);
        verify(recarga);
        verify(modelo);
        verify(formateador);
    }
    @Test
    public void registrarRecargaConCoste0(){
        double km =0.0;
        expect(recarga.getCoste()).andReturn(0.0).times(3);
        expect(recarga.getFecha()).andReturn(new Date());
        expect(recarga.getKm()).andReturn(km).times(2);
        expect(recarga.getCombustible()).andReturn("0").times(2);
        expect(recarga.getMatricula()).andReturn("0").times(3);
        expect(modelo.getVehiculo(isA(String.class))).andReturn(dtoVehiculo);
        replay(recarga);
        replay(modelo);
        formateador.setMsgError(isA(String.class)) ;
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        formateador.setRegistroCompletado(false);
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        replay(formateador);
        controlador.setFormateador(formateador);
        controlador.setModel(modelo);
        controlador.registrarRecarga(recarga);
        verify(recarga);
        verify(modelo);
        verify(formateador);
    }
    @Test
    public void registrarRecargaConMedicionNull(){
        double km =0.0;
        expect(recarga.getMedicion()).andReturn(null);
        expect(recarga.getCoste()).andReturn(10.0).times(3);
        expect(recarga.getFecha()).andReturn(new Date());
        expect(recarga.getKm()).andReturn(km).times(2);
        expect(recarga.getCombustible()).andReturn("0").times(2);
        expect(recarga.getMatricula()).andReturn("0").times(3);
        expect(modelo.getVehiculo(isA(String.class))).andReturn(dtoVehiculo);
        replay(recarga);
        replay(modelo);
        formateador.setMsgError(isA(String.class)) ;
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        formateador.setRegistroCompletado(false);
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        replay(formateador);
        controlador.setFormateador(formateador);
        controlador.setModel(modelo);
        controlador.registrarRecarga(recarga);
        verify(recarga);
        verify(modelo);
        verify(formateador);
    }
    @Test
    public void registrarRecargaConMedicion0(){
        double km =0.0;

        expect(recarga.getMedicion()).andReturn(0.0).times(3);
        expect(recarga.getCoste()).andReturn(10.0).times(3);
        expect(recarga.getFecha()).andReturn(new Date());
        expect(recarga.getKm()).andReturn(km).times(2);
        expect(recarga.getCombustible()).andReturn("0").times(2);
        expect(recarga.getMatricula()).andReturn("0").times(3);
        expect(modelo.getVehiculo(isA(String.class))).andReturn(dtoVehiculo);
        replay(recarga);
        replay(modelo);
        formateador.setMsgError(isA(String.class)) ;
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        formateador.setRegistroCompletado(false);
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        replay(formateador);
        controlador.setFormateador(formateador);
        controlador.setModel(modelo);
        controlador.registrarRecarga(recarga);
        verify(recarga);
        verify(modelo);
        verify(formateador);
    }
    @Test
    public void registrarRecargaConFechaFutura(){
        double km =0.0;
        Calendar futuro= Calendar.getInstance();
        int mes,anio;
        Date ahora = new Date();
        anio=ahora.getYear();
        mes=ahora.getMonth();
        futuro.set(anio+1,mes+1,1);
        expect(recarga.getFecha()).andReturn(futuro.getTime()).times(1);
        expect(recarga.getMedicion()).andReturn(0.0).times(3);
        expect(recarga.getCoste()).andReturn(10.0).times(3);
        expect(recarga.getKm()).andReturn(km).times(2);
        expect(recarga.getCombustible()).andReturn("0").times(2);
        expect(recarga.getMatricula()).andReturn("0").times(3);
        expect(modelo.getVehiculo(isA(String.class))).andReturn(dtoVehiculo);
        replay(recarga);
        replay(modelo);
        formateador.setMsgError(isA(String.class)) ;
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        formateador.setRegistroCompletado(false);
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        replay(formateador);
        controlador.setFormateador(formateador);
        controlador.setModel(modelo);
        controlador.registrarRecarga(recarga);
        verify(recarga);
        verify(modelo);
        verify(formateador);
    }
    @Test
    public void registrarRecargaCamposRegistroNoCompletos(){
        double km =0.0;

        Date ahora = new Date();


        expect(recarga.getFecha()).andReturn(ahora).times(2);
        expect(recarga.getMedicion()).andReturn(1.0).times(3);
        expect(recarga.getCoste()).andReturn(10.0).times(3);
        expect(recarga.getKm()).andReturn(km).times(2);
        expect(recarga.getCombustible()).andReturn("0").times(2);
        expect(recarga.getMatricula()).andReturn("0").times(3);
        expect(modelo.getVehiculo(isA(String.class))).andReturn(dtoVehiculo);
        replay(recarga);
        replay(modelo);

        formateador.setMsgError(isA(String.class)) ;
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        formateador.setRegistroCompletado(false);
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        formateador.isCamposRegistroCompletos();
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return false;
            }
        });
        replay(formateador);
        controlador.setFormateador(formateador);
        controlador.setModel(modelo);
        controlador.registrarRecarga(recarga);
        verify(recarga);
        verify(modelo);
        verify(formateador);

    }

    @Test
    public void registrarRecargaMedicionMenorCapacidadCPrim(){
        double km =0.0;

        Date ahora = new Date();
        expect(dtoVehiculo.getCapTankPrimario()).andReturn(0.0);
        expect(dtoVehiculo.getCombustiblePrimario()).andReturn("0");
        expect(recarga.getFecha()).andReturn(ahora).times(2);
        expect(recarga.getMedicion()).andReturn(1.0).times(4);
        expect(recarga.getCoste()).andReturn(10.0).times(3);
        expect(recarga.getKm()).andReturn(km).times(2);
        expect(recarga.getCombustible()).andReturn("0").times(3);
        expect(recarga.getMatricula()).andReturn("0").times(3);
        expect(modelo.getVehiculo(isA(String.class))).andReturn(dtoVehiculo);
        replay(recarga);
        replay(modelo);
        replay(dtoVehiculo);

        formateador.setMsgError(isA(String.class)) ;
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        formateador.setRegistroCompletado(false);
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        formateador.isCamposRegistroCompletos();
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return true;
            }
        });
        replay(formateador);
        controlador.setFormateador(formateador);
        controlador.setModel(modelo);
        controlador.registrarRecarga(recarga);
        verify(recarga);
        verify(modelo);
        verify(formateador);
        verify(dtoVehiculo);
    }
    @Test
    public void registrarRecargaMedicionMenorCapacidadCSec(){
        double km =0.0;

        Date ahora = new Date();

        expect(dtoVehiculo.getCapTankSecundario()).andReturn(0.0);
        expect(dtoVehiculo.getCombustiblePrimario()).andReturn("aaa");
        expect(recarga.getFecha()).andReturn(ahora).times(2);
        expect(recarga.getMedicion()).andReturn(1.0).times(4);
        expect(recarga.getCoste()).andReturn(10.0).times(3);
        expect(recarga.getKm()).andReturn(km).times(2);
        expect(recarga.getCombustible()).andReturn("0").times(3);
        expect(recarga.getMatricula()).andReturn("0").times(3);
        expect(modelo.getVehiculo(isA(String.class))).andReturn(dtoVehiculo);
        replay(recarga);
        replay(modelo);
        replay(dtoVehiculo);

        formateador.setMsgError(isA(String.class)) ;
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        formateador.setRegistroCompletado(false);
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        formateador.isCamposRegistroCompletos();
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return true;
            }
        });
        replay(formateador);
        controlador.setFormateador(formateador);
        controlador.setModel(modelo);
        controlador.registrarRecarga(recarga);
        verify(recarga);
        verify(modelo);
        verify(formateador);
        verify(dtoVehiculo);

    }
    @Test
    public void registrarRecargaKm0Medicion0(){
        double km =0.0;

        Date ahora = new Date();

        expect(dtoVehiculo.getCapTankSecundario()).andReturn(0.0);
        expect(dtoVehiculo.getCombustiblePrimario()).andReturn("aaa");
        expect(recarga.getFecha()).andReturn(ahora).times(2);
        expect(recarga.getMedicion()).andReturn(1.0).times(4);
        expect(recarga.getCoste()).andReturn(10.0).times(3);
        expect(recarga.getKm()).andReturn(km).times(2);
        expect(recarga.getCombustible()).andReturn("0").times(3);
        expect(recarga.getMatricula()).andReturn("0").times(3);
        expect(modelo.getVehiculo(isA(String.class))).andReturn(dtoVehiculo);
        replay(recarga);
        replay(modelo);
        replay(dtoVehiculo);

        formateador.setMsgError(isA(String.class)) ;
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        formateador.setRegistroCompletado(false);
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        formateador.isCamposRegistroCompletos();
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return true;
            }
        });
        replay(formateador);
        controlador.setFormateador(formateador);
        controlador.setModel(modelo);
        controlador.registrarRecarga(recarga);
        verify(recarga);
        verify(modelo);
        verify(formateador);
        verify(dtoVehiculo);

    }
    @Test
    public void registrarRecargaAnteriorEsNullRegistraDatosDerivadosNuevos(){
        double km =1.0;


        Calendar futuro= Calendar.getInstance();
        int mes,anio;
        Date ahora = new Date();
        anio=ahora.getYear();
        mes=ahora.getMonth();
        futuro.set(anio+1,mes+1,1);

        expect(modelo. isPrimerRegistroDatosDerivadosVehiculo(isA(String.class))).andReturn(true);

        expect(modelo.getUltimaRecargaRegistrada(isA(String.class),isA(Date.class))).andReturn(null);
       // expect(recarga2.getFecha()).andReturn(futuro.getTime());
        expect(recarga2.getKm()).andReturn(0.5);
        expect(modelo.getUltimaRecargaRegistradaMismoCombustible(isA(String.class),isA(String.class))).andReturn(recarga2);
        expect(dtoVehiculo.getKilometraje()).andReturn(100.0).times(3);
        expect(dtoVehiculo.getCapTankSecundario()).andReturn(1.0);
        expect(dtoVehiculo.getCombustiblePrimario()).andReturn("aaa");
        expect(recarga.getFecha()).andReturn(ahora).times(2);
        expect(recarga.getMedicion()).andReturn(1.0).anyTimes();//times(11)
        expect(recarga.getCoste()).andReturn(10.0).anyTimes();
        expect(recarga.getKm()).andReturn(km).times(6);
        expect(recarga.getCombustible()).andReturn("0").times(5);
        expect(recarga.getMatricula()).andReturn("0").times(8);
        expect(modelo.getVehiculo(isA(String.class))).andReturn(dtoVehiculo);
        expect(modelo.getDDVehiculo( "0" ,Calendar.getInstance().get(Calendar.YEAR) ,Calendar.getInstance().get(Calendar.MONTH))).andReturn(null);
        expect(modelo.getDDVCombustible("0" ,Calendar.getInstance().get(Calendar.YEAR),Calendar.getInstance().get(Calendar.MONTH) , "0"  )).andReturn(null);
        expect(modelo.isPrimerRegistroDatosDerivadosCombustible("0" ,  "0" )).andReturn(true).times(1);
        expect(modelo.isPrimerRegistroDatosDerivadosCosteCombustible("0" ,  "0" )).andReturn(true).times(1);

        expect(modelo.getDDVCostesCombustible("0" ,Calendar.getInstance().get(Calendar.YEAR) , Calendar.getInstance().get(Calendar.MONTH)   , "0" )).andReturn(null);

        /* formateador.setMsgError(isA(String.class)) ;
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });*/
        formateador.setRegistroCompletado(true);
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        formateador.isCamposRegistroCompletos();
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return true;
            }
        });
        modelo.registraRecarga(recarga);
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        modelo.addDatosDerivadosCombustible(isA(DTODatosDerivadosCombustible.class));
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
       modelo.addDatosDerivadosCosteCombustible(isA(DTODatosDerivadosCostesCombustible.class));
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
          modelo.addDatosDerivadosVehiculo(isA(DTODatosDerivadosVehiculoTotales.class));
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });




        replay(modelo);
        replay(formateador);
        replay(recarga);
        replay(recarga2);
        replay(dtoVehiculo);
        controlador.setFormateador(formateador);
        controlador.setModel(modelo);
        controlador.registrarRecarga(recarga);
        verify(recarga);
        verify(recarga2);
        verify(modelo);
        verify(formateador);
        verify(dtoVehiculo);

    }
    @Test
    public void registrarRecargaAnteriorEsNullRegistraDatosDerivadosConCambioMes(){
        double km =1.0;


        Calendar futuro= Calendar.getInstance();
        int mes,anio;
        Date ahora = new Date();
        anio=ahora.getYear();
        mes=ahora.getMonth();
        futuro.set(anio+1,mes+1,1);

        expect(dtoDatosDerivadosVehiculo.getCosteRecargasTotalesPorAnio()).andReturn(0.0);
        expect(dtoDatosDerivadosVehiculo.getKmRecargasTotalesPorAnio()).andReturn(0.0);
        expect(dtoDatosDerivadosVehiculo.getNumeroRecargasAnio()).andReturn(0);
        expect(dtoDatosDerivadosCostesCombustible.getSumaCosteXRecargaAnual()).andReturn(0.0);
        expect(dtoDatosDerivadosCostesCombustible.getMaximoCosteXRecargaAnual()).andReturn(0.0);
        expect(dtoDatosDerivadosCostesCombustible.getMinimoCosteXRecargaAnual()).andReturn(0.0).times(2);expect(dtoDatosDerivadosCostesCombustible.getSumaCosteRecargaAnual()).andReturn(0.0);
        expect(dtoDatosDerivadosCostesCombustible.getNumeroRecargasCombustiblePorAnio()).andReturn(0);
        expect(dtoDatosDerivadosCombustible.getSumaCombustibleXKmAnual()).andReturn(0.0);
        expect(dtoDatosDerivadosCombustible.getSumaCombustibleRecargadoPorAnio()).andReturn(0.0);
        expect(dtoDatosDerivadosCombustible.getNumeroRecargasCombustiblePorAnio()).andReturn(0);
        expect(dtoDatosDerivadosCombustible.getMinimoCombustibleAnualRecargado()).andReturn(0.0).times(2);
        expect(dtoDatosDerivadosCombustible.getMinimoCombustibleXKmAnual()).andReturn(0.0).times(2);
        expect(dtoDatosDerivadosCombustible.getMaximoCombustibleXKmAnual()).andReturn(0.0);
        expect(dtoDatosDerivadosCombustible.getMaximoCombustibleAnualRecargado()).andReturn(0.0);
        expect(dtoDatosDerivadosCostesCombustible.getMaximoCosteRecargaAnual()).andReturn(0.0);

        expect(dtoDatosDerivadosCostesCombustible.getMinimoCosteRecargaAnual()).andReturn(0.0).times(2);
        expect(dtoDatosDerivadosCombustible.getAnio()).andReturn(Calendar.getInstance().get(Calendar.YEAR));
        expect(dtoDatosDerivadosVehiculo.getAnio()).andReturn(Calendar.getInstance().get(Calendar.YEAR));
        expect(dtoDatosDerivadosCostesCombustible.getAnio()).andReturn(Calendar.getInstance().get(Calendar.YEAR));
        expect(modelo.isPrimerRegistroDatosDerivadosVehiculo(isA(String.class))).andReturn(false);
        expect(modelo.isPrimerRegistroDatosDerivadosCosteCombustible(isA(String.class),isA(String.class))).andReturn(false);

        expect(modelo.getLastRegistroDatosDerivadosVehiculo("0" )).andReturn(dtoDatosDerivadosVehiculo);
        expect(modelo.getLastRegistroDatosDerivadosCombustible("0","0")).andReturn(dtoDatosDerivadosCombustible);
        expect(modelo.getLastRegistroDatosDerivadosCostesCombustible("0","0")).andReturn(dtoDatosDerivadosCostesCombustible);
        expect(modelo.getUltimaRecargaRegistrada(isA(String.class),isA(Date.class))).andReturn(null);
        //expect(recarga2.getFecha()).andReturn(futuro.getTime());
        expect(recarga2.getKm()).andReturn(0.5);
        expect(modelo.getUltimaRecargaRegistradaMismoCombustible(isA(String.class),isA(String.class))).andReturn(recarga2);
        expect(dtoVehiculo.getCapTankSecundario()).andReturn(1.0);
        expect(dtoVehiculo.getCombustiblePrimario()).andReturn("aaa");
        expect(recarga.getFecha()).andReturn(ahora).times(2);
        expect(recarga.getMedicion()).andReturn(1.0).anyTimes();
        expect(recarga.getCoste()).andReturn(10.0).anyTimes();
        expect(recarga.getKm()).andReturn(km).anyTimes();
        expect(recarga.getCombustible()).andReturn("0").anyTimes();
        expect(recarga.getMatricula()).andReturn("0").anyTimes();
        expect(modelo.getVehiculo(isA(String.class))).andReturn(dtoVehiculo);
         expect(modelo.getDDVehiculo( "0" ,Calendar.getInstance().get(Calendar.YEAR) ,Calendar.getInstance().get(Calendar.MONTH))).andReturn(null);
        expect(modelo.getDDVCombustible("0" ,Calendar.getInstance().get(Calendar.YEAR),Calendar.getInstance().get(Calendar.MONTH) , "0"  )).andReturn(null);
        expect(modelo.isPrimerRegistroDatosDerivadosCombustible("0" ,  "0" )).andReturn(false).times(1);
        expect(modelo.getDDVCostesCombustible("0" ,Calendar.getInstance().get(Calendar.YEAR) , Calendar.getInstance().get(Calendar.MONTH)   , "0" )).andReturn(null);


        formateador.setRegistroCompletado(true);
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        formateador.isCamposRegistroCompletos();
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return true;
            }
        });
        modelo.registraRecarga(recarga);
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });

         modelo.addDatosDerivadosCombustible(isA(DTODatosDerivadosCombustible.class));
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });

        modelo.addDatosDerivadosCosteCombustible(isA(DTODatosDerivadosCostesCombustible.class));
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        modelo.addDatosDerivadosVehiculo(isA(DTODatosDerivadosVehiculoTotales.class));
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });




        replay(modelo);
        replay(formateador);
        replay(recarga);
        replay(recarga2);
        replay(dtoVehiculo);
        replay(dtoDatosDerivadosCombustible);
        replay(dtoDatosDerivadosCostesCombustible);
        replay(dtoDatosDerivadosVehiculo);
        controlador.setFormateador(formateador);
        controlador.setModel(modelo);
        controlador.registrarRecarga(recarga);
        verify(recarga);
        verify(recarga2);
        verify(modelo);
        verify(formateador);
        verify(dtoVehiculo);

    }





    @Test
    public void registrarRecargaAnteriorEsNullRegistraDatosDerivadosConCambioAnio(){
        double km =1.0;


        Calendar futuro= Calendar.getInstance();
        int mes,anio;
        Date ahora = new Date();
        anio=ahora.getYear();
        mes=ahora.getMonth();
        futuro.set(anio+1,mes+1,1);

        expect(dtoDatosDerivadosVehiculo.getCosteRecargasTotalesPorAnio()).andReturn(0.0);
        expect(dtoDatosDerivadosVehiculo.getKmRecargasTotalesPorAnio()).andReturn(0.0);
        expect(dtoDatosDerivadosVehiculo.getNumeroRecargasAnio()).andReturn(0);
        expect(dtoDatosDerivadosCostesCombustible.getSumaCosteXRecargaAnual()).andReturn(0.0);

        expect(dtoDatosDerivadosCostesCombustible.getSumaCosteRecargaAnual()).andReturn(0.0);
        expect(dtoDatosDerivadosCostesCombustible.getNumeroRecargasCombustiblePorAnio()).andReturn(0);
        expect(dtoDatosDerivadosCombustible.getSumaCombustibleXKmAnual()).andReturn(0.0);
        expect(dtoDatosDerivadosCombustible.getSumaCombustibleRecargadoPorAnio()).andReturn(0.0);
        expect(dtoDatosDerivadosCombustible.getNumeroRecargasCombustiblePorAnio()).andReturn(0);
        expect(dtoDatosDerivadosCombustible.getMinimoCombustibleAnualRecargado()).andReturn(0.0).times(2);
        expect(dtoDatosDerivadosCombustible.getMinimoCombustibleXKmAnual()).andReturn(0.0).times(2);
        expect(dtoDatosDerivadosCombustible.getMaximoCombustibleXKmAnual()).andReturn(0.0);
        expect(dtoDatosDerivadosCombustible.getMaximoCombustibleAnualRecargado()).andReturn(0.0);
        expect(dtoDatosDerivadosCostesCombustible.getMaximoCosteRecargaAnual()).andReturn(0.0);

        expect(dtoDatosDerivadosCostesCombustible.getMinimoCosteRecargaAnual()).andReturn(0.0).times(2);
        expect(dtoDatosDerivadosCombustible.getAnio()).andReturn(Calendar.getInstance().get(Calendar.YEAR)+1);
        expect(dtoDatosDerivadosVehiculo.getAnio()).andReturn(Calendar.getInstance().get(Calendar.YEAR)+1);
        expect(dtoDatosDerivadosCostesCombustible.getAnio()).andReturn(Calendar.getInstance().get(Calendar.YEAR)+1);
        expect(modelo.isPrimerRegistroDatosDerivadosVehiculo(isA(String.class))).andReturn(false);
        //expect(modelo. isPrimerRegistroDatosDerivadosCombustible(isA(String.class),isA(String.class))).andReturn(false);
        expect(modelo. isPrimerRegistroDatosDerivadosCosteCombustible(isA(String.class),isA(String.class))).andReturn(false);
         expect(modelo.getLastRegistroDatosDerivadosVehiculo("0" )).andReturn(dtoDatosDerivadosVehiculo);
        expect(modelo.getLastRegistroDatosDerivadosCombustible("0","0")).andReturn(dtoDatosDerivadosCombustible);
        expect(modelo.getLastRegistroDatosDerivadosCostesCombustible("0","0")).andReturn(dtoDatosDerivadosCostesCombustible);
        expect(modelo.getUltimaRecargaRegistrada(isA(String.class),isA(Date.class))).andReturn(null);
       // expect(recarga2.getFecha()).andReturn(futuro.getTime());
        expect(recarga2.getKm()).andReturn(0.5);
        expect(modelo.getUltimaRecargaRegistradaMismoCombustible(isA(String.class),isA(String.class))).andReturn(recarga2);
        expect(dtoVehiculo.getCapTankSecundario()).andReturn(1.0);
        expect(dtoVehiculo.getCombustiblePrimario()).andReturn("aaa");
        expect(recarga.getFecha()).andReturn(ahora).times(2);
        expect(recarga.getMedicion()).andReturn(1.0).anyTimes();
        expect(recarga.getCoste()).andReturn(10.0).anyTimes();
        expect(recarga.getKm()).andReturn(km).anyTimes();
        expect(recarga.getCombustible()).andReturn("0").anyTimes();
        expect(recarga.getMatricula()).andReturn("0").anyTimes();
        expect(modelo.getVehiculo(isA(String.class))).andReturn(dtoVehiculo);
        expect(modelo.getDDVehiculo( "0" ,Calendar.getInstance().get(Calendar.YEAR) ,Calendar.getInstance().get(Calendar.MONTH))).andReturn(null);
        expect(modelo.getDDVCombustible("0" ,Calendar.getInstance().get(Calendar.YEAR),Calendar.getInstance().get(Calendar.MONTH) , "0"  )).andReturn(null);
        expect(modelo.isPrimerRegistroDatosDerivadosCombustible("0" ,  "0" )).andReturn(false).times(1);
        expect(modelo.getDDVCostesCombustible("0" ,Calendar.getInstance().get(Calendar.YEAR) , Calendar.getInstance().get(Calendar.MONTH)   , "0" )).andReturn(null);

        /* formateador.setMsgError(isA(String.class)) ;
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });*/
        formateador.setRegistroCompletado(true);
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        formateador.isCamposRegistroCompletos();
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return true;
            }
        });
        modelo.registraRecarga(recarga);
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });

        modelo.addDatosDerivadosCombustible(isA(DTODatosDerivadosCombustible.class));
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
      /*  modelo.addDatosDerivadosCombustible(isA(DTODatosDerivadosCombustible.class));
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });*/
        modelo.addDatosDerivadosCosteCombustible(isA(DTODatosDerivadosCostesCombustible.class));
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
         modelo.addDatosDerivadosVehiculo(isA(DTODatosDerivadosVehiculoTotales.class));
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });




        replay(modelo);
        replay(formateador);
        replay(recarga);
        replay(dtoVehiculo);
        replay(dtoDatosDerivadosCombustible);
        replay(dtoDatosDerivadosCostesCombustible);
        replay(dtoDatosDerivadosVehiculo);
        replay(recarga2);
        controlador.setFormateador(formateador);
        controlador.setModel(modelo);
        controlador.registrarRecarga(recarga);
        verify(recarga);
        verify(recarga2);
        verify(modelo);
        verify(formateador);
        verify(dtoVehiculo);

    }






}