package com.example.juanalonsoastruga.cargascontrol.Controlador.CURegistrarVehiculo;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTOCombustible;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTOVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Formateador.CURegistrarVehiculo.FormateadorRegistroVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Modelo.CURegistrarVehiculo.InterfaceMRegistrarVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Modelo.CURegistrarVehiculo.ModeloRegistroVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Formateador.CURegistrarVehiculo.InterfaceFRegistroVehiculo;

import org.easymock.Mock;
import org.junit.Before;
import org.junit.Test;

import java.util.ArrayList;
import java.util.List;

import static org.easymock.EasyMock.createMock;
import static org.easymock.EasyMock.expect;
import static org.easymock.EasyMock.isA;
import static org.easymock.EasyMock.replay;
import static org.easymock.EasyMock.verify;
import static org.junit.Assert.*;

public class ControladorRegistroVehiculoTest {

    private ControladorRegistroVehiculo controlador;

    @Mock
    private InterfaceFRegistroVehiculo formateador;
    @Mock
    private InterfaceMRegistrarVehiculo model;
    @Mock
    private List<DTOCombustible> combustibles;
    @Mock
    private DTOVehiculo vehiculo;

    @Before
    public void setUp(){
        formateador =createMock(FormateadorRegistroVehiculo.class);
        model=createMock(ModeloRegistroVehiculo.class);
        vehiculo=createMock(DTOVehiculo.class);
        combustibles= createMock(ArrayList.class );
        controlador = new ControladorRegistroVehiculo(formateador);
    }
    @Test(expected = IllegalArgumentException.class)
    public void setViewNull(){
        controlador.setFormateador(null);
    }
    @Test
    public void setViewCorrect(){
        controlador.setFormateador(formateador);
        assertNotNull(controlador.getFormateador());
    }
    @Test(expected = IllegalArgumentException.class)
    public void setModeloNull(){
        controlador.setModel(null);
    }
    @Test
    public void setModeloCorrect(){
        controlador.setModel(model);
        assertNotNull(controlador.getModel());
    }

    @Test
    public void rellenarCampoIncorrectamenteAno(){

        String msj="Campo Año incorrectamente completado, reinténtelo por favor";
        expect(formateador.setMensajeError(msj)).andReturn(true);
        replay(formateador);
        controlador.setFormateador(formateador);
        controlador.rellenarCampoIncorrectamente("año");
        verify(formateador);

    }
    @Test
    public void rellenarCampoIncorrectamenteKM(){

        String msj="Campo Kilometraje incorrectamente completado,\n reinténtelo por favor";
        expect(formateador.setMensajeError(msj)).andReturn(true);
        replay(formateador);
        controlador.setFormateador(formateador);
        controlador.rellenarCampoIncorrectamente("km");
        verify(formateador);

    }
    @Test
    public void rellenarCampoIncorrectamenteCP(){

        String msj="Campo Capacidad de Carga de Combustible Principal  incorrectamente\n completado,reinténtelo por favor";
        expect(formateador.setMensajeError(msj)).andReturn(true);
        replay(formateador);
        controlador.setFormateador(formateador);
        controlador.rellenarCampoIncorrectamente("CTP");
        verify(formateador);

    }
    @Test
    public void rellenarCampoIncorrectamenteCS(){

        String msj="Campo Capacidad de Carga de Combustible Secundario incorrectamente\n  completado,reinténtelo por favor";
        expect(formateador.setMensajeError(msj)).andReturn(true);
        replay(formateador);
        controlador.setFormateador(formateador);
        controlador.rellenarCampoIncorrectamente("CTS");
        verify(formateador);

    }

    @Test
    public void solicitarGuardadoVehiculoNoRegistrado(){
        expect(formateador.getRegistro()).andReturn(vehiculo);
        expect(vehiculo.getMatricula()).andReturn("1").times(2);
        expect(vehiculo.getNombre()).andReturn("1").times(2);
        expect(vehiculo.getCombustiblePrimario()).andReturn("1").times(3);
        expect(vehiculo.getCapTankPrimario()).andReturn(0.0) ;
        expect(model.isVehiculoRegistrado(vehiculo)).andReturn(false);

        expect(formateador.setMensajeError(isA(String.class))).andReturn(true);
        expect(formateador.setRegistroCompletado( false )).andReturn(true);
        replay(formateador);
        replay(model);
        replay(vehiculo);

        controlador.setModel(model);
        controlador.setFormateador(formateador);

        controlador.solicitarGuardado();

        verify(vehiculo);
        verify(formateador);
        verify(model);
    }

    @Test
    public void solicitarGuardadoVehiculoRegistrado(){
        expect(formateador.getRegistro()).andReturn(vehiculo);
        expect(vehiculo.getMatricula()).andReturn("1").times(2);
        expect(vehiculo.getNombre()).andReturn("1").times(2);
        expect(vehiculo.getCombustiblePrimario()).andReturn("1").times(3);
        expect(model.isVehiculoRegistrado(vehiculo)).andReturn(true);
        expect(formateador.setRegistroCompletado(false)).andReturn(true);
       expect(formateador.setMensajeError(isA(String.class))).andReturn(true);
        expect(vehiculo.getCapTankPrimario()).andReturn(10.0);
        expect(vehiculo.getCombustibleSecundario()).andReturn("");
        expect(vehiculo.getCapTankSecundario()).andReturn(0.0);
        replay(formateador);
        replay(model);
        replay(vehiculo);

        controlador.setModel(model);
        controlador.setFormateador(formateador);

        controlador.solicitarGuardado();

        verify(vehiculo);
        verify(formateador);
        verify(model);
    }

    @Test
    public void solicitarGuardadoVehiculoMatriculaIncorrecta(){
        expect(formateador.getRegistro()).andReturn(vehiculo);
        expect(vehiculo.getMatricula()).andReturn("").times(2);

        expect(formateador.setMensajeError("No se han rellenado campos obligatotios \n (nombre, matricula, combustible primario)")).andReturn(true);
        expect(formateador.setRegistroCompletado(false)).andReturn(true);

        replay(formateador);

        replay(vehiculo);

        controlador.setModel(model);
        controlador.setFormateador(formateador);

        controlador.solicitarGuardado();

        verify(vehiculo);
        verify(formateador);

    }

    @Test
    public void solicitarGuardadoVehiculoNombreIncorrecta(){
        expect(formateador.getRegistro()).andReturn(vehiculo);
        expect(vehiculo.getMatricula()).andReturn("1").times(2);
        expect(vehiculo.getNombre()).andReturn("").times(2);

        expect(formateador.setMensajeError("No se han rellenado campos obligatotios \n (nombre, matricula, combustible primario)")).andReturn(true);
        expect(formateador.setRegistroCompletado(false)).andReturn(true);

        replay(formateador);

        replay(vehiculo);

        controlador.setModel(model);
        controlador.setFormateador(formateador);

        controlador.solicitarGuardado();

        verify(vehiculo);
        verify(formateador);

    }

    @Test
    public void solicitarGuardadoVehiculoCPrimarioIncorrecto(){
        expect(formateador.getRegistro()).andReturn(vehiculo);
        expect(vehiculo.getMatricula()).andReturn("1").times(2);
        expect(vehiculo.getNombre()).andReturn("1").times(2);
        expect(vehiculo.getCombustiblePrimario()).andReturn("").times(2);
        expect(formateador.setMensajeError("No se han rellenado campos obligatotios \n (nombre, matricula, combustible primario)")).andReturn(true);
        expect(formateador.setRegistroCompletado(false)).andReturn(true);

        replay(formateador);

        replay(vehiculo);

        controlador.setModel(model);
        controlador.setFormateador(formateador);

        controlador.solicitarGuardado();

        verify(vehiculo);
        verify(formateador);

    }

    @Test
    public void solicitarGuardadoVehiculoMatriculaNula(){
        expect(formateador.getRegistro()).andReturn(vehiculo);
        expect(vehiculo.getMatricula()).andReturn(null).times(1);

        expect(formateador.setMensajeError("No se han rellenado campos obligatotios \n (nombre, matricula, combustible primario)")).andReturn(true);
        expect(formateador.setRegistroCompletado(false)).andReturn(true);

        replay(formateador);

        replay(vehiculo);

        controlador.setModel(model);
        controlador.setFormateador(formateador);

        controlador.solicitarGuardado();

        verify(vehiculo);
        verify(formateador);

    }


    @Test
    public void solicitarGuardadoVehiculoNombreNulo(){
        expect(formateador.getRegistro()).andReturn(vehiculo);
        expect(vehiculo.getMatricula()).andReturn("1").times(2);
        expect(vehiculo.getNombre()).andReturn(null).times(1);

        expect(formateador.setMensajeError("No se han rellenado campos obligatotios \n (nombre, matricula, combustible primario)")).andReturn(true);
        expect(formateador.setRegistroCompletado(false)).andReturn(true);

        replay(formateador);

        replay(vehiculo);

        controlador.setModel(model);
        controlador.setFormateador(formateador);

        controlador.solicitarGuardado();

        verify(vehiculo);
        verify(formateador);

    }
    @Test
    public void solicitarGuardadoVehiculoCPrimarioNulo(){
        expect(formateador.getRegistro()).andReturn(vehiculo);
        expect(vehiculo.getMatricula()).andReturn("1").times(2);
        expect(vehiculo.getNombre()).andReturn("1").times(2);
        expect(vehiculo.getCombustiblePrimario()).andReturn(null).times(1);
        expect(formateador.setMensajeError("No se han rellenado campos obligatotios \n (nombre, matricula, combustible primario)")).andReturn(true);
        expect(formateador.setRegistroCompletado(false)).andReturn(true);

        replay(formateador);

        replay(vehiculo);

        controlador.setModel(model);
        controlador.setFormateador(formateador);

        controlador.solicitarGuardado();

        verify(vehiculo);
        verify(formateador);

    }
}