package com.example.juanalonsoastruga.cargascontrol.Formateador.CUConsultarEstadisticaCombustible;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTODatosDerivadosCombustible;
import com.example.juanalonsoastruga.cargascontrol.Controlador.CUConsultarEstadisticaCombustible.ControladorConsultarEstadisticaCombustible;
import com.example.juanalonsoastruga.cargascontrol.Controlador.CUConsultarEstadisticaCombustible.InterfaceCConsultarEstadisticaCombustible;

import org.easymock.EasyMock;
import org.easymock.IAnswer;
import org.easymock.Mock;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

import static org.easymock.EasyMock.createMock;
import static org.easymock.EasyMock.expect;
import static org.easymock.EasyMock.replay;
import static org.easymock.EasyMock.verify;

public class FormateadorConsultarEstadisticaCombustibleTest {

    private FormateadorConsultarEstadisticaCombustible formateador;
    @Mock
    private InterfaceCConsultarEstadisticaCombustible controlador;
    @Mock
    private DTODatosDerivadosCombustible dtodatosCombustible;

    @Before
    public void setUp(){
        formateador = new FormateadorConsultarEstadisticaCombustible();
        controlador = createMock(ControladorConsultarEstadisticaCombustible.class);
        dtodatosCombustible = createMock(DTODatosDerivadosCombustible.class);
    }


    @Test(expected=IllegalArgumentException.class)
    public void setPresentadorNull(){
        formateador.setControlador(null);
    }
    @Test
    public void setPresentadorNotNull(){
        formateador.setControlador(controlador);
        Assert.assertNotNull(formateador.getControlador());
        Assert.assertEquals(formateador.getControlador(), controlador);
    }
    @Test
    public void comenzarCU(){
        controlador.comienzoCU("",0);
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        replay(controlador);
        formateador.setControlador(controlador);
        formateador.comienzaCU("",0);
        verify(controlador);
    }



    @Test(expected = IllegalStateException.class)
    public void getMatriculaSinDatos(){
        formateador.getMatricula();
    }
    @Test
    public void getMatriculaConDatos(){
        expect(dtodatosCombustible.getMatriculaVehiculo()).andReturn("");
        replay(dtodatosCombustible);
        formateador.setControlador(controlador);
        formateador.setDatosDerivadosCombustible(dtodatosCombustible);
        Assert.assertEquals( formateador.getMatricula(),"");
        verify(dtodatosCombustible);
    }



    @Test(expected = IllegalStateException.class)
    public void getMesSinDatos(){
        formateador.getMes();
    }
    @Test
    public void getMesConDatos(){
        expect(dtodatosCombustible .getMes()).andReturn(0);
        replay(dtodatosCombustible);
        formateador.setControlador(controlador);
        formateador.setDatosDerivadosCombustible(dtodatosCombustible);
        Assert.assertEquals( formateador.getMes(),1);
        verify(dtodatosCombustible);
    }



    @Test(expected = IllegalStateException.class)
    public void getAnioSinDatos(){
        formateador.getAnio();
    }
    @Test
    public void getAnioConDatos(){
        expect(dtodatosCombustible .getAnio()).andReturn(0);
        replay(dtodatosCombustible);
        formateador.setControlador(controlador);
        formateador.setDatosDerivadosCombustible(dtodatosCombustible);
        Assert.assertEquals( formateador.getAnio(),0);
        verify(dtodatosCombustible);
    }



    @Test(expected = IllegalStateException.class)
    public void getNumerosRecargasMesSinDatos(){
        formateador.getNumerosRecargasMes();
    }
    @Test
    public void getNumerosRecargasMesConDatos(){
        expect(dtodatosCombustible.getNumeroRecargasCombustiblePorMes()).andReturn(0);
        replay(dtodatosCombustible);
        formateador.setControlador(controlador);
        formateador.setDatosDerivadosCombustible(dtodatosCombustible);
        Assert.assertEquals( formateador.getNumerosRecargasMes(),0);
        verify(dtodatosCombustible);
    }



    @Test(expected = IllegalStateException.class)
    public void getNumerosRecargasAnioSinDatos(){
        formateador.getNumerosRecargasAnio();
    }
    @Test
    public void getNumerosRecargasAnioConDatos(){
        expect(dtodatosCombustible.getNumeroRecargasCombustiblePorAnio()).andReturn(0);
        replay(dtodatosCombustible);
        formateador.setControlador(controlador);
        formateador.setDatosDerivadosCombustible(dtodatosCombustible);
        Assert.assertEquals( formateador.getNumerosRecargasAnio(),0);
        verify(dtodatosCombustible);
    }



    @Test(expected = IllegalStateException.class)
    public void getMedicionCombustibleMesSinDatos(){
        formateador.getMedicionCombustibleMes();
    }
    @Test
    public void getMedicionCombustibleMesConDatos(){
        expect(dtodatosCombustible.getSumaCombustibleRecargadoPorMes()).andReturn(0.0);
        replay(dtodatosCombustible);
        formateador.setControlador(controlador);
        formateador.setDatosDerivadosCombustible(dtodatosCombustible);
        Assert.assertEquals( formateador.getMedicionCombustibleMes(),0.0,0,0);
        verify(dtodatosCombustible);
    }



    @Test(expected = IllegalStateException.class)
    public void getMediaMedicionCombustibleMesSinDatos(){
        formateador.getMedicionCombustibleMes();
    }
    @Test
    public void getMediaMedicionCombustibleMesConDatos(){
        expect(dtodatosCombustible.getMediaCombustibleMensualRecargado()).andReturn(0.0);
        replay(dtodatosCombustible);
        formateador.setControlador(controlador);
        formateador.setDatosDerivadosCombustible(dtodatosCombustible);
        Assert.assertEquals(formateador.getMediaMedicionCombustibleMes(),"0");
        verify(dtodatosCombustible);
    }





    @Test(expected = IllegalStateException.class)
    public void getMedicionCombustibleAnioSinDatos(){
        formateador.getMedicionCombustibleAnio();
    }
    @Test
    public void getMedicionCombustibleAnioConDatos(){
        expect(dtodatosCombustible.getSumaCombustibleRecargadoPorAnio()).andReturn(0.0);
        replay(dtodatosCombustible);
        formateador.setControlador(controlador);
        formateador.setDatosDerivadosCombustible(dtodatosCombustible);
        Assert.assertEquals( formateador.getMedicionCombustibleAnio(),0.0,0,0);
        verify(dtodatosCombustible);
    }





    @Test(expected = IllegalStateException.class)
    public void getMediaMedicionCombustibleAnioSinDatos(){
        formateador.getMediaMedicionCombustibleAnio();
    }
    @Test
    public void getMediaMedicionCombustibleAnioConDatos(){
        expect(dtodatosCombustible.getMediaCombustibleAnualRecargado()).andReturn(0.0);
        replay(dtodatosCombustible);
        formateador.setControlador(controlador);
        formateador.setDatosDerivadosCombustible(dtodatosCombustible);
        Assert.assertEquals( formateador.getMediaMedicionCombustibleAnio(),"0");
        verify(dtodatosCombustible);
    }





    @Test(expected = IllegalStateException.class)
    public void getMinimoCombustibleMesSinDatos(){
        formateador.getMinimoCombustibleMes();
    }
    @Test
    public void getMinimoCombustibleMesConDatos(){
        expect(dtodatosCombustible.getMinimoCombustibleMensualRecargado()).andReturn(0.0);
        replay(dtodatosCombustible);
        formateador.setControlador(controlador);
        formateador.setDatosDerivadosCombustible(dtodatosCombustible);
        Assert.assertEquals( formateador.getMinimoCombustibleMes(),0,0);
        verify(dtodatosCombustible);
    }





    @Test(expected = IllegalStateException.class)
    public void  getMaximoCombustibleMesSinDatos(){
        formateador.getMinimoCombustibleMes();
    }
    @Test
    public void  getMaximoCombustibleMesConDatos(){
        expect(dtodatosCombustible.getMaximoCombustibleMensualRecargado()).andReturn(0.0);
        replay(dtodatosCombustible);
        formateador.setControlador(controlador);
        formateador.setDatosDerivadosCombustible(dtodatosCombustible);
        Assert.assertEquals( formateador. getMaximoCombustibleMes(),0.0,0,0);
        verify(dtodatosCombustible);
    }





    @Test(expected = IllegalStateException.class)
    public void  getMinimoCombustibleAnioSinDatos(){
        formateador.getMinimoCombustibleAnio();
    }
    @Test
    public void  getMinimoCombustibleAnioConDatos(){
        expect(dtodatosCombustible.getMinimoCombustibleAnualRecargado()).andReturn(0.0);
        replay(dtodatosCombustible);
        formateador.setControlador(controlador);
        formateador.setDatosDerivadosCombustible(dtodatosCombustible);
        Assert.assertEquals( formateador.getMinimoCombustibleAnio(),0.0,0,0);
        verify(dtodatosCombustible);
    }





    @Test(expected = IllegalStateException.class)
    public void  getMaximoCombustibleAnioSinDatos(){
        formateador.getMaximoCombustibleAnio();
    }
    @Test
    public void  getMaximoCombustibleAnioConDatos(){
        expect(dtodatosCombustible.getMaximoCombustibleAnualRecargado()).andReturn(0.0);
        replay(dtodatosCombustible);
        formateador.setControlador(controlador);
        formateador.setDatosDerivadosCombustible(dtodatosCombustible);
        Assert.assertEquals( formateador.getMaximoCombustibleAnio(),0.0,0,0);
        verify(dtodatosCombustible);
    }





    @Test(expected = IllegalStateException.class)
    public void  getMediaRecargasXKmCombustibleMesSinDatos(){
        formateador.getMediaRecargasXKmCombustibleMes();
    }
    @Test
    public void  getMediaRecargasXKmCombustibleMesConDatos(){
        expect(dtodatosCombustible.getMediaCombustibleXKmMensual()).andReturn(0.0);
        replay(dtodatosCombustible);
        formateador.setControlador(controlador);
        formateador.setDatosDerivadosCombustible(dtodatosCombustible);
        Assert.assertEquals( formateador.getMediaRecargasXKmCombustibleMes(),"0");
        verify(dtodatosCombustible);
    }





    @Test(expected = IllegalStateException.class)
    public void  getMediaRecargasXKmCombustibleAnioSinDatos(){
        formateador.getMediaRecargasXKmCombustibleAnio();
    }
    @Test
    public void  getMediaRecargasXKmCombustibleAnioConDatos(){
        expect(dtodatosCombustible.getMediaCombustibleXKmAnual()).andReturn(0.0);
        replay(dtodatosCombustible);
        formateador.setControlador(controlador);
        formateador.setDatosDerivadosCombustible(dtodatosCombustible);
        Assert.assertEquals( formateador.getMediaRecargasXKmCombustibleAnio(),"0");
        verify(dtodatosCombustible);
    }





    @Test(expected = IllegalStateException.class)
    public void  getMinimoRecargasXKmCombustibleMesSinDatos(){
        formateador.getMinimoRecargasXKmCombustibleMes();
    }
    @Test
    public void  getMinimoRecargasXKmCombustibleMesConDatos(){
        expect(dtodatosCombustible.getMinimoCombustibleXKmMensual()).andReturn(0.0);
        replay(dtodatosCombustible);
        formateador.setControlador(controlador);
        formateador.setDatosDerivadosCombustible(dtodatosCombustible);
        Assert.assertEquals( formateador.getMinimoRecargasXKmCombustibleMes(),"0");
        verify(dtodatosCombustible);
    }





    @Test(expected = IllegalStateException.class)
    public void  getMaximoRecargasXKmCombustibleMesSinDatos(){
        formateador.getMaximoRecargasXKmCombustibleMes();
    }
    @Test
    public void  getMaximoRecargasXKmCombustibleMesConDatos(){
        expect(dtodatosCombustible.getMaximoCombustibleXKmMensual()).andReturn(0.0);
        replay(dtodatosCombustible);
        formateador.setControlador(controlador);
        formateador.setDatosDerivadosCombustible(dtodatosCombustible);
        Assert.assertEquals( formateador.getMaximoRecargasXKmCombustibleMes(),"0");
        verify(dtodatosCombustible);
    }





    @Test(expected = IllegalStateException.class)
    public void  getMinimoRecargasXKmCombustibleAnioSinDatos(){
        formateador.getMinimoRecargasXKmCombustibleAnio();
    }
    @Test
    public void  getMinimoRecargasXKmCombustibleAnioConDatos(){
        expect(dtodatosCombustible.getMinimoCombustibleXKmAnual()).andReturn(0.0);
        replay(dtodatosCombustible);
        formateador.setControlador(controlador);
        formateador.setDatosDerivadosCombustible(dtodatosCombustible);
        Assert.assertEquals( formateador.getMinimoRecargasXKmCombustibleAnio(),"0");
        verify(dtodatosCombustible);
    }






    @Test(expected = IllegalStateException.class)
    public void  getMaximoRecargasXKmCombustibleAnioSinDatos(){
        formateador.getMaximoRecargasXKmCombustibleAnio();
    }
    @Test
    public void  getMaximoRecargasXKmCombustibleAnioConDatos(){
        expect(dtodatosCombustible.getMaximoCombustibleXKmAnual()).andReturn(0.0);
        replay(dtodatosCombustible);
        formateador.setControlador(controlador);
        formateador.setDatosDerivadosCombustible(dtodatosCombustible);
        Assert.assertEquals( formateador.getMaximoRecargasXKmCombustibleAnio(),"0");
        verify(dtodatosCombustible);
    }

    @Test
    public void setDatosDerivadosCombustible(){
         formateador.setDatosDerivadosCombustible(dtodatosCombustible);
        Assert.assertEquals( formateador.getDatos(),dtodatosCombustible);

    }
}