package com.example.juanalonsoastruga.cargascontrol.Formateador.CUConsultarEstadisticaCostesCombustible;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTODatosDerivadosCostesCombustible;
import com.example.juanalonsoastruga.cargascontrol.Controlador.CUConsultarEstadisticaCostesCombustible.ControladorConsultarEstadisticaCosteCombustible;
import com.example.juanalonsoastruga.cargascontrol.Controlador.CUConsultarEstadisticaCostesCombustible.InterfaceCConsultarEstadisticaCosteCombustible;

import org.easymock.EasyMock;
import org.easymock.IAnswer;
import org.easymock.Mock;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

import static org.easymock.EasyMock.createMock;
import static org.easymock.EasyMock.expect;
import static org.easymock.EasyMock.replay;
import static org.easymock.EasyMock.verify;

public class FormateadorConsultarEstadisticaCosteCombustibleTest {

    private FormateadorConsultarEstadisticaCosteCombustible formateador;
    @Mock
    private InterfaceCConsultarEstadisticaCosteCombustible controlador;
    @Mock
    private DTODatosDerivadosCostesCombustible dtodatosCosteCombustible;

    @Before
    public void setUp(){
        formateador =new FormateadorConsultarEstadisticaCosteCombustible();
        controlador =createMock(ControladorConsultarEstadisticaCosteCombustible.class);
        dtodatosCosteCombustible=createMock(DTODatosDerivadosCostesCombustible.class);
    }

   @Test(expected = IllegalArgumentException.class)
    public void setPresenterNull(){
        formateador.setControlador(null);
    }

    @Test
    public void comienzaCu(){
         controlador.comienzaCU("",0);
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        replay(controlador);
        formateador.setControlador(controlador);
        formateador.comienzaCU("",0);
        verify(controlador);
    }

    @Test(expected = IllegalStateException.class)
    public void getMatriculaSinDatos(){
        formateador.getMatriculaVehiculo();
    }
    @Test
    public void getMatriculaConDatos(){
        expect(dtodatosCosteCombustible.getMatriculaVehiculo()).andReturn("");
        replay(dtodatosCosteCombustible);
        formateador.setControlador(controlador);
        formateador.setDatosDerivadosCosteCombustible(dtodatosCosteCombustible);
        Assert.assertEquals( formateador.getMatriculaVehiculo(),"");
        verify(dtodatosCosteCombustible);
    }
    @Test(expected = IllegalStateException.class)
    public void getMesSinDatos(){
        formateador.getMes();
    }
    @Test
    public void getMesConDatos(){
        expect(dtodatosCosteCombustible .getMes()).andReturn(0);
        replay(dtodatosCosteCombustible);
        formateador.setControlador(controlador);
        formateador.setDatosDerivadosCosteCombustible(dtodatosCosteCombustible);
        Assert.assertEquals( formateador.getMes(),1);
        verify(dtodatosCosteCombustible);
    }



    @Test(expected = IllegalStateException.class)
    public void getAnioSinDatos(){
        formateador.getAnio();
    }
    @Test
    public void getAnioConDatos(){
        expect(dtodatosCosteCombustible .getAnio()).andReturn(0);
        replay(dtodatosCosteCombustible);
        formateador.setControlador(controlador);
        formateador.setDatosDerivadosCosteCombustible(dtodatosCosteCombustible);
        Assert.assertEquals( formateador.getAnio(),0);
        verify(dtodatosCosteCombustible);
    }



    @Test(expected = IllegalStateException.class)
    public void getNumerosRecargasMesSinDatos(){
        formateador.getNumeroRecargasMes();
    }
    @Test
    public void getNumerosRecargasMesConDatos(){
        expect(dtodatosCosteCombustible.getNumeroRecargasCombustiblePorMes()).andReturn(0);
        replay(dtodatosCosteCombustible);
        formateador.setControlador(controlador);
        formateador.setDatosDerivadosCosteCombustible(dtodatosCosteCombustible);
        Assert.assertEquals( formateador.getNumeroRecargasMes(),0);
        verify(dtodatosCosteCombustible);
    }



    @Test(expected = IllegalStateException.class)
    public void getNumerosRecargasAnioSinDatos(){
        formateador.getNumeroRecargasAnio();
    }
    @Test
    public void getNumerosRecargasAnioConDatos(){
        expect(dtodatosCosteCombustible.getNumeroRecargasCombustiblePorAnio()).andReturn(0);
        replay(dtodatosCosteCombustible);
        formateador.setControlador(controlador);
        formateador.setDatosDerivadosCosteCombustible(dtodatosCosteCombustible);
        Assert.assertEquals( formateador.getNumeroRecargasAnio(),0);
        verify(dtodatosCosteCombustible);
    }


    @Test(expected = IllegalStateException.class)
    public void getCosteRecargasMesSinDatos(){

        formateador.getCosteRecargasMes();
    }
    @Test
    public void getCosteRecargasMesConDatos(){
        expect(dtodatosCosteCombustible.getSumaCosteRecargaMensual()).andReturn(0.0);
        replay(dtodatosCosteCombustible);
        formateador.setControlador(controlador);
        formateador.setDatosDerivadosCosteCombustible(dtodatosCosteCombustible);
        Assert.assertEquals( formateador.getCosteRecargasMes(),"0");
        verify(dtodatosCosteCombustible);
    }

    @Test(expected = IllegalStateException.class)
    public void getCosteRecargasAnioSinDatos(){

        formateador.getCosteRecargasAnio();
    }
    @Test
    public void getCosteRecargasAnioConDatos(){
        expect(dtodatosCosteCombustible.getSumaCosteRecargaAnual()).andReturn(0.0);
        replay(dtodatosCosteCombustible);
        formateador.setControlador(controlador);
        formateador.setDatosDerivadosCosteCombustible(dtodatosCosteCombustible);
        Assert.assertEquals( formateador.getCosteRecargasAnio(),"0");
        verify(dtodatosCosteCombustible);
    }
    @Test(expected = IllegalStateException.class)
    public void getMediaCosteRecargasMesSinDatos(){

        formateador.getMediaCosteRecargasMes();
    }
    @Test
    public void getMediaCosteRecargasMesConDatos(){
        expect(dtodatosCosteCombustible.getMediaCosteRecargaMensual()).andReturn(0.0);
        replay(dtodatosCosteCombustible);
        formateador.setControlador(controlador);
        formateador.setDatosDerivadosCosteCombustible(dtodatosCosteCombustible);
        Assert.assertEquals( formateador.getMediaCosteRecargasMes(),"0");
        verify(dtodatosCosteCombustible);
    }
    @Test(expected = IllegalStateException.class)
    public void getMediaCosteRecargasAnioSinDatos(){

        formateador.getMediaCosteRecargasAnio();
    }
    @Test
    public void getMediaCosteRecargasAnioConDatos(){
        expect(dtodatosCosteCombustible.getMediaCosteRecargaAnual()).andReturn(0.0);
        replay(dtodatosCosteCombustible);
        formateador.setControlador(controlador);
        formateador.setDatosDerivadosCosteCombustible(dtodatosCosteCombustible);
        Assert.assertEquals( formateador.getMediaCosteRecargasAnio(),"0");
        verify(dtodatosCosteCombustible);
    }
    @Test(expected = IllegalStateException.class)
    public void getMinimoCosteRecargasMesSinDatos(){

        formateador.getMinimoCosteRecargasMes();
    }
    @Test
    public void getMinimoCosteRecargasMesConDatos(){
        expect(dtodatosCosteCombustible.getMinimoCosteRecargaMensual()).andReturn(0.0);
        replay(dtodatosCosteCombustible);
        formateador.setControlador(controlador);
        formateador.setDatosDerivadosCosteCombustible(dtodatosCosteCombustible);
        Assert.assertEquals( formateador.getMinimoCosteRecargasMes(),"0");
        verify(dtodatosCosteCombustible);
    }

    @Test(expected = IllegalStateException.class)
    public void getMaximoCosteRecargasMesSinDatos(){

        formateador.getMaximoCosteRecargasMes();
    }
    @Test
    public void getMaximoCosteRecargasMesConDatos(){
        expect(dtodatosCosteCombustible.getMaximoCosteRecargaMensual()).andReturn(0.0);
        replay(dtodatosCosteCombustible);
        formateador.setControlador(controlador);
        formateador.setDatosDerivadosCosteCombustible(dtodatosCosteCombustible);
        Assert.assertEquals( formateador.getMaximoCosteRecargasMes(),"0");
        verify(dtodatosCosteCombustible);
    }

    @Test(expected = IllegalStateException.class)
    public void getMinimoCosteRecargasAnioSinDatos(){

        formateador.getMinimoCosteRecargasAnio();
    }
    @Test
    public void getMinimoCosteRecargasAnioConDatos(){
        expect(dtodatosCosteCombustible.getMinimoCosteRecargaAnual()).andReturn(0.0);
        replay(dtodatosCosteCombustible);
        formateador.setControlador(controlador);
        formateador.setDatosDerivadosCosteCombustible(dtodatosCosteCombustible);
        Assert.assertEquals( formateador.getMinimoCosteRecargasAnio(),"0");
        verify(dtodatosCosteCombustible);
    }


    @Test(expected = IllegalStateException.class)
    public void getMaximoCosteRecargasAnioSinDatos(){

        formateador.getMaximoCosteRecargasAnio();
    }
    @Test
    public void getMaximoCosteRecargasAnioConDatos(){
        expect(dtodatosCosteCombustible.getMaximoCosteRecargaAnual()).andReturn(0.0);
        replay(dtodatosCosteCombustible);
        formateador.setControlador(controlador);
        formateador.setDatosDerivadosCosteCombustible(dtodatosCosteCombustible);
        Assert.assertEquals( formateador.getMaximoCosteRecargasAnio(),"0");
        verify(dtodatosCosteCombustible);
    }


    @Test(expected = IllegalStateException.class)
    public void getMediaCosteCombustiblesXUnidadMesSinDatos(){

        formateador.getMediaCosteCombustiblesXUnidadMes();
    }
    @Test
    public void getMediaCosteCombustiblesXUnidadMesConDatos(){
        expect(dtodatosCosteCombustible.getMediaCosteXRecargaMensual()).andReturn(0.0);
        replay(dtodatosCosteCombustible);
        formateador.setControlador(controlador);
        formateador.setDatosDerivadosCosteCombustible(dtodatosCosteCombustible);
        Assert.assertEquals( formateador.getMediaCosteCombustiblesXUnidadMes(),"0");
        verify(dtodatosCosteCombustible);
    }

    @Test(expected = IllegalStateException.class)
    public void getMediaCosteCombustiblesXUnidadAnioSinDatos(){

        formateador.getMediaCosteCombustiblesXUnidadAnio();
    }
    @Test
    public void getMediaCosteCombustiblesXUnidadAnioConDatos(){
        expect(dtodatosCosteCombustible.getMediaCosteXRecargaAnual()).andReturn(0.0);
        replay(dtodatosCosteCombustible);
        formateador.setControlador(controlador);
        formateador.setDatosDerivadosCosteCombustible(dtodatosCosteCombustible);
        Assert.assertEquals( formateador.getMediaCosteCombustiblesXUnidadAnio(),"0");
        verify(dtodatosCosteCombustible);
    }

    @Test(expected = IllegalStateException.class)
    public void getMinimoCosteCombustiblesXUnidadMesSinDatos(){

        formateador.getMinimoCosteCombustiblesXUnidadMes();
    }
    @Test
    public void getMinimoCosteCombustiblesXUnidadMesConDatos(){
        expect(dtodatosCosteCombustible.getMinimoCosteXRecargaMensual()).andReturn(0.0);
        replay(dtodatosCosteCombustible);
        formateador.setControlador(controlador);
        formateador.setDatosDerivadosCosteCombustible(dtodatosCosteCombustible);
        Assert.assertEquals( formateador.getMinimoCosteCombustiblesXUnidadMes(),"0");
        verify(dtodatosCosteCombustible);
    }

    @Test(expected = IllegalStateException.class)
    public void getMaximoCosteCombustiblesXUnidadMesSinDatos(){

        formateador.getMaximoCosteCombustiblesXUnidadMes();
    }
    @Test
    public void getMaximoCosteCombustiblesXUnidadMesConDatos(){
        expect(dtodatosCosteCombustible.getMaximoCosteXRecargaMensual()).andReturn(0.0);
        replay(dtodatosCosteCombustible);
        formateador.setControlador(controlador);
        formateador.setDatosDerivadosCosteCombustible(dtodatosCosteCombustible);
        Assert.assertEquals( formateador.getMaximoCosteCombustiblesXUnidadMes(),"0");
        verify(dtodatosCosteCombustible);
    }

    @Test(expected = IllegalStateException.class)
    public void getMinimoCosteCombustiblesXUnidadAnioSinDatos(){

        formateador.getMinimoCosteCombustiblesXUnidadAnio();
    }
    @Test
    public void getMinimoCosteCombustiblesXUnidadAnioConDatos(){
        expect(dtodatosCosteCombustible.getMinimoCosteXRecargaAnual()).andReturn(0.0);
        replay(dtodatosCosteCombustible);
        formateador.setControlador(controlador);
        formateador.setDatosDerivadosCosteCombustible(dtodatosCosteCombustible);
        Assert.assertEquals( formateador.getMinimoCosteCombustiblesXUnidadAnio(),"0");
        verify(dtodatosCosteCombustible);
    }

    @Test(expected = IllegalStateException.class)
    public void getMaximoCosteCombustiblesXUnidadAnioSinDatos(){

        formateador.getMaximoCosteCombustiblesXUnidadAnio();
    }
    @Test
    public void getMaximoCosteCombustiblesXUnidadAnioConDatos(){
        expect(dtodatosCosteCombustible.getMaximoCosteXRecargaAnual()).andReturn(0.0);
        replay(dtodatosCosteCombustible);
        formateador.setControlador(controlador);
        formateador.setDatosDerivadosCosteCombustible(dtodatosCosteCombustible);
        Assert.assertEquals( formateador.getMaximoCosteCombustiblesXUnidadAnio(),"0");
        verify(dtodatosCosteCombustible);
    }
}