package com.example.juanalonsoastruga.cargascontrol.Formateador.CUConsultarEstadisticaVehiculo;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTODatosDerivadosVehiculoTotales;
import com.example.juanalonsoastruga.cargascontrol.Controlador.CUConsultarEstadisticaVehiculo.ControladorConsultarEstadisticaVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Controlador.CUConsultarEstadisticaVehiculo.InterfaceCConsultarEstadisticaVehiculo;

import org.easymock.EasyMock;
import org.easymock.IAnswer;
import org.easymock.Mock;
import org.junit.Before;
import org.junit.Test;

import static org.easymock.EasyMock.createMock;
import static org.easymock.EasyMock.expect;
import static org.easymock.EasyMock.isA;
import static org.easymock.EasyMock.replay;
import static org.easymock.EasyMock.verify;

public class FormateadorConsultarEstadisticaVehiculoTest {

    @Mock
    private InterfaceCConsultarEstadisticaVehiculo controlador;
    private FormateadorConsultarEstadisticaVehiculo formateador;
    @Mock
    private DTODatosDerivadosVehiculoTotales dto;
    @Before
    public void setUp(){
        controlador =createMock(ControladorConsultarEstadisticaVehiculo.class);
        formateador =new FormateadorConsultarEstadisticaVehiculo();
        dto=createMock(DTODatosDerivadosVehiculoTotales.class);
    }

    @Test(expected=IllegalArgumentException.class)
    public void setPresentadorNull(){
        formateador.setControlador(null);
    }
    @Test
    public void setPresentadorNotNull(){

        formateador.setControlador(controlador);
    }


    @Test(expected=IllegalArgumentException.class)
    public void comienzaCUSinMatricula(){

        formateador.setControlador(controlador);
        formateador.comienzaCU(null);
    }
    @Test
    public void comienzaCUConMatricula(){

        formateador.setControlador(controlador);
        formateador.comienzaCU("");

        controlador.comienzaCU(isA(String.class));
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
    }

    @Test(expected=IllegalStateException.class)
    public void getMatriculaSinDatosAsociados(){

        formateador.setControlador(controlador);
        formateador.getMatricula();
    }

    @Test
    public void getMatriculaConDatosAsociados(){
        expect(dto.getMatricula()).andReturn("");
        replay(dto);
        formateador.setControlador(controlador);
        formateador.setDatosDerivados(dto);
        formateador.getMatricula();
        verify(dto);
    }

    @Test(expected=IllegalStateException.class)
    public void getMesSinDatosAsociados(){

        formateador.setControlador(controlador);
        formateador.getMes();
    }

    @Test
    public void getMesConDatosAsociados(){
        expect(dto.getMes()).andReturn(0);
        replay(dto);
        formateador.setControlador(controlador);
        formateador.setDatosDerivados(dto);
        formateador.getMes();
        verify(dto);
    }

    @Test(expected=IllegalStateException.class)
    public void getAnioSinDatosAsociados(){

        formateador.setControlador(controlador);
        formateador.getAnio();
    }

    @Test
    public void getAnioConDatosAsociados(){
        expect(dto.getAnio()).andReturn(0);
        replay(dto);
        formateador.setControlador(controlador);
        formateador.setDatosDerivados(dto);
        formateador.getAnio();
        verify(dto);
    }

    @Test(expected=IllegalStateException.class)
    public void getNRMesSinDatosAsociados(){

        formateador.setControlador(controlador);
        formateador.getNRMes();
    }

    @Test
    public void getNRMesConDatosAsociados(){
        expect(dto.getNumeroRecargasMes()).andReturn(0);
        replay(dto);
        formateador.setControlador(controlador);
        formateador.setDatosDerivados(dto);
        formateador.getNRMes();
        verify(dto);
    }

    @Test(expected=IllegalStateException.class)
    public void getNRAnioSinDatosAsociados(){

        formateador.setControlador(controlador);
        formateador.getNRAnio();
    }

    @Test
    public void getNRAnioConDatosAsociados(){
        expect(dto.getNumeroRecargasAnio()).andReturn(0);
        replay(dto);
        formateador.setControlador(controlador);
        formateador.setDatosDerivados(dto);
        formateador.getNRAnio();
        verify(dto);
    }
    @Test(expected=IllegalStateException.class)
    public void getCosteRecargasMesSinDatosAsociados(){

        formateador.setControlador(controlador);
        formateador.getCosteRecargasMes();
    }

    @Test
    public void getCosteRecargasMesConDatosAsociados(){
        expect(dto.getCosteRecargasTotalesPorMes()).andReturn(0.0);
        replay(dto);
        formateador.setControlador(controlador);
        formateador.setDatosDerivados(dto);
        formateador.getCosteRecargasMes();
        verify(dto);
    }
    @Test(expected=IllegalStateException.class)
    public void getCosteRecargasAnioSinDatosAsociados(){

        formateador.setControlador(controlador);
        formateador.getCosteRecargasAnio();
    }

    @Test
    public void getCosteRecargasAnioConDatosAsociados(){
        expect(dto.getCosteRecargasTotalesPorAnio()).andReturn(0.0);
        replay(dto);
        formateador.setControlador(controlador);
        formateador.setDatosDerivados(dto);
        formateador.getCosteRecargasAnio();
        verify(dto);
    }
    @Test(expected=IllegalStateException.class)
    public void getKmRecargasMesSinDatosAsociados(){

        formateador.setControlador(controlador);
        formateador.getKmRecargasMes();
    }

    @Test
    public void getKmRecargasMesConDatosAsociados(){
        expect(dto.getKmRecargasTotalesPorMes()).andReturn(0.0);
        replay(dto);
        formateador.setControlador(controlador);
        formateador.setDatosDerivados(dto);
        formateador.getKmRecargasMes();
        verify(dto);
    }
    @Test(expected=IllegalStateException.class)
    public void getKmRecargasAnioSinDatosAsociados(){

        formateador.setControlador(controlador);
        formateador.getKmRecargasAnio();
    }

    @Test
    public void getKmRecargasAnioConDatosAsociados(){
        expect(dto.getKmRecargasTotalesPorAnio()).andReturn(0.0);
        replay(dto);
        formateador.setControlador(controlador);
        formateador.setDatosDerivados(dto);
        formateador.getKmRecargasAnio();
        verify(dto);
    }
    @Test(expected=IllegalArgumentException.class)
    public void setDatosDerivadosArgumentoNull(){

        formateador.setControlador(controlador);
        formateador.setDatosDerivados(null);
    }


}