package com.example.juanalonsoastruga.cargascontrol.Formateador.CUConsultarInformacionGasolinera;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTOGasolinera;
import com.example.juanalonsoastruga.cargascontrol.Controlador.CUConsultarInformacionGasolinera.InterfaceCConsultarInformacionGasolinera;
import com.example.juanalonsoastruga.cargascontrol.Controlador.CUConsultarInformacionGasolinera.ControladorConsultarInformacionGasolinera;

import org.easymock.IAnswer;
import org.easymock.Mock;
import org.junit.Before;
import org.junit.Test;

import static org.easymock.EasyMock.createMock;
import static org.easymock.EasyMock.expect;
import static org.easymock.EasyMock.expectLastCall;
import static org.easymock.EasyMock.replay;
import static org.easymock.EasyMock.verify;
import static org.junit.Assert.*;

public class FormateadorConsultarInformacionGasolineraTest {

    private FormateadorConsultarInformacionGasolinera formateador;
    @Mock
    private InterfaceCConsultarInformacionGasolinera controlador;
    @Mock
    private DTOGasolinera dtoGasolinera;
    @Before
    public void setUp(){
        formateador = new FormateadorConsultarInformacionGasolinera();
        controlador = createMock(ControladorConsultarInformacionGasolinera.class);
        dtoGasolinera=createMock(DTOGasolinera.class);
    }


    @Test(expected = IllegalArgumentException.class)
    public void setPresenterNull(){
        formateador.setControlador(null);
    }

    @Test
    public void setPresenterNotNull(){
        formateador.setControlador(controlador);
        assertNotNull(formateador.getControlador());
    }

     @Test(expected = IllegalArgumentException.class)
     public void setDatosGasolineraNull()
     {
         formateador.setDatosGasolinera(null);
     }

    @Test
    public void setDatosGasolineraNotNull()
    {
        formateador.setDatosGasolinera(dtoGasolinera);
    }

     @Test
    public void init(){
        controlador.comienzaCU(0,0,"");
        expectLastCall().andAnswer(new IAnswer<Object>() {
             @Override
             public Object answer() throws Throwable {
                 return null;
             }
         });
        replay(controlador);
        formateador.setControlador(controlador);
        formateador.init(0,0,"");
        verify(controlador);
     }

    @Test(expected = IllegalStateException.class)
    public void getRotulo(){
        formateador.getRotulo();
    }


    @Test
    public void getRotuloNotNull(){
        expect(dtoGasolinera.getRotulo()).andReturn("").times(2);
        replay(dtoGasolinera);
        formateador.setDatosGasolinera(dtoGasolinera);
        assertEquals(formateador.getRotulo(),dtoGasolinera.getRotulo());
        verify(dtoGasolinera);
    }

    @Test(expected = IllegalStateException.class)
    public void getProvincia(){
        formateador.getProvincia();
    }


    @Test
    public void getProvinciaNotNull(){
        expect(dtoGasolinera.getProvincia()).andReturn("").times(2);
        replay(dtoGasolinera);
        formateador.setDatosGasolinera(dtoGasolinera);
        assertEquals(formateador.getProvincia(),dtoGasolinera.getProvincia());
        verify(dtoGasolinera);
    }

    @Test(expected = IllegalStateException.class)
    public void getMunicipio(){
        formateador.getProvincia();
    }


    @Test
    public void getMunicipioNotNull(){
        expect(dtoGasolinera.getMunicipio()).andReturn("").times(2);
        replay(dtoGasolinera);
        formateador.setDatosGasolinera(dtoGasolinera);
        assertEquals(formateador.getMunicipio(),dtoGasolinera.getMunicipio());
        verify(dtoGasolinera);
    }

    @Test(expected = IllegalStateException.class)
    public void getLocalidad(){
        formateador.getLocalidad();
    }

    @Test
    public void getLocalidadNotNull(){
        expect(dtoGasolinera.getLocalidad()).andReturn("").times(2);
        replay(dtoGasolinera);
        formateador.setDatosGasolinera(dtoGasolinera);
        assertEquals(formateador.getLocalidad(),dtoGasolinera.getLocalidad());
        verify(dtoGasolinera);
    }


    @Test(expected = IllegalStateException.class)
    public void getDireccion(){
        formateador.getDireccion();
    }

    @Test
    public void getDireccionNotNull(){
        expect(dtoGasolinera.getDireccion()).andReturn("").times(2);
        replay(dtoGasolinera);
        formateador.setDatosGasolinera(dtoGasolinera);
        assertEquals(formateador.getDireccion(),dtoGasolinera.getDireccion());
        verify(dtoGasolinera);
    }


    @Test(expected = IllegalStateException.class)
    public void getHorario(){
        formateador.getHorario();
    }

    @Test
    public void getHorarioNotNull(){
        expect(dtoGasolinera.getHorario()).andReturn("").times(2);
        replay(dtoGasolinera);
        formateador.setDatosGasolinera(dtoGasolinera);
        assertEquals(formateador.getHorario(),dtoGasolinera.getHorario());
        verify(dtoGasolinera);
    }


    @Test(expected = IllegalStateException.class)
    public void getLatitud(){
        formateador.getLatitud();
    }

    @Test
    public void getLatitudNotNull(){
        expect(dtoGasolinera.getLatitud()).andReturn(0.0).times(2);
        replay(dtoGasolinera);
        formateador.setDatosGasolinera(dtoGasolinera);
        assertEquals(formateador.getLatitud(),dtoGasolinera.getLatitud()+"");
        verify(dtoGasolinera);
    }


    @Test(expected = IllegalStateException.class)
    public void getLongitud(){
        formateador.getLongitud();
    }

    @Test
    public void getLongitudNotNull(){
        expect(dtoGasolinera.getLongitud()).andReturn(0.0).times(2);
        replay(dtoGasolinera);
        formateador.setDatosGasolinera(dtoGasolinera);
        assertEquals(formateador.getLongitud(),dtoGasolinera.getLongitud()+"");
        verify(dtoGasolinera);
    }

    @Test(expected = IllegalStateException.class)
    public void getBiodiesel(){
        formateador.getBiodiesel();
    }

    @Test
    public void getBiodieselNotNull(){
        expect(dtoGasolinera.getBiodiesel()).andReturn(true);
        replay(dtoGasolinera);
        formateador.setDatosGasolinera(dtoGasolinera);
        assertEquals(formateador.getBiodiesel(),"SI");
        verify(dtoGasolinera);
    }

    @Test(expected = IllegalStateException.class)
    public void getBioetanol(){
        formateador.getBioetanol();
    }

    @Test
    public void getBioetanolNotNull(){
        expect(dtoGasolinera.getBioetanol()).andReturn(true);
        replay(dtoGasolinera);
        formateador.setDatosGasolinera(dtoGasolinera);
        assertEquals(formateador.getBioetanol(),"SI");
        verify(dtoGasolinera);
    }

    @Test(expected = IllegalStateException.class)
    public void getGasNaturalComprimido(){
        formateador.getGasNaturalComprimido();
    }

    @Test
    public void getGasNaturalComprimidoNotNull(){
        expect(dtoGasolinera.getGasNaturalComprimido()).andReturn(true);
        replay(dtoGasolinera);
        formateador.setDatosGasolinera(dtoGasolinera);
        assertEquals(formateador.getGasNaturalComprimido(),"SI");
        verify(dtoGasolinera);
    }


    @Test(expected = IllegalStateException.class)
    public void getGasNaturalLicuado(){
        formateador.getGasNaturalLicuado();
    }

    @Test
    public void getGasNaturalLicuadoNotNull(){
        expect(dtoGasolinera.getGasNaturalLicuado()).andReturn(true);
        replay(dtoGasolinera);
        formateador.setDatosGasolinera(dtoGasolinera);
        assertEquals(formateador.getGasNaturalLicuado(),"SI");
        verify(dtoGasolinera);
    }

    @Test(expected = IllegalStateException.class)
    public void getGasLicuadoPetroleo(){
        formateador.getGasLicuadoPetroleo();
    }

    @Test
    public void getGasLicuadoPetroleoNotNull(){
        expect(dtoGasolinera.getGasLicuadoPetroleo()).andReturn(true);
        replay(dtoGasolinera);
        formateador.setDatosGasolinera(dtoGasolinera);
        assertEquals(formateador.getGasLicuadoPetroleo(),"SI");
        verify(dtoGasolinera);
    }

    @Test(expected = IllegalStateException.class)
    public void getGasoleoA(){
        formateador.getGasoleoA();
    }

    @Test
    public void getGasoleoANotNull(){
        expect(dtoGasolinera.getGasoleoA()).andReturn(true);
        replay(dtoGasolinera);
        formateador.setDatosGasolinera(dtoGasolinera);
        assertEquals(formateador.getGasoleoA(),"SI");
        verify(dtoGasolinera);
    }

    @Test(expected = IllegalStateException.class)
    public void getGasoleoB(){
        formateador.getGasoleoB();
    }

    @Test
    public void getGasoleoBNotNull(){
        expect(dtoGasolinera.getGasoleoB()).andReturn(true);
        replay(dtoGasolinera);
        formateador.setDatosGasolinera(dtoGasolinera);
        assertEquals(formateador.getGasoleoB(),"SI");
        verify(dtoGasolinera);
    }

    @Test(expected = IllegalStateException.class)
    public void getGasolina95(){
        formateador.getGasolina95();
    }

    @Test
    public void getGasolina95NotNull(){
        expect(dtoGasolinera.getGasolina95()).andReturn(true);
        replay(dtoGasolinera);
        formateador.setDatosGasolinera(dtoGasolinera);
        assertEquals(formateador.getGasolina95(),"SI");
        verify(dtoGasolinera);
    }

    @Test(expected = IllegalStateException.class)
    public void getGasolina98(){
        formateador.getGasolina98();
    }

    @Test
    public void getGasolina98NotNull(){
        expect(dtoGasolinera.getGasolina98()).andReturn(true);
        replay(dtoGasolinera);
        formateador.setDatosGasolinera(dtoGasolinera);
        assertEquals(formateador.getGasolina98(),"SI");
        verify(dtoGasolinera);
    }

    @Test(expected = IllegalStateException.class)
    public void getNuevoGasoleoA(){
        formateador.getNuevoGasoleoA();
    }

    @Test
    public void getNuevoGasoleoANotNull(){
        expect(dtoGasolinera.getNuevoGasoleoA()).andReturn(true);
        replay(dtoGasolinera);
        formateador.setDatosGasolinera(dtoGasolinera);
        assertEquals(formateador.getNuevoGasoleoA(),"SI");
        verify(dtoGasolinera);
    }
}