package com.example.juanalonsoastruga.cargascontrol.Formateador.CUConsultarInformacionRecarga;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTORecarga;
import com.example.juanalonsoastruga.cargascontrol.Controlador.CUConsultarInformacionRecarga.ControladorConsultarInformacionRecarga;


import org.easymock.EasyMock;
import org.easymock.IAnswer;
import org.easymock.Mock;
import org.junit.Before;
import org.junit.Test;

import java.util.Date;

import static org.easymock.EasyMock.createMock;

import static org.easymock.EasyMock.expect;
import static org.easymock.EasyMock.isA;
import static org.easymock.EasyMock.replay;
import static org.easymock.EasyMock.verify;
import static org.junit.Assert.*;

public class FormateadorConsultarInformacionRecargaTest {

    private FormateadorConsultarInformacionRecarga formateador;
    @Mock
    private ControladorConsultarInformacionRecarga controlador;
    @Mock
    private DTORecarga recarga;

    @Before
    public void setUp(){
        formateador = new FormateadorConsultarInformacionRecarga();
        controlador =createMock(ControladorConsultarInformacionRecarga.class);
        recarga=createMock(DTORecarga.class);
    }


    @Test(expected = IllegalArgumentException.class)
    public void setPresenterNull(){
        formateador.setControlador(null);
    }

    @Test
    public void setPresenterNotNull(){
        formateador.setControlador(controlador);

        assertNotNull(formateador.getControlador());
    }

    @Test
    public void init(){
        controlador.comienzoCU(isA(String.class),isA(Date.class),isA(String.class));
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        replay(controlador);
        formateador.setControlador(controlador);
        formateador.init("",new Date(),"");
        verify(controlador);
    }

    @Test
    public void registraUltimaRecargaConsultada(){
        controlador.registrarIDRecargaConsultada(isA(String.class),isA(Long.class),isA(String.class));
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        replay(controlador);
        formateador.setControlador(controlador);
        formateador.registrarUltimaRecargaConsultada("",(long)0,"");
        verify(controlador);
    }


    @Test(expected = IllegalArgumentException.class)
    public void setRecargaNull(){
        formateador.setRecarga(null);
    }

    @Test
    public void setRecargaNotNull(){
        formateador.setRecarga(recarga);
        expect(recarga.getMatricula()).andReturn("");
        replay(recarga);
        assertNotNull(recarga.getMatricula());
        verify(recarga);
    }


    @Test
    public void getFoto(){

        formateador.setRecarga(recarga);
        expect(recarga.getFoto()).andReturn(null).times(2);
        replay(recarga);
        assertEquals(recarga.getFoto(), formateador.getFoto());
        verify(recarga);
    }


    @Test
    public void getCombustible(){

        formateador.setRecarga(recarga);
        expect(recarga.getCombustible()).andReturn("").times(2);
        replay(recarga);
        assertEquals(recarga.getCombustible(), formateador.getCombustible());
        verify(recarga);
    }


    @Test
    public void getMatricula(){

        formateador.setRecarga(recarga);
        expect(recarga.getMatricula()).andReturn("").times(2);
        replay(recarga);
        assertEquals(recarga.getMatricula(), formateador.getMatricula());
        verify(recarga);
    }




    @Test
    public void getMedicion(){

        formateador.setRecarga(recarga);
        expect(recarga.getMedicion()).andReturn(0.0).times(2);
        replay(recarga);
        assertEquals(recarga.getMedicion(), formateador.getMedicion(),0);
        verify(recarga);
    }

    @Test
    public void getCoste(){

        formateador.setRecarga(recarga);
        expect(recarga.getCoste()).andReturn(0.0).times(2);
        replay(recarga);
        assertEquals(recarga.getCoste(), formateador.getCoste(),0);
        verify(recarga);
    }


    @Test
    public void getKm(){

        formateador.setRecarga(recarga);
        expect(recarga.getKm()).andReturn(0.0).times(2);
        replay(recarga);
        assertEquals(recarga.getKm(), formateador.getKm(),0);
        verify(recarga);
    }



}