package com.example.juanalonsoastruga.cargascontrol.Formateador.CUConsultarInformacionVehiculo;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTOVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Controlador.CUConsultarInformacionVehiculo.ControladorConsultarInformacionVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Controlador.CUConsultarInformacionVehiculo.InterfaceCConsultarInformacion;

import org.easymock.EasyMock;
import org.easymock.IAnswer;
import org.junit.Before;
import org.junit.Test;

import static org.easymock.EasyMock.createMock;
import static org.easymock.EasyMock.expect;
import static org.easymock.EasyMock.replay;
import static org.easymock.EasyMock.verify;
import static org.junit.Assert.*;

public class FormateadorConsultarInformacionVehiculoTest {

    private FormateadorConsultarInformacionVehiculo formateador;
    private InterfaceCConsultarInformacion controlador;
    private DTOVehiculo dto;

    @Before
    public  void setUp(){
        formateador = new FormateadorConsultarInformacionVehiculo();
        controlador =createMock(ControladorConsultarInformacionVehiculo.class);
        dto=createMock(DTOVehiculo.class);
    }

    @Test(expected = IllegalArgumentException.class)
    public void setPresenterNull(){
        formateador.setControler(null);

    }
    @Test
    public void setPresenterNotNull(){
        formateador.setControler(controlador);

        assertNotNull(formateador.getControler());
    }

    @Test
    public void init(){
        controlador.activacionCU("");
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        replay(controlador);
        formateador.setControler(controlador);
        formateador.init("");
        verify(controlador);
    }

    @Test(expected=IllegalArgumentException.class)
    public void setVehiculoNull(){
        formateador.setVehiculo(null);
    }

    @Test
    public void setVehiculoNotNull(){
        expect(dto.getMatricula()).andReturn("");
        replay(dto);
        formateador.setVehiculo(dto);
        assertNotNull(formateador.getVehiculo());
        assertEquals(formateador.getVehiculo().getMatricula(),"");
        verify(dto);
    }
    @Test
    public void getMatricula(){
        expect(dto.getMatricula()).andReturn("").times(2);
        replay(dto);
        formateador.setVehiculo(dto);
        assertNotNull(formateador.getVehiculo());
        assertEquals(formateador.getMatricula(),dto.getMatricula());
        verify(dto);
    }

    @Test
    public void getNombre(){
        expect(dto.getNombre()).andReturn("").times(2);
        replay(dto);
        formateador.setVehiculo(dto);
        assertNotNull(formateador.getVehiculo());
        assertEquals(formateador.getNombre(),dto.getNombre());
        verify(dto);
    }


    @Test
    public void getModelo(){
        expect(dto.getModelo()).andReturn("").times(2);
        replay(dto);
        formateador.setVehiculo(dto);
        assertNotNull(formateador.getVehiculo());
        assertEquals(formateador.getModelo(),dto.getModelo());
        verify(dto);
    }



    @Test
    public void getMarca(){
        expect(dto.getMarca()).andReturn("").times(2);
        replay(dto);
        formateador.setVehiculo(dto);
        assertNotNull(formateador.getVehiculo());
        assertEquals(formateador.getMarca(),dto.getMarca());
        verify(dto);
    }

    @Test
    public void getAño(){
        expect(dto.getAño()).andReturn(0).times(2);
        replay(dto);
        formateador.setVehiculo(dto);
        assertNotNull(formateador.getVehiculo());
        assertEquals( formateador.getAño(),dto.getAño()+"");
        verify(dto);
    }

    @Test
    public void getKm(){
        expect(dto.getKilometraje()).andReturn(0.0).times(2);
        replay(dto);
        formateador.setVehiculo(dto);
        assertNotNull(formateador.getVehiculo());
        assertEquals( formateador.getKm(),((long)dto.getKilometraje())+"");
        verify(dto);
    }

    @Test
    public void getCPrimario(){
        expect(dto.getCombustiblePrimario()).andReturn("").times(2);
        replay(dto);
        formateador.setVehiculo(dto);
        assertNotNull(formateador.getVehiculo());
        assertEquals(formateador.getCPrimario(),dto.getCombustiblePrimario());
        verify(dto);
    }

    @Test
    public void getCTPrimario(){
        expect(dto.getCapTankPrimario()).andReturn(0.0).times(2);
        replay(dto);
        formateador.setVehiculo(dto);
        assertNotNull(formateador.getVehiculo());
        assertEquals( formateador.getCTPrimario(),((long)dto.getCapTankPrimario())+"");
        verify(dto);
    }

    @Test
    public void getCSecundario(){
        expect(dto.getCombustibleSecundario()).andReturn("").times(2);
        replay(dto);
        formateador.setVehiculo(dto);
        assertNotNull(formateador.getVehiculo());
        assertEquals(formateador.getCSecundario(),dto.getCombustibleSecundario());
        verify(dto);
    }

    @Test
    public void getCTSecundario(){
        expect(dto.getCapTankSecundario()).andReturn(0.0).times(2);
        replay(dto);
        formateador.setVehiculo(dto);
        assertNotNull(formateador.getVehiculo());
        assertEquals( formateador.getCTSecundario(),(int) dto.getCapTankSecundario()+"");
        verify(dto);
    }
    @Test
    public void getFoto(){
        expect(dto.getFoto()).andReturn(null).times(2);
        replay(dto);
        formateador.setVehiculo(dto);
        assertNotNull(formateador.getVehiculo());
        assertEquals(formateador.getFoto(), dto.getFoto());
        verify(dto);
    }
}