package com.example.juanalonsoastruga.cargascontrol.Formateador.CUGraficarConsumoCombustible;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.CodigoCombustibleVehiculo;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.CodigoPeriocidadGrafica;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTORecarga;
import com.example.juanalonsoastruga.cargascontrol.Controlador.CUGraficarConsumoCombustible.ControladorGraficarConsumoCombustible;
import com.example.juanalonsoastruga.cargascontrol.Controlador.CUGraficarConsumoCombustible.InterfaceCGraficarConsumoCombustible;

import org.easymock.EasyMock;
import org.easymock.IAnswer;
import org.easymock.Mock;
import org.junit.Before;
import org.junit.Test;

import java.util.List;

import static org.easymock.EasyMock.createMock;
import static org.easymock.EasyMock.expect;
import static org.easymock.EasyMock.replay;
import static org.easymock.EasyMock.verify;
import static org.junit.Assert.*;

public class FormateadorGraficarConsumoCombustibleTest
{
    private FormateadorGraficarConsumoCombustible formateador;
    @Mock
    private InterfaceCGraficarConsumoCombustible controlador;
    @Mock
    private List<DTORecarga> listado;
    @Mock
    private DTORecarga hash;
    @Before
    public void setUp(){
        formateador = new FormateadorGraficarConsumoCombustible();
        controlador =createMock(ControladorGraficarConsumoCombustible.class);
        listado =createMock(List.class);
        hash =createMock(DTORecarga.class);
    }

    @Test(expected = IllegalArgumentException.class)
    public void setPresenterNull(){
        formateador.setControlador(null);
    }

    @Test
    public void setPresenterNotNull(){
        formateador.setControlador(controlador);

        assertNotNull(formateador.getControlador());
    }


    @Test
    public void comienzoCU(){
        controlador.comienzaCU(CodigoPeriocidadGrafica.PERIOCIDAD_GRAFICA1);
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        replay(controlador);
        formateador.setControlador(controlador);
        formateador.comienzaCU("", CodigoCombustibleVehiculo.CPRIM,CodigoPeriocidadGrafica.PERIOCIDAD_GRAFICA1);
        verify(controlador);
    }

    @Test(expected = IllegalArgumentException.class)
    public void preparaDatosVisualizacionInexistentes(){
        formateador.preparaDatosVisualizacion(null);
    }
    @Test
    public void preparaDatosVisualizacionExistentesSizeNot0NotNull(){
        assertNull(formateador.getDataSet());
        expect(listado.size()).andReturn(1).times(2);
        expect(listado.get(0)).andReturn(hash);
        expect(hash.getMedicion()).andReturn(0.0);
        replay(listado);

        formateador.preparaDatosVisualizacion(listado);
        assertNotNull(formateador.getDataSet());
        verify(listado);

    }

    @Test
    public void preparaDatosVisualizacionExistentesSizeNot0Null(){
        assertNull(formateador.getDataSet());
        expect(listado.size()).andReturn(1).times(2);
        expect(listado.get(0)).andReturn(null);

        replay(listado);

        formateador.preparaDatosVisualizacion(listado);
        assertNotNull(formateador.getDataSet());
        verify(listado);

    }
    @Test
    public void getNullDataSet(){
        assertNull(formateador.getDataSet());
    }


}