package com.example.juanalonsoastruga.cargascontrol.Formateador.CUGraficarConsumoCombustibleXKm;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.CodigoCombustibleVehiculo;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.CodigoPeriocidadGrafica;
import com.example.juanalonsoastruga.cargascontrol.Controlador.CUGraficarConsumoCombustibleXKm.InterfaceCGraficarConsumoCombustibleKm;
import com.example.juanalonsoastruga.cargascontrol.Controlador.CUGraficarConsumoCombustibleXKm.ControladorGraficarConsumoCombustibleKm;

import org.easymock.EasyMock;
import org.easymock.IAnswer;
import org.easymock.Mock;
import org.junit.Before;
import org.junit.Test;

import java.util.Hashtable;

import static org.easymock.EasyMock.createMock;
import static org.easymock.EasyMock.expect;
import static org.easymock.EasyMock.replay;
import static org.easymock.EasyMock.verify;
import static org.junit.Assert.*;

public class FormateadorGraficarConsumoCombustibleKmTest {
    private FormateadorGraficarConsumoCombustibleKm formateador;
    @Mock
    private InterfaceCGraficarConsumoCombustibleKm controlador;
    @Mock
    private Hashtable hash;
    @Before
    public void setUp(){
        formateador = new FormateadorGraficarConsumoCombustibleKm();
        controlador =createMock(ControladorGraficarConsumoCombustibleKm.class);
        hash =createMock(Hashtable.class);
    }

    @Test(expected = IllegalArgumentException.class)
    public void setPresenterNull(){
        formateador.setControlador(null);
    }

    @Test
    public void setPresenterNotNull(){
        formateador.setControlador(controlador);

        assertNotNull(formateador.getControlador());
    }

    @Test(expected = IllegalArgumentException.class)
    public void comienzaCUNullMatricula(){
        formateador.comienzaCU(null, CodigoCombustibleVehiculo.CPRIM, CodigoPeriocidadGrafica.PERIOCIDAD_GRAFICA1);

    }

    @Test(expected = IllegalArgumentException.class)
    public void comienzaCUNullCodigoCombustibleVehiculo(){
        formateador.comienzaCU("", null, CodigoPeriocidadGrafica.PERIOCIDAD_GRAFICA1);

    }

    @Test(expected = IllegalArgumentException.class)
    public void comienzaCUNullCodigoPeriocidadGrafica(){
        formateador.comienzaCU("", CodigoCombustibleVehiculo.CPRIM, null);

    }

    @Test
    public void comienzoCU(){
        controlador.comienzaCU(CodigoPeriocidadGrafica.PERIOCIDAD_GRAFICA1);
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        replay(controlador);
        formateador.setControlador(controlador);
        formateador.comienzaCU("",CodigoCombustibleVehiculo.CPRIM,CodigoPeriocidadGrafica.PERIOCIDAD_GRAFICA1);
        verify(controlador);
    }

    @Test(expected = IllegalArgumentException.class)
    public void preparaDatosVisualizacionInexistentes(){
        formateador.preparaDatosVisualizacion(null,0);
    }
    @Test
    public void preparaDatosVisualizacionExistentesSize0(){
        assertNull(formateador.getDataSet());

        formateador.preparaDatosVisualizacion(hash,0);
        assertNotNull(formateador.getDataSet());

    }
    @Test
    public void preparaDatosVisualizacionExistentesSizeNot0(){
        assertNull(formateador.getDataSet());
        expect(hash.get(1)).andReturn(null);
        replay(hash);
        formateador.preparaDatosVisualizacion(hash,1);
        assertNotNull(formateador.getDataSet());
        verify(hash);
    }
    @Test
    public void getNullDataSet(){
        assertNull(formateador.getDataSet());
    }
}