package com.example.juanalonsoastruga.cargascontrol.Formateador.CUGraficarCoste;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.CodigoCombustibleVehiculo;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.CodigoPeriocidadGrafica;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTORecarga;
import com.example.juanalonsoastruga.cargascontrol.Controlador.CUGraficarCoste.InterfaceCGraficarCoste;
import com.example.juanalonsoastruga.cargascontrol.Controlador.CUGraficarCoste.ControladorGraficarCoste;

import org.easymock.EasyMock;
import org.easymock.IAnswer;
import org.easymock.Mock;
import org.junit.Before;
import org.junit.Test;

import java.util.List;

import static org.easymock.EasyMock.createMock;
import static org.easymock.EasyMock.expect;
import static org.easymock.EasyMock.replay;
import static org.easymock.EasyMock.verify;
import static org.junit.Assert.*;

public class FormateadorGraficarCosteTest {
    private FormateadorGraficarCoste formateador;
    @Mock
    private InterfaceCGraficarCoste controlador;
    @Mock
    private List<DTORecarga> listado;
    @Mock
    private DTORecarga dto;
    @Before
    public void setUp(){
        formateador = new FormateadorGraficarCoste();
        controlador =createMock(ControladorGraficarCoste.class);
        listado =createMock(List.class);
        dto=createMock(DTORecarga.class);
    }

    @Test(expected = IllegalArgumentException.class)
    public void setPresenterNull(){
        formateador.setControlador(null);
    }

    @Test
    public void setPresenterNotNull(){
        formateador.setControlador(controlador);

        assertNotNull(formateador.getControlador());
    }

    @Test
    public void comienzaCU(){
        controlador.comienzaCU(CodigoPeriocidadGrafica.PERIOCIDAD_GRAFICA2);
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        replay(controlador);
        formateador.setControlador(controlador);
        formateador.comienzaCU("", CodigoCombustibleVehiculo.CPRIM,CodigoPeriocidadGrafica.PERIOCIDAD_GRAFICA2);
        assertEquals(formateador.getMatricula(),"");
        assertEquals(formateador.getCodigoCombustible(),CodigoCombustibleVehiculo.CPRIM);
        verify(controlador);
    }

    @Test(expected = IllegalArgumentException.class)
    public void preparaDatosVisualizacionInexistentes(){
        formateador.preparaDatosVisualizacion(null);
    }
    @Test
    public void preparaDatosVisualizacionExistentesSize0(){
        assertNull(formateador.getDataSet());
        expect(listado.size()).andReturn(0);
        replay(listado);
        formateador.preparaDatosVisualizacion(listado);
        assertNotNull(formateador.getDataSet());
        verify(listado);
    }

    @Test
    public void preparaDatosVisualizacionExistentesSizeNot0NotNull(){
        assertNull(formateador.getDataSet());
        expect(listado.size()).andReturn(1).times(2);
        expect(listado.get(0)).andReturn(dto);
        expect(dto.getCoste()).andReturn(0.0);
        replay(listado);
        replay(dto);
        formateador.preparaDatosVisualizacion(listado);
        assertNotNull(formateador.getDataSet());
        verify(listado);
        verify(dto);
    }

    @Test
    public void preparaDatosVisualizacionExistentesSizeNot0Null(){
        assertNull(formateador.getDataSet());
        expect(listado.size()).andReturn(1).times(2);
        expect(listado.get(0)).andReturn(null);

        replay(listado);

        formateador.preparaDatosVisualizacion(listado);
        assertNotNull(formateador.getDataSet());
        verify(listado);

    }

    @Test
    public void getNullDataSet(){
        assertNull(formateador.getDataSet());
    }

}