package com.example.juanalonsoastruga.cargascontrol.Formateador.CUGraficarKm;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.CodigoCombustibleVehiculo;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.CodigoPeriocidadGrafica;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTORecarga;
import com.example.juanalonsoastruga.cargascontrol.Controlador.CUGraficarKm.InterfaceCGraficarKm;
import com.example.juanalonsoastruga.cargascontrol.Controlador.CUGraficarKm.ControladorGraficarKm;

import org.easymock.EasyMock;
import org.easymock.IAnswer;
import org.easymock.Mock;
import org.junit.Before;
import org.junit.Test;

import java.util.List;

import static org.easymock.EasyMock.createMock;
import static org.easymock.EasyMock.expect;
import static org.easymock.EasyMock.isA;
import static org.easymock.EasyMock.replay;
import static org.easymock.EasyMock.verify;
import static org.junit.Assert.*;

public class FormateadorGraficarKmTest {
    private FormateadorGraficarKm formateador;
    @Mock
    private InterfaceCGraficarKm controlador;
    @Mock
    private List<DTORecarga> dto;
    @Mock
    private CodigoCombustibleVehiculo codV;
    @Mock
    private CodigoPeriocidadGrafica codG;
    @Before
    public void setUp(){
        formateador = new FormateadorGraficarKm();
        controlador =createMock(ControladorGraficarKm.class);
        dto=createMock(List.class);
        codV =CodigoCombustibleVehiculo.CPRIM;
        codG=CodigoPeriocidadGrafica.PERIOCIDAD_GRAFICA1;
    }

    @Test(expected = IllegalArgumentException.class)
    public void setPresenterNull(){
        formateador.setControlador(null);
    }

    @Test
    public void setPresenterNotNull(){
        formateador.setControlador(controlador);

        assertNotNull(formateador.getControlador());
    }

    @Test
    public void comienzoCU(){
        controlador.comienzaCU(isA(CodigoPeriocidadGrafica.class));
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        replay(controlador);
        formateador.setControlador(controlador);
        formateador.comienzaCU("", codV,codG);
        verify(controlador);
    }

    @Test
    public void getMatricula(){
        controlador.comienzaCU(isA(CodigoPeriocidadGrafica.class));
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        replay(controlador);
        formateador.setControlador(controlador);
        formateador.comienzaCU("", codV,codG);
        assertEquals(formateador.getMatricula(),"");
        verify(controlador);
    }

    @Test
    public void getCodigoCombustible(){
        controlador.comienzaCU(isA(CodigoPeriocidadGrafica.class));
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        replay(controlador);
        formateador.setControlador(controlador);
        formateador.comienzaCU("", codV,codG);
        assertEquals(formateador.getCodigoCombustible(),codV);
        verify(controlador);
    }

    @Test(expected = IllegalArgumentException.class)
    public void preparaDatosVisualizacionInexistentes(){
        formateador.preparaDatosVisualizacion(null);
    }
    @Test
    public void preparaDatosVisualizacionExistentes(){
        assertNull(formateador.getDataSet());
        expect(dto.size()).andReturn(0);
        replay(dto);
        formateador.preparaDatosVisualizacion(dto);
        assertNotNull(formateador.getDataSet());
        verify(dto);
    }
    @Test
    public void getNullDataSet(){
        assertNull(formateador.getDataSet());
    }

}