package com.example.juanalonsoastruga.cargascontrol.Formateador.CUModificarInformacionVehiculo;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTOCombustible;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTOVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Controlador.CUModificarInformacionVehiculo.InterfaceCModificarInformacionVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Controlador.CUModificarInformacionVehiculo.ControladorModificarInformacionVehiculo;

import org.easymock.EasyMock;
import org.easymock.IAnswer;
import org.easymock.Mock;
import org.junit.Before;
import org.junit.Test;

import java.util.List;

import static org.easymock.EasyMock.createMock;
import static org.easymock.EasyMock.expect;
import static org.easymock.EasyMock.isA;
import static org.easymock.EasyMock.replay;
import static org.easymock.EasyMock.verify;
import static org.junit.Assert.*;

public class FormateadorModificarInformacionVehiculoTest {

    private FormateadorModificarInformacionVehiculo formateador;
    @Mock
    private InterfaceCModificarInformacionVehiculo controlador;
    @Mock
    private List<DTOCombustible> combustibles;
    @Mock
    private DTOVehiculo vehiculo;
    @Mock
    private DTOCombustible combustible;
    @Before
    public void setUp(){
        formateador = new FormateadorModificarInformacionVehiculo();
        controlador =createMock(ControladorModificarInformacionVehiculo.class);
        vehiculo=createMock(DTOVehiculo.class);
        combustibles=createMock(List.class);
        combustible=createMock(DTOCombustible.class);
    }
    @Test(expected = IllegalArgumentException.class)
    public void setPresenterNull(){
        formateador.setControlador(null);

    }
    @Test
    public void setPresenterNotNull(){
        formateador.setControlador(controlador);

        assertNotNull(formateador.getControlador());
    }
    @Test
    public void init(){
        controlador.comienzoCU("");
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        replay(controlador);
        formateador.setControlador(controlador);
        formateador.init("");
        verify(controlador);
    }

    @Test
    public void registroVehiculoNulo(){
        assertNull(formateador.getRegistro().getMatricula());
    }

    @Test
    public void registroVehiculoNotNulo(){
        expect(combustibles.size()).andReturn(0);
        replay(combustibles);
        formateador.setInitInfo(combustibles,vehiculo);
        assertNotNull(formateador.getRegistro());
        verify(combustibles);
    }
    @Test(expected = IllegalArgumentException.class)
    public void setInitInfoOnNullCombustibles(){
        formateador.setInitInfo(null,vehiculo);
    }
    @Test(expected = IllegalArgumentException.class)
    public void setInitInfoOnNullVehiculo(){
        formateador.setInitInfo(combustibles,null);
    }
    @Test
    public void setInitInfoCorrectCeroCombustibles(){
        expect(combustibles.size()).andReturn(0);
        replay(combustibles);

        formateador.setInitInfo(combustibles,vehiculo);
        assertEquals(formateador.getInfoCombustible().size(),0);
        verify(combustibles);
    }
    @Test
    public void setInitInfoCorrectNoCeroCombustibles(){
        expect(combustibles.size()).andReturn(1).anyTimes();
        expect(combustibles.get(0)).andReturn(combustible).once();
        expect(combustible.getTipo()).andReturn("");
        replay(combustibles);
        replay(combustible);
        formateador.setInitInfo(combustibles,vehiculo);
        assertEquals(formateador.getInfoCombustible().size(),1);
        verify(combustibles);
    }

    @Test
    public void registroNoCompletadoInicialmente(){
        assertFalse(formateador.isRegistroCompletado());
    }
    @Test
    public void registroCompletadoTrasOperacion(){
        formateador.setRegistroCompletado();
        assertTrue(formateador.isRegistroCompletado());
    }



    @Test
    public void pulsacionBoton(){

        expect(controlador.pulsacionBotonModificarDatos()).andReturn(true);
        replay(controlador);
        formateador.setControlador(controlador);
        formateador.pulsacionBotonModificar();
        verify(controlador);
    }
    @Test
    public void getInfoSpinnerNull(){
        assertNull(formateador.getInfoCombustible());
    }

    @Test
    public void getInfoSpinnerNotNull(){
        expect(combustibles.size()).andReturn(1).anyTimes();
        expect(combustibles.get(0)).andReturn(combustible).once();
        expect(combustible.getTipo()).andReturn("");
        replay(combustibles);
        replay(combustible);
        formateador.setInitInfo(combustibles,vehiculo);
        verify(combustibles);
        verify(combustible);
        assertNotNull(formateador.getInfoCombustible());
    }
    @Test
    public void setMensajeError(){
        assertTrue(formateador.setMensajeError(""));
        assertEquals(formateador.getMensajeError(),"");
    }

    @Test
    public void setRegistroNoCompleto(){
        assertTrue(formateador.setRegistroNoCompleto());
        assertFalse(formateador.isRegistroCompletado());
    }

    @Test
    public void getNombre(){
        expect(combustibles.size()).andReturn(0);
        expect(vehiculo.getNombre()).andReturn("");
        replay(combustibles);
        replay(vehiculo);
        formateador.setInitInfo(combustibles,vehiculo);
        assertEquals(formateador.getNombre(),"");
        verify(vehiculo);
        verify(combustibles);
    }

    @Test
    public void setNombre(){
        expect(combustibles.size()).andReturn(0);
        expect(vehiculo.getNombre()).andReturn("");
        vehiculo.setNombre(isA(String.class)) ;
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        replay(combustibles);
        replay(vehiculo);
        formateador.setInitInfo(combustibles,vehiculo);
        formateador.actualizaNombre("");
        assertEquals(formateador.getNombre(),"");
        verify(vehiculo);
        verify(combustibles);
    }

    @Test
    public void getMatricula(){
        expect(combustibles.size()).andReturn(0);
        expect(vehiculo.getMatricula()).andReturn("");
        replay(combustibles);
        replay(vehiculo);
        formateador.setInitInfo(combustibles,vehiculo);
        assertEquals(formateador.getMatricula(),"");
        verify(vehiculo);
        verify(combustibles);
    }


    @Test
    public void getMarca(){
        expect(combustibles.size()).andReturn(0);
        expect(vehiculo.getMarca()).andReturn("");
        replay(combustibles);
        replay(vehiculo);
        formateador.setInitInfo(combustibles,vehiculo);
        assertEquals(formateador.getMarca(),"");
        verify(vehiculo);
        verify(combustibles);
    }

    @Test
    public void setMarca(){
        expect(combustibles.size()).andReturn(0);
        expect(vehiculo.getMarca()).andReturn("");
        vehiculo.setMarca(isA(String.class)) ;
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        replay(combustibles);
        replay(vehiculo);
        formateador.setInitInfo(combustibles,vehiculo);
        formateador.actualizaMarca("");
        assertEquals(formateador.getMarca(),"");
        verify(vehiculo);
        verify(combustibles);
    }

    @Test
    public void getModelo(){
        expect(combustibles.size()).andReturn(0);
        expect(vehiculo.getModelo()).andReturn("");
        replay(combustibles);
        replay(vehiculo);
        formateador.setInitInfo(combustibles,vehiculo);
        assertEquals(formateador.getModelo(),"");
        verify(vehiculo);
        verify(combustibles);
    }

    @Test
    public void setModelo(){
        expect(combustibles.size()).andReturn(0);
        expect(vehiculo.getModelo()).andReturn("");
        vehiculo.setModelo(isA(String.class)) ;
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        replay(combustibles);
        replay(vehiculo);
        formateador.setInitInfo(combustibles,vehiculo);
        formateador.actualizaModelo("");
        assertEquals(formateador.getModelo(),"");
        verify(vehiculo);
        verify(combustibles);
    }


    @Test
    public void getAnio(){
        expect(combustibles.size()).andReturn(0);
        expect(vehiculo.getAño()).andReturn(0);
        replay(combustibles);
        replay(vehiculo);
        formateador.setInitInfo(combustibles,vehiculo);
        assertEquals(formateador.getAnio(),0);
        verify(vehiculo);
        verify(combustibles);
    }

    @Test
    public void setAnio(){
        expect(combustibles.size()).andReturn(0);
        expect(vehiculo.getAño()).andReturn(0);
        vehiculo.setAño(0) ;
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        replay(combustibles);
        replay(vehiculo);
        formateador.setInitInfo(combustibles,vehiculo);
        formateador.actualizaAno(0);
        assertEquals(formateador.getAnio(),0);
        verify(vehiculo);
        verify(combustibles);
    }

    @Test
    public void getKm(){
        expect(combustibles.size()).andReturn(0);
        expect(vehiculo.getKilometraje()).andReturn(0.0);
        replay(combustibles);
        replay(vehiculo);
        formateador.setInitInfo(combustibles,vehiculo);
        assertEquals(formateador.getKm(),0.0,0);
        verify(vehiculo);
        verify(combustibles);
    }

    @Test
    public void setKm(){
        expect(combustibles.size()).andReturn(0);
        expect(vehiculo.getKilometraje()).andReturn(0.0);
        vehiculo.setKilometraje(0.0);
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        replay(combustibles);
        replay(vehiculo);
        formateador.setInitInfo(combustibles,vehiculo);
        formateador.actualizaKm(0.0);
        assertEquals(formateador.getKm(),0.0,0);
        verify(vehiculo);
        verify(combustibles);
    }

    @Test
    public void getCPrim(){
        expect(combustibles.size()).andReturn(0);
        expect(vehiculo.getCombustiblePrimario()).andReturn("");
        replay(combustibles);
        replay(vehiculo);
        formateador.setInitInfo(combustibles,vehiculo);
        assertEquals(formateador.getCPrim(),"");
        verify(vehiculo);
        verify(combustibles);
    }




    @Test
    public void getCSec(){
        expect(combustibles.size()).andReturn(0);
        expect(vehiculo.getCombustibleSecundario()).andReturn("");
        replay(combustibles);
        replay(vehiculo);
        formateador.setInitInfo(combustibles,vehiculo);
        assertEquals(formateador.getCSec(),"");
        verify(vehiculo);
        verify(combustibles);
    }
    @Test
    public void getCapPrim(){
        expect(combustibles.size()).andReturn(0);
        expect(vehiculo.getCapTankPrimario()).andReturn(0.0);
        replay(combustibles);
        replay(vehiculo);
        formateador.setInitInfo(combustibles,vehiculo);
        assertEquals(formateador.getCapPrim(),0.0,0);
        verify(vehiculo);
        verify(combustibles);
    }

    @Test
    public void getCapSec(){
        expect(combustibles.size()).andReturn(0);
        expect(vehiculo.getCapTankSecundario()).andReturn(0.0);
        replay(combustibles);
        replay(vehiculo);
        formateador.setInitInfo(combustibles,vehiculo);
        assertEquals(formateador.getCapSec(),0.0,0);
        verify(vehiculo);
        verify(combustibles);
    }

    @Test
    public void getFoto(){
        expect(combustibles.size()).andReturn(0);
        expect(vehiculo.getFoto()).andReturn(null);
        replay(combustibles);
        replay(vehiculo);
        formateador.setInitInfo(combustibles,vehiculo);
        assertEquals(formateador.getFoto(),null);
        verify(vehiculo);
        verify(combustibles);
    }

    @Test
    public void setFoto(){
        expect(combustibles.size()).andReturn(0);
        expect(vehiculo.getFoto()).andReturn(null);
        vehiculo.setFoto(null);
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        replay(combustibles);
        replay(vehiculo);
        formateador.setInitInfo(combustibles,vehiculo);
        formateador.actualizaFoto(null);
        assertEquals(formateador.getFoto(),null);
        verify(vehiculo);
        verify(combustibles);
    }
}