package com.example.juanalonsoastruga.cargascontrol.Formateador.CURegistrarRecarga;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTORecarga;
import com.example.juanalonsoastruga.cargascontrol.Controlador.CURegistrarRecarga.InterfaceCRegistrarRecarga;
import com.example.juanalonsoastruga.cargascontrol.Controlador.CURegistrarRecarga.ControladorRegistrarRecarga;

import org.easymock.EasyMock;
import org.easymock.IAnswer;
import org.easymock.Mock;
import org.junit.Before;
import org.junit.Test;

import static org.easymock.EasyMock.createMock;
import static org.easymock.EasyMock.isA;
import static org.easymock.EasyMock.replay;
import static org.easymock.EasyMock.verify;
import static org.junit.Assert.*;

public class FormateadorRegistrarRecargaTest {

    private FormateadorRegistrarRecarga formateador;

    @Mock
    private InterfaceCRegistrarRecarga controlador;



    @Before
    public void setUp(){
        formateador = new FormateadorRegistrarRecarga();
        controlador =createMock(ControladorRegistrarRecarga.class);
    }


    @Test(expected = IllegalArgumentException.class)
    public void setPresenterNull(){
        formateador.setControlador(null);
    }

    @Test
    public void setPresenterNotNull(){
        formateador.setControlador(controlador);

        assertNotNull(formateador.getControlador());
    }

    @Test
    public void comienzoCU(){
        controlador.comienzoCU();
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                formateador.setDinamicData();
                return null;
            }
        });
        replay(controlador);
        formateador.setControlador(controlador);
        formateador.comienzoCU();
        verify(controlador);
    }
    @Test
    public void comprobarDespertarPresentador(){
        assertNull(formateador.getRecarga());
        //Metodo Void
        controlador.comienzoCU();
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                formateador.setDinamicData();
                return null;
            }
        });

        EasyMock.replay(controlador);
        controlador.comienzoCU();
        assertNotNull(formateador.getRecarga());
        verify(controlador);
    }

    @Test
    public void registrarSinFecha(){
        controlador.registroSinFecha();
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {

                return null;
            }
        });
        replay(controlador);
        formateador.setControlador(controlador);
        formateador.registrarSinFecha();
        verify(controlador);
    }

    @Test
    public void comprobarRegistroSinFecha(){
        assertFalse(formateador.isSinFecha());

        controlador.registroSinFecha();
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                formateador.setSinFecha();
                return null;
            }
        });

        replay(controlador);
        controlador.registroSinFecha();
        assertTrue(formateador.isSinFecha());
        assertFalse(formateador.isRegistroCorrecto());
        verify(controlador);
    }

    @Test
    public void registrarSinTiempo(){
        controlador.registroSinTiempo();
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {

                return null;
            }
        });
        replay(controlador);
        formateador.setControlador(controlador);
        formateador.registrarSinTiempo();
        verify(controlador);
    }

    @Test
    public void comprobarRegistroSinTiempo(){
        assertFalse(formateador.isSinTiempo());

        controlador.registroSinTiempo();
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                formateador.setSinTiempo();
                return null;
            }
        });

        replay(controlador);
        controlador.registroSinTiempo();
        assertTrue(formateador.isSinTiempo());
        assertFalse(formateador.isRegistroCorrecto());
        verify(controlador);
    }

    @Test
    public void comprobarRegistroSinTiempoNiFecha(){
        assertFalse(formateador.isSinTiempo());
        assertFalse(formateador.isSinFecha());

        controlador.registroSinTiempo();
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                formateador.setSinTiempo();
                return null;
            }
        });
        controlador.registroSinFecha();
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                formateador.setSinFecha();
                return null;
            }
        });

        replay(controlador);
        controlador.registroSinTiempo();
        controlador.registroSinFecha();
        formateador.setRegistroCompletado(true);
        assertTrue(formateador.isSinTiempo());
        assertTrue(formateador.isSinFecha());
        assertFalse(formateador.isRegistroCorrecto());
        verify(controlador);
    }

    @Test
    public void comprobarRegistroConTiempoyFechaSinCamposCompletos(){
        assertFalse(formateador.isSinTiempo());
        assertFalse(formateador.isSinFecha());

         formateador.setConTiempo();
         formateador.setConFecha();


        formateador.setRegistroCompletado(false);
        assertFalse(formateador.isSinTiempo());
        assertFalse(formateador.isSinFecha());
        assertFalse(formateador.isRegistroCorrecto());
    }

    @Test
    public void comprobarRegistroConTiempoyFechaConCamposCompletos(){
        assertFalse(formateador.isSinTiempo());
        assertFalse(formateador.isSinFecha());

        formateador.setConTiempo();
        formateador.setConFecha();


        formateador.setRegistroCompletado(true);
        assertFalse(formateador.isSinTiempo());
        assertFalse(formateador.isSinFecha());
        assertTrue(formateador.isRegistroCorrecto());
    }

    @Test   (expected = IllegalArgumentException.class)
    public void comprobarRegistroFechaAnoNulo(){
       formateador.registrarFechayTiempo(null,0,0,0,0);
    }

    @Test   (expected = IllegalArgumentException.class)
    public void comprobarRegistroFechaMesNulo(){
        formateador.registrarFechayTiempo(0,null,0,0,0);
    }

    @Test   (expected = IllegalArgumentException.class)
    public void comprobarRegistroFechaDiaNulo(){
        formateador.registrarFechayTiempo(0,0,null,0,0);
    }

    @Test   (expected = IllegalArgumentException.class)
    public void comprobarRegistroFechaHoraNulo(){
        formateador.registrarFechayTiempo(0,0,0,null,0);
    }

    @Test   (expected = IllegalArgumentException.class)
    public void comprobarRegistroFechaMinutoNulo(){
        formateador.registrarFechayTiempo(0,0,0,0,null);
    }

    @Test   (expected = IllegalArgumentException.class)
    public void comprobarRegistroMatriculaNulo(){
        formateador.registrarMatricula(null);
    }

    @Test   (expected = IllegalArgumentException.class)
    public void comprobarRegistroKmNulo(){
        formateador.registrarKm(null);
    }

    @Test   (expected = IllegalArgumentException.class)
    public void comprobarRegistroCosteNulo(){
        formateador.registrarCoste(null);
    }

    @Test   (expected = IllegalArgumentException.class)
    public void comprobarRegistroCombstibleNulo(){
        formateador.registrarCombustible(null);
    }

    @Test   (expected = IllegalArgumentException.class)
    public void comprobarRegistroRecargaNulo(){
        formateador.registrarRecarga(null);
    }


    @Test
    public void setMsgError(){

        formateador.setMsgError("");
        assertEquals("", formateador.getMsgError());
    }

    @Test
    public void registroConCamposIncompletos(){

        formateador.registroConCamposIncompletos();
        assertFalse(formateador.isCamposRegistroCompletos());
    }
    @Test
    public void registroConCamposCompletos(){

        formateador.registroConCamposCompletos();
        assertTrue(formateador.isCamposRegistroCompletos());
    }

    @Test(expected=IllegalArgumentException.class)
    public void registrarMatriculaNull(){

        formateador.registrarMatricula(null);

    }
    @Test
    public void solicitarRegistro(){
        controlador.registrarRecarga(isA(DTORecarga.class));
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {

                return null;
            }
        });replay(controlador);
        formateador.setControlador(controlador);
        formateador.setDinamicData();
        formateador.solicitarRegistro();
        verify(controlador);
    }
}
