package com.example.juanalonsoastruga.cargascontrol.Formateador.CURegistrarVehiculo;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTOCombustible;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTOVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Controlador.CURegistrarVehiculo.ControladorRegistroVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Controlador.CURegistrarVehiculo.InterfaceCRegistroVehiculo;

import org.easymock.IAnswer;
import org.easymock.Mock;
import org.junit.Before;
import org.junit.Test;

import java.util.List;

import static org.easymock.EasyMock.createMock;
import static org.easymock.EasyMock.expect;
import static org.easymock.EasyMock.expectLastCall;
import static org.easymock.EasyMock.replay;
import static org.easymock.EasyMock.verify;
import static org.junit.Assert.*;

public class FormateadorRegistroVehiculoTest {

    private FormateadorRegistroVehiculo formateador;
    @Mock
    private InterfaceCRegistroVehiculo controlador;
    @Mock
    private List<DTOCombustible> combustibles;
    @Mock
    private DTOVehiculo vehiculo;
    @Mock
    private DTOCombustible combustible;

    @Before
    public void setUp(){
        formateador = new FormateadorRegistroVehiculo();
        controlador =createMock(ControladorRegistroVehiculo.class);
        vehiculo=createMock(DTOVehiculo.class);
        combustibles=createMock(List.class);
        combustible=createMock(DTOCombustible.class);
    }
    @Test(expected = IllegalArgumentException.class)
    public void setPresenterNull(){
        formateador.setControlador(null);
    }
    @Test
    public void setPresenterNotNull(){
        formateador.setControlador(controlador);

        assertNotNull(formateador.getControlador());
    }

    @Test
    public void init(){
        controlador.eventoComienzo();
        expectLastCall().andAnswer(new IAnswer<Object>() {
            @Override
            public Object answer() throws Throwable {
                return null;
            }
        });
    }

    @Test
    public void registroVehiculoNulo(){
        assertNull(formateador.getRegistro());
    }

    @Test
    public void registroVehiculoNoNulo(){
        formateador.setControlador(controlador);
        formateador.init();
        assertNotNull(formateador.getRegistro());
    }

    @Test(expected=NullPointerException.class)
    public void getInfoSpinnersNull(){
        formateador.getInfoSpinners();
    }

    @Test
    public void getInfoSpinnersNotNull(){
        expect(combustibles.size()).andReturn(1).times(2);
        expect(combustibles.get(0)).andReturn(combustible);
        expect(combustible.getTipo()).andReturn("");
        expect(combustible.getUnidades()).andReturn("");
        replay(combustible);
        replay(combustibles);
        formateador.setSpinnersCombustible(combustibles);
        formateador.getInfoSpinners();
        verify(combustibles);
        verify(combustible);
    }

    @Test
    public void solicitarGuardado(){
        expect(controlador.solicitarGuardado()).andReturn(true);
        replay(controlador);
        controlador.solicitarGuardado();
        verify(controlador);
    }

    @Test
    public void RegistroNoCompletado(){
        formateador.setControlador(controlador);
        formateador.init();
        assertFalse(formateador.isRegistroCompletado());
    }
    @Test
    public void RegistroCompletado(){
        formateador.setControlador(controlador);
        formateador.init();
        formateador.setRegistroCompletado(true);
        assertTrue(formateador.isRegistroCompletado());
    }

    @Test
    public void setMensajeErrorNull(){
        formateador.setMensajeError(null);
        assertNull(formateador.getMensajeError());
    }
    @Test
    public void setMensajeErrorNotNull(){
        formateador.setMensajeError("");
        assertNotNull(formateador.getMensajeError());
    }
    @Test
    public void setNombre(){
        controlador.eventoComienzo();
        expectLastCall().andAnswer(new IAnswer<Object>() {
            @Override
            public Object answer() throws Throwable {
                return null;
            }
        });

        replay(controlador);
        formateador.setControlador(controlador);
        formateador.init();
        formateador.actualizaNombre("");
        verify(controlador);
    }

    @Test
    public void setMatricula(){
        controlador.eventoComienzo();
        expectLastCall().andAnswer(new IAnswer<Object>() {
            @Override
            public Object answer() throws Throwable {
                return null;
            }
        });

        replay(controlador);
        formateador.setControlador(controlador);
        formateador.init();
        formateador.actualizaMatricula("");
        verify(controlador);
    }

    @Test
    public void setMarca(){
        controlador.eventoComienzo();
        expectLastCall().andAnswer(new IAnswer<Object>() {
            @Override
            public Object answer() throws Throwable {
                return null;
            }
        });

        replay(controlador);
        formateador.setControlador(controlador);
        formateador.init();
        formateador.actualizaMarca("");
        verify(controlador);
    }

    @Test
    public void setModelo(){
        controlador.eventoComienzo();
        expectLastCall().andAnswer(new IAnswer<Object>() {
            @Override
            public Object answer() throws Throwable {
                return null;
            }
        });

        replay(controlador);
        formateador.setControlador(controlador);
        formateador.init();
        formateador.actualizaModelo("");
        verify(controlador);
    }

    @Test
    public void setAno(){
        controlador.eventoComienzo();
        expectLastCall().andAnswer(new IAnswer<Object>() {
            @Override
            public Object answer() throws Throwable {
                return null;
            }
        });

        replay(controlador);
        formateador.setControlador(controlador);
        formateador.init();
        formateador.actualizaAno(0);
        verify(controlador);
    }
    @Test
    public void setKm(){
        controlador.eventoComienzo();
        expectLastCall().andAnswer(new IAnswer<Object>() {
            @Override
            public Object answer() throws Throwable {
                return null;
            }
        });

        replay(controlador);
        formateador.setControlador(controlador);
        formateador.init();
        formateador.actualizaKm(0.0);
        verify(controlador);
    }
    @Test
    public void setCPrimario(){
        controlador.eventoComienzo();
        expectLastCall().andAnswer(new IAnswer<Object>() {
            @Override
            public Object answer() throws Throwable {
                return null;
            }
        });

        replay(controlador);
        formateador.setControlador(controlador);
        formateador.init();
        formateador.actualizaCPrimario("");
        verify(controlador);
    }
    @Test
    public void setCapTP(){
        controlador.eventoComienzo();
        expectLastCall().andAnswer(new IAnswer<Object>() {
            @Override
            public Object answer() throws Throwable {
                return null;
            }
        });

        replay(controlador);
        formateador.setControlador(controlador);
        formateador.init();
        formateador.actualizaCapTP(0.0);
        verify(controlador);
    }
    @Test
    public void setCSecundario(){
        controlador.eventoComienzo();
        expectLastCall().andAnswer(new IAnswer<Object>() {
            @Override
            public Object answer() throws Throwable {
                return null;
            }
        });

        replay(controlador);
        formateador.setControlador(controlador);
        formateador.init();
        formateador.actualizaCSecundario("");
        verify(controlador);
    }
    @Test
    public void setCapTS(){
        controlador.eventoComienzo();
        expectLastCall().andAnswer(new IAnswer<Object>() {
            @Override
            public Object answer() throws Throwable {
                return null;
            }
        });

        replay(controlador);
        formateador.setControlador(controlador);
        formateador.init();
        formateador.actualizaCapTS(0.0);
        verify(controlador);
    }
    @Test
    public void setFoto(){
        controlador.eventoComienzo();
        expectLastCall().andAnswer(new IAnswer<Object>() {
            @Override
            public Object answer() throws Throwable {
                return null;
            }
        });

        replay(controlador);
        formateador.setControlador(controlador);
        formateador.init();
        formateador.actualizaFoto(null);
        verify(controlador);
    }
}