package com.example.juanalonsoastruga.cargascontrol.Modelo.CURegistrarRecarga;

import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTODatosDerivadosCombustible;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTODatosDerivadosCostesCombustible;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTODatosDerivadosVehiculoTotales;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTORecarga;
import com.example.juanalonsoastruga.cargascontrol.CapaTransversal.DTOVehiculo;
import com.example.juanalonsoastruga.cargascontrol.Datos.FachadaDatos;
import com.example.juanalonsoastruga.cargascontrol.Datos.InterfaceFachadaDatos;

import org.easymock.EasyMock;
import org.easymock.IAnswer;
import org.easymock.Mock;
import org.junit.Before;
import org.junit.Test;

import java.util.Date;

import static org.easymock.EasyMock.anyInt;
import static org.easymock.EasyMock.createControl;
import static org.easymock.EasyMock.createMock;
import static org.easymock.EasyMock.expect;
import static org.easymock.EasyMock.expectLastCall;
import static org.easymock.EasyMock.isA;
import static org.easymock.EasyMock.replay;
import static org.easymock.EasyMock.verify;
import static org.junit.Assert.*;

public class ModeloRegistrarRecargaTest {
    private ModeloRegistrarRecarga modelo;
    @Mock
    private DTORecarga recarga;
    @Mock
    private DTOVehiculo vehiculo;
    @Mock
    private InterfaceFachadaDatos datos;
    @Mock
    private DTODatosDerivadosVehiculoTotales dtoDD;
    @Mock
    private DTODatosDerivadosCombustible dtoCombustible;
    @Mock
    private DTODatosDerivadosCostesCombustible dtoCostesCombustible;

    @Before
    public void setUp(){
        modelo= new ModeloRegistrarRecarga();
        recarga=createMock(DTORecarga.class);
        vehiculo=createMock(DTOVehiculo.class);
        datos=createMock(FachadaDatos.class);
        dtoDD=createMock(DTODatosDerivadosVehiculoTotales.class);
        dtoCombustible=createMock(DTODatosDerivadosCombustible.class);
        dtoCostesCombustible=createMock(DTODatosDerivadosCostesCombustible.class);
    }

    @Test(expected = IllegalArgumentException.class)
    public void setRepoNull(){
        modelo.setRepo(null);

    }


    @Test
    public void setRepoNotNull(){
        modelo.setRepo(datos);
        assertNotNull(modelo.getDatos());
    }

    @Test
    public void getDatosDerivadosVehiculo(){
       expect(datos.getDDerivadosVehiculo(isA(String.class),anyInt(),anyInt())).andReturn(dtoDD);
       replay(datos);
       modelo.setRepo(datos);
       assertEquals(modelo.getDDVehiculo("",0,0),dtoDD);
       verify(datos);
    }

    @Test
    public void isPrimerRegistroDatosDerivadosVehiculoUltimoNull(){
        expect(datos.getUltimoRegistroDDerivadosVehiculo(isA(String.class))).andReturn(null);
        replay(datos);
        modelo.setRepo(datos);
        assertTrue(modelo.isPrimerRegistroDatosDerivadosVehiculo(""));
        verify(datos);
    }

    @Test
    public void isPrimerRegistroDatosDerivadosVehiculoUltimoNotNull(){
        expect(datos.getUltimoRegistroDDerivadosVehiculo(isA(String.class))).andReturn(dtoDD);
        replay(datos);
        modelo.setRepo(datos);
        assertFalse(modelo.isPrimerRegistroDatosDerivadosVehiculo(""));
        verify(datos);
    }

    @Test
    public void addDatosDerivadosVehiculo(){
        datos.registraDatosDerivadosVehiculo(dtoDD);
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        replay(datos);
        modelo.setRepo(datos);
        modelo.addDatosDerivadosVehiculo(dtoDD);
        verify(datos);
    }

    @Test
    public void getLastRegistroDatosDerivadosVehiculo(){
        expect(datos.getUltimoRegistroDDerivadosVehiculo(isA(String.class))).andReturn(dtoDD);

        replay(datos);
        modelo.setRepo(datos);
        modelo.getLastRegistroDatosDerivadosVehiculo("");
        verify(datos);
    }

    @Test
    public void actualizaDatosDerivadosVehiculo(){
        datos.actualizaDatosDerivadosVehiculo(dtoDD);
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        replay(datos);
        modelo.setRepo(datos);
        modelo.actualizaDatosDerivadosVehiculo(dtoDD);
        verify(datos);
    }

    @Test
    public void getDDVCombustible(){
        expect(datos.getDDerivadosCombustible(isA(String.class),anyInt(),anyInt(),isA(String.class))).andReturn(dtoCombustible);

        replay(datos);
        modelo.setRepo(datos);
        assertEquals(modelo.getDDVCombustible("",0,0,""),dtoCombustible);
        verify(datos);
    }

    @Test
    public void isPrimerRegistroDatosDerivadosCombustibleTrue(){
        expect(datos.getUltimoRegistroDDerivadosCombustible(isA(String.class),isA(String.class))).andReturn(null);

        replay(datos);
        modelo.setRepo(datos);
        assertTrue(modelo.isPrimerRegistroDatosDerivadosCombustible("",""));
        verify(datos);
    }

    @Test
    public void isPrimerRegistroDatosDerivadosCombustibleFalse(){
        expect(datos.getUltimoRegistroDDerivadosCombustible(isA(String.class),isA(String.class))).andReturn(dtoCombustible);

        replay(datos);
        modelo.setRepo(datos);
        assertFalse(modelo.isPrimerRegistroDatosDerivadosCombustible("",""));
        verify(datos);
    }

    @Test
    public void addDatosDerivadosCombustible(){
        datos.registraDatosDerivadosCombustible(dtoCombustible);
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        replay(datos);
        modelo.setRepo(datos);
        modelo.addDatosDerivadosCombustible(dtoCombustible);
        verify(datos);
    }

    @Test
    public void getLastRegistroDatosDerivadosCombustible(){
        expect(datos.getUltimoRegistroDDerivadosCombustible(isA(String.class),isA(String.class))).andReturn(dtoCombustible);

        replay(datos);
        modelo.setRepo(datos);
        assertEquals(modelo.getLastRegistroDatosDerivadosCombustible("",""),dtoCombustible);
        verify(datos);
    }


    @Test
    public void actualizaDatosDerivadosCombustible(){
        datos.actualizaDatosDerivadosCombustible(dtoCombustible);
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        replay(datos);
        modelo.setRepo(datos);
        modelo.actualizaDatosDerivadosCombustible(dtoCombustible);
        verify(datos);
    }

    @Test
    public void getDDVCostesCombustible(){
        expect(datos.getDDerivadosCosteCombustible(isA(String.class),anyInt(),anyInt(),isA(String.class))).andReturn(dtoCostesCombustible);

        replay(datos);
        modelo.setRepo(datos);
        assertEquals(modelo.getDDVCostesCombustible("",0,0,""),dtoCostesCombustible);
        verify(datos);
    }

    @Test
    public void getLastRegistroDatosDerivadosCostesCombustible(){
        expect(datos.getUltimoRegistroDDerivadosCosteCombustible(isA(String.class),isA(String.class))).andReturn(dtoCostesCombustible);

        replay(datos);
        modelo.setRepo(datos);
        assertEquals(modelo.getLastRegistroDatosDerivadosCostesCombustible("",""),dtoCostesCombustible);
        verify(datos);
    }

    @Test
    public void actualizaDatosDerivadosCosteCombustible(){
        datos.actualizaDatosDerivadosCosteCombustible(dtoCostesCombustible);
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        replay(datos);
        modelo.setRepo(datos);
        modelo.actualizaDatosDerivadosCosteCombustible(dtoCostesCombustible);
        verify(datos);
    }

    @Test
    public void addDatosDerivadosCosteCombustible(){
        datos.registraDatosDerivadosCosteCombustible(dtoCostesCombustible);
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        replay(datos);
        modelo.setRepo(datos);
        modelo.addDatosDerivadosCosteCombustible(dtoCostesCombustible);
        verify(datos);
    }

    @Test
    public void getUltimaRecargaAntesDe(){
        expect(datos.getLastRecargaBefore(isA(String.class),isA(Date.class),isA(String.class))).andReturn(recarga);

        replay(datos);
        modelo.setRepo(datos);
        assertEquals(modelo.getUltimaRecargaAntesDe("",new Date(),""),recarga);
        verify(datos);
    }

    @Test
    public void getUltimaRecargaRegistradaMismoCombustible(){
        expect(datos.getLastRecargaPorCombustible(isA(String.class),isA(String.class))).andReturn(recarga);

        replay(datos);
        modelo.setRepo(datos);
        assertEquals(modelo.getUltimaRecargaRegistradaMismoCombustible("",""),recarga);
        verify(datos);
    }

    @Test
    public void getUltimaRecargaRegistrada(){
        expect(datos.getLastRecargaBefore(isA(String.class),isA(Date.class))).andReturn(recarga);

        replay(datos);
        modelo.setRepo(datos);
        assertEquals(modelo.getUltimaRecargaRegistrada("",new Date()),recarga);
        verify(datos);
    }

    @Test(expected = IllegalArgumentException.class)
    public void registraRecargaNull(){
      modelo.registraRecarga(null);
    }

    @Test
    public void registraRecargaNotNull(){
        datos.setRecarga(recarga);
        EasyMock.expectLastCall().andAnswer(new IAnswer() {
            public Object answer() {
                return null;
            }
        });
        replay(datos);
        modelo.setRepo(datos);
        modelo.registraRecarga(recarga);
        verify(datos);
    }

    @Test
    public void getVehiculo(){
        expect(datos.getVehiculo(isA(String.class))).andReturn(vehiculo);

        replay(datos);
        modelo.setRepo(datos);
        assertEquals(modelo.getVehiculo("" ),vehiculo);
        verify(datos);
    }

}